@@ -20,24 +20,23 @@
 
 #include "ActionClasses.h"
 
-#include <config-amarok.h>
 
-#include "core/support/Amarok.h"
 #include "App.h"
-#include "core/support/Debug.h"
 #include "EngineController.h"
 #include "MainWindow.h"
 #include "amarokconfig.h"
+#include <config.h>
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistModelStack.h"
+#include "widgets/Osd.h"
+#include "KNotificationBackend.h"
 
 #include <KAuthorized>
 #include <KHelpMenu>
 #include <KLocale>
 #include <KToolBar>
-#include <Osd.h>
-#include <EqualizerDialog.h>
-
 
 extern OcsData ocsData;
 
@@ -152,13 +151,10 @@ Menu::helpMenu( QWidget *parent ) //STATIC
 
     KMenu* menu = s_helpMenu->menu();
 
-    // NOTE: We hide "Report Bug..." because we need to add it on our own to name the dialog
-    // so it can be blacklisted from LikeBack.
-    s_helpMenu->action( KHelpMenu::menuReportBug )->setVisible( false );
-
-    // NOTE: "What's This" isn't currently defined for anything in Amarok, so let's remove that too
+    // "What's This" isn't currently defined for anything in Amarok, so let's remove it
     s_helpMenu->action( KHelpMenu::menuWhatsThis )->setVisible( false );
 
+    // Hide the default "About App" dialog, as we replace it with a custom one
     s_helpMenu->action( KHelpMenu::menuAboutApp )->setVisible( false );
 
     return menu;
@@ -175,7 +171,7 @@ PlayPauseAction::PlayPauseAction( KActionCollection *ac, QObject *parent )
     ac->addAction( "play_pause", this );
     setText( i18n( "Play/Pause" ) );
     setShortcut( Qt::Key_Space );
-    setGlobalShortcut( KShortcut( Qt::Key_MediaPlay ) );
+    setGlobalShortcut( KShortcut() );
 
     EngineController *engine = The::engineController();
 
@@ -189,12 +185,12 @@ PlayPauseAction::PlayPauseAction( KActionCollection *ac, QObject *parent )
     connect( this, SIGNAL(triggered()),
              engine, SLOT(playPause()) );
 
-    connect( engine, SIGNAL( stopped( qint64, qint64 ) ),
-             this, SLOT( stopped() ) );
-    connect( engine, SIGNAL( paused() ),
-             this, SLOT( paused() ) );
-    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
-             this, SLOT( playing() ) );
+    connect( engine, SIGNAL(stopped(qint64,qint64)),
+             this, SLOT(stopped()) );
+    connect( engine, SIGNAL(paused()),
+             this, SLOT(paused()) );
+    connect( engine, SIGNAL(trackPlaying(Meta::TrackPtr)),
+             this, SLOT(playing()) );
 }
 
 void
@@ -329,64 +325,20 @@ RandomAction::setCurrentItem( int n )
 ReplayGainModeAction::ReplayGainModeAction( KActionCollection *ac, QObject *parent ) :
     SelectAction( i18n( "&Replay Gain Mode" ), &AmarokConfig::setReplayGainMode, ac, "replay_gain_mode", parent )
 {
-    setItems( QStringList() << i18nc( "Replay Gain state, as in, disabled", "&Off" ) << i18nc( "Item, as in, music", "&Track" )
+    setItems( QStringList() << i18nc( "Replay Gain state, as in, disabled", "&Off" )
+                            << i18nc( "Item, as in, music", "&Track" )
                             << i18n( "&Album" ) );
-    //setIcons( QStringList() << "media-playlist-replaygain-off-amarok" << "media-track-replaygain-amarok" << "media-album-replaygain-amarok" );
-    setCurrentItem( AmarokConfig::replayGainMode() );
-}
-
-//////////////////////////////////////////////////////////////////////////////////////////
-// EqualizerAction
-//////////////////////////////////////////////////////////////////////////////////////////
-EqualizerAction::EqualizerAction( KActionCollection *ac, QObject *parent ) :
-    SelectAction( i18n( "&Equalizer" ), &AmarokConfig::setEqualizerMode, ac, "equalizer_mode", parent )
-{
-    // build a new preset list in menu
-    newList();
-    // set selected preset from config
-    updateContent();
-    connect( this, SIGNAL( triggered( int ) ), this, SLOT( actTrigg( int ) ) );
-}
-
-void
-EqualizerAction::updateContent() //SLOT
-{
-    // this slot update the content of equalizer main window menu
-    // according to config blocking is necessary to prevent
-    // circluar loop between menu and config dialog
-    blockSignals( true );
-    setCurrentItem( AmarokConfig::equalizerMode() );
-    blockSignals( false );
-}
-
-void
-EqualizerAction::newList() //SLOT
-{
-    // this slot build a new list of presets in equalizer menu
-    // or disable this menu if equalizer is not supported
-    if( !The::engineController()->isEqSupported() )
+    EngineController *engine = EngineController::instance();
+    Q_ASSERT( engine );
+    if( engine->supportsGainAdjustments() )
+        setCurrentItem( AmarokConfig::replayGainMode() );
+    else
     {
-        setEnabled( false );
-        setToolTip( i18n("Your current setup does not support the equalizer feature") );
-        return;
+        // Note: it would be nice to set a tooltip that would explain why this is disabled
+        // to users, but tooltips aren't shown in meny anyway :-(
+        actions().at( 1 )->setEnabled( false );
+        actions().at( 2 )->setEnabled( false );
     }
-    setEnabled( true );
-    setToolTip( QString() );
-    setItems( QStringList() << i18nc( "Equalizer state, as in, disabled", "&Off" ) << EqualizerPresets::eqGlobalTranslatedList() );
-}
-
-void
-EqualizerAction::actTrigg( int index ) //SLOT
-{
-    if( !The::engineController()->isEqSupported() )
-        return;
-
-    const QString presetName = EqualizerPresets::eqGlobalList().at( index - 1 );
-    if (presetName.isEmpty())
-        return;
-
-    AmarokConfig::setEqualizerGains( EqualizerPresets::eqCfgGetPresetVal( presetName ) );
-    The::engineController()->eqUpdate();
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -410,7 +362,7 @@ BurnMenuAction::createWidget( QWidget *w )
 
         //addContainer( bar, id );
         w->addAction( this );
-        connect( bar, SIGNAL( destroyed() ), SLOT( slotDestroyed() ) );
+        connect( bar, SIGNAL(destroyed()), SLOT(slotDestroyed()) );
 
         //bar->insertButton( QString::null, id, true, i18n( "Burn" ), index );
 
@@ -434,8 +386,8 @@ BurnMenu::BurnMenu( QWidget* parent )
 {
     s_instance = this;
 
-    addAction( i18n("Current Playlist"), this, SLOT( slotBurnCurrentPlaylist() ) );
-    addAction( i18n("Selected Tracks"), this, SLOT( slotBurnSelectedTracks() ) );
+    addAction( i18n("Current Playlist"), this, SLOT(slotBurnCurrentPlaylist()) );
+    addAction( i18n("Selected Tracks"), this, SLOT(slotBurnSelectedTracks()) );
     //TODO add "album" and "all tracks by artist"
 }
 
@@ -468,8 +420,8 @@ StopAction::StopAction( KActionCollection *ac, QObject *parent )
     ac->addAction( "stop", this );
     setText( i18n( "Stop" ) );
     setIcon( KIcon("media-playback-stop-amarok") );
-    setGlobalShortcut( KShortcut( Qt::Key_MediaStop ) );
-    connect( this, SIGNAL( triggered() ), this, SLOT( stop() ) );
+    setGlobalShortcut( KShortcut() );
+    connect( this, SIGNAL(triggered()), this, SLOT(stop()) );
 
     EngineController *engine = The::engineController();
 
@@ -478,10 +430,10 @@ StopAction::StopAction( KActionCollection *ac, QObject *parent )
     else
         playing();
 
-    connect( engine, SIGNAL( stopped( qint64, qint64 ) ),
-             this, SLOT( stopped() ) );
-    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
-             this, SLOT( playing() ) );
+    connect( engine, SIGNAL(stopped(qint64,qint64)),
+             this, SLOT(stopped()) );
+    connect( engine, SIGNAL(trackPlaying(Meta::TrackPtr)),
+             this, SLOT(playing()) );
 
 }
 
@@ -500,12 +452,6 @@ StopAction::playing()
 void
 StopAction::stop()
 {
-    if( The::playlistActions()->stopAfterMode() )
-    {
-        The::playlistActions()->setStopAfterMode( Playlist::StopNever );
-        The::playlistActions()->setTrackToBeLast( 0 );
-        The::playlistActions()->repaintPlaylist();
-    }
     The::engineController()->stop();
 }
 
@@ -521,27 +467,32 @@ StopPlayingAfterCurrentTrackAction::StopPlayingAfterCurrentTrackAction( KActionC
     setText( i18n( "Stop after current Track" ) );
     setIcon( KIcon("media-playback-stop-amarok") );
     setGlobalShortcut( KShortcut( Qt::META + Qt::SHIFT + Qt::Key_V ) );
-    connect( this, SIGNAL( triggered() ), SLOT( stopPlayingAfterCurrentTrack() ) );
+    connect( this, SIGNAL(triggered()), SLOT(stopPlayingAfterCurrentTrack()) );
 }
 
 void
 StopPlayingAfterCurrentTrackAction::stopPlayingAfterCurrentTrack()
 {
-    if ( !The::playlistActions()->willStopAfterTrack( Playlist::ModelStack::instance()->bottom()->activeId() ) )
+    QString text;
+
+    quint64 activeTrack = Playlist::ModelStack::instance()->bottom()->activeId();
+    if( activeTrack )
     {
-        The::playlistActions()->setStopAfterMode( Playlist::StopAfterCurrent );
-        The::playlistActions()->setTrackToBeLast( Playlist::ModelStack::instance()->bottom()->activeId() );
-        The::playlistActions()->repaintPlaylist();
-        Amarok::OSD::instance()->setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
-        Amarok::OSD::instance()->OSDWidget::show( i18n( "Stop after current track: On" ) );
-    } else {
-        The::playlistActions()->setStopAfterMode( Playlist::StopNever );
-        The::playlistActions()->setTrackToBeLast( 0 );
-        The::playlistActions()->repaintPlaylist();
-        Amarok::OSD::instance()->setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
-        Amarok::OSD::instance()->OSDWidget::show( i18n( "Stop after current track: Off" ) );
+        if( The::playlistActions()->willStopAfterTrack( activeTrack ) )
+        {
+            The::playlistActions()->stopAfterPlayingTrack( 0 );
+            text = i18n( "Stop after current track: Off" );
+        }
+        else
+        {
+            The::playlistActions()->stopAfterPlayingTrack( activeTrack );
+            text = i18n( "Stop after current track: On" );
+        }
     }
+    else
+        text = i18n( "No track playing" );
+
+    Amarok::OSD::instance()->OSDWidget::show( text );
+    if( Amarok::KNotificationBackend::instance()->isEnabled() )
+        Amarok::KNotificationBackend::instance()->show( i18n( "Amarok" ), text );
 }
-
-#include "ActionClasses.moc"
-
