@@ -16,52 +16,49 @@
 
 #include "App.h"
 
-#include <config-amarok.h>
-
-#include "core/support/Amarok.h"
+#include <config.h>
+#include "EngineController.h"
+#include "KNotificationBackend.h"
+#include "PluginManager.h"
+#include "scripting/scriptmanager/ScriptManager.h"
+#include "TrayIcon.h"
 #include "amarokconfig.h"
 #include "amarokurls/AmarokUrl.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Components.h"
-#include "core/interfaces/Logger.h"
-#include "ConfigDialog.h"
-#include "covermanager/CoverFetcher.h"
-#include "dialogs/EqualizerDialog.h"
-#include "dbus/CollectionDBusHandler.h"
-#include "core/support/Debug.h"
-#include "EngineController.h"
-#include "firstruntutorial/FirstRunTutorial.h"
-#include "KNotificationBackend.h"
+#include "configdialog/ConfigDialog.h"
+#include "configdialog/dialogs/PlaybackConfig.h"
 #include "core/capabilities/SourceInfoCapability.h"
-#include "core/meta/support/MetaConstants.h"
+#include "core/interfaces/Logger.h"
 #include "core/meta/Meta.h"
+#include "core/meta/support/MetaConstants.h"
 #include "core/meta/support/MetaUtility.h"
-#include "core/transcoding/TranscodingController.h"
-#include "network/NetworkAccessManagerProxy.h"
-#include "Osd.h"
-#include "PlaybackConfig.h"
-#include "dbus/mpris1/PlayerHandler.h"
 #include "core/playlists/Playlist.h"
 #include "core/playlists/PlaylistFormat.h"
-#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
-#include "playlist/PlaylistActions.h"
-#include "playlist/PlaylistModelStack.h"
-#include "playlist/PlaylistController.h"
-#include "playlistmanager/PlaylistManager.h"
-#include "core/plugins/PluginManager.h"
 #include "core/podcasts/PodcastProvider.h"
+#include "core/support/Amarok.h"
+#include "core/support/Components.h"
+#include "core/support/Debug.h"
+#include "core/transcoding/TranscodingController.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
+#include "core-impl/storage/StorageManager.h"
+#include "covermanager/CoverCache.h"
+#include "covermanager/CoverFetcher.h"
+#include "dbus/CollectionDBusHandler.h"
+#include "dbus/mpris1/PlayerHandler.h"
 #include "dbus/mpris1/RootHandler.h"
-#include "ScriptManager.h"
+#include "dbus/mpris1/TrackListHandler.h"
+#include "dbus/mpris2/Mpris2.h"
+#include "network/NetworkAccessManagerProxy.h"
+#include "playlist/PlaylistActions.h"
+#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModelStack.h"
+#include "playlistmanager/PlaylistManager.h"
+#include "services/ServicePluginManager.h"
+#include "scripting/scriptconsole/ScriptConsole.h"
 #include "statemanagement/ApplicationController.h"
 #include "statemanagement/DefaultApplicationController.h"
-#include "dbus/mpris1/TrackListHandler.h"
-#include "dbus/mpris2/Mpris2DBusHandler.h"
-#include "TrayIcon.h"
-
-
-#ifdef NO_MYSQL_EMBEDDED
-#include "MySqlServerTester.h"
-#endif
+#include "statsyncing/Controller.h"
+#include "widgets/Osd.h"
 
 #include <iostream>
 
@@ -74,7 +71,7 @@
 #include <KIO/CopyJob>
 #include <KJob>
 #include <KJobUiDelegate>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KShortcutsDialog>              //slotConfigShortcuts()
 #include <KStandardDirs>
@@ -82,39 +79,17 @@
 #include <QByteArray>
 #include <QDesktopServices>
 #include <QFile>
-#include <KPixmapCache>
 #include <QStringList>
 #include <QTextDocument>                // for Qt::escape()
 #include <QTimer>                       //showHyperThreadingWarning()
 #include <QtDBus/QtDBus>
 
-#include "shared/taglib_filetype_resolvers/asffiletyperesolver.h"
-#include "shared/taglib_filetype_resolvers/mimefiletyperesolver.h"
-#include "shared/taglib_filetype_resolvers/mp4filetyperesolver.h"
-#include "shared/taglib_filetype_resolvers/wavfiletyperesolver.h"
-#include <audiblefiletyperesolver.h>
-#include <realmediafiletyperesolver.h>
-
-int App::mainThreadId = 0;
-
 #ifdef Q_WS_MAC
 #include <CoreFoundation/CoreFoundation.h>
-extern void setupEventHandler_mac(long);
+extern void setupEventHandler_mac(SRefCon);
 #endif
 
-#ifdef DEBUG
-#include "TestDirectoryLoader.h"
-#endif // DEBUG
-
 QStringList App::s_delayedAmarokUrls = QStringList();
-
-AMAROK_EXPORT KAboutData aboutData( "amarok", 0,
-    ki18n( "Amarok" ), AMAROK_VERSION,
-    ki18n( "The audio player for KDE" ), KAboutData::License_GPL,
-    ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2010, The Amarok Development Squad" ),
-    ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: %1)" ).subs( __DATE__ ),
-             ( "http://amarok.kde.org" ) );
-
 AMAROK_EXPORT OcsData ocsData( "opendesktop" );
 
 App::App()
@@ -127,15 +102,6 @@ App::App()
     // required for last.fm plugin to grab app version
     setApplicationVersion( AMAROK_VERSION );
 
-    PERF_LOG( "Registering taglib plugins" )
-    TagLib::FileRef::addFileTypeResolver(new RealMediaFileTypeResolver);
-    TagLib::FileRef::addFileTypeResolver(new AudibleFileTypeResolver);
-    TagLib::FileRef::addFileTypeResolver(new WAVFileTypeResolver);
-    TagLib::FileRef::addFileTypeResolver(new MP4FileTypeResolver);
-    TagLib::FileRef::addFileTypeResolver(new ASFFileTypeResolver);
-    TagLib::FileRef::addFileTypeResolver(new MimeFileTypeResolver);
-    PERF_LOG( "Done Registering taglib plugins" )
-
     qRegisterMetaType<Meta::DataPtr>();
     qRegisterMetaType<Meta::DataList>();
     qRegisterMetaType<Meta::TrackPtr>();
@@ -152,12 +118,10 @@ App::App()
     qRegisterMetaType<Meta::YearList>();
     qRegisterMetaType<Meta::LabelPtr>();
     qRegisterMetaType<Meta::LabelList>();
+    qRegisterMetaType<Playlists::PlaylistPtr>();
+    qRegisterMetaType<Playlists::PlaylistList>();
 
 
-    //make sure we have enough cache space for all our crazy svg stuff
-    KPixmapCache cache( "Amarok-pixmaps" );
-    cache.setCacheLimit ( 20 * 1024 );
-
 #ifdef Q_WS_MAC
     // this is inspired by OpenSceneGraph: osgDB/FilePath.cpp
 
@@ -201,7 +165,7 @@ App::App()
         }
     }
 
-    setupEventHandler_mac((long)this);
+    setupEventHandler_mac(this);
 #endif
 
     PERF_LOG( "Done App ctor" )
@@ -232,6 +196,7 @@ App::~App()
         {
             AmarokConfig::setResumeTrack( engineTrack->playableUrl().prettyUrl() );
             AmarokConfig::setResumeTime( The::engineController()->trackPositionMs() );
+            AmarokConfig::setResumePaused( The::engineController()->isPaused() );
         }
         else
             AmarokConfig::setResumeTrack( QString() ); //otherwise it'll play previous resume next time!
@@ -263,17 +228,21 @@ App::~App()
     //mainWindow()->deleteBrowsers();
     delete mainWindow();
 
+    Playlist::Controller::destroy();
     Playlist::ModelStack::destroy();
     Playlist::Actions::destroy();
     PlaylistManager::destroy();
-    CollectionManager::destroy();
     CoverFetcher::destroy();
+    CoverCache::destroy();
+    ServicePluginManager::destroy();
+    CollectionManager::destroy();
+    StorageManager::destroy();
     NetworkAccessManagerProxy::destroy();
+    Plugins::PluginManager::destroy();
 
     //this should be moved to App::quit() I guess
     Amarok::Components::applicationController()->shutdown();
 
-
 #ifdef Q_WS_WIN
     // work around for KUniqueApplication being not completely implemented on windows
     QDBusConnectionInterface* dbusService;
@@ -309,26 +278,20 @@ App::handleCliArgs() //static
                 The::playlistManager()->defaultPodcasts()->addPodcast( feedUrl );
             }
             else if( url.protocol() == "amarok" )
-            {
                 s_delayedAmarokUrls.append( url.url() );
-            }
             else
-            {
                 list << url;
-                DEBUG_LINE_INFO
-            }
         }
 
-        int options = Playlist::AppendAndPlay;
+        Playlist::AddOptions options;
         if( args->isSet( "queue" ) )
-           options = Playlist::Queue;
+           options = Playlist::OnQueueToPlaylistAction;
         else if( args->isSet( "append" ) )
-           options = Playlist::Append;
+           options = Playlist::OnAppendToPlaylistAction;
         else if( args->isSet( "load" ) )
-            options = Playlist::Replace;
-
-        if( args->isSet( "play" ) )
-            options |= Playlist::DirectPlay;
+            options = Playlist::OnReplacePlaylistAction;
+        else
+            options = Playlist::OnPlayMediaAction;
 
         The::playlistController()->insertOptioned( list, options );
     }
@@ -376,44 +339,6 @@ App::handleCliArgs() //static
     }
 
     firstTime = false;
-
-#ifdef DEBUG
-    if( args->isSet( "test" ) )
-    {
-        bool ok;
-        int verboseInt = args->getOption( "verbose" ).toInt( &ok );
-        verboseInt     = ok ? verboseInt : 2;
-
-        QStringList testOpt( "amarok" );
-
-        QString verbosity;
-        switch( verboseInt )
-        {
-        case 0:
-            verbosity = "-silent";
-            break;
-        case 1:
-            verbosity = "-v1";
-            break;
-        default:
-        case 2:
-            verbosity = "-v2";
-            break;
-        case 3:
-            verbosity = "-vs";
-            break;
-        }
-        testOpt << verbosity;
-
-        const QString format = args->getOption( "format" );
-        if( format == "xml" || format == "lightxml" )
-            testOpt << QString( '-' + format );
-
-        const bool _stdout = ( args->getOption( "output" ) == "log" ) ? false : true;
-        runUnitTests( testOpt, _stdout );
-    }
-#endif // DEBUG
-
     args->clear();    //free up memory
 }
 
@@ -422,14 +347,6 @@ App::handleCliArgs() //static
 // INIT
 /////////////////////////////////////////////////////////////////////////////////////
 
-void
-App::initCliArgs( int argc, char *argv[] )
-{
-    KCmdLineArgs::reset();
-    KCmdLineArgs::init( argc, argv, &::aboutData ); //calls KCmdLineArgs::addStdCmdLineOptions()
-    initCliArgs();
-}
-
 void
 App::initCliArgs() //static
 {
@@ -457,18 +374,12 @@ App::initCliArgs() //static
     options.add("load", ki18n("Load URLs, replacing current playlist"));
     options.add("d");
     options.add("debug", ki18n("Print verbose debugging information"));
+    options.add("debug-audio", ki18n("Print verbose debugging information from the audio system"));
     options.add("c");
     options.add("coloroff", ki18n("Disable colorization for debug output."));
     options.add("m");
     options.add("multipleinstances", ki18n("Allow running multiple Amarok instances"));
     options.add("cwd <directory>", ki18n( "Base for relative filenames/URLs" ));
-#ifdef DEBUG
-    options.add(":", ki18n("Unit test options:"));
-    options.add("test", ki18n( "Run integrated unit tests" ) );
-    options.add("output <dest>", ki18n( "Destination of test output: 'stdout', 'log'" ), "log" );
-    options.add("format <type>", ki18n( "Format of test output: 'xml', 'lightxml', 'plaintext'" ), "xml" );
-    options.add("verbose <level>", ki18n( "Verbosity from 0-3 (highest)" ), "2" );
-#endif // DEBUG
 
     KCmdLineArgs::addCmdLineOptions( options );   //add our own options
 }
@@ -478,54 +389,6 @@ App::initCliArgs() //static
 // METHODS
 /////////////////////////////////////////////////////////////////////////////////////
 
-#include <id3v1tag.h>
-#include <tbytevector.h>
-#include <QTextCodec>
-#include <KGlobal>
-
-//this class is only used in this module, so I figured I may as well define it
-//here and save creating another header/source file combination
-
-// Local version of taglib's QStringToTString macro. It is here, because taglib's one is
-// not Qt3Support clean (uses QString::utf8()). Once taglib will be clean of qt3support
-// it is safe to use QStringToTString again
-#define Qt4QStringToTString(s) TagLib::String(s.toUtf8().data(), TagLib::String::UTF8)
-
-class ID3v1StringHandler : public TagLib::ID3v1::StringHandler
-{
-    QTextCodec *m_codec;
-
-    virtual TagLib::String parse( const TagLib::ByteVector &data ) const
-    {
-        return Qt4QStringToTString( m_codec->toUnicode( data.data(), data.size() ) );
-    }
-
-    virtual TagLib::ByteVector render( const TagLib::String &ts ) const
-    {
-        const QByteArray qcs = m_codec->fromUnicode( TStringToQString(ts) );
-        return TagLib::ByteVector( qcs, (uint) qcs.length() );
-    }
-
-public:
-    ID3v1StringHandler( int codecIndex )
-            : m_codec( QTextCodec::codecForName( QTextCodec::availableCodecs().at( codecIndex ) ) )
-    {
-        debug() << "codec: " << m_codec;
-        debug() << "codec-name: " << m_codec->name();
-    }
-
-    ID3v1StringHandler( QTextCodec *codec )
-            : m_codec( codec )
-    {
-        debug() << "codec: " << m_codec;
-        debug() << "codec-name: " << m_codec->name();
-    }
-
-    virtual ~ID3v1StringHandler()
-    {}
-};
-
-#undef Qt4QStringToTString
 
 //SLOT
 void App::applySettings( bool firstTime )
@@ -549,47 +412,13 @@ void App::applySettings( bool firstTime )
 
     if( !firstTime )
         emit settingsChanged();
+
+    if( AmarokConfig::enableScriptConsole() && !m_scriptConsole )
+        m_scriptConsole = ScriptConsoleNS::ScriptConsole::instance();
+    else if( !AmarokConfig::enableScriptConsole() && m_scriptConsole )
+        m_scriptConsole.data()->deleteLater();
 }
 
-#ifdef DEBUG
-//SLOT
-void
-App::runUnitTests( const QStringList options, bool _stdout )
-{
-    DEBUG_BLOCK
-
-    QString logPath;
-    if( !_stdout )
-    {
-        const QString location = Amarok::saveLocation( "testresults/" );
-        const QString stamp    = QDateTime::currentDateTime().toString( "yyyy-MM-dd.HH-mm-ss" );
-        logPath                = QDir::toNativeSeparators( location + stamp + "/" );
-
-        // create log folder for this run:
-        QDir logDir( logPath );
-        logDir.mkpath( logPath );
-
-        QFile::remove( QDir::toNativeSeparators( Amarok::saveLocation( "testresults/" ) + "LATEST" ) );
-        QFile::link( logPath, QDir::toNativeSeparators( Amarok::saveLocation( "testresults/" ) + "LATEST" ) );
-
-        QFile logArgs( logPath + "test_options" );
-        if( logArgs.open( QIODevice::WriteOnly ) )
-        {
-            logArgs.write( options.join( " " ).toLatin1() );
-            logArgs.close();
-        }
-    }
-
-    PERF_LOG( "Running Unit Tests" )
-
-    // modifies the playlist asynchronously, so run this last to avoid messing other test results
-    TestDirectoryLoader        *test015 = new TestDirectoryLoader( options, logPath );
-
-    PERF_LOG( "Done Running Unit Tests" )
-    Q_UNUSED( test015 )
-}
-#endif // DEBUG
-
 //SLOT
 void
 App::continueInit()
@@ -598,7 +427,8 @@ App::continueInit()
 
     PERF_LOG( "Begin App::continueInit" )
     const KCmdLineArgs* const args = KCmdLineArgs::parsedArgs();
-    const bool restoreSession = args->count() == 0 || args->isSet( "append" ) || args->isSet( "queue" )
+    const bool restoreSession = args->count() == 0 || args->isSet( "append" )
+                                || args->isSet( "queue" )
                                 || Amarok::config().readEntry( "AppendAsDefault", false );
 
     QTextCodec* utf8codec = QTextCodec::codecForName( "UTF-8" );
@@ -607,30 +437,24 @@ App::continueInit()
     new Amarok::DefaultApplicationController( this );
     Amarok::Components::applicationController()->start();
 
-    KSplashScreen* splash = 0;
-    if( AmarokConfig::showSplashScreen() && !isSessionRestored() )
-    {
-        QPixmap splashimg( KGlobal::dirs()->findResource( "data", "amarok/images/splash_screen.jpg" ) );
-        splash = new KSplashScreen( splashimg, Qt::WindowStaysOnTopHint );
-        splash->show();
-    }
+    // Instantiate statistics synchronization controller. Needs to be before creating
+    // MainWindow as MainWindow connects a signal to StatSyncing::Controller.
+    Amarok::Components::setStatSyncingController( new StatSyncing::Controller( this ) );
 
     PERF_LOG( "Creating MainWindow" )
     m_mainWindow = new MainWindow();
     PERF_LOG( "Done creating MainWindow" )
 
-    if ( AmarokConfig::showTrayIcon() ) {
+    if( AmarokConfig::showTrayIcon() )
         m_tray = new Amarok::TrayIcon( mainWindow() );
-    }
 
     PERF_LOG( "Creating DBus handlers" )
     new Mpris1::RootHandler();
     new Mpris1::PlayerHandler();
     new Mpris1::TrackListHandler();
-    new CollectionDBusHandler( this );
-    new Amarok::Mpris2DBusHandler();
     QDBusConnection::sessionBus().registerService("org.mpris.amarok");
-    QDBusConnection::sessionBus().registerService("org.mpris.MediaPlayer2.amarok");
+    new CollectionDBusHandler( this );
+    new Amarok::Mpris2( this );
     PERF_LOG( "Done creating DBus handlers" )
 
     //DON'T DELETE THIS NEXT LINE or the app crashes when you click the X (unless we reimplement closeEvent)
@@ -670,50 +494,20 @@ App::continueInit()
     //FFmpeg which should not take more than ~200msec.
     Amarok::Components::setTranscodingController( new Transcoding::Controller( this ) );
 
-    if( splash ) // close splash correctly
-    {
-        splash->close();
-        delete splash;
-    }
-
     PERF_LOG( "App init done" )
 
-    // NOTE: First Run Tutorial disabled for 2.1-beta1 release (too buggy / unfinished)
-#if 0
-    const bool firstruntut = config.readEntry( "FirstRunTutorial", true );
-    debug() << "Checking whether to run first run tutorial..." << firstruntut;
-    if( firstruntut )
+    // check that the amarok sql configuration is valid.
+    if( !StorageManager::instance()->getLastErrors().isEmpty() )
     {
-        config.writeEntry( "FirstRunTutorial", false );
-        debug() << "Starting first run tutorial";
-        FirstRunTutorial *frt = new FirstRunTutorial( mainWindow() );
-        QTimer::singleShot( 1000, frt, SLOT( initOverlay() ) );
-    }
-#endif
-
-#ifdef NO_MYSQL_EMBEDDED
-    bool useServer = true;
-    if( !AmarokConfig::useServer() )
-    {
-        useServer = false;
-        AmarokConfig::setUseServer( true );
-    }
-    if( !MySqlServerTester::testSettings(
-             AmarokConfig::host(),
-             AmarokConfig::user(),
-             AmarokConfig::password(),
-             AmarokConfig::port()
-         )
-    )
-    {
-        KMessageBox::messageBox( 0, KMessageBox::Information,
-                ( !useServer ? i18n( "The embedded database was not found; you must set up a database server connection.\nYou must restart Amarok after doing this." ) :
-                              i18n( "The connection details for the database server were invalid.\nYou must enter correct settings and restart Amarok after doing this." ) ),
-                i18n( "Database Error" ) );
+        KMessageBox::error( The::mainWindow(), i18n( "The amarok database reported "
+                 "the following errors:\n%1\nIn most cases you will need to resolve "
+                 "these errors before Amarok will run properly." ).
+                 arg( StorageManager::instance()->getLastErrors().join( "\n" ) ),
+                 i18n( "Database Error" ));
+        StorageManager::instance()->clearLastErrors();
         slotConfigAmarok( "DatabaseConfig" );
     }
     else
-#endif
     {
         handleFirstRun();
     }
@@ -735,17 +529,14 @@ App::continueInit()
 
 void App::slotConfigAmarok( const QString& page )
 {
-    Amarok2ConfigDialog* dialog = static_cast<Amarok2ConfigDialog*>( KConfigDialog::exists( "settings" ) );
-
+    KConfigDialog *dialog = KConfigDialog::exists( "settings" );
     if( !dialog )
     {
         //KConfigDialog didn't find an instance of this dialog, so lets create it :
         dialog = new Amarok2ConfigDialog( mainWindow(), "settings", AmarokConfig::self() );
-
-        connect( dialog, SIGNAL( settingsChanged( const QString& ) ), SLOT( applySettings() ) );
+        connect( dialog, SIGNAL(settingsChanged(QString)), SLOT(applySettings()) );
     }
-
-    dialog->show( page );
+    static_cast<Amarok2ConfigDialog*>( dialog )->show( page );
 }
 
 void App::slotConfigShortcuts()
@@ -758,7 +549,7 @@ KIO::Job *App::trashFiles( const KUrl::List &files )
 {
     KIO::Job *job = KIO::trash( files );
     Amarok::Components::logger()->newProgressOperation( job, i18n("Moving files to trash") );
-    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotTrashResult( KJob* ) ) );
+    connect( job, SIGNAL(result(KJob*)), this, SLOT(slotTrashResult(KJob*)) );
     return job;
 }
 
@@ -773,15 +564,8 @@ void App::quit()
     DEBUG_BLOCK
     The::playlistManager()->completePodcastDownloads();
 
+    // Following signal is relayed to scripts, which may block quitting for a while
     emit prepareToQuit();
-    /*
-    if( MediaBrowser::instance() && MediaBrowser::instance()->blockQuit() )
-    {
-        // don't quit yet, as some media devices still have to finish transferring data
-        QTimer::singleShot( 100, this, SLOT( quit() ) );
-        return;
-    }
-    */
     KApplication::quit();
 }
 
@@ -793,19 +577,7 @@ bool App::event( QEvent *event )
         case QEvent::FileOpen:
         {
             QString file = static_cast<QFileOpenEvent*>( event )->file();
-            //we are only going to receive local files here
-            KUrl url( file );
-            if( Playlists::isPlaylist( url ) )
-            {
-                Playlists::PlaylistPtr playlist =
-                        Playlists::PlaylistPtr::dynamicCast( Playlists::loadPlaylistFile( url ) );
-                The::playlistController()->insertOptioned( playlist, Playlist::AppendAndPlay );
-            }
-            else
-            {
-                Meta::TrackPtr track = CollectionManager::instance()->trackForUrl( url );
-                The::playlistController()->insertOptioned( track, Playlist::AppendAndPlay );
-            }
+            The::playlistController()->insertOptioned( KUrl( file ), Playlist::OnPlayMediaAction );
             return true;
         }
         default:
@@ -843,33 +615,29 @@ void App::handleFirstRun()
     int result = KMessageBox::No;
     if( dir.exists() && dir.isReadable() )
     {
-        result = KMessageBox::questionYesNoCancel(
-            mainWindow(),
-            i18n( "A music path, %1, is set in System Settings.\nWould you like to use that as a collection folder?", musicDir )
-            );
+        result = KMessageBox::questionYesNoCancel( mainWindow(), i18n( "A music path, "
+                "%1, is set in System Settings.\nWould you like to use that as a "
+                "collection folder?", musicDir ) );
     }
 
-    KConfigGroup folderConf = Amarok::config( "Collection Folders" );
-    bool useMusicLocation( false );
     switch( result )
     {
-    case KMessageBox::Yes:
-        if( CollectionManager::instance()->primaryCollection() )
+        case KMessageBox::Yes:
         {
-            CollectionManager::instance()->primaryCollection()->setProperty( "collectionFolders", QStringList() << musicDir );
-            CollectionManager::instance()->startFullScan();
-            useMusicLocation = true;
+            Collections::Collection *coll = CollectionManager::instance()->primaryCollection();
+            if( coll )
+            {
+                coll->setProperty( "collectionFolders", QStringList() << musicDir );
+                CollectionManager::instance()->startFullScan();
+            }
+            break;
         }
-        break;
-
-    case KMessageBox::No:
-        slotConfigAmarok( "CollectionConfig" );
-        break;
-
-    default:
-        break;
+        case KMessageBox::No:
+            slotConfigAmarok( "CollectionConfig" );
+            break;
+        default:
+            break;
     }
-    folderConf.writeEntry( "Use MusicLocation", useMusicLocation );
     config.writeEntry( "First Run", false );
 }
 
