@@ -1,7 +1,7 @@
 /****************************************************************************************
  * Copyright (c) 2004 Frederik Holljen <fh@ez.no>                                       *
  * Copyright (c) 2004,2005 Max Howell <max.howell@methylblue.com>                       *
- * Copyright (c) 2004-2010 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2004-2013 Mark Kretschmann <kretschmann@kde.org>                       *
  * Copyright (c) 2008 Jason A. Donenfeld <Jason@zx2c4.com>                              *
  * Copyright (c) 2009 Artur Szymiec <artur.szymiec@gmail.com>                           *
  *                                                                                      *
@@ -21,8 +21,12 @@
 #ifndef AMAROK_ENGINECONTROLLER_H
 #define AMAROK_ENGINECONTROLLER_H
 
+#include "amarok_export.h"
 #include "core/capabilities/BoundedPlaybackCapability.h"
-#include "core/meta/Meta.h"
+#include "core/meta/Observer.h"
+#include "playback/EqualizerController.h"
+
+#include <KUrl>
 
 #include <QMutex>
 #include <QObject>
@@ -37,10 +41,10 @@
 #include <Phonon/EffectParameter>
 #include <phonon/audiodataoutput.h>
 
-class QTimer;
-
+class Fadeouter;
 namespace Capabilities { class MultiPlayableCapability; class MultiSourceCapability; }
 namespace Phonon { class AudioOutput; class MediaSource; class VolumeFaderEffect; }
+class QTimer;
 
 /**
  * A thin wrapper around Phonon that implements Amarok-specific functionality like
@@ -52,6 +56,8 @@ class AMAROK_EXPORT EngineController : public QObject, public Meta::Observer
     Q_OBJECT
 
 public:
+    static const uint DATAOUTPUT_DATA_SIZE = 512;
+
     /**
      * Construct EngineController. Must be called from the main thread.
      */
@@ -104,14 +110,6 @@ public:
      */
     void setNextTrack( Meta::TrackPtr track );
 
-    /** Returns the media object Amarok is using for playback.
-     *  Provides access to the Phonon MediaObject for components that need more information
-     *  This is not for normal use, except maybe when you really need
-     *  the stateChanged signal.
-     */
-    // const so that it can only be used for info
-    const Phonon::MediaObject* phononMediaObject() const { return m_media.data(); }
-
     /**
      * Gets the volume
      * @return the volume as a percentage
@@ -159,34 +157,9 @@ public:
     bool isSeekable() const;
 
     /**
-     * Phonon kequalizer support is required for Amarok to enable equalizer
-     * this method return whatever phonon support kequalizer effect.
-     *
-     * @return @c true if the phonon support kequalizer effect, @c false otherwise
+     * Returns the associated EqualizerController object.
      */
-    bool isEqSupported() const;
-
-    /**
-     * kequalizer implementation for different backends may have different
-     * gain scale. To properly display it we need to get a scale from effect
-     *
-     * @return maximum gain value for kequalizer parameters.
-     */
-    double eqMaxGain() const;
-
-     /**
-     * kequalizer implementation for different backends may have different
-     * frequency bands. For proper display this will try to extract frequency values
-     * from effect parameters info.
-     *
-     * @return QStringList with band labels (form xxx Hz or xxx kHz).
-     */
-    QStringList eqBandsFreq() const;
-
-    /**
-    * @return whether the currently playing track is from an audiocd
-    */
-    bool isPlayingAudioCd();
+    EqualizerController *equalizerController() const;
 
     /**
      * @return QString with a pretty name for the current track
@@ -205,7 +178,7 @@ public slots:
      * Plays the specified track
      * This happens asynchronously.
      */
-    void play( Meta::TrackPtr track, uint offset = 0 );
+    void play( Meta::TrackPtr track, uint offset = 0, bool startPaused = false );
 
     /**
      * Replays the current track
@@ -245,7 +218,7 @@ public slots:
      *
      * @param ms the position in milliseconds (counting from the start of the track)
      */
-    void seek( int ms );
+    void seekTo( int ms );
 
     /**
      * Seeks forward or backward in the track
@@ -262,21 +235,7 @@ public slots:
      *
      * @param ms the offset from the current position in milliseconds
      */
-    void seekRelative( int ms );
-
-    /**
-     * Seeks forward in the track
-     *
-     * Same as seekRelative()
-     */
-    void seekForward( int ms = 10000 );
-
-    /**
-     * Seeks backward in the track
-     *
-     * Works identically to seekRelative(), but seeks in the opposite direction.
-     */
-    void seekBackward( int ms = 10000 );
+    void seekBy( int ms );
 
     /**
      * Increases the volume
@@ -318,10 +277,20 @@ public slots:
     void toggleMute();
 
     /**
-     * Update equalizer status - enabled,disabled,set values
+     * Return true if current Phonon back-end supports fade-out.
      */
-    void eqUpdate();
+    bool supportsFadeout() const;
 
+    /**
+     * Return true if current Phonon back-end supports our implementation of
+     * Replay Gain adjustment.
+     */
+    bool supportsGainAdjustments() const;
+
+    /**
+     * Return true if the current Phonon backend supports visualizations.
+     */
+    bool supportsAudioDataOutput() const;
 
 Q_SIGNALS:
     /**
@@ -372,11 +341,14 @@ Q_SIGNALS:
      */
     void trackChanged( Meta::TrackPtr track );
 
-    /** Emitted when the metadata of the current track changes.
-        You might want to connect also to trackChanged to get more changes because
-        this signal is only emitted when the track metadata changes while it's playing.
-        Also connecting to currentMetadataChanged would give you information when the current track
-        is a stream. In such a case the meta information from track might be unreliable.
+    /**
+     * Emitted when the metadata of the current track changes.
+     *
+     * You might want to connect also to trackChanged() or trackPlaying() to get more
+     * changes because this signal is only emitted when the track metadata changes
+     * while it's playing, not when new track starts playing. This method now works
+     * correctly also for streams and is preferred to currentMetaDataChanged() because
+     * it providers somehow more filtered values.
      */
     void trackMetadataChanged( Meta::TrackPtr track );
 
@@ -450,7 +422,7 @@ Q_SIGNALS:
      * Called when playback state changes to PlayingState, StoppedState or PausedState.
      */
     void playbackStateChanged();
-    
+
     /**
     *   Is emitted when new audio Data is ready
     *   @param audioData The audio data that is available
@@ -475,12 +447,12 @@ private slots:
     void slotAboutToFinish();
     void slotNewTrackPlaying( const Phonon::MediaSource &source);
     void slotStateChanged( Phonon::State newState, Phonon::State oldState);
-    void slotPlayableUrlFetched(const KUrl&);
+    void slotPlayableUrlFetched( const KUrl &url );
     void slotTick( qint64 );
     void slotTrackLengthChanged( qint64 );
     void slotMetaDataChanged();
-    void slotStopFadeout(); //called after the fade-out has finished
     void slotSeekableChanged( bool );
+    void slotPause();
 
     /**
      * For volume/mute changes from the phonon side
@@ -521,33 +493,46 @@ private:
      *
      * @param url the URL of the media
      * @param offset the position in the media to start at in milliseconds
+     * @param startPaused if true, go to paused state. if false, go to playing state (default)
      */
-    void playUrl( const KUrl &url, uint offset );
-
-    void createFadeoutEffect();
-    void resetFadeout();
+    void playUrl( const KUrl &url, uint offset, bool startPaused = false );
 
     /**
      * Try to detect MetaData spam in Streams etc.
-     * Some streams are doing advertisment in the metadata. We try to filter that out.
-     * Additionally, some Phonon back-ends emit more than one metadataChanged() signals
-     * per on track, so filter it all altogether.
+     *
+     * Some streams are doing advertisment in the metadata. We try to filter that
+     * out. Additionally, some Phonon back-ends emit more than one
+     * metadataChanged() signals per on track, so filter it all altogether.
      */
     bool isInRecentMetaDataHistory( const QVariantMap &meta );
 
+    /**
+     * If m_lastStreamStampPosition is non-negative, update it to current position
+     * and update track length in current stream.
+     */
+    void stampStreamTrackLength();
+
+    /**
+     * emit metadataChanged() with info so that MetaStream::Track that is
+     * currently listening updates its length.
+     *
+     * @param length new track length in milliseconds
+     */
+    void updateStreamLength( qint64 length );
+
     Q_DISABLE_COPY( EngineController )
 
+    EqualizerController                     *m_equalizerController;
     QWeakPointer<Phonon::MediaObject>       m_media;
     QWeakPointer<Phonon::VolumeFaderEffect> m_preamp;
-    QWeakPointer<Phonon::Effect>            m_equalizer;
     QWeakPointer<Phonon::AudioOutput>       m_audio;
     QWeakPointer<Phonon::AudioDataOutput>   m_audioDataOutput;
     QWeakPointer<Phonon::MediaController>   m_controller;
     Phonon::Path                            m_path;
     Phonon::Path                            m_dataPath;
 
-    Phonon::VolumeFaderEffect* m_fader;
-    QTimer* m_fadeoutTimer;
+    QWeakPointer<Fadeouter> m_fadeouter;
+    QWeakPointer<Phonon::VolumeFaderEffect> m_fader;
 
     Meta::TrackPtr  m_currentTrack;
     Meta::AlbumPtr  m_currentAlbum;
@@ -555,13 +540,15 @@ private:
     KUrl            m_nextUrl;
     Capabilities::BoundedPlaybackCapability* m_boundedPlayback;
     Capabilities::MultiPlayableCapability* m_multiPlayback;
-    Capabilities::MultiSourceCapability* m_multiSource;
+    QScopedPointer<Capabilities::MultiSourceCapability> m_multiSource;
     bool m_playWhenFetched;
     int m_volume;
-    bool m_currentIsAudioCd;
     int m_currentAudioCdTrack;
+    QTimer *m_pauseTimer;
 
     QList<QVariantMap> m_metaDataHistory; // against metadata spam
+    // last position (in ms) when the song changed (within the current stream) or -1 for non-stream
+    qint64 m_lastStreamStampPosition;
 
     /**
      * Some flags to prevent feedback loops in volume updates
@@ -586,5 +573,4 @@ namespace The {
     AMAROK_EXPORT EngineController* engineController();
 }
 
-
 #endif
