@@ -101,6 +101,9 @@
 
 #ifdef Q_WS_MAC
 #include "mac/GrowlInterface.h"
+#ifdef HAVE_NOTIFICATION_CENTER
+#include "mac/MacSystemNotify.h"
+#endif
 #endif
 
 #define AMAROK_CAPTION I18N_NOOP( "Amarok" )
@@ -125,7 +128,10 @@ MainWindow::MainWindow()
     s_instance = this;
 
 #ifdef Q_WS_MAC
-    GrowlInterface* growl = new GrowlInterface( qApp->applicationName() );
+    (void)new GrowlInterface( qApp->applicationName() );
+#ifdef HAVE_NOTIFICATION_CENTER
+    (void)new OSXNotify( qApp->applicationName() );
+#endif
 #endif
 
     PERF_LOG( "Instantiate Collection Manager" )
@@ -440,14 +446,6 @@ MainWindow::closeEvent( QCloseEvent *e )
 #endif
 }
 
-bool
-MainWindow::queryExit()
-{
-    DEBUG_BLOCK
-
-    return true; // KMainWindow API expects us to always return true
-}
-
 void
 MainWindow::exportPlaylist() //SLOT
 {
@@ -660,7 +658,7 @@ void
 MainWindow::slotShufflePlaylist()
 {
     m_playlistDock.data()->sortWidget()->trimToLevel();
-    m_playlistDock.data()->sortWidget()->addLevel( QString( "Shuffle" ) );
+    The::playlistActions()->shuffle();
 }
 
 void
@@ -871,7 +869,7 @@ MainWindow::createActions()
     ac->addAction( "prev", action );
     action->setIcon( KIcon("media-skip-backward-amarok") );
     action->setText( i18n( "Previous Track" ) );
-    action->setGlobalShortcut( KShortcut( Qt::Key_MediaPrevious ) );
+    action->setGlobalShortcut( KShortcut() );
     connect( action, SIGNAL(triggered(bool)), pa, SLOT(back()) );
 
     action = new KAction( this );
@@ -902,7 +900,7 @@ MainWindow::createActions()
 
     action = new KAction( KIcon("media-skip-forward-amarok"), i18n( "Next Track" ), this );
     ac->addAction( "next", action );
-    action->setGlobalShortcut( KShortcut( Qt::Key_MediaNext ) );
+    action->setGlobalShortcut( KShortcut() );
     connect( action, SIGNAL(triggered(bool)), pa, SLOT(next()) );
 
     action = new KAction( i18n( "Increase Volume" ), this );
@@ -1123,9 +1121,6 @@ MainWindow::createMenus()
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("bookmark_manager") );
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("cover_manager") );
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("equalizer_dialog") );
-#ifdef QTSCRIPTQTBINDINGS_FOUND
-    m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("script_manager") );
-#endif
 #ifdef DEBUG_BUILD_TYPE
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("network_request_viewer") );
 #endif // DEBUG_BUILD_TYPE
