@@ -18,7 +18,7 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "config.h"
+#include <config.h>
 
 #include "amarok_export.h"
 #include "core/meta/forward_declarations.h"
@@ -27,7 +27,6 @@
 #include <KMainWindow>
 #include <KVBox>
 #include <Phonon/Global>
-#include <Phonon/Global>
 
 #include <QWeakPointer>
 
@@ -174,7 +173,6 @@ class AMAROK_EXPORT MainWindow : public KMainWindow
     protected:
         virtual void closeEvent( QCloseEvent* );
         virtual void changeEvent( QEvent *event );
-        virtual bool queryExit(); 
 
     private slots:
         void setRating1() { setRating( 1 ); }
