@@ -18,9 +18,9 @@
 
 #include "AmarokUrlHandler.h"
 #include "BookmarkGroup.h"
-#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/storage/StorageManager.h"
 #include "core/support/Debug.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 
 #include <QUrl>
 
@@ -75,7 +75,7 @@ void AmarokUrl::initFromString( const QString & urlString )
             
             QStringList argParts = argument.split( '=' );
             debug() << "argument: " << argument << " unescaped: " << unescape( argParts.at( 1 ) );
-            appendArg( argParts.at( 0 ), unescape( argParts.at( 1 ) ) );
+            setArg( argParts.at( 0 ), unescape( argParts.at( 1 ) ) );
         }
     }
 
@@ -111,7 +111,7 @@ QMap<QString, QString> AmarokUrl::args() const
     return m_arguments;
 }
 
-void AmarokUrl::appendArg( const QString &name, const QString &value )
+void AmarokUrl::setArg( const QString &name, const QString &value )
 {
     m_arguments.insert( name, value );
 }
@@ -124,32 +124,15 @@ bool AmarokUrl::run()
 
 QString AmarokUrl::url() const
 {
-    QString url = "amarok://";
-    url += m_command;
-    url += '/';
-    url += m_path;
+    QUrl url;
+    url.setScheme( "amarok" );
+    url.setHost( m_command );
+    url.setPath( m_path );
 
-    if ( url.endsWith( '/' ) )
-        url.chop( 1 );
+    foreach( const QString &argName, m_arguments.keys() )
+        url.addQueryItem( argName, m_arguments[argName] );
 
-    if( m_arguments.size() > 0 )
-    {
-    
-        url += '?';
-        const QStringList args = m_arguments.keys();
-        foreach( const QString &argName, args )
-        {
-            url += argName;
-            url += '=';
-            url += m_arguments.value( argName );
-            url += '&';
-        }
-        url.chop( 1 );
-    }
-
-    QUrl qUrl( url );
-
-    return QString( qUrl.toEncoded() );
+    return url.toEncoded();
 }
 
 bool AmarokUrl::saveToDb()
@@ -161,23 +144,25 @@ bool AmarokUrl::saveToDb()
 
     const int parentId = m_parent ? m_parent->id() : -1;
 
-    SqlStorage * sql =  CollectionManager::instance()->sqlStorage();
+    SqlStorage * sql =  StorageManager::instance()->sqlStorage();
 
     if( m_id != -1 )
     {
         //update existing
         debug() << "Updating bookmark";
         QString query = "UPDATE bookmarks SET parent_id=%1, name='%2', url='%3', description='%4', custom='%5' WHERE id=%6;";
-        query = query.arg( QString::number( parentId ) ).arg( sql->escape( m_name ), sql->escape( url() ), sql->escape( m_description ), sql->escape( m_customValue ) , QString::number( m_id ) );
-        CollectionManager::instance()->sqlStorage()->query( query );
+        query = query.arg( QString::number( parentId ), sql->escape( m_name ), sql->escape( url() ),
+                           sql->escape( m_description ), sql->escape( m_customValue ), QString::number( m_id ) );
+        StorageManager::instance()->sqlStorage()->query( query );
     }
     else
     {
         //insert new
         debug() << "Creating new bookmark in the db";
         QString query = "INSERT INTO bookmarks ( parent_id, name, url, description, custom ) VALUES ( %1, '%2', '%3', '%4', '%5' );";
-        query = query.arg( QString::number( parentId ), sql->escape( m_name ), sql->escape( url() ), sql->escape( m_description ), sql->escape( m_customValue ) );
-        m_id = CollectionManager::instance()->sqlStorage()->insert( query, NULL );
+        query = query.arg( QString::number( parentId ), sql->escape( m_name ), sql->escape( url() ),
+                           sql->escape( m_description ), sql->escape( m_customValue ) );
+        m_id = StorageManager::instance()->sqlStorage()->insert( query, NULL );
     }
 
     return true;
@@ -207,7 +192,7 @@ void AmarokUrl::removeFromDb()
 {
     QString query = "DELETE FROM bookmarks WHERE id=%1";
     query = query.arg( QString::number( m_id ) );
-    CollectionManager::instance()->sqlStorage()->query( query );
+    StorageManager::instance()->sqlStorage()->query( query );
 }
 
 void AmarokUrl::rename( const QString &name )
@@ -243,7 +228,7 @@ QString AmarokUrl::unescape( const QString & in )
     return QUrl::fromPercentEncoding( in.toUtf8() );
 }
 
-bool AmarokUrl::isNull()
+bool AmarokUrl::isNull() const
 {
     return m_command.isEmpty();
 }
