@@ -19,12 +19,27 @@
 
 #include <KHBox>
 
+#include <QIcon>
+
+class FileBrowser;
 class BrowserCategory;
 class BreadcrumbItemButton;
 class BreadcrumbItemMenuButton;
 
+struct BreadcrumbSibling {
+    BreadcrumbSibling( const QIcon &icon_, const QString &name_, const QString &callback_ )
+        : icon( icon_ ), name( name_ ), callback( callback_ ) {}
+
+    QIcon icon;
+    QString name;
+    QString callback;
+};
+typedef QList<BreadcrumbSibling> BreadcrumbSiblingList;
+
 /**
  *  A widget representing a single "breadcrumb" item
+ *  It has a mainButton and optionally a menuButton
+ *
  *  @author Nikolaj Hald Nielsen <nhn@kde.org>
  */
 
@@ -32,15 +47,14 @@ class BrowserBreadcrumbItem : public KHBox
 {
     Q_OBJECT
 public:
-    BrowserBreadcrumbItem( BrowserCategory * category );
+    BrowserBreadcrumbItem( BrowserCategory* category, QWidget* parent = 0 );
 
     /**
      * Overloaded constructor for creating breadcrumb items not bound to a particular BrowserCategory
      */
-    BrowserBreadcrumbItem( const QString &name, const QStringList &childItems, const QString &callback, BrowserCategory * handler );
-
-    bool isAddItem() { return m_category == 0; }
-    
+    BrowserBreadcrumbItem( const QString &name, const QString &callback,
+                           const BreadcrumbSiblingList &childItems, FileBrowser *handler,
+                           QWidget *parent = 0 );
     ~BrowserBreadcrumbItem();
 
     void setActive( bool active );
@@ -52,20 +66,18 @@ public:
 signals:
 
     void activated( const QString &callback );
-    
+
 protected slots:
     void updateSizePolicy();
     void activate();
     void activateSibling();
 
 private:
-    BrowserCategory          *m_category;
     BreadcrumbItemMenuButton *m_menuButton;
     BreadcrumbItemButton     *m_mainButton;
 
     QString m_callback;
     int m_nominalWidth;
-    
 };
 
 #endif
