@@ -18,36 +18,17 @@
 #ifndef COLLECTIONTREEITEM_H
 #define COLLECTIONTREEITEM_H
 
-#include "BrowserDefines.h"
+#include "amarok_export.h"
+#include "browsers/BrowserDefines.h"
 #include "core/collections/Collection.h"
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 
 #include <QList>
 
-
-namespace CustomRoles
-{
-    enum CustomRolesId
-    {
-        SortRole = Qt::UserRole + 1,
-        FilterRole = Qt::UserRole + 2,
-        ByLineRole = Qt::UserRole + 3,
-        /** Boolean value whether given collection knows about used and total capacity */
-        HasCapacityRole = Qt::UserRole + 4,
-        /** Number of bytes used by music and other files in collection (float) */
-        UsedCapacityRole = Qt::UserRole + 5,
-        /** Total capacity of the collection in bytes (float) */
-        TotalCapacityRole = Qt::UserRole + 6,
-        /** The number of collection actions */
-        DecoratorRoleCount = Qt::UserRole + 7,
-        /** The collection actions */
-        DecoratorRole = Qt::UserRole + 8
-    };
-}
-
 class CollectionTreeItemModelBase;
+class QAction;
 
-class CollectionTreeItem : public QObject
+class AMAROK_EXPORT CollectionTreeItem : public QObject
 {
     Q_OBJECT
     Q_ENUMS( Type )
@@ -70,7 +51,7 @@ class CollectionTreeItem : public QObject
 
         ~CollectionTreeItem();
 
-        CollectionTreeItem* parent() { return m_parent; }
+        CollectionTreeItem* parent() const { return m_parent; }
 
         void appendChild( CollectionTreeItem *child );
         void removeChild( int index );
@@ -104,15 +85,15 @@ class CollectionTreeItem : public QObject
          *        CategoryId::CatMenuId enum. Used only for distinction between Artist and
          *        AlbumArtist.
          */
-        void addMatch( Collections::QueryMaker* qm, CategoryId::CatMenuId levelCategory ) const;
+        void addMatch( Collections::QueryMaker *qm, CategoryId::CatMenuId levelCategory ) const;
 
-        bool operator<( const CollectionTreeItem& other ) const;
+        bool operator<( const CollectionTreeItem &other ) const;
 
-        const Meta::DataPtr data() const { return m_data; }
+        const Meta::DataPtr data() const;
         Collections::Collection* parentCollection() const { return m_parentCollection ? m_parentCollection : (m_parent ? m_parent->parentCollection() : 0); }
 
         KUrl::List urls() const;
-        QList<Meta::TrackPtr> descendentTracks();
+        Meta::TrackList descendentTracks();
 
         bool allDescendentTracksLoaded() const;
 
@@ -133,7 +114,7 @@ class CollectionTreeItem : public QObject
         /** Returns a list of collection actions.
             Collection actions are shown on top of the collection tree item as icons (decorations)
         */
-        QList<QAction*> decoratorActions() const;
+        QList<QAction *> decoratorActions() const;
         void prepareForRemoval();
 
         Meta::DataPtr m_data;
@@ -149,4 +130,7 @@ class CollectionTreeItem : public QObject
         mutable bool m_isCounting;
 };
 
+Q_DECLARE_METATYPE( CollectionTreeItem* )
+Q_DECLARE_METATYPE( QList<CollectionTreeItem*> )
+
 #endif
