@@ -68,6 +68,7 @@ CollectionTreeItemModelBase::CollectionTreeItemModelBase( )
     , m_loading1( QPixmap( KStandardDirs::locate("data", "amarok/images/loading1.png" ) ) )
     , m_loading2( QPixmap( KStandardDirs::locate("data", "amarok/images/loading2.png" ) ) )
     , m_currentAnimPixmap( m_loading1 )
+    , m_autoExpand( false )
 {
     m_timeLine = new QTimeLine( 10000, this );
     m_timeLine->setFrameRange( 0, 20 );
@@ -366,10 +367,7 @@ CollectionTreeItemModelBase::parent(const QModelIndex & index) const
      CollectionTreeItem *childItem = static_cast<CollectionTreeItem*>(index.internalPointer());
      CollectionTreeItem *parentItem = childItem->parent();
 
-     if ( (parentItem == m_rootItem) || !parentItem )
-         return QModelIndex();
-
-     return createIndex(parentItem->row(), 0, parentItem);
+     return itemIndex( parentItem );
 }
 
 int
@@ -489,6 +487,24 @@ CollectionTreeItemModelBase::ensureChildrenLoaded( CollectionTreeItem *item )
     }
 }
 
+CollectionTreeItem *
+CollectionTreeItemModelBase::treeItem( const QModelIndex &index ) const
+{
+    if( !index.isValid() || index.model() != this )
+        return 0;
+
+    return static_cast<CollectionTreeItem *>( index.internalPointer() );
+}
+
+QModelIndex
+CollectionTreeItemModelBase::itemIndex( CollectionTreeItem *item ) const
+{
+    if( !item || item == m_rootItem )
+        return QModelIndex();
+
+    return createIndex( item->row(), 0, item );
+}
+
 void CollectionTreeItemModelBase::listForLevel(int level, Collections::QueryMaker * qm, CollectionTreeItem * parent)
 {
     if( qm && parent )
@@ -638,13 +654,14 @@ CollectionTreeItemModelBase::queryDone()
     //reset icon for this item
     if( item && item != m_rootItem )
     {
-        emit dataChanged( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) );
+        emit dataChanged( itemIndex( item ), itemIndex( item ) );
     }
 
     //stop timer if there are no more animations active
     if( m_runningQueries.isEmpty() )
     {
-        emit allQueriesFinished();
+        emit allQueriesFinished( m_autoExpand );
+        m_autoExpand = false; // reset to default value
         m_timeLine->stop();
     }
     qm->deleteLater();
@@ -736,13 +753,9 @@ CollectionTreeItemModelBase::handleSpecialQueryResult( CollectionTreeItem::Type
     else if( type == CollectionTreeItem::NoLabel )
         parent = m_noLabelsQueries.value( qm );
 
-    QModelIndex parentIndex;
     if( parent )
     {
-        if (parent == m_rootItem ) // will never happen in CollectionTreeItemModel
-            parentIndex = QModelIndex();
-        else
-            parentIndex = createIndex( parent->row(), 0, parent );
+        QModelIndex parentIndex = itemIndex( parent );
 
         //if the special query did not return a result we have to remove the
         //the special node itself
@@ -801,7 +814,7 @@ CollectionTreeItemModelBase::handleSpecialQueryResult( CollectionTreeItem::Type
                 //only call populateChildren for the special node if we have not
                 //created it in this method call. The special node ctor takes care
                 //of that itself
-                populateChildren( dataList, specialNode, createIndex( specialNode->row(), 0, specialNode ) );
+                populateChildren( dataList, specialNode, itemIndex( specialNode ) );
             }
             //populate children will call setRequiresUpdate on vaNode
             //but as the special query is based on specialNode's parent,
@@ -834,13 +847,8 @@ void
 CollectionTreeItemModelBase::handleNormalQueryResult( Collections::QueryMaker *qm, const Meta::DataList &dataList )
 {
     CollectionTreeItem *parent = m_childQueries.value( qm );
-    QModelIndex parentIndex;
     if( parent ) {
-        if( parent == m_rootItem ) // will never happen in CollectionTreeItemModel, but will happen in Single!
-            parentIndex = QModelIndex();
-        else
-            parentIndex = createIndex( parent->row(), 0, parent );
-
+        QModelIndex parentIndex = itemIndex( parent );
         populateChildren( dataList, parent, parentIndex );
 
         if ( parent->isDataItem() )
@@ -851,10 +859,6 @@ CollectionTreeItemModelBase::handleNormalQueryResult( Collections::QueryMaker *q
                 //simply insert the item, nothing will change if it is already in the set
                 m_expandedItems.insert( parent->data() );
         }
-        else
-        {
-            m_expandedCollections.insert( parent->parentCollection() );
-        }
     }
 }
 
@@ -1067,29 +1071,35 @@ void CollectionTreeItemModelBase::loadingAnimationTick()
     {
         if( item == m_rootItem )
             continue;
-        emit dataChanged ( createIndex(item->row(), 0, item), createIndex(item->row(), 0, item) );
+        emit dataChanged( itemIndex( item ), itemIndex( item ) );
     }
 }
 
+QString
+CollectionTreeItemModelBase::currentFilter() const
+{
+    return m_currentFilter;
+}
+
 void
 CollectionTreeItemModelBase::setCurrentFilter( const QString &filter )
 {
     m_currentFilter = filter;
-    slotFilter();
+    slotFilter( /* autoExpand */ true );
 }
 
 void
-CollectionTreeItemModelBase::slotFilter()
+CollectionTreeItemModelBase::slotFilter( bool autoExpand )
 {
+    m_autoExpand = autoExpand;
     filterChildren();
-    if ( !m_expandedCollections.isEmpty() )
+
+    // following is not auto-expansion, it is restoring the state before filtering
+    foreach( Collections::Collection *expanded, m_expandedCollections )
     {
-        foreach( Collections::Collection *expanded, m_expandedCollections )
-        {
-            CollectionTreeItem *expandedItem = m_collections.value( expanded->collectionId() ).second;
-            if( expandedItem )
-                emit expandIndex( createIndex( expandedItem->row(), 0, expandedItem ) );
-        }
+        CollectionTreeItem *expandedItem = m_collections.value( expanded->collectionId() ).second;
+        if( expandedItem )
+            emit expandIndex( itemIndex( expandedItem ) );
     }
 }
 
@@ -1123,22 +1133,27 @@ CollectionTreeItemModelBase::slotCollapsed( const QModelIndex &index )
 void
 CollectionTreeItemModelBase::slotExpanded( const QModelIndex &index )
 {
-    if( index.isValid() )
+    if( !index.isValid() )
+        return;
+
+    CollectionTreeItem *item = static_cast<CollectionTreeItem*>( index.internalPointer() );
+    // we are really only interested in the special nodes here.
+    // we have to remember whether the user expanded a various artists/no labels node or not.
+    // otherwise we won't be able to automatically expand the special node after filtering again
+    // there is exactly one special node per type per collection, so use the collection to store that information
+
+    // we also need to store collection expansion state here as they are no longer
+    // added to th expanded set in handleNormalQueryResult()
+    switch( item->type() )
     {
-        CollectionTreeItem *item = static_cast<CollectionTreeItem*>( index.internalPointer() );
-        //we are really only interested in the special nodes here.
-        //we have to remember whether the user expanded a various artists/no labels node or not.
-        //otherwise we won't be able to automatically expand the special node after filtering again
-        //there is exactly one special node per type per collection, so use the collection to store that information
-        switch( item->type() )
-        {
         case CollectionTreeItem::VariousArtist:
         case CollectionTreeItem::NoLabel:
             m_expandedSpecialNodes.insert( item->parentCollection() );
             break;
+        case CollectionTreeItem::Collection:
+            m_expandedCollections.insert( item->parentCollection() );
         default:
             break;
-        }
     }
 }
 
