@@ -22,14 +22,14 @@
 #include "amarok_export.h"
 
 #include "core/collections/QueryMaker.h"
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 #include "CollectionTreeItem.h"
-#include "Expression.h"
 
 #include <QAbstractItemModel>
 #include <QDateTime>
 #include <QHash>
 #include <QPair>
+#include <QPixmap>
 #include <QSet>
 
 namespace Collections
@@ -41,20 +41,6 @@ class QTimeLine;
 
 typedef QPair<Collections::Collection*, CollectionTreeItem* > CollectionRoot;
 
-namespace CategoryId
-{
-    enum CatMenuId {
-    None = 0,
-    Album,
-    Artist,
-    AlbumArtist,
-    Composer,
-    Genre,
-    Year,
-    Label
-    };
-}
-
 /**
 	@author Nikolaj Hald Nielsen <nhn@kde.org>
 */
@@ -63,7 +49,7 @@ class AMAROK_EXPORT CollectionTreeItemModelBase : public QAbstractItemModel
         Q_OBJECT
 
     public:
-        CollectionTreeItemModelBase( );
+        CollectionTreeItemModelBase();
         virtual ~CollectionTreeItemModelBase();
 
         virtual Qt::ItemFlags flags(const QModelIndex &index) const;
@@ -75,75 +61,113 @@ class AMAROK_EXPORT CollectionTreeItemModelBase : public QAbstractItemModel
         virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
         virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
         virtual bool hasChildren ( const QModelIndex & parent = QModelIndex() ) const;
-        
+
         // Writable..
         virtual bool setData( const QModelIndex &index, const QVariant &value, int role = Qt::EditRole );
 
         virtual QStringList mimeTypes() const;
-        virtual QMimeData* mimeData( const QList<CollectionTreeItem*> &items ) const;
         virtual QMimeData* mimeData( const QModelIndexList &indices ) const;
+        virtual QMimeData* mimeData( const QList<CollectionTreeItem *> &items ) const;
 
-        virtual QIcon iconForLevel( int level ) const;
         virtual void listForLevel( int level, Collections::QueryMaker *qm, CollectionTreeItem* parent );
 
+        virtual void setLevels( const QList<CategoryId::CatMenuId> &levelType );
+        virtual QList<CategoryId::CatMenuId> levels() const { return m_levelType; }
+        virtual CategoryId::CatMenuId levelCategory( const int level ) const;
 
-        virtual void setLevels( const QList<int> &levelType ) = 0;
-        virtual QList<int> levels() const { return m_levelType; }
-        virtual int levelCategory( const int level ) const;
-
-        virtual void addDateFilter( qint64 field, Collections::QueryMaker::NumberComparison compare, const expression_element &elem, Collections::QueryMaker *qm ) const;
-
-        virtual void addFilters( Collections::QueryMaker *qm ) const;
-
+        QString currentFilter() const;
         void setCurrentFilter( const QString &filter );
 
         void itemAboutToBeDeleted( CollectionTreeItem *item );
 
+        /**
+         * This should be called every time a drag enters collection browser
+         */
+        void setDragSourceCollections( const QSet<Collections::Collection*> &collections );
+
+        /**
+         * Return true if there are any queries still running. If this returns true,
+         * you can expect allQueriesFinished(bool) signal in some time.
+         */
+        bool hasRunningQueries() const;
+
+        static QIcon iconForCategory( CategoryId::CatMenuId category );
+        static QString nameForCategory( CategoryId::CatMenuId category, bool showYears = false );
+
+        void ensureChildrenLoaded( CollectionTreeItem *item );
+
+        /**
+         * Get a pointer to colleciton tree item given its index. It is not safe to
+         * cache this pointer unless QWeakPointer is used.
+         */
+        CollectionTreeItem *treeItem( const QModelIndex &index ) const;
+
+        /**
+         * Get (create) index for a collection tree item. The caller must ensure this
+         * item is in this model. Invalid model index is returned on null or root item.
+         */
+        QModelIndex itemIndex( CollectionTreeItem *item ) const;
+
     signals:
         void expandIndex( const QModelIndex &index );
-        void allQueriesFinished();
+        void allQueriesFinished( bool autoExpand );
 
     public slots:
         virtual void queryDone();
-        virtual void newResultReady( const QString &collectionId, Meta::DataList data );
+        void newResultReady( Meta::TrackList );
+        void newResultReady( Meta::ArtistList );
+        void newResultReady( Meta::AlbumList );
+        void newResultReady( Meta::GenreList );
+        void newResultReady( Meta::ComposerList );
+        void newResultReady( Meta::YearList );
+        void newResultReady( Meta::LabelList );
+        virtual void newResultReady( Meta::DataList data );
 
-        void slotFilter();
+        /**
+         * Apply the current filter.
+         *
+         * @param autoExpand whether to trigger automatic expansion of the tree after
+         * filtering is done. This should be set to true only if filter is run after
+         * user has actually just typed something and defaults to false.
+         */
+        void slotFilter( bool autoExpand = false );
 
         void slotCollapsed( const QModelIndex &index );
-    
         void slotExpanded( const QModelIndex &index );
 
     private:
         void handleSpecialQueryResult( CollectionTreeItem::Type type, Collections::QueryMaker *qm, const Meta::DataList &dataList );
         void handleNormalQueryResult( Collections::QueryMaker *qm, const Meta::DataList &dataList );
-        QDateTime semanticDateTimeParser( const QString &text ) const;
 
         Collections::QueryMaker::QueryType mapCategoryToQueryType( int levelType ) const;
 
     protected:
+        /** Adds the query maker to the running queries and connects the slots */
+        void addQueryMaker( CollectionTreeItem* item,
+                            Collections::QueryMaker *qm ) const;
+
         virtual void populateChildren(const Meta::DataList &dataList, CollectionTreeItem *parent, const QModelIndex &parentIndex );
         virtual void updateHeaderText();
+
+        virtual QIcon iconForLevel( int level ) const;
         virtual QString nameForLevel( int level ) const;
+
         virtual int levelModifier() const = 0;
         virtual QVariant dataForItem( CollectionTreeItem *item, int role, int level = -1 ) const;
 
         virtual void filterChildren() = 0;
-        void ensureChildrenLoaded( CollectionTreeItem *item );
 
         void markSubTreeAsDirty( CollectionTreeItem *item );
 
         /** Initiates a special search for albums without artists */
-        void handleCompilations( CollectionTreeItem *parent ) const;
+        void handleCompilations( Collections::QueryMaker::QueryType queryType, CollectionTreeItem *parent ) const;
 
         /** Initiates a special search for tracks without label */
         void handleTracksWithoutLabels( Collections::QueryMaker::QueryType queryType, CollectionTreeItem *parent ) const;
 
         QString m_headerText;
         CollectionTreeItem *m_rootItem;
-        QList<int> m_levelType;
-
-        class Private;
-        Private * const d;
+        QList<CategoryId::CatMenuId> m_levelType;
 
         QTimeLine *m_timeLine;
         int m_animFrame;
@@ -154,24 +178,24 @@ class AMAROK_EXPORT CollectionTreeItemModelBase : public QAbstractItemModel
         QSet<Collections::Collection*> m_expandedCollections;
         QSet<Collections::Collection*> m_expandedSpecialNodes;
 
+        /**
+         * Contents of this set are undefined if there is no active drag 'n drop operation.
+         * Additionally, you may _never_ dereference pointers in this set, just compare
+         * them with other pointers
+         */
+        QSet<Collections::Collection*> m_dragSourceCollections;
+
+        QHash<QString, CollectionRoot > m_collections;  //I'll concide this one... :-)
+        mutable QHash<Collections::QueryMaker* , CollectionTreeItem* > m_childQueries;
+        mutable QHash<Collections::QueryMaker* , CollectionTreeItem* > m_compilationQueries;
+        mutable QHash<Collections::QueryMaker* , CollectionTreeItem* > m_noLabelsQueries;
+        mutable QMultiHash<CollectionTreeItem*, Collections::QueryMaker*> m_runningQueries;
+        bool m_autoExpand; // whether to expand tree after queries are done
+
     protected slots:
         void startAnimationTick();
         void loadingAnimationTick();
-        void update();
-
-    private slots:
-        void updateRowHeight();
 };
 
-class CollectionTreeItemModelBase::Private
-{
- public:
-    QHash<QString, CollectionRoot > collections;  //I'll concide this one... :-)
-    QHash<Collections::QueryMaker* , CollectionTreeItem* > childQueries;
-    QHash<Collections::QueryMaker* , CollectionTreeItem* > compilationQueries;
-    QHash<Collections::QueryMaker* , CollectionTreeItem* > noLabelsQueries;
-    QHash<CollectionTreeItem*, Collections::QueryMaker*>   runningQueries;
-    int rowHeight;
-};
 
 #endif
