@@ -21,7 +21,7 @@
 
 #include "amarok_export.h"
 #include "CollectionTreeItemModelBase.h"
-#include "meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 
 #include <QAbstractItemModel>
 #include <QMap>
@@ -37,12 +37,12 @@ class AMAROK_EXPORT SingleCollectionTreeItemModel: public CollectionTreeItemMode
     Q_OBJECT
 
     public:
-        SingleCollectionTreeItemModel( Amarok::Collection * collection,  const QList<int> &levelType );
+        SingleCollectionTreeItemModel( Collections::Collection *collection,
+                                       const QList<CategoryId::CatMenuId> &levelType );
 
         virtual QVariant data(const QModelIndex &index, int role) const;
         virtual bool canFetchMore( const QModelIndex &parent ) const;
         virtual void fetchMore( const QModelIndex &parent );
-        virtual void setLevels( const QList<int> &levelType );
         virtual Qt::ItemFlags flags(const QModelIndex &index) const;
 
     protected:
@@ -51,8 +51,7 @@ class AMAROK_EXPORT SingleCollectionTreeItemModel: public CollectionTreeItemMode
 
     private:
 
-        Amarok::Collection* m_collection;
-        QueryMaker* m_queryMaker;
+        Collections::Collection* m_collection;
 };
 
 #endif
