@@ -19,24 +19,21 @@
 
 #include "FileView.h"
 
+#include "EngineController.h"
+#include "PaletteHandler.h"
+#include "PopupDropperFactory.h"
+#include "SvgHandler.h"
 #include "context/ContextView.h"
 #include "context/popupdropper/libpud/PopupDropper.h"
 #include "context/popupdropper/libpud/PopupDropperItem.h"
+#include "core/playlists/PlaylistFormat.h"
+#include "core/support/Debug.h"
 #include "core-impl/collections/support/CollectionManager.h"
 #include "core-impl/collections/support/FileCollectionLocation.h"
 #include "core-impl/meta/file/File.h"
 #include "core-impl/playlists/types/file/PlaylistFileSupport.h"
-#include "core/interfaces/Logger.h"
-#include "core/playlists/PlaylistFormat.h"
-#include "core/support/Debug.h"
-#include "core/support/Components.h"
+#include "core-impl/support/TrackLoader.h"
 #include "dialogs/TagDialog.h"
-#include "DirectoryLoader.h"
-#include "EngineController.h"
-#include "PaletteHandler.h"
-#include "playlist/PlaylistController.h"
-#include "PopupDropperFactory.h"
-#include "SvgHandler.h"
 
 #include <KAction>
 #include <KIO/CopyJob>
@@ -61,24 +58,21 @@ FileView::FileView( QWidget *parent )
     , m_appendAction( 0 )
     , m_loadAction( 0 )
     , m_editAction( 0 )
-    , m_separator1( 0 )
+    , m_moveToTrashAction( 0 )
     , m_deleteAction( 0 )
     , m_pd( 0 )
     , m_ongoingDrag( false )
-    , m_moveActivated( false )
-    , m_copyActivated( false )
-    , m_moveAction( 0 )
-    , m_copyAction( 0 )
 {
     setFrameStyle( QFrame::NoFrame );
     setItemsExpandable( false );
     setRootIsDecorated( false );
     setAlternatingRowColors( true );
     setUniformRowHeights( true );
+    setEditTriggers( EditKeyPressed );
 
     The::paletteHandler()->updateItemView( this );
-    connect( The::paletteHandler(), SIGNAL(newPalette( const QPalette & )),
-                                    SLOT(newPalette( const QPalette & )) );
+    connect( The::paletteHandler(), SIGNAL(newPalette(QPalette)),
+                                    SLOT(newPalette(QPalette)) );
 }
 
 void
@@ -99,10 +93,10 @@ FileView::contextMenuEvent( QContextMenuEvent *e )
     if( indices.isEmpty() )
         return;
 
-    KMenu* menu = new KMenu( this );
-    QList<QAction *> actions = actionsForIndices( indices );
-    foreach( QAction * action, actions )
-        menu->addAction( action );
+    KMenu menu;
+    foreach( QAction *action, actionsForIndices( indices, PlaylistAction ) )
+        menu.addAction( action );
+    menu.addSeparator();
 
     // Create Copy/Move to menu items
     // ported from old filebrowser
@@ -120,26 +114,34 @@ FileView::contextMenuEvent( QContextMenuEvent *e )
     }
     if( !writableCollections.isEmpty() )
     {
-        QMenu *moveMenu = new QMenu( i18n( "Move to Collection" ), this );
+        QMenu *copyMenu = new QMenu( i18n( "Copy to Collection" ), &menu );
+        copyMenu->setIcon( KIcon( "edit-copy" ) );
         foreach( Collections::Collection *coll, writableCollections )
         {
-            CollectionAction *moveAction = new CollectionAction( coll, this );
-            connect( moveAction, SIGNAL( triggered() ), this, SLOT( slotPrepareMoveTracks() ) );
-            moveMenu->addAction( moveAction );
-        }
-        menu->addMenu( moveMenu );
-
-        QMenu *copyMenu = new QMenu( i18n( "Copy to Collection" ), this );
-        foreach( Collections::Collection *coll, writableCollections )
-        {
-            CollectionAction *copyAction = new CollectionAction( coll, this );
-            connect( copyAction, SIGNAL( triggered() ), this, SLOT( slotPrepareCopyTracks() ) );
+            CollectionAction *copyAction = new CollectionAction( coll, &menu );
+            connect( copyAction, SIGNAL(triggered()), this, SLOT(slotPrepareCopyTracks()) );
             copyMenu->addAction( copyAction );
         }
-        menu->addMenu( copyMenu );
-    }
+        menu.addMenu( copyMenu );
 
-    menu->exec( e->globalPos() );
+        QMenu *moveMenu = new QMenu( i18n( "Move to Collection" ), &menu );
+        moveMenu->setIcon( KIcon( "go-jump" ) );
+        foreach( Collections::Collection *coll, writableCollections )
+        {
+            CollectionAction *moveAction = new CollectionAction( coll, &menu );
+            connect( moveAction, SIGNAL(triggered()), this, SLOT(slotPrepareMoveTracks()) );
+            moveMenu->addAction( moveAction );
+        }
+        menu.addMenu( moveMenu );
+    }
+    foreach( QAction *action, actionsForIndices( indices, OrganizeAction ) )
+        menu.addAction( action );
+    menu.addSeparator();
+
+    foreach( QAction *action, actionsForIndices( indices, EditAction ) )
+        menu.addAction( action );
+
+    menu.exec( e->globalPos() );
 }
 
 void
@@ -148,78 +150,110 @@ FileView::mouseReleaseEvent( QMouseEvent *event )
     QModelIndex index = indexAt( event->pos() );
     if( !index.isValid() )
     {
-        m_lastSelectedIndex = QModelIndex();
+        PrettyTreeView::mouseReleaseEvent( event );
+        return;
+    }
+
+    if( state() == QAbstractItemView::NoState && event->button() == Qt::MidButton )
+    {
+        addIndexToPlaylist( index, Playlist::OnMiddleClickOnSelectedItems );
         event->accept();
         return;
     }
 
-    QModelIndexList indices = selectedIndexes();
-    if( indices.count() == 1 && KGlobalSettings::singleClick() )
+    KFileItem file = index.data( KDirModel::FileItemRole ).value<KFileItem>();
+    if( state() == QAbstractItemView::NoState &&
+        event->button() == Qt::LeftButton &&
+        event->modifiers() == Qt::NoModifier &&
+        KGlobalSettings::singleClick() &&
+        ( file.isDir() || file.isNull() ) )
     {
-        const QVariant qvar = index.data( KDirModel::FileItemRole );
-        if( qvar.canConvert<KFileItem>() )
-        {
-            KFileItem item = index.data( KDirModel::FileItemRole ).value<KFileItem>();
-            if( item.isDir() )
-            {
-                m_lastSelectedIndex = QModelIndex();
-                Amarok::PrettyTreeView::mouseReleaseEvent( event );
-                return;
-            }
-
-            // check if the last selected item was clicked again, if so then trigger editor
-            if( m_lastSelectedIndex != index )
-            {
-                m_lastSelectedIndex = index;
-            }
-            else
-            {
-                QTimer::singleShot( QApplication::doubleClickInterval(), this,
-                                    SLOT(slotEditTriggered())
-                                  );
-                m_lastSelectedIndex = QModelIndex();
-            }
-            event->accept();
-            return;
-        }
+        emit navigateToDirectory( index );
+        event->accept();
+        return;
     }
-    m_lastSelectedIndex = QModelIndex();
-    Amarok::PrettyTreeView::mouseReleaseEvent( event );
-}
 
-void
-FileView::slotEditTriggered()
-{
-    QModelIndexList indices = selectedIndexes();
-    if( indices.count() == 1 && indices.first().isValid() )
-        Amarok::PrettyTreeView::edit( indices.first() );
+    PrettyTreeView::mouseReleaseEvent( event );
 }
 
 void
 FileView::mouseDoubleClickEvent( QMouseEvent *event )
 {
-    m_lastSelectedIndex = QModelIndex();
     QModelIndex index = indexAt( event->pos() );
     if( !index.isValid() )
     {
         event->accept();
         return;
     }
-    emit activated( index );
-    event->accept();
+
+    // swallow middle-button double-clicks
+    if( event->button() == Qt::MidButton )
+    {
+        event->accept();
+        return;
+    }
+
+    if( event->button() == Qt::LeftButton )
+    {
+        KFileItem file = index.data( KDirModel::FileItemRole ).value<KFileItem>();
+        KUrl url = file.url();
+        if( !file.isNull() && ( Playlists::isPlaylist( url ) || MetaFile::Track::isTrack( url ) ) )
+            addIndexToPlaylist( index, Playlist::OnDoubleClickOnSelectedItems );
+        else
+            emit navigateToDirectory( index );
+
+        event->accept();
+        return;
+    }
+
+    PrettyTreeView::mouseDoubleClickEvent( event );
+}
+
+void
+FileView::keyPressEvent( QKeyEvent *event )
+{
+    QModelIndex index = currentIndex();
+    if( !index.isValid() )
+        return;
+
+    switch( event->key() )
+    {
+        case Qt::Key_Enter:
+        case Qt::Key_Return:
+        {
+            KFileItem file = index.data( KDirModel::FileItemRole ).value<KFileItem>();
+            KUrl url = file.url();
+            if( !file.isNull() && ( Playlists::isPlaylist( url ) || MetaFile::Track::isTrack( url ) ) )
+                // right, we test the current item, but then add the selection to playlist
+                addSelectionToPlaylist( Playlist::OnReturnPressedOnSelectedItems );
+            else
+                emit navigateToDirectory( index );
+
+            return;
+        }
+        case Qt::Key_Delete:
+            slotMoveToTrash( Qt::NoButton, event->modifiers() );
+            break;
+        case Qt::Key_F5:
+            emit refreshBrowser();
+            break;
+        default:
+            break;
+    }
+
+    QTreeView::keyPressEvent( event );
 }
 
 void
 FileView::slotAppendToPlaylist()
 {
-    addSelectionToPlaylist( false );
+    addSelectionToPlaylist( Playlist::OnAppendToPlaylistAction );
 }
 
-
 void
 FileView::slotReplacePlaylist()
 {
-    addSelectionToPlaylist( true );
+    addSelectionToPlaylist( Playlist::OnReplacePlaylistAction );
 }
 
 void
@@ -236,57 +270,54 @@ FileView::slotEditTracks()
 void
 FileView::slotPrepareMoveTracks()
 {
-    if( m_moveActivated )
+    if( m_moveDestinationCollection )
         return;
 
     CollectionAction *action = dynamic_cast<CollectionAction*>( sender() );
     if( !action )
         return;
 
-    m_moveActivated = true;
-    m_moveAction = action;
+    m_moveDestinationCollection = action->collection();
 
     const KFileItemList list = selectedItems();
     if( list.isEmpty() )
         return;
 
-    DirectoryLoader* dl = new DirectoryLoader();
-    connect( dl, SIGNAL(finished( const Meta::TrackList& )),
-                 SLOT(slotMoveTracks( const Meta::TrackList& )) );
+    // prevent bug 313003, require full metadata
+    TrackLoader* dl = new TrackLoader( TrackLoader::FullMetadataRequired ); // auto-deletes itself
+    connect( dl, SIGNAL(finished(Meta::TrackList)), SLOT(slotMoveTracks(Meta::TrackList)) );
     dl->init( list.urlList() );
 }
 
 void
 FileView::slotPrepareCopyTracks()
 {
-    if( m_copyActivated )
+    if( m_copyDestinationCollection )
         return;
 
     CollectionAction *action = dynamic_cast<CollectionAction*>( sender() );
     if( !action )
         return;
 
-    m_copyActivated = true;
-    m_copyAction = action;
+    m_copyDestinationCollection = action->collection();
 
     const KFileItemList list = selectedItems();
     if( list.isEmpty() )
         return;
 
-    DirectoryLoader* dl = new DirectoryLoader();
-    connect( dl, SIGNAL(finished( const Meta::TrackList & )),
-                 SLOT(slotCopyTracks( const Meta::TrackList & )) );
+    // prevent bug 313003, require full metadata
+    TrackLoader* dl = new TrackLoader( TrackLoader::FullMetadataRequired ); // auto-deletes itself
+    connect( dl, SIGNAL(finished(Meta::TrackList)), SLOT(slotCopyTracks(Meta::TrackList)) );
     dl->init( list.urlList() );
 }
 
 void
 FileView::slotCopyTracks( const Meta::TrackList& tracks )
 {
-    DEBUG_BLOCK
-    if( !m_copyAction || !m_copyActivated )
+    if( !m_copyDestinationCollection )
         return;
 
-    QSet<Collections::Collection*> collections;
+    QSet<Collections::Collection *> collections;
     foreach( const Meta::TrackPtr &track, tracks )
     {
         collections.insert( track->collection() );
@@ -297,134 +328,140 @@ FileView::slotCopyTracks( const Meta::TrackList& tracks )
         Collections::Collection *sourceCollection = collections.values().first();
         Collections::CollectionLocation *source;
         if( sourceCollection )
-        {
             source = sourceCollection->location();
-        }
         else
-        {
             source = new Collections::FileCollectionLocation();
-        }
-        Collections::CollectionLocation *destination = m_copyAction->collection()->location();
+
+        Collections::CollectionLocation *destination = m_copyDestinationCollection.data()->location();
         source->prepareCopy( tracks, destination );
     }
     else
-    {
         warning() << "Cannot handle copying tracks from multiple collections, doing nothing to be safe";
-    }
-    m_copyActivated = false;
-    m_copyAction = 0;
+
+    m_copyDestinationCollection.clear();
 }
 
 void
 FileView::slotMoveTracks( const Meta::TrackList& tracks )
 {
-    if( !m_moveAction || !m_moveActivated )
+    if( !m_moveDestinationCollection )
         return;
 
-    QSet<Collections::Collection*> collections;
+    QSet<Collections::Collection *> collections;
     foreach( const Meta::TrackPtr &track, tracks )
     {
         collections.insert( track->collection() );
     }
+
     if( collections.count() == 1 )
     {
         Collections::Collection *sourceCollection = collections.values().first();
         Collections::CollectionLocation *source;
         if( sourceCollection )
-        {
             source = sourceCollection->location();
-        }
         else
-        {
             source = new Collections::FileCollectionLocation();
-        }
-        Collections::CollectionLocation *destination = m_moveAction->collection()->location();
 
+        Collections::CollectionLocation *destination = m_moveDestinationCollection.data()->location();
         source->prepareMove( tracks, destination );
     }
     else
-    {
         warning() << "Cannot handle moving tracks from multiple collections, doing nothing to be safe";
-    }
-    m_moveActivated = false;
-    m_moveAction = 0;
+
+    m_moveDestinationCollection.clear();
 }
 
 QList<QAction *>
-FileView::actionsForIndices( const QModelIndexList &indices )
+FileView::actionsForIndices( const QModelIndexList &indices, ActionType type )
 {
     QList<QAction *> actions;
 
     if( indices.isEmpty() )
         return actions; // get out of here!
 
-    if( m_appendAction == 0 )
+    if( !m_appendAction )
     {
         m_appendAction = new QAction( KIcon( "media-track-add-amarok" ), i18n( "&Add to Playlist" ),
-                                     this );
+                                      this );
         m_appendAction->setProperty( "popupdropper_svg_id", "append" );
-        connect( m_appendAction, SIGNAL( triggered() ), this, SLOT( slotAppendToPlaylist() ) );
+        connect( m_appendAction, SIGNAL(triggered()), this, SLOT(slotAppendToPlaylist()) );
     }
+    if( type & PlaylistAction )
+        actions.append( m_appendAction );
 
-    actions.append( m_appendAction );
-
-    if( m_loadAction == 0 )
+    if( !m_loadAction )
     {
-        m_loadAction = new QAction( KIcon( "folder-open" ),
-                                   i18nc( "Replace the currently loaded tracks with these",
-                                          "&Replace Playlist" ),
-                                   this
-                                  );
+        m_loadAction = new QAction( i18nc( "Replace the currently loaded tracks with these",
+                                           "&Replace Playlist" ), this );
         m_loadAction->setProperty( "popupdropper_svg_id", "load" );
-        connect( m_loadAction, SIGNAL( triggered() ), this, SLOT( slotReplacePlaylist() ) );
+        connect( m_loadAction, SIGNAL(triggered()), this, SLOT(slotReplacePlaylist()) );
     }
+    if( type & PlaylistAction )
+        actions.append( m_loadAction );
 
-    actions.append( m_loadAction );
+    if( !m_moveToTrashAction )
+    {
+        m_moveToTrashAction = new KAction( KIcon( "user-trash" ), i18n( "&Move to Trash" ), this );
+        m_moveToTrashAction->setProperty( "popupdropper_svg_id", "delete_file" );
+        // key shortcut is only for display purposes here, actual one is determined by View in Model/View classes
+        m_moveToTrashAction->setShortcut( Qt::Key_Delete );
+        connect( m_moveToTrashAction, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
+                 this, SLOT(slotMoveToTrash(Qt::MouseButtons,Qt::KeyboardModifiers)) );
+    }
+    if( type & OrganizeAction )
+        actions.append( m_moveToTrashAction );
 
-    if( m_editAction == 0 )
+    if( !m_deleteAction )
+    {
+        m_deleteAction = new KAction( KIcon( "remove-amarok" ), i18n( "&Delete" ), this );
+        m_deleteAction->setProperty( "popupdropper_svg_id", "delete_file" );
+        // key shortcut is only for display purposes here, actual one is determined by View in Model/View classes
+        m_deleteAction->setShortcut( Qt::SHIFT + Qt::Key_Delete );
+        connect( m_deleteAction, SIGNAL(triggered(bool)), SLOT(slotDelete()) );
+    }
+    if( type & OrganizeAction )
+        actions.append( m_deleteAction );
+
+    if( !m_editAction )
     {
         m_editAction = new QAction( KIcon( "media-track-edit-amarok" ),
                                     i18n( "&Edit Track Details" ), this );
         m_editAction->setProperty( "popupdropper_svg_id", "edit" );
-        connect( m_editAction, SIGNAL( triggered() ), this, SLOT( slotEditTracks() ) );
+        connect( m_editAction, SIGNAL(triggered()), this, SLOT(slotEditTracks()) );
     }
-
-    actions.append( m_editAction );
-
-    if( m_separator1 == 0 )
+    if( type & EditAction )
     {
-            m_separator1 = new QAction( this );
-            m_separator1->setSeparator( true );
+        actions.append( m_editAction );
+        Meta::TrackList tracks = tracksForEdit();
+        m_editAction->setVisible( !tracks.isEmpty() );
     }
 
-    actions.append( m_separator1 );
-
-    if( m_deleteAction == 0 )
-    {
-        m_deleteAction = new KAction( KIcon( "media-track-remove-amarok" ), i18n( "&Delete" ), this );
-        m_deleteAction->setProperty( "popupdropper_svg_id", "delete_file" );
-        connect( m_deleteAction, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
-                 this, SLOT(slotDelete(Qt::MouseButtons,Qt::KeyboardModifiers)) );
-    }
-
-    actions.append( m_deleteAction );
-
-    Meta::TrackList tracks = tracksForEdit();
-    m_editAction->setEnabled( !tracks.isEmpty() );
-
     return actions;
 }
 
 void
-FileView::addSelectionToPlaylist( bool replace )
+FileView::addSelectionToPlaylist( Playlist::AddOptions options )
 {
-    QModelIndexList indices = selectedIndexes();
+    addIndicesToPlaylist( selectedIndexes(), options );
+}
 
-    if( indices.count() == 0 )
+void
+FileView::addIndexToPlaylist( const QModelIndex &idx, Playlist::AddOptions options )
+{
+    addIndicesToPlaylist( QModelIndexList() << idx, options );
+}
+
+void
+FileView::addIndicesToPlaylist( QModelIndexList indices, Playlist::AddOptions options )
+{
+    if( indices.isEmpty() )
         return;
-    QList<KUrl> urls;
 
-    foreach( const QModelIndex& index, indices )
+    // let tracks & playlists appear in playlist as they are shown in the view:
+    qSort( indices );
+
+    QList<KUrl> urls;
+    foreach( const QModelIndex &index, indices )
     {
         KFileItem file = index.data( KDirModel::FileItemRole ).value<KFileItem>();
         KUrl url = file.url();
@@ -434,14 +471,12 @@ FileView::addSelectionToPlaylist( bool replace )
         }
     }
 
-    The::playlistController()->insertOptioned( urls, replace ? Playlist::Replace : Playlist::AppendAndPlay );
+    The::playlistController()->insertOptioned( urls, options );
 }
 
 void
 FileView::startDrag( Qt::DropActions supportedActions )
 {
-    m_lastSelectedIndex = QModelIndex();
-
     //setSelectionMode( QAbstractItemView::NoSelection );
     // When a parent item is dragged, startDrag() is called a bunch of times. Here we prevent that:
     m_dragMutex.lock();
@@ -476,7 +511,7 @@ FileView::startDrag( Qt::DropActions supportedActions )
 
     if( m_pd )
     {
-        connect( m_pd, SIGNAL( fadeHideFinished() ), m_pd, SLOT( clear() ) );
+        connect( m_pd, SIGNAL(fadeHideFinished()), m_pd, SLOT(clear()) );
         m_pd->hide();
     }
 
@@ -510,8 +545,8 @@ FileView::tracksForEdit() const
     if( indices.isEmpty() )
         return tracks;
 
-    foreach( const QModelIndex& index, indices )
-    {   
+    foreach( const QModelIndex &index, indices )
+    {
         KFileItem item = index.data( KDirModel::FileItemRole ).value<KFileItem>();
         Meta::TrackPtr track = CollectionManager::instance()->trackForUrl( item.url() );
         if( track )
@@ -521,7 +556,7 @@ FileView::tracksForEdit() const
 }
 
 void
-FileView::slotDelete( Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers )
+FileView::slotMoveToTrash( Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers )
 {
     Q_UNUSED( buttons )
     DEBUG_BLOCK
@@ -530,10 +565,10 @@ FileView::slotDelete( Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers
     if( indices.isEmpty() )
         return;
 
-    const bool skipTrash = modifiers.testFlag( Qt::ShiftModifier );
+    const bool deleting = modifiers.testFlag( Qt::ShiftModifier );
     QString caption;
     QString labelText;
-    if( skipTrash )
+    if( deleting  )
     {
         caption = i18nc( "@title:window", "Confirm Delete" );
         labelText = i18np( "Are you sure you want to delete this item?",
@@ -548,14 +583,6 @@ FileView::slotDelete( Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers
                            indices.count() );
     }
 
-    KDialog dialog;
-    dialog.setCaption( caption );
-    dialog.setButtons( KDialog::Ok | KDialog::Cancel );
-    QLabel label( labelText, &dialog );
-    dialog.setMainWidget( &label );
-    if( dialog.exec() != QDialog::Accepted )
-        return;
-
     KUrl::List urls;
     QStringList filepaths;
     foreach( const QModelIndex& index, indices )
@@ -565,22 +592,22 @@ FileView::slotDelete( Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers
         urls << file.url();
     }
 
-    const bool cont = KMessageBox::warningContinueCancelList(
-        0, labelText, filepaths, caption, KStandardGuiItem::remove() ) == KMessageBox::Continue;
+    KGuiItem confirmButton = deleting ? KStandardGuiItem::del() : KStandardGuiItem::remove();
 
-    if( !cont )
+    if( KMessageBox::warningContinueCancelList( this, labelText, filepaths, caption, confirmButton ) != KMessageBox::Continue )
         return;
 
-    KIO::Job *job = skipTrash
-        ? static_cast<KIO::Job*>( KIO::del( urls, KIO::HideProgressInfo ) )
-        : static_cast<KIO::Job*>( KIO::trash( urls, KIO::HideProgressInfo ) );
-
-    if( job )
+    if( deleting )
     {
-        QString statusText = i18ncp( "@info:status", "Moving to trash: 1 file",
-                                     "Moving to trash: %1 files", urls.count() );
-        Amarok::Components::logger()->newProgressOperation( job, statusText );
+        KIO::del( urls, KIO::HideProgressInfo );
+        return;
     }
+
+    KIO::trash( urls, KIO::HideProgressInfo );
 }
 
-#include "FileView.moc"
+void
+FileView::slotDelete()
+{
+    slotMoveToTrash( Qt::NoButton, Qt::ShiftModifier );
+}
