@@ -17,7 +17,7 @@
 #ifndef PLAYLISTBROWSERNSPLAYLISTBROWSER_H
 #define PLAYLISTBROWSERNSPLAYLISTBROWSER_H
 
-#include "BrowserCategoryList.h"
+#include "browsers/BrowserCategoryList.h"
 
 #include <QMap>
 
@@ -37,13 +37,7 @@ class PlaylistBrowser : public BrowserCategoryList
         PlaylistBrowser( const QString &name, QWidget *parent = 0 );
         ~PlaylistBrowser();
 
-
-    public slots:
-        void addCategory( int category );
-
     private:
-        PodcastCollection* m_localPodcasts;
-        PodcastCategory* m_podcastCategory;
         QMap<int, int> m_categoryIndexMap;
     };
 }
