@@ -20,8 +20,10 @@
 
 #include "AmarokMimeData.h"
 #include "playlistmanager/PlaylistManager.h"
-#include "playlist/PlaylistModelStack.h"
+#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModel.h"
 #include "core/support/Debug.h"
+#include "widgets/PrettyTreeRoles.h"
 
 #include <KIcon>
 
@@ -39,28 +41,17 @@ lessThanPlaylistTitles( const Playlists::PlaylistPtr &lhs, const Playlists::Play
 PlaylistBrowserModel::PlaylistBrowserModel( int playlistCategory )
     : m_playlistCategory( playlistCategory )
 {
-    //common, unconditional actions
-    m_appendAction = new QAction( KIcon( "media-track-add-amarok" ), i18n( "&Add to Playlist" ),
-                                  this );
-    m_appendAction->setProperty( "popupdropper_svg_id", "append" );
-    connect( m_appendAction, SIGNAL( triggered() ), this, SLOT( slotAppend() ) );
+    connect( The::playlistManager(), SIGNAL(updated(int)), SLOT(slotUpdate(int)) );
 
-    m_loadAction = new QAction( KIcon( "folder-open" ),
-                                i18nc( "Replace the currently loaded tracks with these",
-                                       "&Replace Playlist" ),
-                                this );
-    m_loadAction->setProperty( "popupdropper_svg_id", "load" );
-    connect( m_loadAction, SIGNAL( triggered() ), this, SLOT( slotLoad() ) );
+    connect( The::playlistManager(), SIGNAL(playlistAdded(Playlists::PlaylistPtr,int)),
+             SLOT(slotPlaylistAdded(Playlists::PlaylistPtr,int)) );
+    connect( The::playlistManager(), SIGNAL(playlistRemoved(Playlists::PlaylistPtr,int)),
+             SLOT(slotPlaylistRemoved(Playlists::PlaylistPtr,int)) );
+    connect( The::playlistManager(), SIGNAL(playlistUpdated(Playlists::PlaylistPtr,int)),
+             SLOT(slotPlaylistUpdated(Playlists::PlaylistPtr,int)) );
 
-    connect( The::playlistManager(), SIGNAL(updated( int )), SLOT(slotUpdate( int )) );
-
-    connect( The::playlistManager(), SIGNAL( playlistAdded( Playlists::PlaylistPtr, int ) ),
-             SLOT( slotPlaylistAdded( Playlists::PlaylistPtr,int ) ) );
-    connect( The::playlistManager(), SIGNAL( playlistRemoved( Playlists::PlaylistPtr, int ) ),
-             SLOT( slotPlaylistRemoved( Playlists::PlaylistPtr,int ) ) );
-
-    connect( The::playlistManager(), SIGNAL(renamePlaylist( Playlists::PlaylistPtr )),
-             SLOT(slotRenamePlaylist( Playlists::PlaylistPtr )) );
+    connect( The::playlistManager(), SIGNAL(renamePlaylist(Playlists::PlaylistPtr)),
+             SLOT(slotRenamePlaylist(Playlists::PlaylistPtr)) );
 
     m_playlists = loadPlaylists();
 }
@@ -68,57 +59,17 @@ PlaylistBrowserModel::PlaylistBrowserModel( int playlistCategory )
 QVariant
 PlaylistBrowserModel::data( const QModelIndex &index, int role ) const
 {
-    //Special negative index to support empty provider groups (PlaylistsByProviderProxy)
-    if( index.row() == -1 && index.column() == PlaylistBrowserModel::ProviderColumn )
-    {
-        QVariantList displayList;
-        QVariantList iconList;
-        QVariantList playlistCountList;
-        QVariantList providerActionsCountList;
-        QVariantList providerActionsList;
-        QVariantList providerByLineList;
-
-        //get data from empty providers
-        PlaylistProviderList providerList =
-                The::playlistManager()->providersForCategory( m_playlistCategory );
-        foreach( Playlists::PlaylistProvider *provider, providerList )
-        {
-            if( provider && ( provider->playlistCount() > 0 || provider->playlists().count() > 0 ) )
-                continue;
-
-            displayList << provider->prettyName();
-            iconList << provider->icon();
-            playlistCountList << provider->playlists().count();
-            providerActionsCountList << provider->providerActions().count();
-            providerActionsList <<  QVariant::fromValue( provider->providerActions() );
-            //TODO: after string freeze possibly add a string indicating it's empty.
-            providerByLineList << QString();
-        }
-
-        switch( role )
-        {
-            case Qt::DisplayRole:
-            case DescriptionRole:
-            case Qt::ToolTipRole: return displayList;
-            case Qt::DecorationRole: return iconList;
-            case PlaylistBrowserModel::ActionCountRole: return providerActionsCountList;
-            case PlaylistBrowserModel::ActionRole: return providerActionsList;
-            case PlaylistBrowserModel::ByLineRole: return providerByLineList;
-            case Qt::EditRole: return QVariant();
-        }
-    }
-
-    if( !index.isValid() )
-        return QVariant();
-
     int row = REMOVE_TRACK_MASK(index.internalId());
     Playlists::PlaylistPtr playlist = m_playlists.value( row );
 
     QString name;
-    QString description;
     KIcon icon;
     int playlistCount = 0;
     QList<QAction *> providerActions;
+    QList<Playlists::PlaylistProvider *> providers =
+        The::playlistManager()->getProvidersForPlaylist( playlist );
+    Playlists::PlaylistProvider *provider = providers.count() == 1 ? providers.first() : 0;
+    Meta::TrackPtr track;
 
     switch( index.column() )
     {
@@ -126,14 +77,13 @@ PlaylistBrowserModel::data( const QModelIndex &index, int role ) const
         {
             if( IS_TRACK(index) )
             {
-                Meta::TrackPtr track = playlist->tracks()[index.row()];
+                track = playlist->tracks()[index.row()];
                 name = track->prettyName();
                 icon = KIcon( "amarok_track" );
             }
             else
             {
                 name = playlist->prettyName();
-                description = playlist->description();
                 icon = KIcon( "amarok_playlist" );
             }
             break;
@@ -150,54 +100,50 @@ PlaylistBrowserModel::data( const QModelIndex &index, int role ) const
 
         case PlaylistBrowserModel::ProviderColumn: //source
         {
-            QList<Playlists::PlaylistProvider *> providers =
-                    The::playlistManager()->getProvidersForPlaylist( playlist );
-
             if( providers.count() > 1 )
             {
                 QVariantList nameData;
-                QVariantList descriptionData;
                 QVariantList iconData;
                 QVariantList playlistCountData;
                 QVariantList providerActionsData;
                 foreach( Playlists::PlaylistProvider *provider, providers )
                 {
-                    name = description = provider->prettyName();
+                    name = provider->prettyName();
                     nameData << name;
-                    descriptionData << description;
                     icon = provider->icon();
                     iconData << QVariant( icon );
                     playlistCount = provider->playlists().count();
-                    //TODO: after string freeze add a "loading" for -1"
-                    playlistCountData << i18ncp( "number of playlists from one source",
-                                                 "One Playlist", "%1 playlists",
-                                                 playlistCount );
+                    if( playlistCount >= 0 )
+                        playlistCountData << i18ncp(
+                                "number of playlists from one source",
+                                "One Playlist", "%1 playlists",
+                                playlistCount );
+                    else
+                        playlistCountData << i18nc(
+                                "normally number of playlists, but they are still loading",
+                                "Loading..." );
                     providerActions << provider->providerActions();
                     providerActionsData << QVariant::fromValue( providerActions );
                 }
                 switch( role )
                 {
                 case Qt::DisplayRole:
-                case Qt::EditRole: return nameData;
-                case DescriptionRole:
-                case Qt::ToolTipRole: return descriptionData;
-                case Qt::DecorationRole: return iconData;
-                case PlaylistBrowserModel::ByLineRole:
+                case Qt::EditRole:
+                case Qt::ToolTipRole:
+                    return nameData;
+                case Qt::DecorationRole:
+                    return iconData;
+                case PrettyTreeRoles::ByLineRole:
                     return playlistCountData;
-                case PlaylistBrowserModel::ActionRole:
+                case PrettyTreeRoles::DecoratorRoleCount:
+                    return providerActions.count();
+                case PrettyTreeRoles::DecoratorRole:
                     return providerActionsData;
                 }
             }
-            else if( providers.count() )
+            else if( provider )
             {
-                Playlists::PlaylistProvider *provider = providers.first();
-                //prevent crash
-                if( !provider )
-                {
-                    error() << "Invalid PlaylistProvider.";
-                    break;
-                }
-                name = description = provider->prettyName();
+                name = provider->prettyName();
                 icon = provider->icon();
                 playlistCount = provider->playlistCount();
                 providerActions << provider->providerActions();
@@ -206,35 +152,40 @@ PlaylistBrowserModel::data( const QModelIndex &index, int role ) const
             break;
         }
 
-        default: return QVariant();
+        default: break;
     }
 
 
     switch( role )
     {
         case Qt::DisplayRole:
-        case Qt::EditRole: return name;
-        case DescriptionRole:
-        case Qt::ToolTipRole: return description;
-        case Qt::DecorationRole: return QVariant( icon );
-        case PlaylistBrowserModel::ByLineRole:
-            return i18ncp( "number of playlists from one source",
-                           "One Playlist", "%1 playlists",
-                           playlistCount );
-        case PlaylistBrowserModel::ActionRole:
-            return QVariant::fromValue( index.column() == PlaylistBrowserModel::ProviderColumn ?
-                    providerActions : actionsFor( index ) );
-        case PlaylistBrowserModel::ActionCountRole:
-            return QVariant( index.column() == PlaylistBrowserModel::ProviderColumn ?
-                    providerActions.count() : actionsFor( index ).count() );
+        case Qt::EditRole:
+        case Qt::ToolTipRole:
+            return name;
+        case Qt::DecorationRole:
+            return QVariant( icon );
+        case PrettyTreeRoles::ByLineRole:
+            if( IS_TRACK(index) )
+                return QVariant();
+            else
+                return i18ncp( "number of playlists from one source", "One playlist",
+                               "%1 playlists", playlistCount );
+        case PlaylistBrowserModel::ProviderRole:
+            return provider ? QVariant::fromValue( provider ) : QVariant();
+        case PlaylistBrowserModel::PlaylistRole:
+            return playlist ? QVariant::fromValue( playlist ) : QVariant();
+        case PlaylistBrowserModel::TrackRole:
+            return track ? QVariant::fromValue( track ) : QVariant();
 
-        default: return QVariant();
+        default:
+            return QVariant();
     }
 }
 
 bool
 PlaylistBrowserModel::setData( const QModelIndex &idx, const QVariant &value, int role )
 {
+
     if( !idx.isValid() )
         return false;
 
@@ -254,7 +205,7 @@ PlaylistBrowserModel::setData( const QModelIndex &idx, const QVariant &value, in
                     if( !track )
                         return false;
                     debug() << QString( "Copy track \"%1\" to \"%2\"." )
-                            .arg( track->prettyName() ).arg( provider->prettyName() );
+                            .arg( track->prettyName(),  provider->prettyName() );
     //                return !provider->addTrack( track ).isNull();
                     provider->addTrack( track ); //ignore result since UmsPodcastProvider returns NULL
                     return true;
@@ -262,13 +213,22 @@ PlaylistBrowserModel::setData( const QModelIndex &idx, const QVariant &value, in
                 else
                 {
                     Playlists::PlaylistPtr playlist = playlistFromIndex( idx );
-                    if( !playlist )
+                    if( !playlist || ( playlist->provider() == provider ) )
                         return false;
+
+                    foreach( Playlists::PlaylistPtr tempPlaylist , provider->playlists() )
+                    {
+                        if ( tempPlaylist->name() == playlist->name() )
+                            return false;
+                    }
+
                     debug() << QString( "Copy playlist \"%1\" to \"%2\"." )
-                            .arg( playlist->prettyName() ).arg( provider->prettyName() );
+                            .arg( playlist->prettyName(), provider->prettyName() );
+
                     return !provider->addPlaylist( playlist ).isNull();
                 }
             }
+
             //return true even for the data we didn't handle to get QAbstractItemModel::setItemData to work
             //TODO: implement setItemData()
             return true;
@@ -278,6 +238,7 @@ PlaylistBrowserModel::setData( const QModelIndex &idx, const QVariant &value, in
             debug() << "changing group of item " << idx.internalId() << " to " << value.toString();
             Playlists::PlaylistPtr item = m_playlists.value( idx.internalId() );
             item->setGroups( value.toStringList() );
+
             return true;
         }
     }
@@ -290,11 +251,7 @@ PlaylistBrowserModel::index( int row, int column, const QModelIndex &parent) con
 {
     if( !parent.isValid() )
     {
-        //there are valid indexes available with row == -1 for empty groups and providers
-        if( row == -1 && column >= 0 )
-            return createIndex( row, column, row );
-
-        if( row < m_playlists.count() )
+        if( row >= 0 && row < m_playlists.count() )
             return createIndex( row, column, row );
     }
     else //if it has a parent it is a track
@@ -327,7 +284,7 @@ PlaylistBrowserModel::hasChildren( const QModelIndex &parent ) const
         return false;
     if( !parent.isValid() )
     {
-        return m_playlists.isEmpty();
+        return !m_playlists.isEmpty();
     }
     else if( !IS_TRACK(parent) )
     {
@@ -351,7 +308,7 @@ PlaylistBrowserModel::rowCount( const QModelIndex &parent ) const
     else if( !IS_TRACK(parent) )
     {
         Playlists::PlaylistPtr playlist = m_playlists.value( parent.internalId() );
-        return playlist->tracks().count();
+        return playlist->trackCount();
     }
 
     return 0;
@@ -401,6 +358,8 @@ PlaylistBrowserModel::fetchMore ( const QModelIndex &parent )
     if( !IS_TRACK(parent) )
     {
         Playlists::PlaylistPtr playlist = m_playlists.value( parent.internalId() );
+         // TODO: following doesn't seem to be needed, PlaylistBrowserModel seems to be able to cope with async track loading fine
+        playlist->makeLoadingSync();
         playlist->triggerTrackLoad();
     }
 }
@@ -427,6 +386,7 @@ PlaylistBrowserModel::flags( const QModelIndex &idx ) const
     return Qt::ItemIsEnabled | Qt::ItemIsEditable | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled |
            Qt::ItemIsDropEnabled;
 }
+
 QVariant
 PlaylistBrowserModel::headerData( int section, Qt::Orientation orientation, int role ) const
 {
@@ -479,48 +439,77 @@ bool
 PlaylistBrowserModel::dropMimeData( const QMimeData *data, Qt::DropAction action, int row, int column,
                                  const QModelIndex &parent )
 {
-    Q_UNUSED( column );
+    DEBUG_BLOCK
+    debug() << "Dropped on" << parent << "row" << row << "column" << column << "action" << action;
     if( action == Qt::IgnoreAction )
         return true;
 
+    //drop on track is not possible
+    if( IS_TRACK(parent) )
+        return false;
+
     const AmarokMimeData* amarokMime = dynamic_cast<const AmarokMimeData*>( data );
     if( !amarokMime )
         return false;
 
     if( data->hasFormat( AmarokMimeData::PLAYLIST_MIME ) )
     {
+        // TODO: is this ever called????
         Playlists::PlaylistList playlists = amarokMime->playlists();
 
         foreach( Playlists::PlaylistPtr playlist, playlists )
         {
             if( !m_playlists.contains( playlist ) )
-            {
-                debug() << "unknown playlist dragged in: " << playlist->prettyName();
-                debug() << "TODO: start synchronization";
-            }
+                debug() << "Unknown playlist dragged in: " << playlist->prettyName();
         }
 
         return true;
     }
     else if( data->hasFormat( AmarokMimeData::TRACK_MIME ) )
     {
-        debug() << "Dropped track on " << parent << " at row: " << row;
-
-        Playlists::PlaylistPtr playlist = playlistFromIndex( parent );
-        if( !playlist )
-            return false;
-
         Meta::TrackList tracks = amarokMime->tracks();
-        bool allAdded = true;
-        foreach( Meta::TrackPtr track, tracks )
-            /*allAdded = */playlist->addTrack( track, row )/* ? allAdded : false*/;
+        if( !parent.isValid() && row == -1 && column == -1 )
+        {
+            debug() << "Dropped tracks on empty area: create new playlist in a default provider";
+            The::playlistManager()->save( tracks, Amarok::generatePlaylistName( tracks ) );
+            return true;
+        }
+        else if( !parent.isValid() )
+        {
+            warning() << "Dropped tracks between root items, this is not supported!";
+            return false;
+        }
+        else
+        {
+            debug() << "Dropped tracks on " << parent << " at row: " << row;
 
-        return allAdded;
+            Playlists::PlaylistPtr playlist = playlistFromIndex( parent );
+            if( !playlist )
+                return false;
+
+            foreach( Meta::TrackPtr track, tracks )
+                playlist->addTrack( track, ( row >= 0 ) ? row++ : -1 ); // increment only if positive
+
+            return true;
+        }
     }
 
     return false;
 }
 
+void
+PlaylistBrowserModel::metadataChanged( Playlists::PlaylistPtr playlist )
+{
+    int indexNumber = m_playlists.indexOf( playlist );
+    if( indexNumber == -1 )
+    {
+        error() << "This playlist is not in the list of this model.";
+        return;
+    }
+    QModelIndex playlistIdx = index( indexNumber, 0 );
+    emit dataChanged( playlistIdx, playlistIdx );
+}
+
 void
 PlaylistBrowserModel::trackAdded( Playlists::PlaylistPtr playlist, Meta::TrackPtr track,
                                           int position )
@@ -554,25 +543,19 @@ PlaylistBrowserModel::trackRemoved( Playlists::PlaylistPtr playlist, int positio
 void
 PlaylistBrowserModel::slotRenamePlaylist( Playlists::PlaylistPtr playlist )
 {
-    if( playlist->provider()->category() != m_playlistCategory )
+    if( !playlist->provider() || playlist->provider()->category() != m_playlistCategory )
         return;
 
-    //search index of this Playlist
-    // HACK: matches first to match same name, but there could be
-    // several playlists with the same name
-    int row = -1;
-    foreach( const Playlists::PlaylistPtr p, m_playlists )
+    int row = 0;
+    foreach( Playlists::PlaylistPtr p, m_playlists )
     {
-        row++;
-        if( p->name() == playlist->name() )
+        if( p == playlist )
+        {
+            emit renameIndex( index( row, 0 ) );
             break;
+        }
+        row++;
     }
-    if( row == -1 )
-        return;
-
-    QModelIndex idx = index( row, 0, QModelIndex() );
-    debug() << idx;
-    emit( renameIndex( idx ) );
 }
 
 void
@@ -612,34 +595,6 @@ PlaylistBrowserModel::loadPlaylists()
     return playlists;
 }
 
-void
-PlaylistBrowserModel::slotLoad()
-{
-    QAction *action = qobject_cast<QAction *>( QObject::sender() );
-    if( action == 0 )
-        return;
-
-    QModelIndexList indexes = action->data().value<QModelIndexList>();
-
-    Meta::TrackList tracks = tracksFromIndexes( indexes );
-    if( !tracks.isEmpty() )
-        The::playlistController()->insertOptioned( tracks, Playlist::LoadAndPlay );
-}
-
-void
-PlaylistBrowserModel::slotAppend()
-{
-    QAction *action = qobject_cast<QAction *>( QObject::sender() );
-    if( action == 0 )
-        return;
-
-    QModelIndexList indexes = action->data().value<QModelIndexList>();
-
-    Meta::TrackList tracks = tracksFromIndexes( indexes );
-    if( !tracks.isEmpty() )
-        The::playlistController()->insertOptioned( tracks, Playlist::AppendAndPlay );
-}
-
 void
 PlaylistBrowserModel::slotPlaylistAdded( Playlists::PlaylistPtr playlist, int category )
 {
@@ -669,7 +624,7 @@ PlaylistBrowserModel::slotPlaylistRemoved( Playlists::PlaylistPtr playlist, int
     int position = m_playlists.indexOf( playlist );
     if( position == -1 )
     {
-        error() << "signal removed playlist not in m_playlists";
+        error() << "signal received for removed playlist not in m_playlists";
         return;
     }
 
@@ -678,6 +633,32 @@ PlaylistBrowserModel::slotPlaylistRemoved( Playlists::PlaylistPtr playlist, int
     endRemoveRows();
 }
 
+void
+PlaylistBrowserModel::slotPlaylistUpdated( Playlists::PlaylistPtr playlist, int category )
+{
+    if( category != m_playlistCategory )
+        return;
+
+    int position = m_playlists.indexOf( playlist );
+    if( position == -1 )
+    {
+        error() << "signal received for updated playlist not in m_playlists";
+        return;
+    }
+
+    //TODO: this should work by signaling a change in the model data, but QtGroupingProxy doesn't
+    //work like that ATM
+//    const QModelIndex &idx = index( position, 0 );
+//    emit dataChanged( idx, idx );
+
+    //HACK: remove and readd so QtGroupingProxy can put it in the correct groups.
+    beginRemoveRows( QModelIndex(), position, position );
+    endRemoveRows();
+
+    beginInsertRows( QModelIndex(), position, position );
+    endInsertRows();
+}
+
 Meta::TrackList
 PlaylistBrowserModel::tracksFromIndexes( const QModelIndexList &list ) const
 {
@@ -688,6 +669,7 @@ PlaylistBrowserModel::tracksFromIndexes( const QModelIndexList &list ) const
             tracks << trackFromIndex( index );
         else if( Playlists::PlaylistPtr playlist = playlistFromIndex( index ) )
         {
+            playlist->makeLoadingSync();
             //first trigger a load of the tracks or we'll end up with an empty list
             playlist->triggerTrackLoad();
             tracks << playlist->tracks();
@@ -740,34 +722,6 @@ PlaylistBrowserModel::providerForIndex( const QModelIndex &idx ) const
     return m_playlists.at( playlistRow )->provider();
 }
 
-QActionList
-PlaylistBrowserModel::actionsFor( const QModelIndex &idx ) const
-{
-    //wheter we use the list from m_appendAction of m_loadAction does not matter they are the same
-    QModelIndexList actionList = m_appendAction->data().value<QModelIndexList>();
-
-    actionList << idx;
-    QVariant value = QVariant::fromValue( actionList );
-    m_appendAction->setData( value );
-    m_loadAction->setData( value );
-
-    QActionList actions;
-    actions << m_appendAction << m_loadAction;
-
-    if( !IS_TRACK(idx) )
-    {
-        Playlists::PlaylistPtr playlist = m_playlists.value( idx.internalId() );
-        actions << playlist->actions();
-    }
-    else
-    {
-        Playlists::PlaylistPtr playlist = m_playlists.value( idx.parent().internalId() );
-        actions << playlist->trackActions( idx.row() );
-    }
-
-    return actions;
-}
-
 Playlists::PlaylistProvider *
 PlaylistBrowserModel::getProviderByName( const QString &name )
 {
