@@ -15,6 +15,8 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "ContextView"
+
 /*
   Significant parts of this code is inspired and/or copied from KDE plasma sources,
   available at kdebase/workspace/plasma
@@ -22,23 +24,27 @@
 
 #include "ContextView.h"
 
-#include "Amarok.h"
+#include <config.h>
+
 #include "Context.h"
 #include "ContextScene.h"
-#include "Debug.h"
 #include "Svg.h"
 #include "Theme.h"
 #include "amarokconfig.h"
 #include "amarokurls/AmarokUrlHandler.h"
 #include "amarokurls/ContextUrlRunner.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core/meta/Meta.h"
+#include "EngineController.h"
 
 #include <plasma/dataenginemanager.h>
 
+#include <Phonon/AudioOutput>
+#include <QParallelAnimationGroup>
+#include <QSequentialAnimationGroup>
 #include <QWheelEvent>
 
-
-#define DEBUG_PREFIX "ContextView"
-
 namespace Context
 {
 
@@ -47,17 +53,18 @@ ContextView* ContextView::s_self = 0;
 
 ContextView::ContextView( Plasma::Containment *cont, Plasma::Corona *corona, QWidget* parent )
     : Plasma::View( cont, parent )
-    , EngineObserver( The::engineController() )
     , m_curState( Home )
-    , m_firstPlayingState( true )
-    , m_appletExplorer( 0 )
+    , m_urlRunner(0)
+    , m_appletExplorer(0)
+    , m_collapseAnimations(0)
+    , m_queuedAnimations(0)
+    , m_collapseGroupTimer(0)
 {
     Q_UNUSED( corona )
     DEBUG_BLOCK
 
-    s_self = this;
-
-    scene()->setItemIndexMethod( QGraphicsScene::BspTreeIndex );
+    // using QGraphicsScene::BspTreeIndex leads to crashes in some Qt versions
+    scene()->setItemIndexMethod( QGraphicsScene::NoIndex );
     //TODO: Figure out a way to use rubberband and ScrollHandDrag
     //setDragMode( QGraphicsView::RubberBandDrag );
     setTransformationAnchor( QGraphicsView::NoAnchor );
@@ -65,9 +72,7 @@ ContextView::ContextView( Plasma::Containment *cont, Plasma::Corona *corona, QWi
     setInteractive( true );
     setAcceptDrops( true );
     setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-   // setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-    setMouseTracking( true );
-    setScreen( -1 );
+    // setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 
     //make background transparent
     QPalette p = palette();
@@ -76,35 +81,35 @@ ContextView::ContextView( Plasma::Containment *cont, Plasma::Corona *corona, QWi
     p.setColor( QPalette::Base, c );
     setPalette( p );
 
-    PERF_LOG( "Accessing Plasma::Theme" );
-    // here we initialize all the Plasma paths to Amarok paths
-    Plasma::Theme::defaultTheme()->setUseGlobalSettings( false );
-    Theme::defaultTheme()->setThemeName( "Amarok-Mockup" );
-    PERF_LOG( "Access to Plasma::Theme complete" )
     contextScene()->setAppletMimeType( "text/x-amarokappletservicename" );
 
     cont->setPos( 0, 0 );
     cont->updateConstraints();
     Containment* amarokContainment = qobject_cast<Containment* >( cont );
     if( amarokContainment )
-    {
         amarokContainment->setView( this );
-    //    amarokContainment->addCurrentTrack();
-    }
-
-    m_appletExplorer = new AppletExplorer( cont );
-    m_appletExplorer->setContainment( amarokContainment );
-    m_appletExplorer->setPos( 0, cont->size().height() - m_appletExplorer->size().height() );
-    m_appletExplorer->setZValue( m_appletExplorer->zValue() + 1000 );
-    m_appletExplorer->hide();
-
-    connect( m_appletExplorer, SIGNAL( addAppletToContainment( const QString&, const int ) ),
-             amarokContainment, SLOT( addApplet( const QString&, const int ) ) );
-
-    connect( m_appletExplorer, SIGNAL( appletExplorerHid() ), this, SIGNAL( appletExplorerHid() ) );
 
     m_urlRunner = new ContextUrlRunner();
     The::amarokUrlHandler()->registerRunner( m_urlRunner, "context" );
+
+    m_queuedAnimations = new QSequentialAnimationGroup( this );
+    m_collapseAnimations = new QParallelAnimationGroup( this );
+    connect( m_collapseAnimations, SIGNAL(finished()),
+             this, SLOT(slotCollapseAnimationsFinished()) );
+
+    m_collapseGroupTimer = new QTimer( this );
+    m_collapseGroupTimer->setSingleShot( true );
+    connect( m_collapseGroupTimer, SIGNAL(timeout()), SLOT(slotStartCollapseAnimations()) );
+
+    EngineController* const engine = The::engineController();
+
+    connect( engine, SIGNAL(trackChanged(Meta::TrackPtr)),
+             this, SLOT(slotTrackChanged(Meta::TrackPtr)) );
+    connect( engine, SIGNAL(trackMetadataChanged(Meta::TrackPtr)),
+             this, SLOT(slotMetadataChanged(Meta::TrackPtr)) );
+
+    // keep this assignment at bottom so that premature usage of ::self() asserts out
+    s_self = this;
 }
 
 ContextView::~ContextView()
@@ -165,35 +170,22 @@ void ContextView::clearNoSave()
 }
 
 
-void ContextView::enginePlaybackEnded( qint64 finalPosition, qint64 trackLength, EngineObserver::PlaybackEndedReason reason )
+void ContextView::slotTrackChanged( Meta::TrackPtr track )
 {
-    Q_UNUSED( finalPosition )
-    Q_UNUSED( trackLength )
-    Q_UNUSED( reason )
-    DEBUG_BLOCK
-
-    messageNotify( Home );
-}
-
-
-void ContextView::engineNewTrackPlaying()
-{
-    DEBUG_BLOCK
-    messageNotify( Current );
-    m_firstPlayingState = false;
+    if( track )
+        messageNotify( Current );
+    else
+        messageNotify( Home );
 }
 
 
 void
-ContextView::engineNewMetaData( const QHash<qint64, QString> &newMetaData, bool trackChanged )
+ContextView::slotMetadataChanged( Meta::TrackPtr track )
 {
-    Q_UNUSED( newMetaData )
-    Q_UNUSED( trackChanged )
     DEBUG_BLOCK
 
     // if we are listening to a stream, take the new metadata as a "new track"
-    Meta::TrackPtr track = The::engineController()->currentTrack();
-    if( track && track->type() == "stream" )
+    if( track && The::engineController()->isStream() )
         messageNotify( Current );
 }
 
@@ -213,6 +205,7 @@ ContextView::loadConfig()
 {
     contextScene()->clearContainments();
 
+    PERF_LOG( "Start to load config" );
     int numContainments = contextScene()->containments().size();
     KConfig conf( "amarok_homerc", KConfig::FullConfig );
     for( int i = 0; i < numContainments; i++ )
@@ -221,37 +214,116 @@ ContextView::loadConfig()
         if( containment )
         {
             KConfigGroup cg( &conf, QString( "Containment %1" ).arg( i ) );
+#ifdef QT_QTOPENGL_FOUND
+            // Special case: If this is the first time that the user runs an Amarok version
+            // containing the Analyzer applet, modify the user's config so that the applet
+            // will become active. We do this for discoverability and prettiness.
+            // Remove this code in a future Amarok release (possibly 3.0)
+            const bool firstTimeWithAnalyzer = Amarok::config( "Context View" ).readEntry( "firstTimeWithAnalyzer", true );
+            if( firstTimeWithAnalyzer )
+            {
+                QStringList plugins = cg.readEntry( "plugins", QStringList() );
+                if( EngineController::instance()->supportsAudioDataOutput() && !plugins.contains( "analyzer" ) )
+                {
+                    Amarok::config( "Context View" ).writeEntry( "firstTimeWithAnalyzer", false );
+
+                    // Put the Analyzer applet at position #2, which is most likely below the Currenttrack applet.
+                    if( !plugins.empty() )
+                        plugins.insert( 1, "analyzer" );
+
+                    cg.writeEntry( "plugins", plugins );
+                }
+            }
+#endif
             containment->loadConfig( cg );
         }
     }
+    PERF_LOG( "Done loading config" );
 }
 
-Plasma::Applet*
-ContextView::addApplet( const QString& name, const QStringList& args )
+void
+ContextView::addCollapseAnimation( QAbstractAnimation *anim )
 {
-    QVariantList argList;
-    QStringListIterator i(args);
-    while( i.hasNext() )
-        argList << QVariant( i.next() );
+    if( !anim )
+    {
+        debug() << "failed to add collapsing animation";
+        return;
+    }
 
-    if( !containment() )
-        contextScene()->addContainment( "amarok_containment_vertical" );
+    if( m_collapseAnimations->state() == QAbstractAnimation::Running ||
+        m_collapseGroupTimer->isActive() )
+    {
+        m_queuedAnimations->addAnimation( anim );
+    }
+    else
+    {
+        m_collapseAnimations->addAnimation( anim );
+        m_collapseGroupTimer->start( 0 );
+    }
+}
 
-    return containment()->addApplet( name, argList );
+void
+ContextView::slotCollapseAnimationsFinished()
+{
+    m_collapseGroupTimer->stop();
+    m_collapseAnimations->clear();
+
+    while( m_queuedAnimations->animationCount() > 0 )
+    {
+        if( QAbstractAnimation *anim = m_queuedAnimations->takeAnimation(0) )
+            m_collapseAnimations->addAnimation( anim );
+    }
+
+    if( m_collapseAnimations->animationCount() > 0 )
+        m_collapseGroupTimer->start( 0 );
+}
+
+void
+ContextView::slotStartCollapseAnimations()
+{
+    if( m_collapseAnimations->animationCount() > 0 )
+        m_collapseAnimations->start( QAbstractAnimation::KeepWhenStopped );
 }
 
 void
 ContextView::hideAppletExplorer()
 {
-    m_appletExplorer->hide();
+    if( m_appletExplorer )
+        m_appletExplorer->hide();
 }
 
 void
 ContextView::showAppletExplorer()
 {
+    if( !m_appletExplorer )
+    {
+        Context::Containment *cont = qobject_cast<Context::Containment*>( containment() );
+        m_appletExplorer = new AppletExplorer( cont );
+        m_appletExplorer->setContainment( cont );
+        m_appletExplorer->setZValue( m_appletExplorer->zValue() + 1000 );
+        m_appletExplorer->setFlag( QGraphicsItem::ItemIsSelectable );
+
+        connect( m_appletExplorer, SIGNAL(addAppletToContainment(QString,int)),
+                 cont, SLOT(addApplet(QString,int)) );
+        connect( m_appletExplorer, SIGNAL(appletExplorerHid()), SIGNAL(appletExplorerHid()) );
+        connect( m_appletExplorer, SIGNAL(geometryChanged()), SLOT(slotPositionAppletExplorer()) );
+
+        qreal height = m_appletExplorer->effectiveSizeHint( Qt::PreferredSize ).height();
+        m_appletExplorer->resize( rect().width() - 2, height );
+        m_appletExplorer->setPos( 0, rect().height() - height - 2 );
+    }
     m_appletExplorer->show();
 }
 
+void
+ContextView::slotPositionAppletExplorer()
+{
+    if( !m_appletExplorer )
+        return;
+    qreal height = m_appletExplorer->effectiveSizeHint( Qt::PreferredSize ).height();
+    m_appletExplorer->setPos( 0, rect().height() - height - 2 );
+}
+
 
 ContextScene*
 ContextView::contextScene()
@@ -262,29 +334,28 @@ ContextView::contextScene()
 void
 ContextView::resizeEvent( QResizeEvent* event )
 {
-    Q_UNUSED( event )
-
-    if ( testAttribute( Qt::WA_PendingResizeEvent ) )
+    Plasma::View::resizeEvent( event );
+    if( testAttribute( Qt::WA_PendingResizeEvent ) )
         return; // lets not do this more than necessary, shall we?
 
-   updateContainmentsGeometry();
-}
+    QRectF rect( pos(), maximumViewportSize() );
+    containment()->setGeometry( rect );
+    scene()->setSceneRect( rect );
+    scene()->update( rect );
 
-
-void
-ContextView::updateContainmentsGeometry()
-{
-    containment()->resize( rect().size() );
-    containment()->setPos( rect().topLeft() );
-    m_appletExplorer->resize( rect().width(), m_appletExplorer->size().height() );
-    m_appletExplorer->setPos( 0, rect().height() - m_appletExplorer->size().height() - 5 );
+    if( m_appletExplorer )
+    {
+        qreal height = m_appletExplorer->effectiveSizeHint( Qt::PreferredSize ).height();
+        m_appletExplorer->resize( rect.width() - 2, height );
+        m_appletExplorer->setPos( 0, rect.height() - height - 2 );
+    }
 }
 
 void
 ContextView::wheelEvent( QWheelEvent* event )
 {
     if( event->orientation() != Qt::Horizontal )
-        QGraphicsView::wheelEvent( event );
+        Plasma::View::wheelEvent( event );
 }
 
 QStringList
@@ -323,4 +394,3 @@ QStringList ContextView::currentAppletNames()
 } // Context namespace
 
 #include "ContextView.moc"
-
