@@ -20,27 +20,29 @@
 
 #include "CurrentTrack.h"
 
-#include "amarokurls/AmarokUrl.h"
-#include "core/support/Amarok.h"
 #include "App.h"
-#include "core/support/Debug.h"
 #include "EngineController.h"
 #include "GlobalCurrentTrackActions.h"
-#include "CollectionManager.h"
-#include "covermanager/CoverViewDialog.h"
-#include "core/capabilities/EditCapability.h"
-#include "core/capabilities/ActionsCapability.h"
-#include "core/capabilities/BookmarkThisCapability.h"
-#include "core/capabilities/FindInSourceCapability.h"
-#include "core/meta/support/MetaUtility.h"
 #include "MainWindow.h"
 #include "PaletteHandler.h"
 #include "PluginManager.h"
 #include "SvgHandler.h"
+#include "amarokurls/AmarokUrl.h"
 #include "context/widgets/RatingWidget.h"
 #include "context/widgets/TextScrollingWidget.h"
 #include "context/widgets/DropPixmapItem.h"
 #include "context/widgets/RecentlyPlayedListWidget.h"
+#include "core/capabilities/ActionsCapability.h"
+#include "core/capabilities/BookmarkThisCapability.h"
+#include "core/capabilities/FindInSourceCapability.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core/meta/Meta.h"
+#include "core/meta/Statistics.h"
+#include "core/meta/TrackEditor.h"
+#include "core/meta/support/MetaUtility.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "covermanager/CoverViewDialog.h"
 #include "dialogs/TagDialog.h"
 
 #include <KConfigDialog>
@@ -95,11 +97,13 @@ CurrentTrack::init()
     // Call the base implementation.
     Context::Applet::init();
 
+    setToolTip( i18n( "Right-click to configure" ) );
+
     m_ratingWidget = new RatingWidget( this );
     m_ratingWidget->setSpacing( 2 );
     m_ratingWidget->setMinimumSize( m_albumWidth + 10, 30 );
     m_ratingWidget->setMaximumSize( m_albumWidth + 10, 30 );
-    connect( m_ratingWidget, SIGNAL( ratingChanged( int ) ), SLOT( trackRatingChanged( int ) ) );
+    connect( m_ratingWidget, SIGNAL(ratingChanged(int)), SLOT(trackRatingChanged(int)) );
 
     QLabel *collectionLabel = new QLabel( i18n( "Local Collection" ) );
     collectionLabel->setAttribute( Qt::WA_NoSystemBackground );
@@ -214,7 +218,7 @@ CurrentTrack::trackRatingChanged( int rating )
     if( !track )
         return;
 
-    track->setRating( rating );
+    track->statistics()->setRating( rating );
 }
 
 QList<QAction*>
@@ -308,7 +312,6 @@ CurrentTrack::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
     if( data.isEmpty() || name != QLatin1String("current") )
         return;
 
-    DEBUG_BLOCK
     if( data.contains( QLatin1String("notrack" ) ) )
     {
         if( m_view != Stopped )
@@ -568,7 +571,6 @@ CurrentTrack::clearTrackActions()
 void
 CurrentTrack::resizeCover( const QPixmap &cover, qreal width )
 {
-    DEBUG_BLOCK
     QPixmap coverWithBorders;
     if( !cover.isNull() )
     {
@@ -659,6 +661,8 @@ CurrentTrack::artistsCounted( QStringList results )
 void
 CurrentTrack::createConfigurationInterface( KConfigDialog *parent )
 {
+    parent->setButtons( KDialog::Ok | KDialog::Cancel );
+
     KConfigGroup configuration = config();
     QWidget *settings = new QWidget;
     ui_Settings.setupUi( settings );
@@ -764,16 +768,12 @@ CurrentTrack::setupLayoutActions( Meta::TrackPtr track )
         actions << btc->bookmarkAction();
     }
 
-    if( m_showEditTrackDetailsAction && track->has<EditCapability>() )
+    if( m_showEditTrackDetailsAction && track->editor() )
     {
-        QScopedPointer<EditCapability> ec( track->create<EditCapability>() );
-        if( ec && ec->isEditable() )
-        {
-            QAction *editAction = new QAction( KIcon("media-track-edit-amarok"),
-                                               i18n("Edit Track Details"), this );
-            connect( editAction, SIGNAL(triggered()), SLOT(editTrack()) );
-            m_customActions << editAction;
-        }
+        QAction *editAction = new QAction( KIcon("media-track-edit-amarok"),
+                                           i18n("Edit Track Details"), this );
+        connect( editAction, SIGNAL(triggered()), SLOT(editTrack()) );
+        m_customActions << editAction;
     }
 
     if( track->has<FindInSourceCapability>() )
@@ -805,7 +805,8 @@ CurrentTrack::setupLayoutActions( Meta::TrackPtr track )
             m_findInSourceSignalMapper->setMapping( act, QLatin1String("artist") );
             m_customActions << act;
 
-            KPluginInfo::List services = The::pluginManager()->plugins( QLatin1String("Service") );
+            // show a special action if the amazon plugin is enabled
+            KPluginInfo::List services = The::pluginManager()->plugins( Plugins::PluginManager::Service );
             foreach( const KPluginInfo &service, services )
             {
                 if( service.pluginName() == QLatin1String("amarok_service_amazonstore") )
