@@ -75,7 +75,7 @@ PhotosApplet::init()
     settingsAction->setEnabled( true );
     settingsAction->setText( i18n( "Settings" ) );
     m_settingsIcon = addRightHeaderAction( settingsAction );
-    connect( m_settingsIcon, SIGNAL( clicked() ), this, SLOT( showConfigurationInterface() ) );
+    connect( m_settingsIcon, SIGNAL(clicked()), this, SLOT(showConfigurationInterface()) );
 
     m_widget = new PhotosScrollWidget( this );
     m_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
@@ -114,7 +114,7 @@ void
 PhotosApplet::stopped()
 {
     DEBUG_BLOCK
-    setHeaderText( i18n( "Photos: No track playing" ) );
+    setHeaderText( i18n( "Photos: No Track Playing" ) );
     m_widget->clear();
     m_widget->hide();
     setBusy( false );
@@ -128,14 +128,16 @@ void
 PhotosApplet::photoAdded()
 {
     setBusy( false );
-    setHeaderText( i18nc( "@title:window Number of photos of artist", "%1 Photos: %2",
-                          m_widget->count(), m_currentArtist ) );
+    setHeaderText( i18ncp( "@title:window Number of photos of artist",
+                           "1 Photo: %2",
+                           "%1 Photos: %2",
+                           m_widget->count(),
+                           m_currentArtist ) );
 }
 
 void
 PhotosApplet::dataUpdated( const QString& name, const Plasma::DataEngine::Data& data ) // SLOT
 {
-    DEBUG_BLOCK
     if( name != QLatin1String("photos") || data.isEmpty() )
         return;
 
@@ -209,6 +211,8 @@ PhotosApplet::dataUpdated( const QString& name, const Plasma::DataEngine::Data&
 void
 PhotosApplet::createConfigurationInterface( KConfigDialog *parent )
 {
+    parent->setButtons( KDialog::Ok | KDialog::Cancel );
+
     KConfigGroup configuration = config();
     QWidget *settings = new QWidget;
     ui_Settings.setupUi( settings );
@@ -218,7 +222,7 @@ PhotosApplet::createConfigurationInterface( KConfigDialog *parent )
     ui_Settings.animationComboBox->setCurrentIndex( ui_Settings.animationComboBox->findText( m_Animation ) );
     ui_Settings.photosSpinBox->setValue( m_nbPhotos );
     ui_Settings.additionalkeywordsLineEdit->setText( m_KeyWords.join(", ") );
-    connect( parent, SIGNAL( accepted() ), this, SLOT( saveSettings( ) ) );
+    connect( parent, SIGNAL(accepted()), this, SLOT(saveSettings()) );
 }
 
 void
