@@ -20,15 +20,16 @@
 #include "ArtistWidget.h"
 
 //Amarok
-#include "amarokurls/AmarokUrl.h"
-#include "core/collections/Collection.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/collections/QueryMaker.h"
-#include "core/support/Amarok.h"
-#include "core/support/Debug.h"
-#include "playlist/PlaylistModelStack.h"
 #include "PaletteHandler.h"
 #include "SvgHandler.h"
+#include "amarokurls/AmarokUrl.h"
+#include "core/collections/Collection.h"
+#include "core/collections/QueryMaker.h"
+#include "core/meta/Meta.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "playlist/PlaylistController.h"
 
 //KDE
 #include <KColorUtils>
@@ -47,6 +48,7 @@
 #include <QPixmapCache>
 #include <QSignalMapper>
 #include <QTextDocument>
+#include <QTimer>
 
 #include <cmath>
 
@@ -127,7 +129,6 @@ ArtistWidget::ArtistWidget( const SimilarArtistPtr &artist,
     m_navigateButton->setMaximumSize( QSizeF( 22, 22 ) );
     m_navigateButton->setIcon( KIcon( "edit-find" ) );
     m_navigateButton->setToolTip( i18n( "Show in Media Sources" ) );
-    m_navigateButton->hide();
     connect( m_navigateButton, SIGNAL(clicked()), this, SLOT(navigateToArtist()) );
     
     m_lastfmStationButton = new Plasma::PushButton( this );
@@ -146,7 +147,7 @@ ArtistWidget::ArtistWidget( const SimilarArtistPtr &artist,
     m_similarArtistButton = new Plasma::PushButton( this );
     m_similarArtistButton->setMaximumSize( QSizeF( 22, 22 ) );
     m_similarArtistButton->setIcon( KIcon( "similarartists-amarok" ) );
-    m_similarArtistButton->setToolTip( i18n( "Show Similar Artists of %1", m_artist->name() ) );
+    m_similarArtistButton->setToolTip( i18n( "Show Similar Artists to %1", m_artist->name() ) );
     connect( m_similarArtistButton, SIGNAL(clicked()), this, SIGNAL(showSimilarArtists()) );
 
     QGraphicsLinearLayout *buttonsLayout = new QGraphicsLinearLayout( Qt::Horizontal );
@@ -181,10 +182,15 @@ ArtistWidget::ArtistWidget( const SimilarArtistPtr &artist,
     m_match->setText( i18n( "Match: %1%", QString::number( m_artist->match() ) ) );
     m_nameLabel->setText( m_artist->name() );
 
+    QTimer::singleShot( 0, this, SLOT(updateInfo()) );
+}
+
+void
+ArtistWidget::updateInfo()
+{
     fetchPhoto();
     fetchInfo();
     fetchTopTrack();
-    queryArtist();
 }
 
 ArtistWidget::~ArtistWidget()
@@ -389,26 +395,6 @@ ArtistWidget::parseTopTrack( const KUrl &url, QByteArray data, NetworkAccessMana
     setTopTrack( topTrack );
 }
 
-void
-ArtistWidget::queryArtist()
-{
-    // Figure out of this applet is present in the local collection,
-    // and show the "show in collection" button if so
-    m_navigateButton->hide();
-
-    Collections::QueryMaker *qm = CollectionManager::instance()->queryMaker();
-    
-    qm->setQueryType( Collections::QueryMaker::Artist );
-    qm->addFilter( Meta::valArtist, m_nameLabel->text() );
-    qm->limitMaxResultSize( 1 );
-    qm->setAutoDelete( true );
-
-    connect( qm, SIGNAL(newResultReady(QString,Meta::ArtistList)),
-             SLOT(resultReady(QString,Meta::ArtistList)) );
-
-    qm->run();
-}
-
 SimilarArtistPtr
 ArtistWidget::artist() const
 {
@@ -427,10 +413,8 @@ ArtistWidget::clear()
 void
 ArtistWidget::openArtistUrl()
 {
-    // somehow Last.fm decides to supply this url without the scheme
-    KUrl artistUrl = QString( "http://%1" ).arg( m_artist->url().url() );
-    if( artistUrl.isValid() )
-        QDesktopServices::openUrl( artistUrl );
+    if( m_artist->url().isValid() )
+        QDesktopServices::openUrl( m_artist->url() );
 }
 
 void
@@ -482,8 +466,8 @@ ArtistWidget::setTopTrack( const QString &topTrack )
         qm->limitMaxResultSize( 1 );
         qm->setAutoDelete( true );
 
-        connect( qm, SIGNAL(newResultReady(QString,Meta::TrackList)),
-                 SLOT(resultReady(QString,Meta::TrackList)) );
+        connect( qm, SIGNAL(newResultReady(Meta::TrackList)),
+                 SLOT(resultReady(Meta::TrackList)) );
          
         qm->run();
     }
@@ -573,7 +557,7 @@ ArtistWidget::fullBio() const
 void
 ArtistWidget::addTopTrackToPlaylist()
 {
-    The::playlistController()->insertOptioned( m_topTrack, Playlist::AppendAndPlay );
+    The::playlistController()->insertOptioned( m_topTrack, Playlist::OnAppendToPlaylistAction );
 }
 
 void
@@ -582,7 +566,7 @@ ArtistWidget::navigateToArtist()
     AmarokUrl url;
     url.setCommand( "navigate" );
     url.setPath( "collections" );
-    url.appendArg( "filter", "artist:\"" + m_artist->name() + "\"" );
+    url.setArg( "filter", "artist:\"" + AmarokUrl::escape( m_artist->name() ) + '\"' );
     url.run();
 }
 
@@ -591,15 +575,7 @@ ArtistWidget::addLastfmArtistStation()
 {
     const QString url = "lastfm://artist/" + m_artist->name() + "/similarartists";
     Meta::TrackPtr lastfmtrack = CollectionManager::instance()->trackForUrl( KUrl( url ) );
-    The::playlistController()->insertOptioned( lastfmtrack, Playlist::AppendAndPlay );
-}
-
-void
-ArtistWidget::resultReady( const QString &collectionId, const Meta::ArtistList &artists )
-{
-    Q_UNUSED( collectionId )
-    if( artists.length() > 0 )
-        m_navigateButton->show();
+    The::playlistController()->insertOptioned( lastfmtrack, Playlist::OnAppendToPlaylistAction );
 }
 
 ArtistsListWidget::ArtistsListWidget( QGraphicsWidget *parent )
@@ -717,13 +693,11 @@ ArtistsListWidget::widget( const QString &artistName )
 }
 
 void
-ArtistWidget::resultReady( const QString &collectionId, const Meta::TrackList &tracks )
+ArtistWidget::resultReady( const Meta::TrackList &tracks )
 {
-    Q_UNUSED( collectionId )
     if( !tracks.isEmpty() )
     {
         m_topTrack = tracks.first();
-        m_navigateButton->show();
         m_topTrackButton->show();
     }
 }
