@@ -19,7 +19,10 @@
 #include "VerticalToolbarContainment.h"
 
 #include "ContextView.h"
+#include "core/interfaces/Logger.h"
+#include "core/support/Components.h"
 #include "core/support/Debug.h"
+#include "EngineController.h"
 #include "PaletteHandler.h"
 #include "VerticalAppletLayout.h"
 
@@ -45,8 +48,8 @@ Context::VerticalToolbarContainment::VerticalToolbarContainment( QObject *parent
     connect( this, SIGNAL(appletRemoved(Plasma::Applet*)), SLOT(appletRemoved(Plasma::Applet*)) );
     connect( this, SIGNAL(appletRemoved(Plasma::Applet*)), SIGNAL(geometryChanged()) );
 
-    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*, int)), SIGNAL(appletAdded(Plasma::Applet*, int)) );
-    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*, int)), SIGNAL(geometryChanged()) );
+    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*,int)), SIGNAL(appletAdded(Plasma::Applet*,int)) );
+    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*,int)), SIGNAL(geometryChanged()) );
     connect( m_applets, SIGNAL(noApplets(bool)), SLOT(showEmptyText(bool)) );
 
 }
@@ -136,21 +139,25 @@ Context::VerticalToolbarContainment::updateGeometry()
     m_applets->refresh();
 }
 
-Plasma::Applet*
+void
 Context::VerticalToolbarContainment::addApplet( const QString& pluginName, const int loc ) // SLOT
 {
     DEBUG_BLOCK
 
+    if( pluginName == "analyzer" && !EngineController::instance()->supportsAudioDataOutput() )
+    {
+        Amarok::Components::logger()->longMessage( i18n( "Error: Visualizations are not supported by your current Phonon backend." ),
+                                                   Amarok::Logger::Error ) ;
+
+        return;
+    }
+
     Plasma::Applet* applet = Plasma::Containment::addApplet( pluginName );
 
-    if( applet == 0 )
-        debug() << "FAILED ADDING APPLET TO CONTAINMENT!! NOT FOUND!!";
-    else
-        m_applets->addApplet( applet, loc );
+    Q_ASSERT_X( applet, "addApplet", "FAILED ADDING APPLET TO CONTAINMENT!! NOT FOUND!!" );
 
-     applet->setFlag(QGraphicsItem::ItemIsMovable, false);
-
-    return applet;
+    m_applets->addApplet( applet, loc );
+    applet->setFlag( QGraphicsItem::ItemIsMovable, false );
 }
 
 void
