@@ -19,13 +19,14 @@
 
 #include "PopupDropper.h"
 #include "PopupDropper_p.h"
-#include "PopupDropperAction.h"
 #include "PopupDropperItem.h"
 
 #include <QtDebug>
 #include <QGraphicsItem>
 #include <QGraphicsScene>
 #include <QtSvg/QSvgRenderer>
+#include <QAction>
+#include <QMenu>
 #include <QPalette>
 #include <QTimeLine>
 #include <QWidget>
@@ -67,11 +68,11 @@ PopupDropperPrivate::PopupDropperPrivate( PopupDropper* parent, bool sa, QWidget
 {
     if( widget )
         widgetRect = widget->rect();
-	windowBackgroundBrush.setColor( windowColor );
-	hoveredBorderPen.setColor( Qt::blue );
+    windowBackgroundBrush.setColor( windowColor );
+    hoveredBorderPen.setColor( Qt::blue );
     hoveredBorderPen.setWidth( 2 );
     hoveredBorderPen.setStyle( Qt::SolidLine );
-	QColor hoveredFillColor = QColor( Qt::blue );
+    QColor hoveredFillColor = QColor( Qt::blue );
     hoveredFillColor.setAlpha( 32 );
     hoveredFillBrush.setColor( hoveredFillColor );
     hoveredFillBrush.setStyle( Qt::SolidPattern );
@@ -80,11 +81,11 @@ PopupDropperPrivate::PopupDropperPrivate( PopupDropper* parent, bool sa, QWidget
     //qDebug() << "on create, view size = " << view->size();
     deleteTimer.setSingleShot( true );
     fadeHideTimer.setDirection( QTimeLine::Backward );
-    connect( &fadeHideTimer, SIGNAL( frameChanged(int) ), this, SLOT( fadeHideTimerFrameChanged(int) ) );
-    connect( &fadeShowTimer, SIGNAL( frameChanged(int) ), this, SLOT( fadeShowTimerFrameChanged(int) ) );
-    connect( &fadeHideTimer, SIGNAL( finished() ), this, SLOT( fadeHideTimerFinished() ) );
-    connect( &fadeShowTimer, SIGNAL( finished() ), this, SLOT( fadeShowTimerFinished() ) );
-    connect( &deleteTimer, SIGNAL( timeout() ), this, SLOT( deleteTimerFinished() ) );
+    connect( &fadeHideTimer, SIGNAL(frameChanged(int)), this, SLOT(fadeHideTimerFrameChanged(int)) );
+    connect( &fadeShowTimer, SIGNAL(frameChanged(int)), this, SLOT(fadeShowTimerFrameChanged(int)) );
+    connect( &fadeHideTimer, SIGNAL(finished()), this, SLOT(fadeHideTimerFinished()) );
+    connect( &fadeShowTimer, SIGNAL(finished()), this, SLOT(fadeShowTimerFinished()) );
+    connect( &deleteTimer, SIGNAL(timeout()), this, SLOT(deleteTimerFinished()) );
 }
 
 PopupDropperPrivate::~PopupDropperPrivate()
@@ -115,6 +116,8 @@ void PopupDropperPrivate::fadeHideTimerFrameChanged( int frame ) //SLOT
         int alpha = (int)( color.alpha() * val );
         color.setAlpha( alpha );
         q->setPalette( color );
+        foreach( PopupDropperItem* pdi, pdiItems )
+            pdi->setSubitemOpacity( val );
     }
 }
 
@@ -127,6 +130,8 @@ void PopupDropperPrivate::fadeShowTimerFrameChanged( int frame ) //SLOT
         int alpha = (int)( color.alpha() * val );
         color.setAlpha( alpha );
         q->setPalette( color );
+        foreach( PopupDropperItem* pdi, pdiItems )
+            pdi->setSubitemOpacity( val );
     }
 }
 
@@ -141,6 +146,8 @@ void PopupDropperPrivate::fadeShowTimerFinished() //SLOT
 {
     q->setPalette( windowColor ); 
     queuedHide = false;
+    foreach( PopupDropperItem* pdi, pdiItems )
+        pdi->setSubitemOpacity( 1.0 );
 }
 
 void PopupDropperPrivate::dragEntered()
@@ -157,7 +164,7 @@ void PopupDropperPrivate::dragLeft()
     {
         view->setAcceptDrops( false );
         //qDebug() << "View entered, hiding";
-        connect( q, SIGNAL( fadeHideFinished() ), q, SLOT( subtractOverlay() ) );
+        connect( q, SIGNAL(fadeHideFinished()), q, SLOT(subtractOverlay()) );
         q->hide();
     }
     q->updateAllOverlays();
@@ -177,14 +184,14 @@ void PopupDropperPrivate::deleteTimerFinished() //SLOT
     //qDebug() << "Delete Timer Finished";
     if( !view->entered() && quitOnDragLeave )
     {
-        connect( q, SIGNAL( fadeHideFinished() ), q, SLOT( subtractOverlay() ) );
+        connect( q, SIGNAL(fadeHideFinished()), q, SLOT(subtractOverlay()) );
         q->hide();
     }
 }
 
 void PopupDropperPrivate::reposItems()
 {
-    qreal partitionsize, my_min, my_max, vert_center, item_min;
+    qreal partitionsize, my_min, my_max;
     //qDebug() << "allItems.size() = " << allItems.size();
     int counter = 0;
     for( int i = 0; i < allItems.size(); i++ )
@@ -195,7 +202,6 @@ void PopupDropperPrivate::reposItems()
         my_min = ( counter * partitionsize ) + verticalmargin;
         my_max = ( ( counter + 1 ) * partitionsize ) - verticalmargin;
         //qDebug() << "my_min = " << my_min << ", my_max = " << my_max;
-        vert_center = ( ( my_max - my_min ) / 2 ) + my_min; //gives us our center line...now center the item around it
         PopupDropperItem* pItem = dynamic_cast<PopupDropperItem*>( allItems.at( i ) );
         QGraphicsLineItem* qglItem = 0;
         if( pItem )
@@ -204,8 +210,6 @@ void PopupDropperPrivate::reposItems()
             //qDebug() << "item " << i << " is a PDI ";
             //If the svgElementRect is too high, resize it to fit
             pItem->svgElementRect().setHeight( my_max - my_min - ( 2 * verticalmargin ) );
-            item_min = vert_center - ( pItem->svgElementRect().height() / 2 );
-            //qDebug() << "vert_center = " << vert_center << ", ited->min = " << item_min;
             pItem->setPos( 0, my_min );
             pItem->borderRectItem()->setRect( 0 - pItem->borderWidth(), 0, scene->width() + 2*pItem->borderWidth(), my_max - my_min );
             pItem->scaleAndReposSvgItem();
@@ -274,7 +278,9 @@ void PopupDropper::initOverlay( QWidget* parent, PopupDropperPrivate* priv )
     pdp->view->setBackgroundRole( QPalette::Window );
     pdp->view->setAutoFillBackground( true );
     pdp->fadeHideTimer.setFrameRange( 0, pdp->frameMax );
+    pdp->fadeHideTimer.setUpdateInterval( 20 ); // 50 fps
     pdp->fadeShowTimer.setFrameRange( 0, pdp->frameMax );
+    pdp->fadeShowTimer.setUpdateInterval( 20 ); // 50 fps
 }
 
 void PopupDropper::addOverlay()
@@ -314,11 +320,11 @@ void PopupDropper::addOverlay( PopupDropperPrivate* newD )
 bool PopupDropper::subtractOverlay()
 {
     //qDebug() << "subtractOverlay, with d pointer " << d;
-    disconnect( this, SLOT( subtractOverlay() ) );
+    disconnect( this, SLOT(subtractOverlay()) );
     while( !isHidden() && d->fadeHideTimer.state() == QTimeLine::Running )
     {
         //qDebug() << "singleshotting subtractOverlay";
-        QTimer::singleShot( 0, this, SLOT( subtractOverlay() ) );
+        QTimer::singleShot( 0, this, SLOT(subtractOverlay()) );
         return false;
     }
     //qDebug() << "in PopupDropper " << this;
@@ -350,11 +356,6 @@ bool PopupDropper::subtractOverlay()
 }
 
 PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, const QString &text )
-{
-    return addSubmenu( pd, 0, QString(), text );
-}
-
-PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, QSvgRenderer *renderer, const QString &elementId, const QString &text )
 {
     //qDebug() << "addSubmenu, this is " << this << " and passed-in PopupDropper is " << (*pd);
     if( !(*pd) )
@@ -372,20 +373,17 @@ PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, QSvgRenderer *ren
     initOverlay( d->widget, newD );
     PopupDropperItem* pdi = new PopupDropperItem();
 
-    PopupDropperAction* pda = 0;
+    QAction* action = new QAction( text, this );
 
-    if( renderer )
-        pda = new PopupDropperAction( renderer, elementId, text, this );
-    else
-        pda = new PopupDropperAction( text, this );
-
-    connect( pda, SIGNAL( hovered() ), this, SLOT( activateSubmenu() ) );
-    pdi->setAction( pda );
+    connect( action, SIGNAL(hovered()), this, SLOT(activateSubmenu()) );
+    pdi->setAction( action );
     pdi->setSubmenuTrigger( true );
     pdi->setHoverIndicatorShowStyle( PopupDropperItem::OnHover );
-    d->submenuMap[pda] = newD;
+    d->submenuMap[action] = newD;
     delete (*pd);
     (*pd) = 0;
+    foreach( PopupDropperItem* item, newD->pdiItems )
+        item->setPopupDropper( this );
     //qDebug() << "d->submenuMap[pda] = " << d->submenuMap[pda];
     addItem( pdi );
     return pdi;
@@ -397,7 +395,7 @@ void PopupDropper::activateSubmenu()
     if( isHidden() || d->fadeHideTimer.state() == QTimeLine::Running )
         return;
     PopupDropperPrivate* oldd = d;
-    addOverlay( d->submenuMap[static_cast<PopupDropperAction*>(QObject::sender())] );
+    addOverlay( d->submenuMap[static_cast<QAction*>(QObject::sender())] );
     //qDebug() << "d->pdiItems.size() = " << d->pdiItems.size() << " for " << d;
     foreach( PopupDropperItem* item, d->pdiItems )
         addItem( item, false, false );
@@ -405,6 +403,37 @@ void PopupDropper::activateSubmenu()
     show();
 }
 
+bool PopupDropper::addMenu( const QMenu *menu )
+{
+    Q_UNUSED(menu)
+    if( !menu )
+        return false;
+        
+    if( menu->actions().size() == 0 )
+        return true;
+
+    PopupDropperItem *pdi = 0;
+    foreach( QAction *action, menu->actions() )
+    {
+        if( !action->menu() )
+        {
+            pdi = new PopupDropperItem();
+            pdi->setAction( action );
+            addItem( pdi );
+        }
+        else
+        {
+            PopupDropper *pd = new PopupDropper( 0 );
+            bool success = pd->addMenu( action->menu() );
+            if( success )
+                pdi = addSubmenu( &pd, action->text() );
+        }
+        pdi = 0;
+    }
+    
+    return true;
+}
+
 bool PopupDropper::standalone() const
 {
     return d->standalone;
@@ -435,6 +464,8 @@ void PopupDropper::show()
         QColor color = d->windowColor;
         color.setAlpha( 0 );
         setPalette( color );
+        foreach( PopupDropperItem* pdi, d->pdiItems )
+            pdi->setSubitemOpacity( 0.0 );
         d->fadeShowTimer.start();
         //qDebug() << "Timer started";
     }
@@ -470,8 +501,8 @@ void PopupDropper::hide()
         //qDebug() << "show timer running, queueing hide";
         d->fadeShowTimer.stop();
         d->queuedHide = true;
-        QTimer::singleShot( 0, d, SLOT( fadeShowTimerFinished() ) );
-        QTimer::singleShot( 0, this, SLOT( hide() ) );
+        QTimer::singleShot( 0, d, SLOT(fadeShowTimerFinished()) );
+        QTimer::singleShot( 0, this, SLOT(hide()) );
         return;
     }
 
@@ -480,7 +511,7 @@ void PopupDropper::hide()
     if( d->fadeHideTimer.state() == QTimeLine::Running || d->queuedHide )
     {
         //qDebug() << "hide timer running or queued hide";
-        QTimer::singleShot( 0, this, SLOT( hide() ) );
+        QTimer::singleShot( 0, this, SLOT(hide()) );
         return;
     }
 
@@ -496,7 +527,7 @@ void PopupDropper::hide()
     else  //time is zero, or no fade
     {
         //qDebug() << "time is zero, or no fade";
-        QTimer::singleShot( 0, d, SLOT( fadeHideTimerFinished() ) );
+        QTimer::singleShot( 0, d, SLOT(fadeHideTimerFinished()) );
         return;
     }
 }
@@ -504,7 +535,7 @@ void PopupDropper::hide()
 void PopupDropper::hideAllOverlays()
 {
     //qDebug() << "Entered hideAllOverlays";
-    connect( this, SIGNAL( fadeHideFinished() ), this, SLOT( slotHideAllOverlays() ) );
+    connect( this, SIGNAL(fadeHideFinished()), this, SLOT(slotHideAllOverlays()) );
     hide();
     //qDebug() << "Leaving hideAllOverlays";
 }
@@ -512,7 +543,7 @@ void PopupDropper::hideAllOverlays()
 void PopupDropper::slotHideAllOverlays()
 {
     //qDebug() << "Entered slotHideAllOverlays()";
-    disconnect( this, SIGNAL( fadeHideFinished() ), this, SLOT( slotHideAllOverlays() ) );
+    disconnect( this, SIGNAL(fadeHideFinished()), this, SLOT(slotHideAllOverlays()) );
     //qDebug() << "m_viewStack.size() = " << m_viewStack.size();
     for( int i = m_viewStack.size() - 1; i >= 0; --i )
     {
@@ -549,21 +580,21 @@ void PopupDropper::clear()
 {
     while( !isHidden() && d->fadeHideTimer.state() == QTimeLine::Running )
     {
-        QTimer::singleShot(0, this, SLOT( clear() ) );
+        QTimer::singleShot(0, this, SLOT(clear()) );
         return;
     }
     //qDebug() << "Clear happening!";
-    disconnect( this, SLOT( clear() ) );
+    disconnect( this, SLOT(clear()) );
     do
     {
         foreach( QGraphicsItem* item, d->allItems )
         {
             if( dynamic_cast<PopupDropperItem*>(item) )
             {
-                if( dynamic_cast<PopupDropperItem*>(item)->submenuTrigger() )
+                if( dynamic_cast<PopupDropperItem*>(item)->isSubmenuTrigger() )
                 {
                     //qDebug() << "Disconnecting action";
-                    disconnect( dynamic_cast<PopupDropperItem*>(item)->action(), SIGNAL( hovered() ), this, SLOT( activateSubmenu() ) );
+                    disconnect( dynamic_cast<PopupDropperItem*>(item)->action(), SIGNAL(hovered()), this, SLOT(activateSubmenu()) );
                 }
                 dynamic_cast<PopupDropperItem*>(item)->deleteLater();
             }
@@ -824,9 +855,9 @@ void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer )
 void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer, bool appendToList )
 {
     //qDebug() << "adding item";
-    //FIXME: Make separators use something graphical instead of just ignoring them
+    //FIXME: Make separators do something graphical instead of just ignoring them
     PopupDropperItem *pItem = static_cast<PopupDropperItem*>( item );
-    if( pItem->action()->isSeparator() )
+    if( pItem->isSeparator() )
         return;
     if( useSharedRenderer )
         pItem->setSharedRenderer( d->sharedRenderer );
@@ -870,20 +901,60 @@ void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer, bool
     d->scene->addItem( pItem );
 }
 
-void PopupDropper::addSeparator( PopupDropperAction* separator )
+QList<PopupDropperItem*> PopupDropper::items() const
 {
-    if( separator && !separator->isSeparator() )
+    QList<PopupDropperItem*> list;
+    foreach( PopupDropperItem *item, d->pdiItems )
+        list.append( item );
+
+    return list;
+}
+
+//Won't currently work for > 1 level of submenu!
+//TODO: Figure out a better way. (Does anything else work > 1 level?)
+QList<PopupDropperItem*> PopupDropper::submenuItems( const PopupDropperItem *item ) const
+{
+    QList<PopupDropperItem*> list;
+    if( !item || !item->isSubmenuTrigger() || !d->submenuMap.contains( item->action() ) )
+        return list;
+
+    PopupDropperPrivate *pdp = d->submenuMap[item->action()];
+    foreach( PopupDropperItem *pdi, pdp->pdiItems )
+        list.append( pdi );
+
+    return list;
+}
+
+//Goes through and calls the callback on all items, including submenuItems
+//which can be adjusted differently by checking isSubmenuTrigger()
+void PopupDropper::forEachItem( void callback(void*) )
+{
+    forEachItemPrivate( d, callback );
+}
+
+void PopupDropper::forEachItemPrivate( PopupDropperPrivate *pdp, void callback(void* item) )
+{
+    foreach( PopupDropperItem *item, pdp->pdiItems )
+        callback( item );
+    foreach( QAction *action, pdp->submenuMap.keys() )
+        forEachItemPrivate( pdp->submenuMap[action], callback );
+}
+
+void PopupDropper::addSeparator( PopupDropperItem* separator )
+{
+
+    if( !separator )
     {
         //qDebug() << "Action is not a separator!";
         return;
     }
 
-    if( separator && separator->separatorStyle() == PopupDropperAction::TextSeparator )
+    separator->setSeparator( true );
+
+    if( separator->separatorStyle() == PopupDropperItem::TextSeparator )
     {
         //qDebug() << "Separator style is text";
-        PopupDropperItem* pdi = new PopupDropperItem();
-        pdi->setAction( separator );
-        addItem( pdi );
+        addItem( separator );
     }
 
     //qDebug() << "Separator style is line";
