@@ -15,6 +15,7 @@
  ****************************************************************************************/
 
 #define DEBUG_PREFIX "TextScrollingWidget"
+
 #include "TextScrollingWidget.h"
 
 #include "core/support/Debug.h"
@@ -26,7 +27,6 @@
 #include <QGraphicsSimpleTextItem>
 #include <QGraphicsSceneHoverEvent>
 #include <QPainter>
-#include <QTextDocument>
 #include <QTimer>
 #include <QPropertyAnimation>
 
@@ -79,6 +79,21 @@ public:
         p->restore();
     }
 
+    void setScrollingText( const QString &str )
+    {
+        text = str;
+    }
+
+    void setText( const QString &str )
+    {
+        text = str;
+        textItem->setText( text );
+
+        if( animation )
+            animation.data()->stop();
+
+    }
+
     qreal             width;          // box width
     qreal             delta;          // complete delta
     qreal             currentDelta;   // current delta
@@ -119,12 +134,7 @@ void
 TextScrollingWidget::setScrollingText( const QString &text )
 {
     Q_D( TextScrollingWidget );
-    if( d->text != text )
-    {
-        QTextDocument doc;
-        doc.setHtml( text );
-        d->text = doc.toPlainText();
-    }
+    d->setScrollingText( text );
     updateGeometry();
 }
 
@@ -132,12 +142,7 @@ void
 TextScrollingWidget::setText( const QString &text )
 {
     Q_D( TextScrollingWidget );
-    QTextDocument doc;
-    doc.setHtml( text );
-    d->text = doc.toPlainText();
-    d->textItem->setText( d->text );
-    if( d->animation )
-        d->animation.data()->stop();
+    d->setText( text );
 }
 
 void
@@ -287,7 +292,7 @@ TextScrollingWidget::startAnimation( QAbstractAnimation::Direction direction )
         animation->setEndValue( 1.0 );
         animation->setEasingCurve( QEasingCurve::InOutQuad );
         d->animation = animation;
-        connect( animation, SIGNAL( finished() ), this, SLOT( animationFinished() ) );
+        connect( animation, SIGNAL(finished()), this, SLOT(animationFinished()) );
     }
     else
     {
