@@ -51,9 +51,7 @@ class DaapTrack : public Meta::Track
         virtual KUrl playableUrl() const;
         virtual QString uidUrl() const;
         virtual QString prettyUrl() const;
-
-        virtual bool isPlayable() const;
-        virtual bool isEditable() const;
+        virtual QString notPlayableReason() const;
 
         virtual AlbumPtr album() const;
         virtual ArtistPtr artist() const;
@@ -74,12 +72,6 @@ class DaapTrack : public Meta::Track
         virtual QString comment() const;
         virtual void setComment ( const QString &newComment );
 
-        virtual double score() const;
-        virtual void setScore ( double newScore );
-
-        virtual int rating() const;
-        virtual void setRating ( int newRating );
-
         virtual qint64 length() const;
 
         virtual int filesize() const;
@@ -92,16 +84,8 @@ class DaapTrack : public Meta::Track
         virtual int discNumber() const;
         virtual void setDiscNumber ( int newDiscNumber );
 
-        virtual int playCount() const;
-
         virtual QString type() const;
 
-        virtual void beginMetaDataUpdate() {}    //read only
-        virtual void endMetaDataUpdate() {}      //read only
-
-        virtual void subscribe ( Observer *observer );
-        virtual void unsubscribe ( Observer *observer );
-
         virtual bool inCollection() const;
         virtual Collections::Collection* collection() const;
 
