@@ -21,33 +21,32 @@
 
 #include "SqlCollectionLocation.h"
 
+#include "MetaTagLib.h" // for getting the uid
 #include "core/collections/CollectionLocationDelegate.h"
+#include <core/storage/SqlStorage.h>
+#include "core/interfaces/Logger.h"
 #include "core/support/Components.h"
 #include "core/support/Debug.h"
-#include "core/interfaces/Logger.h"
-#include "core/collections/support/SqlStorage.h"
 #include "core/meta/Meta.h"
 #include "core/meta/support/MetaUtility.h"
-#include "core-impl/collections/db/ScanManager.h"
-#include "MountPointManager.h"
-#include "SqlCollection.h"
-#include "SqlMeta.h"
-#include "transcoding/TranscodingJob.h"
 #include "core/transcoding/TranscodingController.h"
-#include <shared/MetaTagLib.h> // for getting the uid
+#include "core-impl/collections/db/MountPointManager.h"
+#include "core-impl/collections/db/sql/SqlCollection.h"
+#include "core-impl/collections/db/sql/SqlMeta.h"
+#include "transcoding/TranscodingJob.h"
 
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 
 #include <kdiskfreespaceinfo.h>
+#include <KFileItem>
 #include <kjob.h>
 #include <KSharedPtr>
 #include <kio/job.h>
 #include <kio/jobclasses.h>
 #include <kio/deletejob.h>
 
-
 using namespace Collections;
 
 SqlCollectionLocation::SqlCollectionLocation( SqlCollection *collection )
@@ -89,7 +88,7 @@ SqlCollectionLocation::isWritable() const
     bool path_exists_with_space = false;
     bool path_exists_writable = false;
     QStringList folders = actualLocation();
-    foreach(QString path, folders)
+    foreach( const QString &path, folders )
     {
         float used = KDiskFreeSpaceInfo::freeSpaceInfo( path ).used();
         float total = KDiskFreeSpaceInfo::freeSpaceInfo( path ).size();
@@ -152,39 +151,39 @@ SqlCollectionLocation::remove( const Meta::TrackPtr &track )
 bool
 SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
 {
-    debug() << "SqlCollectionLocation::insert from:"<<track->playableUrl()<<"to"<<url;
-
     if( !QFile::exists( url ) )
-        return false; // won't insert bogus file information
-
-    debug() << "File on disk, insert to dbase"<<track->name()<<"uid:"<<track->uidUrl();
+    {
+        warning() << Q_FUNC_INFO << "file" << url << "does not exist, not inserting into db";
+        return false;
+    }
 
     // -- the target url
     SqlRegistry *registry = m_collection->registry();
     int deviceId = m_collection->mountPointManager()->getIdForUrl( url );
     QString rpath = m_collection->mountPointManager()->getRelativePath( deviceId, url );
-    int directoryId = registry->getDirectory( QFileInfo(url).path() );
+    int directoryId = registry->getDirectory( QFileInfo( url ).path() );
 
     // -- the track uid (we can't use the original one from the old collection)
     Meta::FieldHash fileTags = Meta::Tag::readTags( url );
     QString uid = fileTags.value( Meta::valUniqueId ).toString();
+    uid = m_collection->generateUidUrl( uid ); // add the right prefix
 
-    // -- the the track from the registry
-    KSharedPtr<Meta::SqlTrack> metaTrack;
-    metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrackFromUid( uid ) );
+    // -- the track from the registry
+    Meta::SqlTrackPtr metaTrack;
+    metaTrack = Meta::SqlTrackPtr::staticCast( registry->getTrackFromUid( uid ) );
 
-    if( metaTrack ) {
+    if( metaTrack )
+    {
         warning() << "Location is inserting a file with the same uid as an already existing one.";
-        // TODO: in addition another file with the same url could already exist.
         metaTrack->setUrl( deviceId, rpath, directoryId );
+    } else
+        metaTrack = Meta::SqlTrackPtr::staticCast( registry->getTrack( deviceId, rpath, directoryId, uid ) );
 
-    } else {
-        metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrack( deviceId, rpath, directoryId, uid ) );
-    }
+    Meta::ConstStatisticsPtr origStats = track->statistics();
 
     // -- set the values
     metaTrack->setWriteFile( false ); // no need to write the tags back
-    metaTrack->beginMetaDataUpdate();
+    metaTrack->beginUpdate();
 
     if( !track->name().isEmpty() )
         metaTrack->setTitle( track->name() );
@@ -204,10 +203,10 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     if( !track->comment().isEmpty() )
         metaTrack->setComment( track->comment() );
 
-    if( track->score() > 0 )
-        metaTrack->setScore( track->score() );
-    if( track->rating() > 0 )
-        metaTrack->setRating( track->rating() );
+    if( origStats->score() > 0 )
+        metaTrack->setScore( origStats->score() );
+    if( origStats->rating() > 0 )
+        metaTrack->setRating( origStats->rating() );
 
     /* These tags change when transcoding. Prefer to read those from file */
     if( fileTags.value( Meta::valLength, 0 ).toLongLong() > 0 )
@@ -234,12 +233,12 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     if( track->discNumber() > 0 )
         metaTrack->setDiscNumber( track->discNumber() );
 
-    if( track->lastPlayed().isValid() )
-        metaTrack->setLastPlayed( track->lastPlayed() );
-    if( track->firstPlayed().isValid() )
-        metaTrack->setFirstPlayed( track->firstPlayed() );
-    if( track->playCount() > 0 )
-        metaTrack->setPlayCount( track->playCount() );
+    if( origStats->lastPlayed().isValid() )
+        metaTrack->setLastPlayed( origStats->lastPlayed() );
+    if( origStats->firstPlayed().isValid() )
+        metaTrack->setFirstPlayed( origStats->firstPlayed() );
+    if( origStats->playCount() > 0 )
+        metaTrack->setPlayCount( origStats->playCount() );
 
     Meta::ReplayGainTag modes[] = { Meta::ReplayGain_Track_Gain,
         Meta::ReplayGain_Track_Peak,
@@ -267,16 +266,15 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
 
         if( track->album()->hasImage() && !metaTrack->album()->hasImage() )
             metaTrack->album()->setImage( track->album()->image() );
-
     }
 
-    metaTrack->endMetaDataUpdate();
+    metaTrack->endUpdate();
     metaTrack->setWriteFile( true );
 
     // we have a first shot at the meta data (expecially ratings and playcounts from media
     // collections) but we still need to trigger the collection scanner
     // to get the album and other meta data correct.
-    m_collection->scanManager()->delayedIncrementalScan( QFileInfo(url).path() );
+    // TODO m_collection->directoryWatcher()->delayedIncrementalScan( QFileInfo(url).path() );
 
     return true;
 }
@@ -343,8 +341,8 @@ SqlCollectionLocation::showDestinationDialog( const Meta::TrackList &tracks,
     delegate->setTranscodingConfiguration( configuration );
     delegate->setCaption( operationText( configuration ) );
 
-    connect( delegate, SIGNAL( accepted() ), SLOT( slotDialogAccepted() ) );
-    connect( delegate, SIGNAL( rejected() ), SLOT( slotDialogRejected() ) );
+    connect( delegate, SIGNAL(accepted()), SLOT(slotDialogAccepted()) );
+    connect( delegate, SIGNAL(rejected()), SLOT(slotDialogRejected()) );
     delegate->show();
 }
 
@@ -383,7 +381,7 @@ SqlCollectionLocation::slotJobFinished( KJob *job )
     DEBUG_BLOCK
 
     Meta::TrackPtr track = m_jobs.value( job );
-    if( job->error() )
+    if( job->error()  && job->error() != KIO::ERR_FILE_ALREADY_EXIST )
     {
         //TODO: proper error handling
         warning() << "An error occurred when copying a file: " << job->errorString();
@@ -428,7 +426,6 @@ SqlCollectionLocation::slotRemoveJobFinished( KJob *job )
 
     if( !startNextRemoveJob() )
     {
-        m_collection->scanManager()->unblockScan();
         slotRemoveOperationFinished();
     }
 
@@ -450,7 +447,6 @@ void SqlCollectionLocation::slotTransferJobFinished( KJob* job )
         m_originalUrls[track] = track->playableUrl();
     }
     debug () << "m_originalUrls" << m_originalUrls;
-    m_collection->scanManager()->unblockScan();
     slotCopyOperationFinished();
 }
 
@@ -468,7 +464,6 @@ void SqlCollectionLocation::slotTransferJobAborted()
             insert( track, m_destinations[ track ] ); // was already copied, so have to insert it in the db
         m_originalUrls[track] = track->playableUrl();
     }
-    m_collection->scanManager()->unblockScan();
     abort();
 }
 
@@ -478,15 +473,13 @@ SqlCollectionLocation::copyUrlsToCollection( const QMap<Meta::TrackPtr, KUrl> &s
                                              const Transcoding::Configuration &configuration )
 {
     DEBUG_BLOCK
-    m_collection->scanManager()->blockScan();  //make sure the collection scanner does not run while we are coyping stuff
-
     m_sources = sources;
 
     QString statusBarTxt = operationInProgressText( configuration, sources.count() );
     m_transferjob = new TransferJob( this, configuration );
     Amarok::Components::logger()->newProgressOperation( m_transferjob, statusBarTxt, this,
                                                         SLOT(slotTransferJobAborted()) );
-    connect( m_transferjob, SIGNAL(result( KJob * )), SLOT(slotTransferJobFinished( KJob * )) );
+    connect( m_transferjob, SIGNAL(result(KJob*)), SLOT(slotTransferJobFinished(KJob*)) );
     m_transferjob->start();
 }
 
@@ -495,15 +488,10 @@ SqlCollectionLocation::removeUrlsFromCollection(  const Meta::TrackList &sources
 {
     DEBUG_BLOCK
 
-    m_collection->scanManager()->blockScan();  //make sure the collection scanner does not run while we are deleting stuff
-
     m_removetracks = sources;
 
     if( !startNextRemoveJob() ) //this signal needs to be called no matter what, even if there are no job finishes to call it
-    {
-        m_collection->scanManager()->unblockScan();
         slotRemoveOperationFinished();
-    }
 }
 
 void
@@ -525,14 +513,23 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         src.cleanPath();
 
         bool hasMoodFile = QFile::exists( moodFile( src ).toLocalFile() );
+        bool isJustCopy = configuration.isJustCopy( track );
 
-        if( configuration.isJustCopy() )
+        if( isJustCopy )
             debug() << "copying from " << src << " to " << dest;
         else
             debug() << "transcoding from " << src << " to " << dest;
 
-        QFileInfo info( dest.pathOrUrl() );
-        QDir dir = info.dir();
+        KFileItem srcInfo( src, src.toMimeDataString(), KFileItem::Unknown );
+        if( !srcInfo.isFile() )
+        {
+            warning() << "Source track" << src << "was no file";
+            source()->transferError( track, i18n( "Source track does not exist: %1", src.pathOrUrl() ) );
+            return true; // Attempt to copy/move the next item in m_sources
+        }
+
+        QFileInfo destInfo( dest.pathOrUrl() );
+        QDir dir = destInfo.dir();
         if( !dir.exists() )
         {
             if( !dir.mkpath( "." ) )
@@ -544,7 +541,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         }
 
         KIO::JobFlags flags;
-        if( configuration.isJustCopy() )
+        if( isJustCopy )
         {
             flags = KIO::HideProgressInfo;
             if( m_overwriteFiles )
@@ -558,7 +555,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
 
         if( src.equals( dest ) )
         {
-            warning() << "move to itself found: " << info.absoluteFilePath();
+            warning() << "move to itself found: " << destInfo.absoluteFilePath();
             m_transferjob->slotJobFinished( 0 );
             if( m_sources.isEmpty() )
                 return false;
@@ -578,7 +575,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         else
         {
             //later on in the case that remove is called, the file will be deleted because we didn't apply moveByDestination to the track
-            if( configuration.isJustCopy() )
+            if( isJustCopy )
                 job = KIO::file_copy( src, dest, -1, flags );
             else
             {
@@ -600,13 +597,13 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         }
         if( job )   //just to be safe
         {
-            connect( job, SIGNAL( result( KJob* ) ), SLOT( slotJobFinished( KJob* ) ) );
-            connect( job, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
+            connect( job, SIGNAL(result(KJob*)), SLOT(slotJobFinished(KJob*)) );
+            connect( job, SIGNAL(result(KJob*)), m_transferjob, SLOT(slotJobFinished(KJob*)) );
             m_transferjob->addSubjob( job );
 
             if( moodJob )
             {
-                connect( moodJob, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
+                connect( moodJob, SIGNAL(result(KJob*)), m_transferjob, SLOT(slotJobFinished(KJob*)) );
                 m_transferjob->addSubjob( moodJob );
             }
 
@@ -614,7 +611,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
             if( track->artist() )
                 name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
 
-            if( configuration.isJustCopy() )
+            if( isJustCopy )
                 m_transferjob->emitInfo( i18n( "Transferring: %1", name ) );
             else
                 m_transferjob->emitInfo( i18n( "Transcoding: %1", name ) );
@@ -656,7 +653,7 @@ bool SqlCollectionLocation::startNextRemoveJob()
             if( QFile::exists( srcMoodFile.toLocalFile() ) )
                 KIO::del( srcMoodFile, KIO::HideProgressInfo );
            
-            connect( job, SIGNAL( result( KJob* ) ), SLOT( slotRemoveJobFinished( KJob* ) ) );
+            connect( job, SIGNAL(result(KJob*)), SLOT(slotRemoveJobFinished(KJob*)) );
             QString name = track->prettyName();
             if( track->artist() )
                 name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
@@ -674,7 +671,7 @@ KUrl
 SqlCollectionLocation::moodFile( const KUrl &track ) const
 {
     KUrl moodPath = track;
-    moodPath.setFileName( "." + moodPath.fileName().replace( QRegExp( "(\\.\\w{2,5})$" ), ".mood" ) );
+    moodPath.setFileName( '.' + moodPath.fileName().replace( QRegExp( "(\\.\\w{2,5})$" ), ".mood" ) );
     return moodPath;
 }
 
@@ -690,8 +687,8 @@ TransferJob::TransferJob( SqlCollectionLocation * location, const Transcoding::C
 
 bool TransferJob::addSubjob( KJob* job )
 {
-    connect( job, SIGNAL( processedAmount( KJob *, KJob::Unit, qulonglong ) ),
-             this, SLOT( propagateProcessedAmount( KJob *, KJob::Unit, qulonglong ) ) );
+    connect( job, SIGNAL(processedAmount(KJob*,KJob::Unit,qulonglong)),
+             this, SLOT(propagateProcessedAmount(KJob*,KJob::Unit,qulonglong)) );
     //KCompositeJob::addSubjob doesn't handle progress reporting.
     return KCompositeJob::addSubjob( job );
 }
@@ -701,6 +698,15 @@ void TransferJob::emitInfo(const QString& message)
     emit infoMessage( this, message );
 }
 
+void TransferJob::slotResult( KJob *job )
+{
+    // When copying without overwriting some files might already be
+    // there and it is not a reason for stopping entire transfer.
+    if ( job->error() == KIO::ERR_FILE_ALREADY_EXIST )
+        removeSubjob( job );
+    else
+        KCompositeJob::slotResult( job );
+}
 
 void TransferJob::start()
 {
@@ -712,7 +718,7 @@ void TransferJob::start()
         emitResult();
         return;
     }
-    QTimer::singleShot( 0, this, SLOT( doWork() ) );
+    QTimer::singleShot( 0, this, SLOT(doWork()) );
 }
 
 void TransferJob::doWork()
