@@ -301,7 +301,7 @@ SqlScanResultProcessor::commitTrack( CollectionScanner::Track *track,
     metaTrack->setType( track->filetype() );
 
     if( m_type == GenericScanManager::FullScan ||
-        !track->bpm() >= 0 )
+        track->bpm() >= 0 )
         metaTrack->setBpm( track->bpm() );
 
     if( m_type == GenericScanManager::FullScan ||
@@ -488,7 +488,7 @@ SqlScanResultProcessor::relocateTracksToNewDirectory( int oldDirId, int newDirId
     if( res.count() != 1 )
     {
         warning() << "relocateTracksToNewDirectory(): no or multiple entries when"
-                  << "quering directory with id" << newDirId;
+                  << "querying directory with id" << newDirId;
         return false;
     }
     int newDirDeviceId = res.at( 0 ).toInt();
