@@ -20,4 +20,19 @@ Handler::ReadCapability::~ReadCapability()
 {
     // nothing to do here
 }
+
+bool
+Handler::ReadCapability::libIsCompilation( const Meta::MediaDeviceTrackPtr &track )
+{
+    Q_UNUSED( track )
+    // provide default implementation so that MTP collection doesn't need to override.
+    return false;
+}
+
+qreal Handler::ReadCapability::libGetReplayGain( const Meta::MediaDeviceTrackPtr &track )
+{
+    Q_UNUSED( track )
+    return 0.0;
+}
+
 #include "ReadCapability.moc"
