@@ -16,6 +16,9 @@
  ****************************************************************************************/
 
 #include "MemoryMeta.h"
+
+#include "core/meta/Statistics.h"
+#include "core/meta/TrackEditor.h"
 #include "core-impl/capabilities/AlbumActionsCapability.h"
 #include "covermanager/CoverCache.h"
 
@@ -50,7 +53,7 @@ Base::removeTrack( Track *track )
 }
 
 Album::Album( const Meta::AlbumPtr &other )
-    : Base( other->name() )
+    : MemoryMeta::Base( other->name() )
     , m_isCompilation( other->isCompilation() )
     , m_canUpdateCompilation( other->canUpdateCompilation() )
     , m_image( other->image() )
@@ -145,6 +148,9 @@ Album::updateCachedValues()
         Track *memoryTrack = static_cast<Track *>( track.data() );
         Meta::AlbumPtr album = memoryTrack->originalTrack()->album();
 
+        if( !album )
+            continue;
+
         if( !m_isCompilation )
             m_isCompilation = album->isCompilation();
         if( !m_canUpdateCompilation )
@@ -183,6 +189,18 @@ Track::~Track()
         static_cast<Year *>( m_year.data() )->removeTrack( this );
 }
 
+Meta::TrackEditorPtr
+Track::editor()
+{
+    return m_track->editor();
+}
+
+Meta::StatisticsPtr
+Track::statistics()
+{
+    return m_track->statistics();
+}
+
 void
 Track::setAlbum( Album *album )
 {
@@ -280,11 +298,14 @@ MapChanger::addExistingTrack( Meta::TrackPtr track, Track *memoryTrack )
     if( !album )
     {
         Meta::ArtistPtr albumArtist;
-        if( !albumArtistName.isEmpty() )
+        const bool isCompilation = trackAlbum ? trackAlbum->isCompilation() : false;
+        // we create even empty-named album artists if the album is not compilation
+        // because Collection Browser wouldn't show these otherwise
+        if( !albumArtistName.isEmpty() || !isCompilation )
         {
             /* Even if MemoryQueryMaker doesn't need albumArtists to be in MemoryCollection
-            * maps, we add her into artist map so that album artist has the same instance as
-            * indentically-named artist (of potentially different tracks) */
+             * maps, we add her into artist map so that album artist has the same instance as
+             * indentically-named artist (of potentially different tracks) */
             albumArtist = m_mc->artistMap().value( albumArtistName );
             if( !albumArtist )
             {
@@ -360,7 +381,7 @@ MapChanger::removeTrack( Meta::TrackPtr track )
      * if( entity && entity->tracks().count() == 1 ) because it would be racy: entity could
      * have referenced a track that is no longer in MemoryCollection, but not yet destroyed.
      *
-     * When track to remove is MemoryMeta::Track, copy and reassing Meta:: entities so
+     * When track to remove is MemoryMeta::Track, copy and reassign Meta:: entities so
      * that the track is detached from MemoryCollection completely. */
 
     Meta::ArtistPtr artist = track->artist();
@@ -478,7 +499,7 @@ MapChanger::referencedAsAlbumArtist( const Meta::ArtistPtr &artist, const AlbumM
     return false;
 }
 
-bool MapChanger::entitiesDiffer( const Meta::MetaBase *first, const Meta::MetaBase *second )
+bool MapChanger::entitiesDiffer( const Meta::Base *first, const Meta::Base *second )
 {
     if( !first && !second )
         return false;  // both null -> do not differ
