@@ -54,8 +54,7 @@ UpnpBrowseCollection::UpnpBrowseCollection( const DeviceInfo& dev )
 
     // experimental code, will probably be moved to a better place
     OrgKdeKDirNotifyInterface *notify = new OrgKdeKDirNotifyInterface("", "", QDBusConnection::sessionBus(), this );
-    Q_ASSERT(connect( notify, SIGNAL( FilesChanged(const QStringList &) ),
-                      this, SLOT( slotFilesChanged(const QStringList &) ) ));
+    connect( notify, SIGNAL(FilesChanged(QStringList)), SLOT(slotFilesChanged(QStringList)) );
 }
 
 UpnpBrowseCollection::~UpnpBrowseCollection()
@@ -97,7 +96,7 @@ void UpnpBrowseCollection::invalidateTracksIn( const QString &dir )
      * If performance is really affected we can use some
      * kind of a prefix tree instead of a hash.
      */
-    foreach( QString key, m_tracksInContainer.keys() ) {
+    foreach( const QString &key, m_tracksInContainer.keys() ) {
         if( key.startsWith( dir ) ) {
             debug() << key << " matches " << dir;
             foreach( TrackPtr track, m_tracksInContainer[dir] ) {
@@ -144,8 +143,8 @@ UpnpBrowseCollection::startIncrementalScan( const QString &directory )
     url.setPath( directory );
     KIO::ListJob *listJob = KIO::listRecursive( url, KIO::HideProgressInfo );
     addJob( listJob );
-    Q_ASSERT( connect( listJob, SIGNAL(entries(KIO::Job *, const KIO::UDSEntryList& )),
-                       this, SLOT(entries(KIO::Job *, const KIO::UDSEntryList&)), Qt::UniqueConnection ) );
+    Q_ASSERT( connect( listJob, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+                       this, SLOT(entries(KIO::Job*,KIO::UDSEntryList)), Qt::UniqueConnection ) );
     Q_ASSERT( connect( listJob, SIGNAL(result(KJob*)),
                        this, SLOT(done(KJob*)), Qt::UniqueConnection ) );
     listJob->start();
@@ -158,7 +157,7 @@ UpnpBrowseCollection::entries( KIO::Job *job, const KIO::UDSEntryList &list )
     DEBUG_BLOCK;
     int count = 0;
     KIO::SimpleJob *sj = static_cast<KIO::SimpleJob *>( job );
-    foreach( KIO::UDSEntry entry, list ) {
+    foreach( const KIO::UDSEntry &entry, list ) {
         if( entry.contains( KIO::UPNP_CLASS )
             && entry.stringValue( KIO::UPNP_CLASS ).startsWith( "object.item.audioItem" ) ) {
             createTrack( entry, sj->url().prettyUrl() );
