@@ -17,7 +17,10 @@
 #ifndef AMAROK_META_FILE_H
 #define AMAROK_META_FILE_H
 
+#include "amarok_export.h"
 #include "core/meta/Meta.h"
+#include "core/meta/Statistics.h"
+#include "core/meta/TrackEditor.h"
 
 namespace MetaFile
 {
@@ -25,22 +28,20 @@ namespace MetaFile
 
     typedef KSharedPtr<Track> TrackPtr;
 
-    class AMAROK_EXPORT Track : public Meta::Track
+    class AMAROK_EXPORT Track : public Meta::Track, public Meta::Statistics, Meta::TrackEditor
     {
         public:
             Track( const KUrl &url );
             virtual ~Track();
 
-        //methods inherited from Meta::MetaBase
+        //methods inherited from Meta::Base
             virtual QString name() const;
 
         //methods inherited from Meta::Track
             virtual KUrl playableUrl() const;
             virtual QString prettyUrl() const;
             virtual QString uidUrl() const;
-
-            virtual bool isPlayable() const;
-            virtual bool isEditable() const;
+            virtual QString notPlayableReason() const;
 
             virtual Meta::AlbumPtr album() const;
             virtual Meta::ArtistPtr artist() const;
@@ -51,12 +52,6 @@ namespace MetaFile
             virtual qreal bpm() const;
             virtual QString comment() const;
 
-            virtual double score() const;
-            virtual void setScore( double newScore );
-
-            virtual int rating() const;
-            virtual void setRating( int newRating );
-
             virtual int trackNumber() const;
             virtual int discNumber() const;
 
@@ -65,22 +60,50 @@ namespace MetaFile
             virtual int sampleRate() const;
             virtual int bitrate() const;
             virtual QDateTime createDate() const;
-            virtual QDateTime lastPlayed() const;
-            virtual QDateTime firstPlayed() const;
-            virtual int playCount() const;
 
             virtual qreal replayGain( Meta::ReplayGainTag mode ) const;
 
             virtual QString type() const;
 
-            virtual void finishedPlaying( double playedFraction );
             virtual bool inCollection() const;
             virtual Collections::Collection *collection() const;
 
             virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const;
             virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type );
 
+            virtual Meta::TrackEditorPtr editor();
+            virtual Meta::StatisticsPtr statistics();
+
+        // Meta::TrackEditor methods:
+            virtual void setAlbum( const QString &newAlbum );
+            virtual void setAlbumArtist( const QString &newAlbumArtist );
+            virtual void setArtist( const QString &newArtist );
+            virtual void setComposer( const QString &newComposer );
+            virtual void setGenre( const QString &newGenre );
+            virtual void setYear( int newYear );
+            virtual void setTitle( const QString &newTitle );
+            virtual void setComment( const QString &newComment );
+            virtual void setTrackNumber( int newTrackNumber );
+            virtual void setDiscNumber( int newDiscNumber );
+            virtual void setBpm( const qreal newBpm );
+
+        // Meta::Statistics methods:
+            virtual double score() const;
+            virtual void setScore( double newScore );
+
+            virtual int rating() const;
+            virtual void setRating( int newRating );
+
+            virtual int playCount() const;
+            virtual void setPlayCount( int newPlayCount );
+
+        // combined Meta::TrackEditor, Meta::Statistics methods:
+            virtual void beginUpdate();
+            virtual void endUpdate();
+
         // MetaFile::Track own methods:
+            bool isEditable() const;
+
             /**
              * Return true if file at @param url is a track.
              *
@@ -90,34 +113,24 @@ namespace MetaFile
              */
             static bool isTrack( const KUrl &url );
 
-            virtual void beginMetaDataUpdate();
-            virtual void endMetaDataUpdate();
-
             virtual QImage getEmbeddedCover() const;
-
             virtual void setCollection( Collections::Collection *newCollection );
-            virtual void setTitle( const QString &newTitle );
-            virtual void setAlbum( const QString &newAlbum );
-            virtual void setAlbumArtist( const QString &newAlbumArtist );
-            virtual void setArtist( const QString &newArtist );
-            virtual void setGenre( const QString &newGenre );
-            virtual void setComposer( const QString &newComposer );
-            virtual void setYear( int newYear );
-            virtual void setTrackNumber( int newTrackNumber );
-            virtual void setBpm( const qreal newBpm );
-            virtual void setComment( const QString &newComment );
-            virtual void setDiscNumber( int newDiscNumber );
-            virtual void setLastPlayed( const QDateTime &newTime );
-            virtual void setFirstPlayed( const QDateTime &newTime );
-            virtual void setPlayCount( int newCount );
 
             // publish method so that it can be called by Private.
-            using Meta::MetaBase::notifyObservers;
+            using Meta::Track::notifyObservers;
 
             class Private;
 
         private:
             Private * const d;
+
+            /**
+             * Must be called at end of every set*() method, with d->lock locked for
+             * writing. Takes care of writing back the fields, re-reading them and
+             * notifying observers.
+             */
+            void commitIfInNonBatchUpdate( qint64 field, const QVariant &value );
+            void commitIfInNonBatchUpdate();
     };
 }
 
