@@ -19,13 +19,15 @@
 #include "amarokurls/BookmarkMetaActions.h"
 #include "amarokurls/PlayUrlRunner.h"
 #include "core/capabilities/ActionsCapability.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 #include "core/meta/TrackEditor.h"
 #include "core/support/Debug.h"
 #include "core-impl/capabilities/timecode/TimecodeLoadCapability.h"
 #include "core-impl/capabilities/timecode/TimecodeWriteCapability.h"
 #include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/storage/StorageManager.h"
 #include "core-impl/meta/proxy/MetaProxy.h"
+#include "core-impl/meta/file/FileTrackProvider.h"
 #include "core-impl/podcasts/sql/SqlPodcastProvider.h"
 
 #include <QDate>
@@ -33,6 +35,8 @@
 
 using namespace Podcasts;
 
+static FileTrackProvider myFileTrackProvider; // we need it to be available for lookups
+
 class TimecodeWriteCapabilityPodcastImpl : public Capabilities::TimecodeWriteCapability
 {
     public:
@@ -117,7 +121,7 @@ SqlPodcastEpisode::SqlPodcastEpisode( const QStringList &result, SqlPodcastChann
     : Podcasts::PodcastEpisode( Podcasts::PodcastChannelPtr::staticCast( sqlChannel ) )
     , m_channel( sqlChannel )
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     QStringList::ConstIterator iter = result.constBegin();
     m_dbId = (*(iter++)).toInt();
     m_url = KUrl( *(iter++) );
@@ -239,7 +243,7 @@ SqlPodcastEpisode::setupLocalFile()
     /* following won't write to actual file, because MetaProxy::Track hasn't yet looked
      * up the underlying track. It will just set some cached values. */
     writeTagsToFile();
-    proxyTrack->lookupTrack( CollectionManager::instance()->fileTrackProvider() );
+    proxyTrack->lookupTrack( &myFileTrackProvider );
 }
 
 SqlPodcastEpisode::~SqlPodcastEpisode()
@@ -433,7 +437,7 @@ SqlPodcastEpisode::writeTagsToFile()
 void
 SqlPodcastEpisode::updateInDb()
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
 
     QString boolTrue = sqlStorage->boolTrue();
     QString boolFalse = sqlStorage->boolFalse();
@@ -504,7 +508,7 @@ SqlPodcastEpisode::updateInDb()
 void
 SqlPodcastEpisode::deleteFromDb()
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     sqlStorage->query(
         QString( "DELETE FROM podcastepisodes WHERE id = %1;" ).arg( dbId() ) );
 }
@@ -527,9 +531,10 @@ SqlPodcastChannel::SqlPodcastChannel( SqlPodcastProvider *provider,
                                             const QStringList &result )
     : Podcasts::PodcastChannel()
     , m_episodesLoaded( false )
+    , m_trackCacheIsValid( false )
     , m_provider( provider )
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     QStringList::ConstIterator iter = result.constBegin();
     m_dbId = (*(iter++)).toInt();
     m_url = KUrl( *(iter++) );
@@ -539,7 +544,7 @@ SqlPodcastChannel::SqlPodcastChannel( SqlPodcastProvider *provider,
     m_description = *(iter++);
     m_copyright = *(iter++);
     m_directory = KUrl( *(iter++) );
-    m_labels = QStringList( QString( *(iter++) ).split( ",", QString::SkipEmptyParts ) );
+    m_labels = QStringList( QString( *(iter++) ).split( ',', QString::SkipEmptyParts ) );
     m_subscribeDate = QDate::fromString( *(iter++) );
     m_autoScan = sqlStorage->boolTrue() == *(iter++);
     m_fetchType = (*(iter++)).toInt() == DownloadWhenAvailable ? DownloadWhenAvailable : StreamOrDownloadOnDemand;
@@ -553,6 +558,7 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
                                             Podcasts::PodcastChannelPtr channel )
     : Podcasts::PodcastChannel()
     , m_dbId( 0 )
+    , m_trackCacheIsValid( false )
     , m_provider( provider )
     , m_filenameLayout( "%default%" )
 {
@@ -731,7 +737,7 @@ SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
     notifyObserversTrackAdded( Meta::TrackPtr::dynamicCast( sqlEpisode ), i );
 
     applyPurge();
-
+    m_trackCacheIsValid = false;
     return PodcastEpisodePtr::dynamicCast( sqlEpisode );
 }
 
@@ -759,60 +765,96 @@ SqlPodcastChannel::applyPurge()
                     purgeIndex++;
             }
         }
+        m_trackCacheIsValid = false;
     }
 }
 
 void
 SqlPodcastChannel::updateInDb()
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
 
     QString boolTrue = sqlStorage->boolTrue();
     QString boolFalse = sqlStorage->boolFalse();
     #define escape(x) sqlStorage->escape(x)
-    QString insert = "INSERT INTO podcastchannels("
-    "url,title,weblink,image,description,copyright,directory,labels,"
-    "subscribedate,autoscan,fetchtype,haspurge,purgecount,writetags,filenamelayout) "
-    "VALUES ( '%1','%2','%3','%4','%5','%6','%7','%8','%9',%10,%11,%12,%13,%14,'%15' );";
-
-    QString update = "UPDATE podcastchannels SET url='%1',title='%2'"
-    ",weblink='%3',image='%4',description='%5',copyright='%6',directory='%7'"
-    ",labels='%8',subscribedate='%9',autoscan=%10,fetchtype=%11,haspurge=%12,"
-    "purgecount=%13, writetags=%14, filenamelayout='%15' WHERE id=%16;";
-    //if we don't have a database ID yet we should insert;
-    QString command = m_dbId ? update : insert;
-
-    command = command.arg( escape(m_url.url()) ); //%1
-    command = command.arg( escape(m_title) ); //%2
-    command = command.arg( escape(m_webLink.url()) ); //%3
-    command = command.arg( escape(m_imageUrl.url()) ); //image  //%4
-    command = command.arg( escape(m_description) ); //%5
-    command = command.arg( escape(m_copyright) ); //%6
-    command = command.arg( escape(m_directory.url()) ); //%7
-    command = command.arg( escape(m_labels.join( "," )) ); //%8
-    command = command.arg( escape(m_subscribeDate.toString()) ); //%9
-    command = command.arg( m_autoScan ? boolTrue : boolFalse ); //%10
-    command = command.arg( QString::number(m_fetchType) ); //%11
-    command = command.arg( m_purge ? boolTrue : boolFalse ); //%12
-    command = command.arg( QString::number(m_purgeCount) ); //%13
-    command = command.arg( m_writeTags ? boolTrue : boolFalse ); //%14
-    command = command.arg( escape(m_filenameLayout) ); //%15
-
+    QString command;
+    QTextStream stream( &command );
     if( m_dbId )
-        sqlStorage->query( command.arg( m_dbId ) );
+    {
+        stream << "UPDATE podcastchannels ";
+        stream << "SET url='";
+        stream << escape(m_url.url());
+        stream << "', title='";
+        stream << escape(m_title);
+        stream << "', weblink='";
+        stream << escape(m_webLink.url());
+        stream << "', image='";
+        stream << escape(m_imageUrl.url());
+        stream << "', description='";
+        stream << escape(m_description);
+        stream << "', copyright='";
+        stream << escape(m_copyright);
+        stream << "', directory='";
+        stream << escape(m_directory.url());
+        stream << "', labels='";
+        stream << escape(m_labels.join( "," ));
+        stream << "', subscribedate='";
+        stream << escape(m_subscribeDate.toString());
+        stream << "', autoscan=";
+        stream << (m_autoScan ? boolTrue : boolFalse);
+        stream << ", fetchtype=";
+        stream << QString::number(m_fetchType);
+        stream << ", haspurge=";
+        stream << (m_purge ? boolTrue : boolFalse);
+        stream << ", purgecount=";
+        stream << QString::number(m_purgeCount);
+        stream << ", writetags=";
+        stream << (m_writeTags ? boolTrue : boolFalse);
+        stream << ", filenamelayout='";
+        stream << escape(m_filenameLayout);
+        stream << "' WHERE id=";
+        stream << m_dbId;
+        stream << ";";
+        kDebug() << command;
+        sqlStorage->query( command );
+    }
     else
+    {
+        stream << "INSERT INTO podcastchannels(";
+        stream << "url,title,weblink,image,description,copyright,directory,labels,";
+        stream << "subscribedate,autoscan,fetchtype,haspurge,purgecount,writetags,filenamelayout) ";
+        stream << "VALUES ( '";
+        stream << escape(m_url.url()) << "', '";
+        stream << escape(m_title) << "', '";
+        stream << escape(m_webLink.url()) << "', '";
+        stream << escape(m_imageUrl.url()) << "', '";
+        stream << escape(m_description) << "', '";
+        stream << escape(m_copyright) << "', '";
+        stream << escape(m_directory.url()) << "', '";
+        stream << escape(m_labels.join( "," )) << "', '";
+        stream << escape(m_subscribeDate.toString()) << "', ";
+        stream << (m_autoScan ? boolTrue : boolFalse) << ", ";
+        stream << QString::number(m_fetchType) << ", ";
+        stream << (m_purge ? boolTrue : boolFalse) << ", ";
+        stream << QString::number(m_purgeCount) << ", ";
+        stream << (m_writeTags ? boolTrue : boolFalse) << ", '";
+        stream << escape(m_filenameLayout);
+        stream << "');";
+        kDebug() << command;
         m_dbId = sqlStorage->insert( command, "podcastchannels" );
+    }
 }
 
 void
 SqlPodcastChannel::deleteFromDb()
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     foreach( SqlPodcastEpisodePtr sqlEpisode, m_episodes )
     {
        sqlEpisode->deleteFromDb();
        m_episodes.removeOne( sqlEpisode );
     }
+    m_trackCacheIsValid = false;
 
     sqlStorage->query(
         QString( "DELETE FROM podcastchannels WHERE id = %1;" ).arg( dbId() ) );
@@ -823,7 +865,7 @@ SqlPodcastChannel::loadEpisodes()
 {
     m_episodes.clear();
 
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
 
     //If purge is enabled we must limit the number of results
     QString command;
@@ -868,12 +910,17 @@ SqlPodcastChannel::loadEpisodes()
     }
 
     m_episodesLoaded = true;
+    m_trackCacheIsValid = false;
 }
 
 Meta::TrackList
 Podcasts::SqlPodcastChannel::tracks()
 {
-    return Podcasts::SqlPodcastEpisode::toTrackList( m_episodes );
+    if ( !m_trackCacheIsValid ) {
+        m_episodesAsTracksCache = Podcasts::SqlPodcastEpisode::toTrackList( m_episodes );
+        m_trackCacheIsValid = true;
+    }
+    return m_episodesAsTracksCache;
 }
 
 void
