@@ -16,49 +16,49 @@
  ****************************************************************************************/
 
 #include "SqlPodcastProvider.h"
-#include <kprogressdialog.h>
 
+#include "MainWindow.h"
+#include "OpmlWriter.h"
+#include "SvgHandler.h"
+#include "QStringx.h"
+#include "browsers/playlistbrowser/PodcastModel.h"
 #include "context/popupdropper/libpud/PopupDropper.h"
 #include "context/popupdropper/libpud/PopupDropperItem.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 #include "core/interfaces/Logger.h"
 #include "core/podcasts/PodcastImageFetcher.h"
 #include "core/podcasts/PodcastReader.h"
 #include "core/support/Amarok.h"
 #include "core/support/Components.h"
 #include "core/support/Debug.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "MainWindow.h"
-#include "OpmlWriter.h"
+#include "core-impl/storage/StorageManager.h"
+#include "core-impl/podcasts/sql/PodcastSettingsDialog.h"
 #include "playlistmanager/sql/SqlPlaylistGroup.h"
-#include "PodcastModel.h"
-#include "PodcastSettingsDialog.h"
-#include "QStringx.h"
-#include "SvgHandler.h"
 
 #include "ui_SqlPodcastProviderSettingsWidget.h"
 
 #include <KCodecs>
-#include <KLocale>
 #include <KFileDialog>
 #include <KIO/CopyJob>
 #include <KIO/DeleteJob>
 #include <KIO/Job>
 #include <KIO/NetAccess>
+#include <KLocale>
+#include <KProgressDialog>
 #include <KStandardDirs>
 #include <KUrl>
 #include <Solid/Networking>
 
 #include <QAction>
-#include <QFile>
-#include <QDir>
-#include <QTimer>
 #include <QCheckBox>
+#include <QDir>
+#include <QFile>
 #include <QMap>
+#include <QTimer>
 
 using namespace Podcasts;
 
-static const int PODCAST_DB_VERSION = 5;
+static const int PODCAST_DB_VERSION = 6;
 static const QString key( "AMAROK_PODCAST" );
 static const QString PODCAST_TMP_POSTFIX( ".tmp" );
 
@@ -71,15 +71,15 @@ SqlPodcastProvider::SqlPodcastProvider()
         , m_configureChannelAction( 0 )
         , m_deleteAction( 0 )
         , m_downloadAction( 0 )
+        , m_keepAction( 0 )
         , m_removeAction( 0 )
-        , m_renameAction( 0 )
         , m_updateAction( 0 )
         , m_writeTagsAction( 0 )
         , m_podcastImageFetcher( 0 )
 {
-    connect( m_updateTimer, SIGNAL( timeout() ), SLOT( autoUpdate() ) );
+    connect( m_updateTimer, SIGNAL(timeout()), SLOT(autoUpdate()) );
 
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
 
     if( !sqlStorage )
     {
@@ -165,7 +165,7 @@ void
 SqlPodcastProvider::loadPodcasts()
 {
     m_channels.clear();
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 
@@ -195,13 +195,13 @@ SqlPodcastProvider::sqlEpisodeForString( const QString &string )
     if( string.isEmpty() )
         return SqlPodcastEpisodePtr();
 
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return SqlPodcastEpisodePtr();
 
     QString command = "SELECT id, url, channel, localurl, guid, "
             "title, subtitle, sequencenumber, description, mimetype, pubdate, "
-            "duration, filesize, isnew FROM podcastepisodes "
+            "duration, filesize, isnew, iskeep FROM podcastepisodes "
             "WHERE guid='%1' OR url='%1' OR localurl='%1' ORDER BY id DESC;";
     command = command.arg( sqlStorage->escape( string ) );
     QStringList dbResult = sqlStorage->query( command );
@@ -237,13 +237,13 @@ SqlPodcastProvider::sqlEpisodeForString( const QString &string )
 
     //The episode was found in the database but it's channel didn't have it in it's list.
     //That probably is because it's beyond the purgecount limit or the tracks were not loaded yet.
-    return SqlPodcastEpisodePtr( new SqlPodcastEpisode( dbResult.mid( 0, 14 ), channel ) );
+    return SqlPodcastEpisodePtr( new SqlPodcastEpisode( dbResult.mid( 0, 15 ), channel ) );
 }
 
 bool
 SqlPodcastProvider::possiblyContainsTrack( const KUrl &url ) const
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return false;
 
@@ -279,31 +279,25 @@ SqlPodcastProvider::playlists()
     return playlistList;
 }
 
-QList<QAction *>
+QActionList
 SqlPodcastProvider::providerActions()
 {
     if( m_providerActions.isEmpty() )
     {
         QAction *updateAllAction = new QAction( KIcon( "view-refresh-amarok" ),
-                                         i18n( "&Update All Channels" ),
-                                         this
-                                       );
+                i18n( "&Update All Channels" ), this );
         updateAllAction->setProperty( "popupdropper_svg_id", "update" );
-        connect( updateAllAction, SIGNAL( triggered() ), this, SLOT( updateAll() ) );
+        connect( updateAllAction, SIGNAL(triggered()), this, SLOT(updateAll()) );
         m_providerActions << updateAllAction;
 
         QAction *configureAction = new QAction( KIcon( "configure" ),
-            i18n( "&Configure General Settings" ),
-            this
-        );
+                i18n( "&Configure General Settings" ), this );
         configureAction->setProperty( "popupdropper_svg_id", "configure" );
-        connect( configureAction, SIGNAL( triggered() ), this, SLOT( slotConfigureProvider() ) );
+        connect( configureAction, SIGNAL(triggered()), this, SLOT(slotConfigureProvider()) );
         m_providerActions << configureAction;
 
         QAction *exportOpmlAction = new QAction( KIcon( "document-export" ),
-                                                 i18n( "&Export subscriptions to OPML file" ),
-                                                 this
-                                               );
+                i18n( "&Export subscriptions to OPML file" ), this );
         connect( exportOpmlAction, SIGNAL(triggered()), SLOT(slotExportOpml()) );
         m_providerActions << exportOpmlAction;
     }
@@ -311,149 +305,145 @@ SqlPodcastProvider::providerActions()
     return m_providerActions;
 }
 
-QList<QAction *>
-SqlPodcastProvider::playlistActions( Playlists::PlaylistPtr playlist )
+QActionList
+SqlPodcastProvider::playlistActions( const Playlists::PlaylistList &playlists )
 {
-    QList<QAction *> actions;
+    QActionList actions;
+    SqlPodcastChannelList sqlChannels;
+    foreach( const Playlists::PlaylistPtr &playlist, playlists )
+    {
+        SqlPodcastChannelPtr sqlChannel = SqlPodcastChannel::fromPlaylistPtr( playlist );
+        if( sqlChannel )
+            sqlChannels << sqlChannel;
+    }
 
-    Podcasts::SqlPodcastChannelPtr sqlChannel = Podcasts::SqlPodcastChannel::fromPlaylistPtr( playlist );
-    if( sqlChannel.isNull() )
+    if( sqlChannels.isEmpty() )
         return actions;
 
     //TODO: add export OPML action for selected playlists only. Use the QAction::data() trick.
     if( m_configureChannelAction == 0 )
     {
-        m_configureChannelAction = new QAction(
-            KIcon( "configure" ),
-            i18n( "&Configure" ),
-            this
-        );
+        m_configureChannelAction = new QAction( KIcon( "configure" ), i18n( "&Configure" ), this );
         m_configureChannelAction->setProperty( "popupdropper_svg_id", "configure" );
-        connect( m_configureChannelAction, SIGNAL( triggered() ), SLOT( slotConfigureChannel() ) );
+        connect( m_configureChannelAction, SIGNAL(triggered()), SLOT(slotConfigureChannel()) );
+    }
+    //only one channel can be configured at a time.
+    if( sqlChannels.count() == 1 )
+    {
+        m_configureChannelAction->setData( QVariant::fromValue( sqlChannels.first() ) );
+        actions << m_configureChannelAction;
     }
 
-    //only one channel can be configured at a time.
-    if( m_configureChannelAction->data().isNull() )
-        m_configureChannelAction->setData( QVariant::fromValue( sqlChannel ) );
-
-    actions << m_configureChannelAction;
-
-    Podcasts::SqlPodcastChannelList actionChannels;
     if( m_removeAction == 0 )
     {
-        m_removeAction = new QAction(
-            KIcon( "news-unsubscribe" ),
-            i18n( "&Remove Subscription" ),
-            this
-        );
+        m_removeAction = new QAction( KIcon( "news-unsubscribe" ), i18n( "&Remove Subscription" ), this );
         m_removeAction->setProperty( "popupdropper_svg_id", "remove" );
-        connect( m_removeAction, SIGNAL( triggered() ), SLOT( slotRemoveChannels() ) );
+        connect( m_removeAction, SIGNAL(triggered()), SLOT(slotRemoveChannels()) );
     }
-    else
-    {
-        actionChannels = m_removeAction->data().value<Podcasts::SqlPodcastChannelList>();
-    }
-    m_removeAction->setObjectName( "deleteAction" );
-
-    actionChannels << sqlChannel;
-    m_removeAction->setData( QVariant::fromValue( actionChannels ) );
-
+    m_removeAction->setData( QVariant::fromValue( sqlChannels ) );
     actions << m_removeAction;
 
-    actionChannels.clear();
     if( m_updateAction == 0 )
     {
-        m_updateAction = new QAction(
-            KIcon( "view-refresh-amarok" ),
-            i18n( "&Update Channel" ),
-            this
-        );
+        m_updateAction = new QAction( KIcon( "view-refresh-amarok" ), i18n( "&Update Channel" ), this );
         m_updateAction->setProperty( "popupdropper_svg_id", "update" );
-        connect( m_updateAction, SIGNAL( triggered() ), SLOT( slotUpdateChannels() ) );
+        connect( m_updateAction, SIGNAL(triggered()), SLOT(slotUpdateChannels()) );
     }
-    else
-    {
-        actionChannels = m_updateAction->data().value<Podcasts::SqlPodcastChannelList>();
-    }
-
-    actionChannels << sqlChannel;
-    m_updateAction->setData( QVariant::fromValue( actionChannels ) );
-
+    m_updateAction->setData( QVariant::fromValue( sqlChannels ) );
     actions << m_updateAction;
 
     return actions;
 }
 
-QList<QAction *>
-SqlPodcastProvider::trackActions( Playlists::PlaylistPtr playlist, int trackIndex )
+QActionList
+SqlPodcastProvider::trackActions( const QMultiHash<Playlists::PlaylistPtr, int> &playlistTracks )
 {
-    QList<QAction *> actions;
-    Podcasts::SqlPodcastChannelPtr sqlChannel = Podcasts::SqlPodcastChannel::fromPlaylistPtr( playlist );
-    if( sqlChannel.isNull() )
+    SqlPodcastEpisodeList episodes;
+    foreach( const Playlists::PlaylistPtr &playlist, playlistTracks.uniqueKeys() )
+    {
+        SqlPodcastChannelPtr sqlChannel = SqlPodcastChannel::fromPlaylistPtr( playlist );
+        if( !sqlChannel )
+            continue;
+
+        SqlPodcastEpisodeList channelEpisodes = sqlChannel->sqlEpisodes();
+        QList<int> trackPositions = playlistTracks.values( playlist );
+        qSort( trackPositions );
+        foreach( int trackPosition, trackPositions )
+        {
+            if( trackPosition >= 0 && trackPosition < channelEpisodes.count() )
+                episodes << channelEpisodes.at( trackPosition );
+        }
+    }
+
+    QActionList actions;
+    if( episodes.isEmpty() )
         return actions;
 
-    Podcasts::SqlPodcastEpisodeList sqlEpisodes = sqlChannel->sqlEpisodes();
-    if( trackIndex >= sqlEpisodes.count() )
-        return actions;
-
-    Podcasts::SqlPodcastEpisodePtr sqlEpisode = sqlEpisodes.at( trackIndex );
-    if( sqlEpisode.isNull() )
-        return actions;
+    if( m_downloadAction == 0 )
+    {
+        m_downloadAction = new QAction( KIcon( "go-down" ), i18n( "&Download Episode" ), this );
+        m_downloadAction->setProperty( "popupdropper_svg_id", "download" );
+        connect( m_downloadAction, SIGNAL(triggered()), SLOT(slotDownloadEpisodes()) );
+    }
 
     if( m_deleteAction == 0 )
     {
-        m_deleteAction = new QAction(
-            KIcon( "edit-delete" ),
-            i18n( "&Delete Downloaded Episode" ),
-            this
-        );
+        m_deleteAction = new QAction( KIcon( "edit-delete" ),
+            i18n( "&Delete Downloaded Episode" ), this );
         m_deleteAction->setProperty( "popupdropper_svg_id", "delete" );
-        connect( m_deleteAction, SIGNAL( triggered() ), SLOT( slotDeleteDownloadedEpisodes() ) );
+        m_deleteAction->setObjectName( "deleteAction" );
+        connect( m_deleteAction, SIGNAL(triggered()), SLOT(slotDeleteDownloadedEpisodes()) );
     }
-    m_deleteAction->setObjectName( "deleteAction" );
 
     if( m_writeTagsAction == 0 )
     {
-        m_writeTagsAction = new QAction(
-            KIcon( "media-track-edit-amarok" ),
-            i18n( "&Write Feed Information to File" ),
-            this
-        );
+        m_writeTagsAction = new QAction( KIcon( "media-track-edit-amarok" ),
+            i18n( "&Write Feed Information to File" ), this );
         m_writeTagsAction->setProperty( "popupdropper_svg_id", "edit" );
-        connect( m_writeTagsAction, SIGNAL( triggered() ), SLOT( slotWriteTagsToFiles() ) );
+        connect( m_writeTagsAction, SIGNAL(triggered()), SLOT(slotWriteTagsToFiles()) );
     }
 
-    Podcasts::SqlPodcastEpisodeList actionEpisodes;
-    if( !sqlEpisode->localUrl().isEmpty() )
+    if( m_keepAction == 0 )
     {
-        actionEpisodes = m_deleteAction->data().value<Podcasts::SqlPodcastEpisodeList>();
-        actionEpisodes << sqlEpisode;
-        m_deleteAction->setData( QVariant::fromValue( actionEpisodes ) );
-        //these lists are the same anyway
-        m_writeTagsAction->setData( QVariant::fromValue( actionEpisodes ) );
-        actions << m_deleteAction;
-        actions << m_writeTagsAction;
+        m_keepAction = new QAction( KIcon( "podcast-amarok" ),
+                i18n( "&Keep downloaded file" ), this );
+        m_keepAction->setToolTip( i18n( "Toggle the \"keep\" downloaded file status of "
+                "this podcast episode. Downloaded files with this status wouldn't be "
+                "deleted even if we apply a purge." ) );
+        m_keepAction->setProperty( "popupdropper_svg_id", "keep" );
+        m_keepAction->setCheckable( true );
+        connect( m_keepAction, SIGNAL(triggered(bool)), SLOT(slotSetKeep()) );
     }
-    else
+
+    SqlPodcastEpisodeList remoteEpisodes;
+    SqlPodcastEpisodeList keptDownloadedEpisodes, unkeptDownloadedEpisodes;
+    foreach( const SqlPodcastEpisodePtr &episode, episodes )
     {
-        if( m_downloadAction == 0 )
-        {
-            m_downloadAction = new QAction(
-                KIcon( "go-down" ),
-                i18n( "&Download Episode" ),
-                this
-            );
-            m_downloadAction->setProperty( "popupdropper_svg_id", "download" );
-            connect( m_downloadAction, SIGNAL( triggered() ), SLOT( slotDownloadEpisodes() ) );
-        }
+        if( episode->localUrl().isEmpty() )
+            remoteEpisodes << episode;
         else
         {
-            actionEpisodes = m_downloadAction->data().value<Podcasts::SqlPodcastEpisodeList>();
+            if( episode->isKeep() )
+                keptDownloadedEpisodes << episode;
+            else
+                unkeptDownloadedEpisodes << episode;
         }
-        actionEpisodes << sqlEpisode;
-        m_downloadAction->setData( QVariant::fromValue( actionEpisodes ) );
+    }
+
+    if( !remoteEpisodes.isEmpty() )
+    {
+        m_downloadAction->setData( QVariant::fromValue( remoteEpisodes ) );
         actions << m_downloadAction;
     }
+    if( !( keptDownloadedEpisodes + unkeptDownloadedEpisodes ).isEmpty() )
+    {
+        m_deleteAction->setData( QVariant::fromValue( keptDownloadedEpisodes + unkeptDownloadedEpisodes ) );
+        actions << m_deleteAction;
+
+        m_keepAction->setChecked( unkeptDownloadedEpisodes.isEmpty() );
+        m_keepAction->setData( QVariant::fromValue( keptDownloadedEpisodes + unkeptDownloadedEpisodes ) );
+        actions << m_keepAction;
+    }
 
     return actions;
 }
@@ -470,7 +460,7 @@ SqlPodcastProvider::addPodcast( const KUrl &url )
     KUrl kurl = KUrl( url );
     debug() << "importing " << kurl.url();
 
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 
@@ -515,10 +505,10 @@ SqlPodcastProvider::subscribe( const KUrl &url )
     }
 
     PodcastReader *podcastReader = new PodcastReader( this );
-    connect( podcastReader, SIGNAL( finished( PodcastReader * ) ),
-             SLOT( slotReadResult( PodcastReader * ) ) );
-    connect( podcastReader, SIGNAL( statusBarSorryMessage( const QString & ) ),
-            this, SLOT( slotStatusBarSorryMessage( const QString & ) ) );
+    connect( podcastReader, SIGNAL(finished(PodcastReader*)),
+             SLOT(slotReadResult(PodcastReader*)) );
+    connect( podcastReader, SIGNAL(statusBarSorryMessage(QString)),
+            this, SLOT(slotStatusBarSorryMessage(QString)) );
     connect( podcastReader,
         SIGNAL(statusBarNewProgressOperation( KIO::TransferJob *, const QString &,
                                               Podcasts::PodcastReader* )),
@@ -535,13 +525,13 @@ SqlPodcastProvider::addChannel( Podcasts::PodcastChannelPtr channel )
 {
     Podcasts::SqlPodcastChannelPtr sqlChannel =
             SqlPodcastChannelPtr( new Podcasts::SqlPodcastChannel( this, channel ) );
-    m_channels <<  sqlChannel;
+    m_channels << sqlChannel;
 
     if( sqlChannel->episodes().count() == 0 )
         updateSqlChannel( sqlChannel );
 
-    emit playlistAdded( Playlists::PlaylistPtr::dynamicCast( sqlChannel ) );
-    return Podcasts::PodcastChannelPtr::dynamicCast( sqlChannel );
+    emit playlistAdded( Playlists::PlaylistPtr( sqlChannel.data() ) );
+    return PodcastChannelPtr( sqlChannel.data() );
 }
 
 Podcasts::PodcastEpisodePtr
@@ -587,7 +577,7 @@ SqlPodcastProvider::removeSubscription( Podcasts::SqlPodcastChannelPtr sqlChanne
     //Remove it when we know it's supposed to be empty.
     if( m_channels.isEmpty() )
     {
-        SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+        SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
         if( !sqlStorage )
             return;
         debug() << "Unsubscribed from last channel, cleaning out the podcastepisodes table.";
@@ -618,9 +608,9 @@ SqlPodcastProvider::configureProvider()
     m_providerSettingsDialog->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
     m_providerSettingsDialog->setMainWidget( settingsWidget );
 
-    connect( settings.m_baseDirUrl, SIGNAL( textChanged(QString) ), SLOT( slotConfigChanged() ) );
-    connect( settings.m_autoUpdateInterval, SIGNAL( valueChanged(int) ),
-             SLOT( slotConfigChanged() ) );
+    connect( settings.m_baseDirUrl, SIGNAL(textChanged(QString)), SLOT(slotConfigChanged()) );
+    connect( settings.m_autoUpdateInterval, SIGNAL(valueChanged(int)),
+             SLOT(slotConfigChanged()) );
 
     m_providerSettingsDialog->setWindowTitle( i18n( "Configure Local Podcasts" ) );
     m_providerSettingsDialog->enableButtonApply( false );
@@ -766,12 +756,11 @@ SqlPodcastProvider::configureChannel( Podcasts::SqlPodcastChannelPtr sqlChannel
         /* changed from purge to no-purge or increase purge count:
         we need to reload all episodes from the database. */
         sqlChannel->loadEpisodes();
-        emit( updated() );
     }
     else
-    {
         sqlChannel->applyPurge();
-    }
+
+    emit updated();
 
     if( oldSaveLocation != sqlChannel->saveLocation() )
     {
@@ -789,156 +778,6 @@ SqlPodcastProvider::configureChannel( Podcasts::SqlPodcastChannelPtr sqlChannel
         startTimer();
 }
 
-QList<QAction *>
-SqlPodcastProvider::episodeActions( Podcasts::PodcastEpisodeList episodes )
-{
-    QList< QAction * > actions;
-    if( episodes.isEmpty() )
-        return actions;
-
-    Podcasts::PodcastEpisodeList actionEpisodes;
-    if( m_deleteAction == 0 )
-    {
-        m_deleteAction = new QAction(
-            KIcon( "edit-delete" ),
-            i18n( "&Delete Downloaded Episode" ),
-            this
-        );
-        m_deleteAction->setProperty( "popupdropper_svg_id", "delete" );
-        connect( m_deleteAction, SIGNAL( triggered() ), SLOT( slotDeleteDownloadedEpisodes() ) );
-    }
-
-
-    actionEpisodes.clear();
-    if( m_writeTagsAction == 0 )
-    {
-        m_writeTagsAction = new QAction(
-            KIcon( "media-track-edit-amarok" ),
-            i18n( "&Write Feed Information to File" ),
-            this
-        );
-        m_writeTagsAction->setProperty( "popupdropper_svg_id", "edit" );
-        connect( m_writeTagsAction, SIGNAL( triggered() ), SLOT( slotWriteTagsToFiles() ) );
-    }
-
-    bool hasDownloaded = false;
-    foreach( Podcasts::PodcastEpisodePtr episode, episodes )
-    {
-        Podcasts::SqlPodcastEpisodePtr sqlEpisode
-                = Podcasts::SqlPodcastEpisodePtr::dynamicCast( episode );
-        if( sqlEpisode.isNull() )
-            break;
-
-        if( !sqlEpisode->localUrl().isEmpty() )
-        {
-            hasDownloaded = true;
-            break;
-        }
-    }
-    if( hasDownloaded )
-    {
-        Podcasts::PodcastEpisodeList actionEpisodes = m_deleteAction->data().value<Podcasts::PodcastEpisodeList>();
-        actionEpisodes << episodes;
-        m_deleteAction->setData( QVariant::fromValue( actionEpisodes ) );
-        //these lists are the same anyway
-        m_writeTagsAction->setData( QVariant::fromValue( actionEpisodes ) );
-        actions << m_deleteAction;
-        actions << m_writeTagsAction;
-    }
-    else
-    {
-        if( m_downloadAction == 0 )
-        {
-            m_downloadAction = new QAction(
-                KIcon( "go-down" ),
-                i18n( "&Download Episode" ),
-                this
-            );
-            m_downloadAction->setProperty( "popupdropper_svg_id", "download" );
-            connect( m_downloadAction, SIGNAL( triggered() ), SLOT( slotDownloadEpisodes() ) );
-        }
-        else
-        {
-            actionEpisodes = m_downloadAction->data().value<Podcasts::PodcastEpisodeList>();
-        }
-        actionEpisodes << episodes;
-        m_downloadAction->setData( QVariant::fromValue( actionEpisodes ) );
-        actions << m_downloadAction;
-    }
-
-    return actions;
-}
-
-QList<QAction *>
-SqlPodcastProvider::channelActions( Podcasts::PodcastChannelList channels )
-{
-    QList< QAction * > actions;
-
-    if( channels.isEmpty() )
-        return actions;
-
-    if( m_configureChannelAction == 0 )
-    {
-        m_configureChannelAction = new QAction(
-            KIcon( "configure" ),
-            i18n( "&Configure" ),
-            this
-        );
-        m_configureChannelAction->setProperty( "popupdropper_svg_id", "configure" );
-        connect( m_configureChannelAction, SIGNAL( triggered() ), SLOT( slotConfigureChannel() ) );
-    }
-
-    //only one playlist can be renamed at a time.
-    if( m_configureChannelAction->data().isNull() )
-        m_configureChannelAction->setData( QVariant::fromValue( channels.first() ) );
-
-    actions << m_configureChannelAction;
-
-    Podcasts::PodcastChannelList actionChannels;
-    if( m_removeAction == 0 )
-    {
-        m_removeAction = new QAction(
-            KIcon( "news-unsubscribe" ),
-            i18n( "&Remove Subscription" ),
-            this
-        );
-        m_removeAction->setProperty( "popupdropper_svg_id", "remove" );
-        connect( m_removeAction, SIGNAL( triggered() ), SLOT( slotRemoveChannels() ) );
-    }
-    else
-    {
-        actionChannels = m_removeAction->data().value<Podcasts::PodcastChannelList>();
-    }
-
-    actionChannels << channels;
-    m_removeAction->setData( QVariant::fromValue( actionChannels ) );
-
-    actions << m_removeAction;
-
-    actionChannels.clear();
-    if( m_updateAction == 0 )
-    {
-        m_updateAction = new QAction(
-            KIcon( "view-refresh-amarok" ),
-            i18n( "&Update Channel" ),
-            this
-        );
-        m_updateAction->setProperty( "popupdropper_svg_id", "update" );
-        connect( m_updateAction, SIGNAL( triggered() ), SLOT( slotUpdateChannels() ) );
-    }
-    else
-    {
-        actionChannels = m_updateAction->data().value<Podcasts::PodcastChannelList>();
-    }
-
-    actionChannels << channels;
-    m_updateAction->setData( QVariant::fromValue( actionChannels ) );
-
-    actions << m_updateAction;
-
-    return actions;
-}
-
 void
 SqlPodcastProvider::deleteDownloadedEpisodes( Podcasts::SqlPodcastEpisodeList &episodes )
 {
@@ -995,6 +834,19 @@ SqlPodcastProvider::slotDownloadEpisodes()
         downloadEpisode( episode );
 }
 
+void
+SqlPodcastProvider::slotSetKeep()
+{
+    QAction *action = qobject_cast<QAction *>( QObject::sender() );
+    if( action == 0 )
+        return;
+
+    Podcasts::SqlPodcastEpisodeList episodes = action->data().value<Podcasts::SqlPodcastEpisodeList>();
+
+    foreach( Podcasts::SqlPodcastEpisodePtr episode, episodes )
+        episode->setKeep( action->isChecked() );
+}
+
 QPair<bool, bool>
 SqlPodcastProvider::confirmUnsubscribe( Podcasts::SqlPodcastChannelPtr channel )
 {
@@ -1107,6 +959,8 @@ SqlPodcastProvider::deleteDownloadedEpisode( Podcasts::SqlPodcastEpisodePtr epis
     KIO::del( episode->localUrl(), KIO::HideProgressInfo );
 
     episode->setLocalUrl( KUrl() );
+
+    emit episodeDeleted( Podcasts::PodcastEpisodePtr::dynamicCast( episode ) );
 }
 
 Podcasts::SqlPodcastChannelPtr
@@ -1141,12 +995,12 @@ SqlPodcastProvider::completePodcastDownloads()
         m_completedDownloads = 0;
         foreach( KJob *job, m_downloadJobMap.keys() )
         {
-            connect( job, SIGNAL( percent( KJob *, unsigned long ) ),
-                    SLOT( slotDownloadProgress( KJob *, unsigned long ) )
+            connect( job, SIGNAL(percent(KJob*,ulong)),
+                    SLOT(slotDownloadProgress(KJob*,ulong))
                    );
         }
-        connect( this, SIGNAL( totalPodcastDownloadProgress( int ) ),
-                 progressDialog.progressBar(), SLOT( setValue( int ) ) );
+        connect( this, SIGNAL(totalPodcastDownloadProgress(int)),
+                 progressDialog.progressBar(), SLOT(setValue(int)) );
         int result = progressDialog.exec();
         if( result == QDialog::Rejected )
         {
@@ -1192,12 +1046,12 @@ SqlPodcastProvider::updateSqlChannel( Podcasts::SqlPodcastChannelPtr channel )
 
     PodcastReader *podcastReader = new PodcastReader( this );
 
-    connect( podcastReader, SIGNAL( finished( PodcastReader * ) ),
-             SLOT( slotReadResult( PodcastReader * ) ) );
-    connect( podcastReader, SIGNAL( statusBarSorryMessage( const QString & ) ),
-            this, SLOT( slotStatusBarSorryMessage( const QString & ) ) );
-    connect( podcastReader, SIGNAL( statusBarNewProgressOperation( KIO::TransferJob *, const QString &, Podcasts::PodcastReader* ) ),
-                this, SLOT( slotStatusBarNewProgressOperation( KIO::TransferJob *, const QString &, Podcasts::PodcastReader* ) ) );
+    connect( podcastReader, SIGNAL(finished(PodcastReader*)),
+             SLOT(slotReadResult(PodcastReader*)) );
+    connect( podcastReader, SIGNAL(statusBarSorryMessage(QString)),
+            this, SLOT(slotStatusBarSorryMessage(QString)) );
+    connect( podcastReader, SIGNAL(statusBarNewProgressOperation(KIO::TransferJob*,QString,Podcasts::PodcastReader*)),
+                this, SLOT(slotStatusBarNewProgressOperation(KIO::TransferJob*,QString,Podcasts::PodcastReader*)) );
     
     m_updatingChannels++;
     podcastReader->update( Podcasts::PodcastChannelPtr::dynamicCast( channel ) );
@@ -1268,7 +1122,7 @@ SqlPodcastProvider::downloadEpisode( Podcasts::SqlPodcastEpisodePtr sqlEpisode )
         return;
     }
 
-    foreach( struct PodcastEpisodeDownload download, m_downloadJobMap.values() )
+    foreach( struct PodcastEpisodeDownload download, m_downloadJobMap )
     {
         if( download.episode == sqlEpisode )
         {
@@ -1327,18 +1181,18 @@ SqlPodcastProvider::downloadEpisode( Podcasts::SqlPodcastEpisodePtr sqlEpisode )
                                                         : i18n( "Downloading Podcast \"%1\""
                                                                 , sqlEpisode->title() ),
                                                         transferJob,
-                                                        SLOT( kill() )
+                                                        SLOT(kill())
                                                       );
 
-    connect( transferJob, SIGNAL( data( KIO::Job *, const QByteArray & ) ),
-             SLOT( addData( KIO::Job *, const QByteArray & ) ) );
+    connect( transferJob, SIGNAL(data(KIO::Job*,QByteArray)),
+             SLOT(addData(KIO::Job*,QByteArray)) );
     //need to connect to finished instead of result because it's always emitted.
     //We need to cleanup after a download is cancled regardless of the argument in
     //KJob::kill()
-    connect( transferJob, SIGNAL( finished( KJob * ) ),
-             SLOT( downloadResult( KJob * ) ) );
-    connect( transferJob, SIGNAL( redirection( KIO::Job *, const KUrl& ) ),
-             SLOT( redirected( KIO::Job *, const KUrl& ) ) );
+    connect( transferJob, SIGNAL(finished(KJob*)),
+             SLOT(downloadResult(KJob*)) );
+    connect( transferJob, SIGNAL(redirection(KIO::Job*,KUrl)),
+             SLOT(redirected(KIO::Job*,KUrl)) );
 }
 
 void
@@ -1553,6 +1407,8 @@ SqlPodcastProvider::downloadResult( KJob *job )
             if( sqlChannel->writeTags() )
                 sqlEpisode->writeTagsToFile();
             //TODO: force a redraw of the view so the icon can be updated in the PlaylistBrowser
+
+            emit episodeDownloaded( Podcasts::PodcastEpisodePtr::dynamicCast( sqlEpisode ) );
         }
         else
         {
@@ -1582,7 +1438,7 @@ SqlPodcastProvider::redirected( KIO::Job *job, const KUrl &redirectedUrl )
 void
 SqlPodcastProvider::createTables() const
 {
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 
@@ -1615,7 +1471,8 @@ SqlPodcastProvider::createTables() const
                                 ",pubdate "  + sqlStorage->textColumnType() +
                                 ",duration INTEGER"
                                 ",filesize INTEGER"
-                                ",isnew BOOL ) ENGINE = MyISAM;" ) );
+                                ",isnew BOOL"
+                                ",iskeep BOOL) ENGINE = MyISAM;" ) );
 
     sqlStorage->query( "CREATE FULLTEXT INDEX url_podchannel ON podcastchannels( url );" );
     sqlStorage->query( "CREATE FULLTEXT INDEX url_podepisode ON podcastepisodes( url );" );
@@ -1629,7 +1486,7 @@ SqlPodcastProvider::updateDatabase( int fromVersion, int toVersion )
     debug() << QString( "Updating Podcast tables from version %1 to version %2" )
             .arg( fromVersion ).arg( toVersion );
 
-    SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 #define escape(x) sqlStorage->escape(x)
@@ -1676,7 +1533,8 @@ SqlPodcastProvider::updateDatabase( int fromVersion, int toVersion )
                                     ",pubdate "  + sqlStorage->textColumnType() +
                                     ",duration INTEGER"
                                     ",filesize INTEGER"
-                                    ",isnew BOOL ) ENGINE = MyISAM;" ) );
+                                    ",isnew BOOL"
+                                    ",iskeep BOOL) ENGINE = MyISAM;" ) );
 
         sqlStorage->query( "INSERT INTO podcastchannels_temp SELECT * FROM podcastchannels;" );
         sqlStorage->query( "INSERT INTO podcastepisodes_temp SELECT * FROM podcastepisodes;" );
@@ -1704,7 +1562,7 @@ SqlPodcastProvider::updateDatabase( int fromVersion, int toVersion )
         sqlStorage->query( setWriteTagsQuery );
     }
 
-    if(fromVersion < 5 && toVersion == 5 )
+    if( fromVersion < 5 && toVersion == 5 )
     {
         QString updateChannelQuery = QString ( "ALTER TABLE podcastchannels"
                                                " ADD filenamelayout VARCHAR(1024);" );
@@ -1713,6 +1571,14 @@ SqlPodcastProvider::updateDatabase( int fromVersion, int toVersion )
         sqlStorage->query( setWriteTagsQuery );
     }
 
+    if( fromVersion < 6 && toVersion == 6 )
+    {
+        QString updateEpisodeQuery = QString ( "ALTER TABLE podcastepisodes"
+                                               " ADD iskeep BOOL;" );
+        sqlStorage->query( updateEpisodeQuery );
+        QString setIsKeepQuery = QString( "UPDATE podcastepisodes SET iskeep=FALSE;" );
+        sqlStorage->query( setIsKeepQuery );
+    }
 
     QString updateAdmin = QString( "UPDATE admin SET version=%1 WHERE component='%2';" );
     sqlStorage->query( updateAdmin.arg( toVersion ).arg( escape( key ) ) );
@@ -1727,12 +1593,12 @@ SqlPodcastProvider::fetchImage( SqlPodcastChannelPtr channel )
     {
         m_podcastImageFetcher = new PodcastImageFetcher();
         connect( m_podcastImageFetcher,
-                 SIGNAL( imageReady( Podcasts::PodcastChannelPtr, QImage ) ),
-                 SLOT( channelImageReady( Podcasts::PodcastChannelPtr, QImage ) )
+                 SIGNAL(imageReady(Podcasts::PodcastChannelPtr,QImage)),
+                 SLOT(channelImageReady(Podcasts::PodcastChannelPtr,QImage))
                );
         connect( m_podcastImageFetcher,
-                 SIGNAL( done( PodcastImageFetcher * ) ),
-                 SLOT( podcastImageFetcherDone( PodcastImageFetcher * ) )
+                 SIGNAL(done(PodcastImageFetcher*)),
+                 SLOT(podcastImageFetcherDone(PodcastImageFetcher*))
                );
     }
 
