@@ -24,11 +24,13 @@
 #include "core/collections/Collection.h"
 #include "core/collections/CollectionLocationDelegate.h"
 #include "core/collections/QueryMaker.h"
+#include "core/meta/Meta.h"
 #include "core/support/Components.h"
 #include "core/support/Debug.h"
 #include "core/transcoding/TranscodingConfiguration.h"
 #include "core/transcoding/TranscodingController.h"
 
+#include <QDir>
 #include <QTimer>
 
 using namespace Collections;
@@ -138,8 +140,8 @@ CollectionLocation::prepareCopy( Collections::QueryMaker *qm, CollectionLocation
     m_destination = destination;
     m_removeSources = false;
     m_isRemoveAction = false;
-    connect( qm, SIGNAL( newResultReady( Meta::TrackList ) ), SLOT( resultReady( Meta::TrackList ) ) );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ) );
+    connect( qm, SIGNAL(newResultReady(Meta::TrackList)), SLOT(resultReady(Meta::TrackList)) );
+    connect( qm, SIGNAL(queryDone()), SLOT(queryDone()) );
     qm->setQueryType( Collections::QueryMaker::Track );
     qm->run();
 }
@@ -186,8 +188,8 @@ CollectionLocation::prepareMove( Collections::QueryMaker *qm, CollectionLocation
     m_destination = destination;
     m_isRemoveAction = false;
     m_removeSources = true;
-    connect( qm, SIGNAL( newResultReady( Meta::TrackList ) ), SLOT( resultReady( Meta::TrackList ) ) );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ) );
+    connect( qm, SIGNAL(newResultReady(Meta::TrackList)), SLOT(resultReady(Meta::TrackList)) );
+    connect( qm, SIGNAL(queryDone()), SLOT(queryDone()) );
     qm->setQueryType( Collections::QueryMaker::Track );
     qm->run();
 }
@@ -222,8 +224,8 @@ CollectionLocation::prepareRemove( Collections::QueryMaker *qm )
     m_isRemoveAction = true;
     m_removeSources = false;
 
-    connect( qm, SIGNAL( newResultReady( Meta::TrackList ) ), SLOT( resultReady( Meta::TrackList ) ) );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ) );
+    connect( qm, SIGNAL(newResultReady(Meta::TrackList)), SLOT(resultReady(Meta::TrackList)) );
+    connect( qm, SIGNAL(queryDone()), SLOT(queryDone()) );
     qm->setQueryType( Collections::QueryMaker::Track );
     qm->run();
 }
@@ -301,7 +303,7 @@ CollectionLocation::getDestinationTranscodingConfig()
     Collection *destCollection = destination() ? destination()->collection() : 0;
     if( !destCollection )
         return configuration;
-    if( !destCollection->hasCapabilityInterface( Capabilities::Capability::Transcode ) )
+    if( !destCollection->has<Capabilities::TranscodeCapability>() )
         return configuration;
     QScopedPointer<Capabilities::TranscodeCapability> tc(
         destCollection->create<Capabilities::TranscodeCapability>() );
@@ -313,12 +315,6 @@ CollectionLocation::getDestinationTranscodingConfig()
     if( tcC )
         availableEncoders = tcC->availableEncoders();
 
-    if( availableEncoders.isEmpty() )
-    {
-        debug() << "FFmpeg is not installed or does not support any of the required formats.";
-        return configuration;
-    }
-
     Transcoding::Configuration saved = tc->savedConfiguration();
     if( saved.isValid() && ( saved.isJustCopy() || availableEncoders.contains( saved.encoder() ) ) )
         return saved;
@@ -332,9 +328,16 @@ CollectionLocation::getDestinationTranscodingConfig()
     if( collection() && collection() == destination()->collection() )
         operation = CollectionLocationDelegate::Move; // organizing
     configuration = delegate->transcode( tc->playableFileTypes(), &saveConfiguration,
-                                         operation, destCollection->prettyName() );
-    if( configuration.isValid() && saveConfiguration )
-        tc->setSavedConfiguration( configuration );
+                                         operation, destCollection->prettyName(),
+                                         saved );
+    if( configuration.isValid() )
+    {
+        if( saveConfiguration )
+            tc->setSavedConfiguration( configuration );
+        else //save the trackSelection value for restore anyway
+            tc->setSavedConfiguration( Transcoding::Configuration( Transcoding::INVALID,
+                                        configuration.trackSelection() ) );
+    }
     return configuration; // may be invalid, it means user has hit cancel
 }
 
@@ -547,7 +550,7 @@ CollectionLocation::slotFinishRemove()
 
     if( !dirsToRemove.isEmpty() && delegate->deleteEmptyDirs( this ) )
     {
-        debug() << "Removeing empty directories";
+        debug() << "Removing empty directories";
         dirsToRemove.removeDuplicates();
         dirsToRemove.sort();
         while( !dirsToRemove.isEmpty() )
@@ -617,25 +620,25 @@ CollectionLocation::queryDone()
 void
 CollectionLocation::setupConnections()
 {
-    connect( this, SIGNAL( prepareOperation( Meta::TrackList, bool, const Transcoding::Configuration & ) ),
-             m_destination, SLOT( slotPrepareOperation( Meta::TrackList, bool, const Transcoding::Configuration & ) ) );
-    connect( m_destination, SIGNAL( operationPrepared() ), SLOT( slotOperationPrepared() ) );
-    connect( this, SIGNAL( startCopy( QMap<Meta::TrackPtr, KUrl>, const Transcoding::Configuration & ) ),
-             m_destination, SLOT( slotStartCopy( QMap<Meta::TrackPtr, KUrl>, const Transcoding::Configuration & ) ) );
-    connect( m_destination, SIGNAL( finishCopy() ),
-             this, SLOT( slotFinishCopy() ) );
-    connect( this, SIGNAL( aborted() ), SLOT( slotAborted() ) );
-    connect( m_destination, SIGNAL( aborted() ), SLOT( slotAborted() ) );
+    connect( this, SIGNAL(prepareOperation(Meta::TrackList,bool,Transcoding::Configuration)),
+             m_destination, SLOT(slotPrepareOperation(Meta::TrackList,bool,Transcoding::Configuration)) );
+    connect( m_destination, SIGNAL(operationPrepared()), SLOT(slotOperationPrepared()) );
+    connect( this, SIGNAL(startCopy(QMap<Meta::TrackPtr,KUrl>,Transcoding::Configuration)),
+             m_destination, SLOT(slotStartCopy(QMap<Meta::TrackPtr,KUrl>,Transcoding::Configuration)) );
+    connect( m_destination, SIGNAL(finishCopy()),
+             this, SLOT(slotFinishCopy()) );
+    connect( this, SIGNAL(aborted()), SLOT(slotAborted()) );
+    connect( m_destination, SIGNAL(aborted()), SLOT(slotAborted()) );
 }
 
 void
 CollectionLocation::setupRemoveConnections()
 {
-    connect( this, SIGNAL( aborted() ), SLOT( slotAborted() ) );
-    connect( this, SIGNAL( startRemove() ),
-             this, SLOT( slotStartRemove() ) );
-    connect( this, SIGNAL( finishRemove() ),
-             this, SLOT( slotFinishRemove() ) );
+    connect( this, SIGNAL(aborted()), SLOT(slotAborted()) );
+    connect( this, SIGNAL(startRemove()),
+             this, SLOT(slotStartRemove()) );
+    connect( this, SIGNAL(finishRemove()),
+             this, SLOT(slotFinishRemove()) );
 }
 
 void
