@@ -19,8 +19,8 @@
 #ifndef AMAROK_COLLECTIONLOCATION_H
 #define AMAROK_COLLECTIONLOCATION_H
 
-#include "shared/amarok_export.h"
-#include "core/meta/Meta.h"
+#include "core/amarokcore_export.h"
+#include "core/meta/forward_declarations.h"
 #include "core/transcoding/TranscodingConfiguration.h"
 
 #include <QList>
@@ -29,10 +29,11 @@
 #include <KUrl>
 
 namespace Collections {
+    class Collection;
     class QueryMaker;
 
 /**
-    This base class defines the the methods necessary to allow the copying and moving
+    This base class defines the methods necessary to allow the copying and moving
     of tracks between different collections in a generic way.
 
     This class should be used as follows in client code:
@@ -397,7 +398,7 @@ class AMAROK_CORE_EXPORT CollectionLocation : public QObject
         void setSource( CollectionLocation *source );
 
         //only used in the source CollectionLocation
-        CollectionLocation * m_destination;
+        CollectionLocation *m_destination;
         //only used in destination CollectionLocation
         CollectionLocation *m_source;
 
