@@ -25,40 +25,44 @@
 
 namespace Plugins {
 
+/** Baseclass for Amarok plugins.
+ *
+ *  This class is subclassed for the different type of Amarok plugins.
+ *  - CollectionFactory
+ *  - ServiceFactory
+ *  - ImportFactory
+ *  - StorageFactory
+ *
+ *  Plugins need to set the m_info variables.
+ *  Plugins also need to set a valid category in their Desktop file.
+ */
 class AMAROK_CORE_EXPORT PluginFactory : public QObject
 {
     Q_OBJECT
-    Q_ENUMS( Type )
     Q_PROPERTY( KPluginInfo info READ info )
-    Q_PROPERTY( Type pluginType READ pluginType )
 
 public:
-    enum Type { Unknown, Collection, Service };
     PluginFactory( QObject *parent, const QVariantList &args );
     virtual ~PluginFactory() = 0;
 
-    /**
-     * Initialize the service plugin of this type.
+    /** Initialize the service plugin of this type.
+     *
      * Reimplemented by subclasses, which must set
      * m_initialized = true when the function has finished.
+     *
+     * This function is called by the PluginManager after
+     * setting the plugin in the different sub-plugin managers
      * */
     virtual void init() = 0;
 
-    /**
-     * Indicates whether or not the plugin has been initialized.
-     * @return Initialized or not
-     */
-    bool isInitialized() const;
-
     KPluginInfo info() const;
-    Type pluginType() const;
 
 protected:
-    bool m_initialized;
-    KPluginInfo m_info;
-    Type m_type;
+    bool m_initialized; ///< set after init was called
+    KPluginInfo m_info; ///< contains the KPluginInfo returned by info(). Should be set in the constructor
 };
 
 } // namespace Plugins
 
+
 #endif /* AMAROK_PLUGINFACTORY_H */
