@@ -17,9 +17,10 @@
 #ifndef AMAROK_COVERFETCHUNIT_H
 #define AMAROK_COVERFETCHUNIT_H
 
-#include "core/meta/Meta.h"
+#include "core/meta/Meta.h" // for AlbumPtr default argument
 
 #include <KSharedPtr>
+#include <KUrl>
 
 #include <QStringList>
 #include <QXmlStreamReader>
@@ -47,11 +48,10 @@ namespace CoverFetch
         Discogs,        //!< Use Discogs as provider for cover images
         Google,         //!< Use Google image search as provider
         LastFm,         //!< Use Last.fm as provider for cover images
-        Yahoo           //!< Use Yahoo! BOSS image search as provider
     };
 
-    typedef QHash< QString, QString > Metadata;
-    typedef QHash< KUrl, Metadata > Urls;
+    typedef QHash<QString, QString> Metadata;
+    typedef QHash<KUrl, Metadata> Urls;
 }
 
 /**
@@ -102,7 +102,7 @@ public:
     CoverFetchPayload( const Meta::AlbumPtr album, enum Type type, const CoverFetch::Source src );
     virtual ~CoverFetchPayload();
 
-    Meta::AlbumPtr album() const { return m_album; }
+    Meta::AlbumPtr album() const;
     CoverFetch::Source source() const;
     enum Type type() const;
     const CoverFetch::Urls &urls() const;
@@ -154,7 +154,7 @@ public:
     explicit CoverFetchSearchPayload( const QString &query = QString(),
                                       const CoverFetch::Source src = CoverFetch::LastFm,
                                       unsigned int page = 0,
-                                      Meta::AlbumPtr album = Meta::AlbumPtr(0) );
+                                      Meta::AlbumPtr album = Meta::AlbumPtr() );
     ~CoverFetchSearchPayload();
 
     QString query() const;
@@ -212,9 +212,6 @@ private:
     /// prepare urls from xml provided by Last.fm
     void prepareLastFmUrls( QXmlStreamReader &xml );
 
-    /// prepare urls from xml provided by Yahoo! Image Search
-    void prepareYahooUrls( QXmlStreamReader &xml );
-
     /// prepare urls from xml provided by Google Image Search
     void prepareGoogleUrls();
 
