@@ -47,6 +47,13 @@ void FetchCoverAction::init()
     setText( i18np("Fetch Cover", "Fetch Covers", m_albums.count()) );
     setIcon( KIcon("insert-image") );
     setToolTip( i18np("Fetch the artwork for this album", "Fetch artwork for %1 albums", m_albums.count()) );
+
+    bool enabled = !m_albums.isEmpty();
+    foreach( Meta::AlbumPtr album, m_albums )
+    {
+        enabled &= album->canUpdateImage();
+    }
+    setEnabled( enabled );
 }
 
 void FetchCoverAction::slotTriggered()
@@ -141,7 +148,7 @@ void SetCustomCoverAction::init()
 void
 SetCustomCoverAction::slotTriggered()
 {
-    if( !m_albums.first() || m_albums.first()->tracks().isEmpty() )
+    if( m_albums.isEmpty() || m_albums.first()->tracks().isEmpty() )
         return;
 
     const QString& startPath = m_albums.first()->tracks().first()->playableUrl().directory();
@@ -155,15 +162,6 @@ SetCustomCoverAction::slotTriggered()
     dlg.setCaption( i18n("Select Cover Image File") );
     dlg.setInlinePreviewShown( true );
 
-    // TODO: auto error handling was disabled to allow entering an http address
-    // in the dialog (commit 8b192500da7d31e314ce04759233d78fe6ce57b9). Now with
-    // kde 4.4 and 4.3.3 this is no longer necessary (BR 197945).
-
-    // trueg: there could be another implementation of the file module which does not use KFileWidget!
-    if ( KFileWidget *fileWidget = dynamic_cast<KFileWidget*>( dlg.fileWidget() ) )
-        if ( KDirLister *dirLister = fileWidget->dirOperator()->dirLister() )
-            dirLister->setAutoErrorHandlingEnabled( false, qobject_cast<QWidget*>( parent() ) );
-
     dlg.exec();
     KUrl file = dlg.selectedUrl();
 
@@ -174,7 +172,6 @@ SetCustomCoverAction::slotTriggered()
         if( file.isLocalFile() )
         {
             image.load( file.path() );
-
         }
         else
         {
