@@ -17,25 +17,29 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "CoverManager"
+
 #include "CoverManager.h"
-#include "CoverViewDialog.h"
 
-#include "Amarok.h"
 #include "amarokconfig.h"
-#include "collection/Collection.h"
-#include "CollectionManager.h"
-#include "CompoundProgressBar.h"
-#include "Debug.h"
-#include "meta/capabilities/CurrentTrackActionsCapability.h"
-#include "meta/Meta.h"
-#include "QueryMaker.h"
-#include <config-amarok.h>
-#include "PixmapViewer.h"
-#include "playlist/PlaylistModelStack.h"
+#include <config.h>
+#include "core/capabilities/ActionsCapability.h"
+#include "core/collections/Collection.h"
+#include "core/collections/QueryMaker.h"
+#include "core/meta/Meta.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "covermanager/CoverFetchingActions.h"
+#include "covermanager/CoverViewDialog.h"
+#include "playlist/PlaylistController.h"
+#include "statusbar/CompoundProgressBar.h"
 #include "widgets/LineEdit.h"
+#include "widgets/PixmapViewer.h"
 
+#include <KApplication>
 #include <KIO/NetAccess>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMenu>    //showCoverMenu()
 #include <KPushButton>
 #include <KSqueezedTextLabel> //status label
@@ -46,14 +50,13 @@
 #include <QAction>
 #include <QDesktopWidget>
 #include <QProgressBar>
+#include <QSplitter>
 #include <QStringList>
 #include <QTimer>    //search filter timer
 #include <QToolButton>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 
-#define DEBUG_PREFIX "CoverManager"
-
 static QString artistToSelectInInitFunction;
 CoverManager *CoverManager::s_instance = 0;
 
@@ -81,14 +84,13 @@ class ArtistItem : public QTreeWidgetItem
 };
 
 
-CoverManager::CoverManager()
-        : QSplitter( 0 )
+CoverManager::CoverManager( QWidget *parent )
+        : KDialog( parent )
         , m_currentView( AllAlbums )
         , m_timer( new QTimer( this ) )    //search filter timer
         , m_fetchingCovers( false )
         , m_coversFetched( 0 )
         , m_coverErrors( 0 )
-        , m_isClosing( false )
         , m_isLoadingCancelled( false )
 {
     DEBUG_BLOCK
@@ -99,12 +101,18 @@ CoverManager::CoverManager()
 
     // Sets caption and icon correctly (needed e.g. for GNOME)
     kapp->setTopWidget( this );
-    setWindowTitle( KDialog::makeStandardCaption( i18n("Cover Manager") ) );
+    setButtons( 0 );
+    setCaption( i18n("Cover Manager") );
     setAttribute( Qt::WA_DeleteOnClose );
-    setContentsMargins( 4, 4, 4, 4 );
+
+    connect( this, SIGNAL(hidden()), SLOT(delayedDestruct()) );
+    connect( this, SIGNAL(closeClicked()), SLOT(delayedDestruct()) );
+
+    m_splitter = new QSplitter( this );
+    setMainWidget( m_splitter );
 
     //artist listview
-    m_artistView = new QTreeWidget( this );
+    m_artistView = new QTreeWidget( m_splitter );
     m_artistView->setHeaderLabel( i18n( "Albums By" ) );
     m_artistView->setSortingEnabled( false );
     m_artistView->setTextElideMode( Qt::ElideRight );
@@ -119,26 +127,25 @@ CoverManager::CoverManager()
     item->setIcon(0, SmallIcon( "media-optical-audio-amarok" ) );
     m_items.append( item );
 
-    Amarok::Collection *coll = CollectionManager::instance()->primaryCollection();
-    QueryMaker *qm = coll->queryMaker();
+    Collections::Collection *coll = CollectionManager::instance()->primaryCollection();
+    Collections::QueryMaker *qm = coll->queryMaker();
     qm->setAutoDelete( true );
-    qm->setQueryType( QueryMaker::Artist );
-    qm->setAlbumQueryMode( QueryMaker::OnlyNormalAlbums );
+    qm->setQueryType( Collections::QueryMaker::Artist );
+    qm->setAlbumQueryMode( Collections::QueryMaker::OnlyNormalAlbums );
     qm->orderBy( Meta::valArtist );
 
-    connect( qm, SIGNAL( newResultReady( QString, Meta::ArtistList ) ),
-             this, SLOT( slotArtistQueryResult( QString, Meta::ArtistList ) ) );
+    connect( qm, SIGNAL(newResultReady(Meta::ArtistList)),
+             this, SLOT(slotArtistQueryResult(Meta::ArtistList)) );
 
-    connect( qm, SIGNAL( queryDone() ), this, SLOT( slotContinueConstruction() ) );
+    connect( qm, SIGNAL(queryDone()), this, SLOT(slotContinueConstruction()) );
 
     qm->run();
 }
 
 void
-CoverManager::slotArtistQueryResult( QString collectionId, Meta::ArtistList artists ) //SLOT
+CoverManager::slotArtistQueryResult( Meta::ArtistList artists ) //SLOT
 {
     DEBUG_BLOCK
-    Q_UNUSED( collectionId );
     foreach( Meta::ArtistPtr artist, artists )
         m_artistList << artist;
 }
@@ -155,7 +162,7 @@ CoverManager::slotContinueConstruction() //SLOT
     }
     m_artistView->insertTopLevelItems( 0, m_items );
 
-    KVBox *vbox = new KVBox( this );
+    KVBox *vbox = new KVBox( m_splitter );
     KHBox *hbox = new KHBox( vbox );
 
     vbox->setSpacing( 4 );
@@ -176,9 +183,9 @@ CoverManager::slotContinueConstruction() //SLOT
     m_viewButton = new KPushButton( hbox );
 
     m_viewMenu = new KMenu( m_viewButton );
-    m_selectAllAlbums          = m_viewMenu->addAction( i18n("All Albums"),           this, SLOT( slotShowAllAlbums() ) );
-    m_selectAlbumsWithCover    = m_viewMenu->addAction( i18n("Albums With Cover"),    this, SLOT( slotShowAlbumsWithCover() ) );
-    m_selectAlbumsWithoutCover = m_viewMenu->addAction( i18n("Albums Without Cover"), this, SLOT( slotShowAlbumsWithoutCover() ) );
+    m_selectAllAlbums          = m_viewMenu->addAction( i18n("All Albums"),           this, SLOT(slotShowAllAlbums()) );
+    m_selectAlbumsWithCover    = m_viewMenu->addAction( i18n("Albums With Cover"),    this, SLOT(slotShowAlbumsWithCover()) );
+    m_selectAlbumsWithoutCover = m_viewMenu->addAction( i18n("Albums Without Cover"), this, SLOT(slotShowAlbumsWithoutCover()) );
 
     QActionGroup *viewGroup = new QActionGroup( m_viewButton );
     viewGroup->setExclusive( true );
@@ -218,37 +225,12 @@ CoverManager::slotContinueConstruction() //SLOT
     QSize sz = Amarok::config( "Cover Manager" ).readEntry( "Window Size", size );
     resize( sz.width(), sz.height() );
 
-    setStretchFactor( indexOf( m_artistView ), 1 );
-    setStretchFactor( indexOf( vbox ), 4 );
-
-    show();
+    m_splitter->setStretchFactor( m_splitter->indexOf( m_artistView ), 1 );
+    m_splitter->setStretchFactor( m_splitter->indexOf( vbox ), 4 );
 
     m_fetcher = The::coverFetcher();
 
-    QTimer::singleShot( 0, this, SLOT(init()) );
-}
-
-CoverManager::~CoverManager()
-{
-    DEBUG_BLOCK
-
-    Amarok::config( "Cover Manager" ).writeEntry( "Window Size", size() );
-    s_instance = 0;
-
-    m_isClosing = true;
-    m_isLoadingCancelled = true;
-    qDeleteAll( m_coverItems );
-    delete m_coverView;
-    m_coverView = 0;
-}
-
-void
-CoverManager::init()
-{
-    DEBUG_BLOCK
-
     QTreeWidgetItem *item = 0;
-
     int i = 0;
     if ( !artistToSelectInInitFunction.isEmpty() )
     {
@@ -262,21 +244,30 @@ CoverManager::init()
     }
 
     // signals and slots connections
-    connect( m_artistView, SIGNAL(itemSelectionChanged() ),
-                           SLOT( slotArtistSelected() ) );
-    connect( m_coverView,  SIGNAL(itemActivated( QListWidgetItem* )),
-                           SLOT(coverItemClicked( QListWidgetItem* )) );
+    connect( m_artistView, SIGNAL(itemSelectionChanged()),
+                           SLOT(slotArtistSelected()) );
+    connect( m_coverView,  SIGNAL(itemActivated(QListWidgetItem*)),
+                           SLOT(coverItemClicked(QListWidgetItem*)) );
     connect( m_timer,      SIGNAL(timeout()),
                            SLOT(slotSetFilter()) );
-    connect( m_searchEdit, SIGNAL(textChanged( const QString& )),
+    connect( m_searchEdit, SIGNAL(textChanged(QString)),
                            SLOT(slotSetFilterTimeout()) );
 
     if( item == 0 )
         item = m_artistView->invisibleRootItem()->child( 0 );
 
     item->setSelected( true );
+    show();
 }
 
+CoverManager::~CoverManager()
+{
+    Amarok::config( "Cover Manager" ).writeEntry( "Window Size", size() );
+    qDeleteAll( m_coverItems );
+    delete m_coverView;
+    m_coverView = 0;
+    s_instance = 0;
+}
 
 void
 CoverManager::viewCover( Meta::AlbumPtr album, QWidget *parent ) //static
@@ -289,8 +280,6 @@ CoverManager::viewCover( Meta::AlbumPtr album, QWidget *parent ) //static
 void
 CoverManager::metadataChanged( Meta::AlbumPtr album )
 {
-    DEBUG_BLOCK
-
     const QString albumName = album->name();
     foreach( CoverViewItem *item, m_coverItems )
     {
@@ -303,8 +292,6 @@ CoverManager::metadataChanged( Meta::AlbumPtr album )
 void
 CoverManager::fetchMissingCovers() //SLOT
 {
-    DEBUG_BLOCK
-
     m_fetchCovers.clear();
     for( int i = 0, coverCount = m_coverView->count(); i < coverCount; ++i )
     {
@@ -314,6 +301,8 @@ CoverManager::fetchMissingCovers() //SLOT
             m_fetchCovers += coverItem->albumPtr();
     }
 
+    debug() << QString( "Fetching %1 missing covers" ).arg( m_fetchCovers.size() );
+
     ProgressBar *fetchProgressBar = new ProgressBar( this );
     fetchProgressBar->setDescription( i18n( "Fetching" ) );
     fetchProgressBar->setMaximum( m_fetchCovers.size() );
@@ -326,16 +315,15 @@ CoverManager::fetchMissingCovers() //SLOT
     updateStatusBar();
     m_fetchButton->setEnabled( false );
     connect( m_fetcher, SIGNAL(finishedSingle(int)), SLOT(updateFetchingProgress(int)) );
-
 }
 
 void
-CoverManager::showOnce( const QString &artist )
+CoverManager::showOnce( const QString &artist, QWidget* parent )
 {
     if( !s_instance )
     {
         artistToSelectInInitFunction = artist;
-        new CoverManager();
+        new CoverManager( parent );
     }
     else
     {
@@ -357,11 +345,11 @@ CoverManager::slotArtistSelected() //SLOT
     //this can be a bit slow
     QApplication::setOverrideCursor( Qt::WaitCursor );
 
-    Amarok::Collection *coll = CollectionManager::instance()->primaryCollection();
+    Collections::Collection *coll = CollectionManager::instance()->primaryCollection();
 
-    QueryMaker *qm = coll->queryMaker();
+    Collections::QueryMaker *qm = coll->queryMaker();
     qm->setAutoDelete( true );
-    qm->setQueryType( QueryMaker::Album );
+    qm->setQueryType( Collections::QueryMaker::Album );
     qm->orderBy( Meta::valAlbum );
 
     qm->beginOr();
@@ -381,25 +369,20 @@ CoverManager::slotArtistSelected() //SLOT
     qm->excludeFilter( Meta::valAlbum, QString(), true, true );
     qm->endAndOr();
 
-    m_albumList.clear();
+    connect( qm, SIGNAL(newResultReady(Meta::AlbumList)),
+             this, SLOT(slotAlbumQueryResult(Meta::AlbumList)) );
 
-    connect( qm, SIGNAL( newResultReady( QString, Meta::AlbumList ) ),
-             this, SLOT( slotAlbumQueryResult( QString, Meta::AlbumList ) ) );
-
-    connect( qm, SIGNAL( queryDone() ), this, SLOT( slotArtistQueryDone() ) );
+    connect( qm, SIGNAL(queryDone()), this, SLOT(slotArtistQueryDone()) );
 
     qm->run();
 }
 
 void
-CoverManager::slotAlbumQueryResult( QString collectionId, Meta::AlbumList albums ) //SLOT
+CoverManager::slotAlbumQueryResult( Meta::AlbumList albums ) //SLOT
 {
-    DEBUG_BLOCK
-    Q_UNUSED( collectionId );
-    m_albumList += albums;
+    m_albumList = albums;
 }
 
-
 void
 CoverManager::slotAlbumFilterTriggered( QAction *action ) //SLOT
 {
@@ -424,15 +407,21 @@ CoverManager::slotArtistQueryDone() //SLOT
     m_progress->show();
 
     uint x = 0;
+    debug() << "Loading covers for selected artist(s)";
 
     //the process events calls below causes massive flickering in the m_albumList
     //so we hide this view and only show it when all items has been inserted. This
     //also provides quite a massive speed improvement when loading covers.
     m_coverView->hide();
     m_coverViewSpacer->show();
-    foreach( Meta::AlbumPtr album, m_albumList )
+    foreach( const Meta::AlbumPtr &album, m_albumList )
     {
-        kapp->processEvents();
+        kapp->processEvents( QEventLoop::ExcludeSocketNotifiers );
+        if( isHidden() )
+        {
+            m_progress->endProgressOperation( m_coverView );
+            return;
+        }
         /*
          * Loading is stopped if cancelled by the user, or the number of albums
          * has changed. The latter occurs when the artist selection changes.
@@ -451,12 +440,8 @@ CoverManager::slotArtistQueryDone() //SLOT
             m_progress->setProgress( m_coverView, x );
         }
     }
-
     m_progress->endProgressOperation( m_coverView );
 
-    if( m_isClosing )
-        return;
-
     // makes sure View is retained when artist selection changes
     changeView( m_currentView, true );
 
@@ -528,15 +513,13 @@ CoverManager::slotSetFilterTimeout() //SLOT
 void
 CoverManager::changeView( CoverManager::View id, bool force ) //SLOT
 {
-    DEBUG_BLOCK
-
     if( !force && m_currentView == id )
         return;
 
     //clear the iconview without deleting items
     m_coverView->clearSelection();
-
-    while ( m_coverView->count() > 0 )
+    int itemsCount = m_coverView->count();
+    while( itemsCount-- > 0 )
        m_coverView->takeItem( 0 );
 
     foreach( QListWidgetItem *item, m_coverItems )
@@ -608,28 +591,6 @@ CoverManager::loadCover( const QString &artist, const QString &album )
     }
 }
 
-void
-CoverManager::playSelectedAlbums()
-{
-    Amarok::Collection *coll = CollectionManager::instance()->primaryCollection();
-    QueryMaker *qm = coll->queryMaker();
-    foreach( CoverViewItem *item, selectedItems() )
-    {
-        qm->addMatch( item->albumPtr() );
-    }
-    The::playlistController()->insertOptioned( qm, Playlist::AppendAndPlay );
-}
-
-QList<CoverViewItem*>
-CoverManager::selectedItems()
-{
-    QList<CoverViewItem*> selectedItems;
-    foreach( QListWidgetItem *item, m_coverView->selectedItems() )
-        selectedItems.append( static_cast<CoverViewItem*>(item) );
-
-    return selectedItems;
-}
-
 void
 CoverManager::progressAllDone()
 {
@@ -659,7 +620,7 @@ CoverManager::updateStatusBar()
             m_progress->endProgressOperation( m_fetcher );
 
             disconnect( m_fetcher, SIGNAL(finishedSingle(int)), this, SLOT(updateFetchingProgress(int)) );
-            QTimer::singleShot( 2000, this, SLOT( updateStatusBar() ) );
+            QTimer::singleShot( 2000, this, SLOT(updateStatusBar()) );
         }
 
         if( m_fetchCovers.size() == 1 )
@@ -729,12 +690,13 @@ CoverManager::updateStatusBar()
                     Amarok::manipulateThe( artist, false );
                     artists.append( artist );
                 }
-                text += i18n( " by " ) + artists.join( ", " );
+                text = i18n( "%1 by %2", text, artists.join( i18nc("Separator for artists", ", ")) );
             }
         }
 
         if( missingCounter )
-            text += i18n(" - ( <b>%1</b> without cover )", missingCounter );
+            text = i18np("%2 - ( <b>1</b> without cover )", "%2 - ( <b>%1</b> without cover )",
+                         missingCounter, text );
 
         m_fetchButton->setEnabled( missingCounter );
     }
@@ -770,64 +732,84 @@ CoverView::CoverView( QWidget *parent, const char *name, Qt::WFlags f )
     setGridSize( QSize(120, 160) );
     setTextElideMode( Qt::ElideRight );
     setContextMenuPolicy( Qt::DefaultContextMenu );
+    setMouseTracking( true ); // required for setting status text when itemEntered signal is emitted
 
-    connect( this, SIGNAL( itemEntered( QListWidgetItem * ) ), SLOT( setStatusText( QListWidgetItem * ) ) );
-    connect( this, SIGNAL( viewportEntered() ), CoverManager::instance(), SLOT( updateStatusBar() ) );
+    connect( this, SIGNAL(itemEntered(QListWidgetItem*)), SLOT(setStatusText(QListWidgetItem*)) );
+    connect( this, SIGNAL(viewportEntered()), CoverManager::instance(), SLOT(updateStatusBar()) );
 }
 
 void
 CoverView::contextMenuEvent( QContextMenuEvent *event )
 {
     QList<QListWidgetItem*> items = selectedItems();
+    const int itemsCount = items.count();
 
-    if( items.count() > 0 )
+    KMenu menu;
+    menu.addTitle( i18n( "Cover Image" ) );
+
+    if( itemsCount == 1 )
     {
-        QMap<QString,QList<QAction *> >  cacs;
-        KMenu menu;
-        menu.addTitle( i18n( "Cover Image" ) );
-
-        for (int x = 0; x < items.count(); ++x)
+        // only one item selected: get all custom actions this album is capable of.
+        CoverViewItem *item = dynamic_cast<CoverViewItem*>( items.first() );
+        QList<QAction *> actions;
+        Meta::AlbumPtr album = item->albumPtr();
+        if( album )
         {
-            CoverViewItem *item = dynamic_cast<CoverViewItem*>(items.value(x));
-            Meta::AlbumPtr album = item->albumPtr();
+            QScopedPointer<Capabilities::ActionsCapability> ac( album->create<Capabilities::ActionsCapability>() );
+            if( ac )
+            {
+                actions = ac->actions();
+                foreach( QAction *action, actions )
+                    menu.addAction( action );
+            }
+        }
+        menu.exec( event->globalPos() );
+    }
+    else if( itemsCount > 1 )
+    {
+        // multiple albums selected: only unset cover and fetch cover actions
+        // make sense here, and perhaps (un)setting compilation flag (TODO).
+
+        Meta::AlbumList unsetAlbums;
+        Meta::AlbumList fetchAlbums;
+
+        foreach( QListWidgetItem *item, items )
+        {
+            CoverViewItem *cvItem = dynamic_cast<CoverViewItem*>(item);
+            Meta::AlbumPtr album = cvItem->albumPtr();
             if( album )
             {
-                Meta::CustomActionsCapability *cac = album->create<Meta::CustomActionsCapability>();
-                if( cac )
+                QScopedPointer<Capabilities::ActionsCapability> ac( album->create<Capabilities::ActionsCapability>() );
+                if( ac )
                 {
-                    QList<QAction *> actions = cac->customActions();
-
+                    QList<QAction *> actions = ac->actions();
                     foreach( QAction *action, actions )
                     {
-                        if( !action->text().isEmpty() )
-                        {
-                            cacs[action->text()].append(action);
-                        }
+                        if( qobject_cast<FetchCoverAction*>(action) )
+                            fetchAlbums << album;
+                        else if( qobject_cast<UnsetCoverAction*>(action) )
+                            unsetAlbums << album;
                     }
                 }
             }
         }
 
-        if( cacs.count() > 0 )
+        if( itemsCount == fetchAlbums.count() )
         {
-            menu.addSeparator();
-
-            foreach ( QList<QAction *> actionList, cacs )
-            {
-                if( actionList.count() == items.count() )
-                {
-                    MultipleAction *maction = new MultipleAction( this, actionList );
-                    menu.addAction( maction );
-                }
-            }
+            FetchCoverAction *fetchAction = new FetchCoverAction( this, fetchAlbums );
+            menu.addAction( fetchAction );
+        }
+        if( itemsCount == unsetAlbums.count() )
+        {
+            UnsetCoverAction *unsetAction = new UnsetCoverAction( this, unsetAlbums );
+            menu.addAction( unsetAction );
         }
         menu.exec( event->globalPos() );
     }
     else
         QListView::contextMenuEvent( event );
-    // TODO
-    // Play, Load and Append to playlist actions
-    // Set custom cover action
+
+    // TODO: Play, Load and Append to playlist actions
 }
 
 void
@@ -837,16 +819,9 @@ CoverView::setStatusText( QListWidgetItem *item )
     if ( !item )
         return;
 
-    bool sampler = false;
-    //compilations have valDummy for artist.  see QueryBuilder::addReturnValue(..) for explanation
-    //FIXME: Don't rely on other independent code, use an sql query
-    if( item->artist().isEmpty() )
-        sampler = true;
-
-    QString tipContent = i18n( "%1 - %2", sampler ? i18n("Various Artists") : item->artist() , item->album() );
-
+    const QString artist = item->albumPtr()->isCompilation() ? i18n( "Various Artists" ) : item->artist();
+    const QString tipContent = i18n( "%1 - %2", artist , item->album() );
     CoverManager::instance()->setStatusText( tipContent );
-
     #undef item
 }
 
@@ -857,7 +832,6 @@ CoverView::setStatusText( QListWidgetItem *item )
 CoverViewItem::CoverViewItem( QListWidget *parent, Meta::AlbumPtr album )
     : QListWidgetItem( parent )
     , m_albumPtr( album)
-    , m_parent( parent )
 {
     m_album = album->prettyName();
     if( album->hasAlbumArtist() )
@@ -866,10 +840,7 @@ CoverViewItem::CoverViewItem( QListWidget *parent, Meta::AlbumPtr album )
         m_artist = i18n( "No Artist" );
     setText( album->prettyName() );
 
-    const bool isSuppressing = album->suppressImageAutoFetch();
-    album->setSuppressImageAutoFetch( true );
-    setIcon( album->image( 100 ) );
-    album->setSuppressImageAutoFetch( isSuppressing );
+    loadCover();
 
     CoverManager::instance()->subscribeTo( album );
 }
@@ -888,7 +859,7 @@ CoverViewItem::loadCover()
 {
     const bool isSuppressing = m_albumPtr->suppressImageAutoFetch();
     m_albumPtr->setSuppressImageAutoFetch( true );
-    setIcon( m_albumPtr->image( 100 ) );
+    setIcon( QPixmap::fromImage( m_albumPtr->image( 100 ) ) );
     m_albumPtr->setSuppressImageAutoFetch( isSuppressing );
 }
 
