@@ -17,17 +17,18 @@
 
 #include "TrackListHandler.h"
 
+#include "ActionClasses.h"
 #include "amarokconfig.h"
 #include "App.h"
-#include "core-impl/collections/support/CollectionManager.h"
+#include "core/meta/Meta.h"
 #include "core/meta/support/MetaUtility.h"
+#include "core/podcasts/PodcastProvider.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "dbus/mpris1/PlayerHandler.h"
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistController.h"
+#include "playlistmanager/PlaylistManager.h"
 #include "playlist/PlaylistModelStack.h"
-#include "dbus/mpris1/PlayerHandler.h"
-#include "ActionClasses.h"
-
-
 
 #include "Mpris1TrackListAdaptor.h"
 
@@ -39,19 +40,18 @@ namespace Mpris1
     {
         new Mpris1TrackListAdaptor(this);
         QDBusConnection::sessionBus().registerObject( "/TrackList", this );
-        connect( The::playlist()->qaim(), SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), this, SLOT( slotTrackListChange() ) );
-        connect( The::playlist()->qaim(), SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ), this, SLOT( slotTrackListChange() ) );
+        connect( The::playlist()->qaim(), SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(slotTrackListChange()) );
+        connect( The::playlist()->qaim(), SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(slotTrackListChange()) );
     }
 
-    int TrackListHandler::AddTrack( const QString& url, bool playImmediately )
+    int TrackListHandler::AddTrack( const QString &url, bool playImmediately )
     {
         Meta::TrackPtr track = CollectionManager::instance()->trackForUrl( url );
         if( track )
         {
-            if( playImmediately )
-                The::playlistController()->insertOptioned( track, Playlist::AppendAndPlayImmediately );
-            else
-                The::playlistController()->insertOptioned( track, Playlist::Append );
+            Playlist::AddOptions options = playImmediately ?
+                    Playlist::OnPlayMediaAction : Playlist::OnAppendToPlaylistAction;
+            The::playlistController()->insertOptioned( track, options );
             return 0;
         }
         else
@@ -116,6 +116,33 @@ namespace Mpris1
     {
         emit TrackListChange( The::playlist()->qaim()->rowCount() );
     }
+
+    void TrackListHandler::UpdateAllPodcasts()
+    {
+        foreach( Playlists::PlaylistProvider *provider,
+                 The::playlistManager()->providersForCategory( PlaylistManager::PodcastChannel ) )
+        {
+            Podcasts::PodcastProvider *podcastProvider = dynamic_cast<Podcasts::PodcastProvider*>( provider );
+            if( podcastProvider )
+                podcastProvider->updateAll();
+        }
+    }
+
+    void TrackListHandler::AddPodcast( const QString& url )
+    {
+        //RSS 1.0/2.0 or Atom feed URL
+        Podcasts::PodcastProvider *podcastProvider = The::playlistManager()->defaultPodcasts();
+        if( podcastProvider )
+        {
+            if( !url.isEmpty() )
+                podcastProvider->addPodcast( Podcasts::PodcastProvider::toFeedUrl( url.trimmed() ) );
+            else
+                error() << "Invalid input string";
+        }
+        else
+            error() << "PodcastChannel provider is null";
+    }
+
 }
 
 #include "TrackListHandler.moc"
