@@ -1,6 +1,7 @@
 /****************************************************************************************
  * Copyright (c) 2006 Giovanni Venturi <giovanni@kde-it.org>                            *
  * Copyright (c) 2010 Ralf Engels <ralf-engels@gmx.de>                                  *
+ * Copyright (c) 2010 Sergey Ivanov <123kash@gmail.com>                                 *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -18,14 +19,17 @@
 #ifndef AMAROK_EDITFILTERDIALOG_H
 #define AMAROK_EDITFILTERDIALOG_H
 
-#include "core/meta/Meta.h"
-#include "ui_EditFilterDialog.h"
+#include "core/meta/forward_declarations.h"
+#include "widgets/MetaQueryWidget.h"
+#include "widgets/TokenPool.h"
 
 #include <KDialog>
-
 #include <QList>
 
-class QWidget;
+namespace Ui
+{
+    class EditFilterDialog;
+}
 
 class EditFilterDialog : public KDialog
 {
@@ -35,24 +39,48 @@ class EditFilterDialog : public KDialog
         explicit EditFilterDialog( QWidget* parent, const QString &text = QString() );
         ~EditFilterDialog();
 
-        QString filter() const;
+        QString filter();
 
     signals:
         void filterChanged( const QString &filter );
 
+    private slots:
+        void slotTokenSelected( Token *token );
+        void slotTokenDestroyed( QObject *token );
+        void slotAttributeChanged( const MetaQueryWidget::Filter &filter );
+        void slotInvert( bool checked );
+        void slotSeparatorChange();
+        void slotSearchEditChanged( const QString &filterText );
+        void slotReset();
+        void accept();
+
+        void updateAttributeEditor();
+        void updateSearchEdit();
+
+        /** Parses the given text and set's the dropTarget accordingly. */
+        void updateDropTarget( const QString &filterText );
+
     private:
-        Ui::EditFilterDialog m_ui;
+        void initTokenPool();
+        Token *tokenForField( const qint64 field );
 
-        bool m_appended;               // true if a filter appended
-        QString m_filterText;          // the resulting filter string
-        QString m_previousFilterText;  // the previous resulting filter string
+        struct Filter
+        {
+            MetaQueryWidget::Filter filter;
+            bool inverted;
+        };
 
-    protected slots:
-        virtual void slotAttributeChanged();
-        virtual void slotAppend();
-        virtual void slotClear();
-        virtual void slotUndo();
-        virtual void slotOk();
+        Filter &filterForToken( Token *token );
+
+        Ui::EditFilterDialog *m_ui;
+        Token *m_curToken;
+        QMap< Token *, Filter > m_filters;
+
+        QString m_separator;
+
+        /** True if we are already updating the status.
+            This blocks recursive calls to updateWidgets or parsteTextFilters. */
+        bool m_isUpdating;
 };
 
 #endif /* AMAROK_EDITFILTERDIALOG_H */
