@@ -42,7 +42,7 @@ int main( int argc, char *argv[] )
         "amarok", 0,
         ki18n( "Amarok" ), AMAROK_VERSION,
         ki18n( "The audio player for KDE" ), KAboutData::License_GPL,
-        ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2012, The Amarok Development Squad" ),
+        ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2013, The Amarok Development Squad" ),
         ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: %1)" ).subs( __DATE__ ),
         ( "http://amarok.kde.org" ) );
 
@@ -155,7 +155,7 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( "oggb4mp3", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Harald Sitter"), ki18n("Phonon, Lord-President of KDE Multimedia (apachelogger)"), "harald.sitter@kdemail.net" );
         ocsData.addCredit( "apachelogger", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("John Atkinson"), ki18n(( "Assorted patches" )), "john@fauxnetic.co.uk" );
+    aboutData.addCredit( ki18n("John Atkinson"), ki18n("Assorted patches"), "john@fauxnetic.co.uk" );
         ocsData.addCredit( "fauxnetic", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Kenneth Wesley Wimer II"), ki18n("Icons"), "kwwii@bootsplash.org" );
         ocsData.addCredit( "kwwii", aboutData.credits().last() );
