@@ -17,6 +17,8 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "MoodbarManager"
+
 /*
 The mood file loading and rendering code is based on the Amarok 1.4 moodbar implementation
 by Gav Wood and Joseph Rabinoff, ported to Qt 4 with only a few modifications by me.
@@ -28,11 +30,10 @@ even porting to qtscript so it could be run, as needed, by Amarok.
 - Nikolaj
 */
 
-#define DEBUG_PREFIX "MoodbarManager"
-
 #include "MoodbarManager.h"
 
 #include "amarokconfig.h"
+#include "core/meta/Meta.h"
 #include "core/support/Debug.h"
 #include "PaletteHandler.h"
 
@@ -40,7 +41,6 @@ even porting to qtscript so it could be run, as needed, by Amarok.
 #include <QFileInfo>
 #include <QPainter>
 
-
 #define NUM_HUES 12
 
 namespace The
@@ -57,7 +57,7 @@ namespace The
 }
 
 MoodbarManager::MoodbarManager()
-    : m_cache( new KPixmapCache( "Amarok-moodbars" ) )
+    : m_cache( new KImageCache( "Amarok-moodbars", 10 * 1024 ) )
     , m_lastPaintMode( 0 )
 {
     connect( The::paletteHandler(), SIGNAL(newPalette(QPalette)), SLOT(paletteChanged(QPalette)) );
@@ -145,17 +145,18 @@ QPixmap MoodbarManager::getMoodbar( Meta::TrackPtr track, int width, int height,
     if( m_lastPaintMode != AmarokConfig::moodbarPaintStyle() )
     {
         m_lastPaintMode = AmarokConfig::moodbarPaintStyle();
-        m_cache->discard();
+        m_cache->clear();
         m_moodDataMap.clear();
         emit moodbarStyleChanged();
     }
 
 
-    //Do we alrady have this pixmap cached?
-    const QString pixmapKey = QString( "mood:%1-%2x%3%4" ).arg( track->uidUrl() ).arg(width).arg(height).arg( rtl?"r":"" );
+    //Do we already have this pixmap cached?
+    const QString pixmapKey = QString( "mood:%1-%2x%3%4" ).arg( track->uidUrl(), QString::number( width ),
+                                                                QString::number( height ), QString( rtl?"r":"" ) );
     QPixmap moodbar;
-    
-    if( m_cache->find( pixmapKey, moodbar ) )
+
+    if( m_cache->findPixmap( pixmapKey, &moodbar ) )
         return moodbar;
         
     //No? Ok, then create it reusing as much info as possible
@@ -172,7 +173,7 @@ QPixmap MoodbarManager::getMoodbar( Meta::TrackPtr track, int width, int height,
             moodFilePath = m_moodFileMap.value( track );
         else
             moodFilePath = moodPath( track->playableUrl().path() );
-        
+
         data = readMoodFile( moodFilePath );
 
         if ( data.size() > 10 )
@@ -189,7 +190,7 @@ QPixmap MoodbarManager::getMoodbar( Meta::TrackPtr track, int width, int height,
         return moodbar;
 
     moodbar = drawMoodbar( data, width, height, rtl );
-    m_cache->insert( pixmapKey, moodbar );
+    m_cache->insertPixmap( pixmapKey, moodbar );
     
     return moodbar;
 }
@@ -310,7 +311,7 @@ MoodbarColorList MoodbarManager::readMoodFile( const KUrl &moodFileUrl )
 
         switch( paintStyle )
         {
-        case 1: // Angry
+        case Angry: // Angry
             threshold  = samples / 360 * 9;
             rangeStart = 45;
             rangeDelta = -45;
@@ -318,7 +319,7 @@ MoodbarColorList MoodbarManager::readMoodFile( const KUrl &moodFileUrl )
             val        = 100;
             break;
 
-        case 2: // Frozen
+        case Frozen: // Frozen
             threshold  = samples / 360 * 1;
             rangeStart = 140;
             rangeDelta = 160;
@@ -326,14 +327,23 @@ MoodbarColorList MoodbarManager::readMoodFile( const KUrl &moodFileUrl )
             val        = 100;
             break;
 
-        case 3: // Happy
+        case Happy: // Happy
             threshold  = samples / 360 * 2;
             rangeStart = 0;
             rangeDelta = 359;
             sat        = 150;
-            val        = 200;
+            val        = 250;
             break;
 
+        case Normal: // old "normal" mode, don't change moodfile's RGB values
+            threshold  = samples / 360 * 3;
+            rangeStart = 0;
+            rangeDelta = 359;
+            sat        = 100;
+            val        = 100;
+            break;
+
+        case SystemColours:
         default: // Default (system colours)
             threshold  = samples / 360 * 3;
             rangeStart = The::paletteHandler()->highlightColor().hsvHue();
@@ -514,7 +524,7 @@ void MoodbarManager::paletteChanged( const QPalette &palette )
     const int paintStyle = AmarokConfig::moodbarPaintStyle();
     if( paintStyle == 0 ) // system default colour
     {
-        m_cache->discard();
+        m_cache->clear();
         m_moodDataMap.clear();
     }
 }
