@@ -21,6 +21,7 @@
 #ifndef AMAROK_PLAYLISTACTIONS_H
 #define AMAROK_PLAYLISTACTIONS_H
 
+#include "amarok_export.h"
 #include "core/support/Debug.h"
 #include "core/playlists/Playlist.h"
 
@@ -43,14 +44,6 @@ namespace Playlist
 {
 class TrackNavigator;
 
-enum StopAfterMode
-{
-    StopNever = 0,
-    StopAfterCurrent,
-    StopAfterQueue
-};
-
-
 /**
  * This class is a central hub between the playlist model stack, the playlist navigators,
  * and the track playback engine. It ties them together to provide simple "Play", "Play
@@ -93,16 +86,15 @@ public:
      */
     void requestTrack( quint64 id );
 
-    StopAfterMode stopAfterMode() const { return m_stopAfterMode; }
-    void setStopAfterMode( StopAfterMode m ) { m_stopAfterMode = m; }
-    void setTrackToBeLast( quint64 id ) { m_trackToBeLast = id; }
-    bool willStopAfterTrack( const quint64 id ) const { return m_trackToBeLast == id; }
-
     /**
-     * Make sure that there are enough tracks in the current playlist
-     * if it is dynamic and the user removed tracks.
+     * Request that Amarok stops playback after playing certain track.
+     * @param id playlist id of a track to stop playing after or 0 to disable
+     *           stopping after some track. If not specified, enable stopping
+     *           after currently playing track.
      */
-    void normalizeDynamicPlaylist();
+    void stopAfterPlayingTrack( quint64 id = -1 );
+
+    bool willStopAfterTrack( const quint64 id ) const;
 
     // This shouldn't be in Actions, it doesn't make sense
     int queuePosition( quint64 id );
@@ -123,12 +115,38 @@ public slots:
     void play( const quint64 id, bool now = true );
     void next();
     void back();
-    void playlistModeChanged(); //!< Changes the tracknavigator
+    void enableDynamicMode( bool enable );
+
+    /** Changes the tracknavigator depending on the current configuration */
+    void playlistModeChanged();
+
     void repopulateDynamicPlaylist();
-    void queue( QList<int> rows );
-    void dequeue( QList<int> rows );
+
+    /**
+      * Shuffles tracks (that are visible in the top model) at the bottom model level
+      */
+    void shuffle();
+
+    /**
+     * Adds a list of top playlist model rows to the queue.
+     */
+    void queue( const QList<int> &rows );
+
+    /**
+     * Adds a list of playlist item unique ids to the queue.
+     */
+    void queue( const QList<quint64> &ids );
+
+    void dequeue( const QList<int> &rows );
     void restoreDefaultPlaylist();
 
+    /**
+     * Make sure that there are enough tracks in the current playlist
+     * if it is dynamic and the user removed tracks.
+     */
+    void normalizeDynamicPlaylist();
+
+
     /**
     * Repaint the playlist.
     * Useful when triggering a change that will modify the visual appearance of one or more items in the playlist
@@ -138,8 +156,9 @@ public slots:
 signals:
     void navigatorChanged();
 
-protected slots:
+private slots:
     void slotTrackPlaying( Meta::TrackPtr engineTrack );
+    void slotPlayingStopped( qint64 finalPosition, qint64 trackLength );
 
 private:
     Actions();
@@ -148,9 +167,12 @@ private:
     void init();
 
     quint64 m_nextTrackCandidate;
-    quint64 m_trackToBeLast;
+    /**
+     * Playlist id if a track where the playback should be stopped or 0 if playback
+     * shouldn't be stopped after certain track
+     */
+    quint64 m_stopAfterPlayingTrackId;
     TrackNavigator* m_navigator;                //!< the strategy of what to do when a track finishes playing
-    Playlist::StopAfterMode m_stopAfterMode;
     bool m_waitingForNextTrack;
 
     static Actions* s_instance; //!< instance variable
