@@ -23,10 +23,11 @@
 #ifndef AMAROK_PLAYLISTMODEL_H
 #define AMAROK_PLAYLISTMODEL_H
 
-#include "proxymodels/AbstractModel.h"
-#include "core/support/Amarok.h"
 #include "UndoCommands.h"
-#include "core/meta/Meta.h"
+#include "amarok_export.h"
+#include "core/meta/Observer.h"
+#include "core/support/Amarok.h"
+#include "playlist/proxymodels/AbstractModel.h"
 
 #include <QAbstractListModel>
 #include <QHash>
@@ -39,7 +40,6 @@ class QMimeData;
 class QModelIndex;
 class TestPlaylistModels;
 
-
 namespace Playlist
 {
 
@@ -71,15 +71,17 @@ class AMAROK_EXPORT Model : public QAbstractListModel, public Meta::Observer, pu
         QAbstractItemModel* qaim() const { return const_cast<Model*>( this ); }
 
         quint64 activeId() const;
-        int activeRow() const { return m_activeRow; } // returns -1 if there is no active row
-        QSet<int> allRowsForTrack( const Meta::TrackPtr track ) const;
         Meta::TrackPtr activeTrack() const;
-        bool containsTrack( const Meta::TrackPtr track ) const;
-        virtual bool exportPlaylist( const QString &path ) const;
-        int firstRowForTrack( const Meta::TrackPtr track ) const;
+        int activeRow() const { return m_activeRow; } // returns -1 if there is no active row
+
+        bool containsTrack( const Meta::TrackPtr& track ) const;
+        int firstRowForTrack( const Meta::TrackPtr& track ) const;
+        QSet<int> allRowsForTrack( const Meta::TrackPtr& track ) const;
+
         quint64 idAt( const int row ) const;
         bool rowExists( int row ) const { return (( row >= 0 ) && ( row < m_items.size() ) ); }
         int rowForId( const quint64 id ) const; // returns -1 if the id is invalid
+        int rowFromBottomModel( const int row ) { return row; }
         int rowToBottomModel( const int row ) { return row; }
         void setActiveId( const quint64 id ) { setActiveRow( rowForId( id ) ); }
         void setActiveRow( int row );
@@ -93,8 +95,9 @@ class AMAROK_EXPORT Model : public QAbstractListModel, public Meta::Observer, pu
         quint64 totalSize() const { return m_totalSize; }
         Meta::TrackPtr trackAt( int row ) const;
         Meta::TrackPtr trackForId( const quint64 id ) const;
+
+        virtual bool exportPlaylist( const QString &path, bool relative = false ) const;
         virtual Meta::TrackList tracks() const;
-        virtual QString generatePlaylistName() const;
 
         // Inherited from Meta::Observer
         using Observer::metadataChanged;
@@ -107,6 +110,8 @@ class AMAROK_EXPORT Model : public QAbstractListModel, public Meta::Observer, pu
         /** Set the columns that are displayed in the tooltip */
         static void setTooltipColumns( bool columns[] );
 
+        static void enableToolTip( bool enable );
+
     signals:
         void activeTrackChanged( quint64 );
         void queueChanged();
@@ -114,6 +119,11 @@ class AMAROK_EXPORT Model : public QAbstractListModel, public Meta::Observer, pu
     protected:
         int rowForItem( Item *item ) const { return m_items.indexOf( item ); }
 
+    private slots:
+        void saveState();
+        void queueSaveState();
+        void insertTracksFromTrackLoader( const Meta::TrackList &tracks );
+
     private:
         QString tooltipFor( Meta::TrackPtr track ) const;
 
@@ -152,6 +162,9 @@ class AMAROK_EXPORT Model : public QAbstractListModel, public Meta::Observer, pu
         int m_setStateOfItem_batchMaxRow;
 
         static bool s_tooltipColumns[NUM_COLUMNS];
+        static bool s_showToolTip;
+
+        QTimer *m_saveStateTimer;
 };
 
 } // namespace Playlist
