@@ -116,7 +116,10 @@ void LayoutManager::updateCurrentLayout( const PlaylistLayout &layout )
         }
 
 
-        Amarok::Components::logger()->longMessage( i18n( "Current layout '%1' is read only. Creating a new layout '%2' with your changes and setting this as active", m_activeLayout, newLayoutName ) );
+        Amarok::Components::logger()->longMessage( i18n( "Current layout '%1' is read only. " \
+                    "Creating a new layout '%2' with your changes and setting this as active",
+                                                         m_activeLayout, newLayoutName )
+                                                 );
 
         addUserLayout( newLayoutName, layout );
         setActiveLayout( newLayoutName );
@@ -247,7 +250,7 @@ LayoutItemConfig LayoutManager::parseItemConfig( const QDomElement &elem ) const
             QDomNode elementNode = elements.item( index2 );
             index2++;
 
-            int value = internalColumnNames.indexOf( elementNode.toElement().attribute( "value", "Title" ) );
+            int value = columnForName( elementNode.toElement().attribute( "value", "Title" ) );
             QString prefix = elementNode.toElement().attribute( "prefix", QString() );
             QString sufix = elementNode.toElement().attribute( "suffix", QString() );
             qreal size = elementNode.toElement().attribute( "size", "0.0" ).toDouble();
@@ -352,7 +355,7 @@ QDomElement LayoutManager::createItemElement( QDomDocument doc, const QString &n
 
             elementElement.setAttribute ( "prefix", element.prefix() );
             elementElement.setAttribute ( "suffix", element.suffix() );
-            elementElement.setAttribute ( "value", internalColumnNames[element.value()] );
+            elementElement.setAttribute ( "value", internalColumnName( static_cast<Playlist::Column>( element.value() ) ) );
             elementElement.setAttribute ( "size", QString::number( element.size() ) );
             elementElement.setAttribute ( "bold", element.bold() ? "true" : "false" );
             elementElement.setAttribute ( "italic", element.italic() ? "true" : "false" );
