@@ -18,7 +18,7 @@
 #ifndef AMAROK_PLAYLISTABSTRACTMODEL_H
 #define AMAROK_PLAYLISTABSTRACTMODEL_H
 
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 #include "playlist/PlaylistDefines.h"
 #include "playlist/PlaylistItem.h"
 
@@ -99,7 +99,7 @@ public:
      * @param track the track.
      * @return collection of rows, empty if the track pointer is invalid.
      */
-    virtual QSet<int> allRowsForTrack( const Meta::TrackPtr track ) const = 0;
+    virtual QSet<int> allRowsForTrack( const Meta::TrackPtr& track ) const = 0;
 
     /**
      * Clears the current search term.
@@ -114,7 +114,7 @@ public:
      * @param track the track to check for.
      * @return true if the track is present, otherwise false.
      */
-    virtual bool containsTrack( const Meta::TrackPtr track ) const = 0;
+    virtual bool containsTrack( const Meta::TrackPtr& track ) const = 0;
 
     /**
      * Get the current search fields bitmask.
@@ -191,7 +191,7 @@ public:
      * @param track the track.
      * @return the row, or -1 if the track pointer is not found.
      */
-    virtual int firstRowForTrack( const Meta::TrackPtr track ) const = 0;
+    virtual int firstRowForTrack( const Meta::TrackPtr& track ) const = 0;
 
     /**
      * Returns the unique 64-bit id for the given row in the current model.
