@@ -17,15 +17,16 @@
 #include "PlaylistFileProvider.h"
 #include "App.h"
 #include "core-impl/playlists/types/file/PlaylistFileSupport.h"
-#include "core/capabilities/EditablePlaylistCapability.h"
 #include "core/support/Amarok.h"
 #include "core/support/Debug.h"
 #include "core/support/Components.h"
 #include "core/interfaces/Logger.h"
+#include "core-impl/playlists/types/file/asx/ASXPlaylist.h"
 #include "core-impl/playlists/types/file/m3u/M3UPlaylist.h"
 #include "core-impl/playlists/types/file/pls/PLSPlaylist.h"
 #include "core-impl/playlists/types/file/xspf/XSPFPlaylist.h"
 #include "playlist/PlaylistModelStack.h"
+#include "playlistmanager/PlaylistManager.h"
 
 #include <KDialog>
 #include <KInputDialog>
@@ -33,12 +34,10 @@
 #include <KUrl>
 
 #include <QAction>
+#include <QDir>
 #include <QLabel>
 #include <QString>
-
-//For removing multiple tracks from different playlists with one QAction
-typedef QMultiMap<Playlists::PlaylistPtr, Meta::TrackPtr> PlaylistTrackMap;
-Q_DECLARE_METATYPE( PlaylistTrackMap )
+#include <QTimer>
 
 using Playlist::ModelStack;
 
@@ -47,12 +46,29 @@ namespace Playlists {
 PlaylistFileProvider::PlaylistFileProvider()
  : UserPlaylistProvider()
  , m_playlistsLoaded( false )
- , m_defaultFormat( Playlists::XSPF )
- , m_renameAction( 0 )
- , m_deleteAction( 0 )
- , m_removeTrackAction( 0 )
+ , m_saveLaterTimer( 0 )
 {
-    //playlists are lazy loaded
+    //playlists are lazy loaded but we can count how many we'll load already
+    QStringList keys = loadedPlaylistsConfig().keyList();
+    foreach( const QString &key, keys )
+    {
+        KUrl url( key );
+        //Don't load these from the config file, they are read from the directory anyway
+        if( url.upUrl().equals( Amarok::saveLocation( "playlists" ) ) )
+            continue;
+        m_urlsToLoad << url;
+    }
+    //also add all files in the $KDEHOME/share/apps/amarok/playlists
+    QDir playlistDir = QDir( Amarok::saveLocation( "playlists" ), "",
+                             QDir::Name,
+                             QDir::Files | QDir::Readable );
+    foreach( const QString &file, playlistDir.entryList() )
+    {
+        KUrl url( playlistDir.path() );
+        url.addPath( file );
+        if( Playlists::isPlaylist( url ) )
+            m_urlsToLoad << url;
+    }
 }
 
 PlaylistFileProvider::~PlaylistFileProvider()
@@ -61,7 +77,6 @@ PlaylistFileProvider::~PlaylistFileProvider()
     //remove all, well add them again soon
     loadedPlaylistsConfig().deleteGroup();
     //Write loaded playlists to config file
-    debug() << m_playlists.size()  << " Playlists loaded";
     foreach( Playlists::PlaylistFilePtr playlistFile, m_playlists )
     {
         KUrl url = playlistFile->uidUrl();
@@ -82,22 +97,29 @@ PlaylistFileProvider::prettyName() const
     return i18n( "Playlist Files on Disk" );
 }
 
+KIcon PlaylistFileProvider::icon() const
+{
+    return KIcon( "folder-documents" );
+}
+
 int
 PlaylistFileProvider::playlistCount() const
 {
-    if( m_playlistsLoaded )
-        return m_playlists.count();
-    //count the entries in the config file
-    return loadedPlaylistsConfig().keyList().count();
+    return m_playlists.count() + m_urlsToLoad.count();
 }
 
 Playlists::PlaylistList
 PlaylistFileProvider::playlists()
 {
-    if( !m_playlistsLoaded )
-        loadPlaylists();
-
     Playlists::PlaylistList playlists;
+
+    if( !m_playlistsLoaded )
+    {
+        //trigger a lazy load the playlists
+        QTimer::singleShot(0, this, SLOT(loadPlaylists()) );
+        return playlists;
+    }
+
     foreach( const Playlists::PlaylistFilePtr &playlistFile, m_playlists )
     {
         Playlists::PlaylistPtr playlist = Playlists::PlaylistPtr::dynamicCast( playlistFile );
@@ -107,107 +129,21 @@ PlaylistFileProvider::playlists()
     return playlists;
 }
 
-QList<QAction *>
-PlaylistFileProvider::playlistActions( Playlists::PlaylistPtr playlist )
-{
-    QList<QAction *> actions;
-
-    Playlists::PlaylistFilePtr playlistFile = Playlists::PlaylistFilePtr::dynamicCast( playlist );
-    if( !playlistFile )
-    {
-        error() << "Action requested for a non-file playlist";
-        return actions;
-    }
-
-    if( m_renameAction == 0 )
-    {
-        m_renameAction =  new QAction( KIcon( "media-track-edit-amarok" ), i18n( "&Rename..." ), this );
-        m_renameAction->setProperty( "popupdropper_svg_id", "edit" );
-        connect( m_renameAction, SIGNAL( triggered() ), this, SLOT( slotRename() ) );
-    }
-    //only one playlist can be renamed at a time.
-    if( m_renameAction->data().isNull() )
-        m_renameAction->setData( QVariant::fromValue( playlistFile ) );
-
-    actions << m_renameAction;
-
-    if( m_deleteAction == 0 )
-    {
-        m_deleteAction = new QAction( KIcon( "media-track-remove-amarok" ), i18n( "&Delete..." ),
-                                      this );
-        m_deleteAction->setProperty( "popupdropper_svg_id", "delete" );
-        connect( m_deleteAction, SIGNAL( triggered() ), SLOT( slotDelete() ) );
-    }
-    m_deleteAction->setObjectName( "deleteAction" );
-
-    Playlists::PlaylistFileList actionList =
-            m_deleteAction->data().value<Playlists::PlaylistFileList>();
-    actionList << playlistFile;
-    m_deleteAction->setData( QVariant::fromValue( actionList ) );
-
-    actions << m_deleteAction;
-
-    return actions;
-}
-
-QList<QAction *>
-PlaylistFileProvider::trackActions( Playlists::PlaylistPtr playlist, int trackIndex )
-{
-    Q_UNUSED( trackIndex );
-    QList<QAction *> actions;
-
-    if( trackIndex < 0 )
-        return actions;
-
-    int trackCount = playlist->trackCount();
-    if( trackCount == -1 )
-        trackCount = playlist->tracks().size();
-
-    if( trackIndex >= trackCount )
-        return actions;
-
-    if( m_removeTrackAction == 0 )
-    {
-        m_removeTrackAction = new QAction( this );
-        m_removeTrackAction->setIcon( KIcon( "media-track-remove-amarok" ) );
-        m_removeTrackAction->setProperty( "popupdropper_svg_id", "delete" );
-        connect( m_removeTrackAction, SIGNAL( triggered() ), SLOT( slotRemove() ) );
-    }
-
-    m_removeTrackAction->setObjectName( "deleteAction" );
-    //Add the playlist/track combination to a QMultiMap that is stored in the action.
-    //In the slot we use this data to remove that track from the playlist.
-    PlaylistTrackMap playlistMap = m_removeTrackAction->data().value<PlaylistTrackMap>();
-    Meta::TrackPtr track = playlist->tracks()[trackIndex];
-    //only add action to map if playlist/track combo is not in there yet.
-    if( !playlistMap.keys().contains( playlist ) ||
-           !playlistMap.values( playlist ).contains( track )
-      )
-    {
-        playlistMap.insert( playlist, track );
-    }
-    m_removeTrackAction->setData( QVariant::fromValue( playlistMap ) );
-
-    if( playlistMap.keys().count() > 1 )
-        m_removeTrackAction->setText( i18n( "Remove tracks" ) );
-    else
-        m_removeTrackAction->setText( i18nc( "Remove a track from a saved playlist",
-                                             "Remove From \"%1\"", playlist->name() ) );
-
-    if( playlistMap.keys().count() > 1 )
-        m_removeTrackAction->setText( i18n( "Remove" ) );
-
-    actions << m_removeTrackAction;
-
-    return actions;
-}
-
 Playlists::PlaylistPtr
 PlaylistFileProvider::save( const Meta::TrackList &tracks, const QString &name )
 {
     DEBUG_BLOCK
+
     QString filename = name.isEmpty() ? QDateTime::currentDateTime().toString( "ddd MMMM d yy hh-mm") : name;
-    filename = QString( filename ).replace( QLatin1Char('/'), QLatin1Char('-') );
+    filename.replace( QLatin1Char('/'), QLatin1Char('-') );
+    filename.replace( QLatin1Char('\\'), QLatin1Char('-') );
+
+    Playlists::PlaylistFormat format = Playlists::getFormat( filename );
+    if( format == Playlists::Unknown ) // maybe the name just had a dot in it. We just add .xspf
+    {
+        format = Playlists::XSPF;
+        filename.append( QLatin1String( ".xspf" ) );
+    }
 
     KUrl path( Amarok::saveLocation( "playlists" ) );
     path.addPath( Amarok::vfatPath( filename ) );
@@ -217,46 +153,39 @@ PlaylistFileProvider::save( const Meta::TrackList &tracks, const QString &name )
         return Playlists::PlaylistPtr();
     }
 
-    Playlists::PlaylistFormat format = m_defaultFormat;
-    QString ext = Amarok::extension( path.fileName() );
-    if( ext.isEmpty() )
-    {
-        ext = QLatin1String("xspf");
-        path.setFileName( QString("%1.%2").arg(Amarok::vfatPath(filename), ext) );
-    }
-    else
-    {
-        format = Playlists::getFormat( path );
-    }
-
     Playlists::PlaylistFile *playlistFile = 0;
     switch( format )
     {
+        case Playlists::ASX:
+            playlistFile = new Playlists::ASXPlaylist( path, this );
+            break;
         case Playlists::PLS:
-            playlistFile = new Playlists::PLSPlaylist( tracks );
+            playlistFile = new Playlists::PLSPlaylist( path, this );
             break;
         case Playlists::M3U:
-            playlistFile = new Playlists::M3UPlaylist( tracks );
+            playlistFile = new Playlists::M3UPlaylist( path, this );
             break;
         case Playlists::XSPF:
-            playlistFile = new Playlists::XSPFPlaylist( tracks );
+            playlistFile = new Playlists::XSPFPlaylist( path, this );
             break;
-        default:
-            error() << QString("Do not support filetype with extension \"%1!\"").arg( ext );
+        case Playlists::XML:
+        case Playlists::RAM:
+        case Playlists::SMIL:
+        case Playlists::Unknown:
+            // this should not happen since we set the format to XSPF above.
             return Playlists::PlaylistPtr();
     }
     playlistFile->setName( filename );
-    debug() << "Forcing save of playlist!";
-    playlistFile->save( path, true );
-    playlistFile->setProvider( this );
+    playlistFile->addTracks( tracks );
+    playlistFile->save( true );
 
     Playlists::PlaylistFilePtr playlistPtr( playlistFile );
     m_playlists << playlistPtr;
     //just in case there wasn't one loaded before.
     m_playlistsLoaded = true;
-    Playlists::PlaylistPtr playlist = Playlists::PlaylistPtr::dynamicCast( playlistPtr );
-    emit playlistAdded( playlist );
 
+    Playlists::PlaylistPtr playlist( playlistFile );
+    emit playlistAdded( playlist );
     return playlist;
 }
 
@@ -292,26 +221,27 @@ PlaylistFileProvider::import( const KUrl &path )
         return false;
     }
 
-    Playlists::PlaylistFilePtr playlistFile = Playlists::loadPlaylistFile( path );
+    Playlists::PlaylistFilePtr playlistFile = Playlists::loadPlaylistFile( path, this );
     if( !playlistFile )
         return false;
-    playlistFile->setProvider( this );
+
     m_playlists << playlistFile;
     //just in case there wasn't one loaded before.
     m_playlistsLoaded = true;
-    emit playlistAdded( Playlists::PlaylistPtr::dynamicCast( playlistFile ) );
+
+    emit playlistAdded( PlaylistPtr( playlistFile.data() ) );
     return true;
 }
 
 void
-PlaylistFileProvider::rename( Playlists::PlaylistPtr playlist, const QString &newName )
+PlaylistFileProvider::renamePlaylist( Playlists::PlaylistPtr playlist, const QString &newName )
 {
     DEBUG_BLOCK
     playlist->setName( newName );
 }
 
 bool
-PlaylistFileProvider::deletePlaylists( Playlists::PlaylistList playlists )
+PlaylistFileProvider::deletePlaylists( const Playlists::PlaylistList &playlists )
 {
     Playlists::PlaylistFileList playlistFiles;
     foreach( Playlists::PlaylistPtr playlist, playlists )
@@ -327,20 +257,6 @@ PlaylistFileProvider::deletePlaylists( Playlists::PlaylistList playlists )
 bool
 PlaylistFileProvider::deletePlaylistFiles( Playlists::PlaylistFileList playlistFiles )
 {
-    DEBUG_BLOCK
-    KDialog dialog;
-    dialog.setCaption( i18n( "Confirm Delete" ) );
-    dialog.setButtons( KDialog::Ok | KDialog::Cancel );
-    QLabel label( i18np( "Are you sure you want to delete this playlist?",
-                         "Are you sure you want to delete these %1 playlist files?",
-                         playlistFiles.count() )
-                    , &dialog
-                  );
-    dialog.setButtonText( KDialog::Ok, i18n( "Yes, delete from disk." ) );
-    dialog.setMainWidget( &label );
-    if( dialog.exec() != QDialog::Accepted )
-        return false;
-
     foreach( Playlists::PlaylistFilePtr playlistFile, playlistFiles )
     {
         m_playlists.removeAll( playlistFile );
@@ -356,22 +272,16 @@ PlaylistFileProvider::deletePlaylistFiles( Playlists::PlaylistFileList playlistF
 void
 PlaylistFileProvider::loadPlaylists()
 {
-    DEBUG_BLOCK
-    //load the playlists defined in the config
-    QStringList keys = loadedPlaylistsConfig().keyList();
-    // debug() << "keys " << keys;
+    if( m_urlsToLoad.isEmpty() )
+        return;
 
-    //ConfigEntry: name, file
-    foreach( const QString &key, keys )
+    //arbitrary number of playlists to load during one mainloop run: 5
+    for( int i = 0; i < qMin( m_urlsToLoad.count(), 5 ); i++ )
     {
-        KUrl url( key );
-        //Don't load these from the config file, they are read from the directory anyway
-        if( url.upUrl().equals( Amarok::saveLocation( "playlists" ) ) )
-            continue;
-
-        QString groups = loadedPlaylistsConfig().readEntry( key );
-        Playlists::PlaylistFilePtr playlist = Playlists::loadPlaylistFile( url );
-        if( playlist.isNull() )
+        KUrl url = m_urlsToLoad.takeFirst();
+        QString groups = loadedPlaylistsConfig().readEntry( url.url() );
+        Playlists::PlaylistFilePtr playlist = Playlists::loadPlaylistFile( url, this );
+        if( !playlist )
         {
             Amarok::Components::logger()->longMessage(
                     i18n("The playlist file \"%1\" could not be loaded.", url.fileName() ),
@@ -380,95 +290,48 @@ PlaylistFileProvider::loadPlaylists()
             continue;
         }
 
-        //since class PlaylistFile can be used without a collection we set it manually
-        playlist->setProvider( this );
-
         if( !groups.isEmpty() && playlist->isWritable() )
             playlist->setGroups( groups.split( ',',  QString::SkipEmptyParts ) );
 
         m_playlists << playlist;
+        emit playlistAdded( PlaylistPtr( playlist.data() ) );
     }
 
-    //also add all files in the $KDEHOME/share/apps/amarok/playlists
-    QDir playlistDir = QDir( Amarok::saveLocation( "playlists" ), "",
-                             QDir::Name,
-                             QDir::Files | QDir::Readable );
-    foreach( const QString &file, playlistDir.entryList() )
+    //give the mainloop time to run
+    if( !m_urlsToLoad.isEmpty() )
+        QTimer::singleShot( 0, this, SLOT(loadPlaylists()) );
+}
+
+void
+PlaylistFileProvider::saveLater( Playlists::PlaylistFilePtr playlist )
+{
+    //WARNING: this assumes the playlistfile uses it's m_url for uidUrl
+    if( playlist->uidUrl().isEmpty() )
+        return;
+
+    if( !m_saveLaterPlaylists.contains( playlist ) )
+        m_saveLaterPlaylists << playlist;
+
+    if( !m_saveLaterTimer )
     {
-        KUrl url( playlistDir.path() );
-        url.addPath( file );
-        debug() << QString( "Trying to open %1 as a playlist file" ).arg( url.url() );
-        Playlists::PlaylistFilePtr playlist = Playlists::loadPlaylistFile( url );
-        if( playlist.isNull() )
-        {
-            Amarok::Components::logger()->longMessage(
-                    i18n("The playlist file \"%1\" could not be loaded.", url.fileName() ),
-                    Amarok::Logger::Error
-                );
-            continue;
-        }
-        playlist->setProvider( this );
-
-        m_playlists << playlist;
+        m_saveLaterTimer = new QTimer( this );
+        m_saveLaterTimer->setSingleShot( true );
+        m_saveLaterTimer->setInterval( 0 );
+        connect( m_saveLaterTimer, SIGNAL(timeout()), SLOT(slotSaveLater()) );
     }
 
-    m_playlistsLoaded = true;
+    m_saveLaterTimer->start();
 }
 
 void
-PlaylistFileProvider::slotDelete()
+PlaylistFileProvider::slotSaveLater() //SLOT
 {
-    QAction *action = qobject_cast<QAction *>( QObject::sender() );
-    if( action == 0 )
-        return;
-
-    //only one playlist can be selected at this point
-    Playlists::PlaylistFileList playlists = action->data().value<Playlists::PlaylistFileList>();
-
-    if( playlists.count() > 0 )
-        deletePlaylistFiles( playlists );
-}
-
-void
-PlaylistFileProvider::slotRename()
-{
-    QAction *action = qobject_cast<QAction *>( QObject::sender() );
-    if( action == 0 )
-        return;
-
-    //only one playlist can be renamed at a time.
-    Playlists::PlaylistFilePtr playlist = action->data().value<Playlists::PlaylistFilePtr>();
-    if( playlist.isNull() )
-        return;
-
-    //TODO: inline rename
-    bool ok;
-    const QString newName = KInputDialog::getText( i18n("Change playlist"),
-                i18n("Enter new name for playlist:"), playlist->name(),
-                                                   &ok );
-    if( ok )
-        playlist->setName( newName.trimmed() );
-}
-
-void
-PlaylistFileProvider::slotRemove()
-{
-    QAction *action = qobject_cast<QAction *>( QObject::sender() );
-    if( action == 0 )
-        return;
-
-    PlaylistTrackMap playlistMap = action->data().value<PlaylistTrackMap>();
-    QList< Playlists::PlaylistPtr > uniquePlaylists = playlistMap.uniqueKeys();
-
-    foreach( Playlists::PlaylistPtr playlist, uniquePlaylists )
+    foreach( Playlists::PlaylistFilePtr playlist, m_saveLaterPlaylists )
     {
-        QList< Meta::TrackPtr > tracks = playlistMap.values( playlist );
-        foreach( Meta::TrackPtr track, tracks )
-            playlist->removeTrack( playlist->tracks().indexOf( track ) );
+        playlist->save( true ); //TODO: read relative type when loading
     }
 
-    //clear the data
-    action->setData( QVariant() );
+    m_saveLaterPlaylists.clear();
 }
 
 KConfigGroup
