@@ -16,9 +16,9 @@
 
 #include "SqlPlaylist.h"
 
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 #include "core/support/Debug.h"
-#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/storage/StorageManager.h"
 #include "core-impl/meta/proxy/MetaProxy.h"
 #include "core-impl/meta/stream/Stream.h"
 #include "core-impl/meta/timecode/TimecodeMeta.h"
@@ -38,7 +38,6 @@ SqlPlaylist::SqlPlaylist( const QString & name, const Meta::TrackList
     , m_tracks( tracks )
     , m_provider( provider)
     , m_name( name )
-    , m_description( i18n( "Playlist in database" ) )
     , m_urlId( urlId )
     , m_tracksLoaded( true )
 {
@@ -54,8 +53,7 @@ SqlPlaylist::SqlPlaylist( const QStringList & resultRow,
 {
     m_dbId = resultRow[0].toInt();
     m_name = resultRow[2];
-    m_description = resultRow[3];
-    m_urlId = resultRow[4];
+    m_urlId = resultRow[3];
 }
 
 
@@ -104,7 +102,7 @@ SqlPlaylist::saveToDb( bool tracks )
     if( m_parent )
         parentId = m_parent->id();
 
-    SqlStorage *sql = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sql = StorageManager::instance()->sqlStorage();
 
     //figure out if we have a urlId and if this id is already in the db, if so, update it instead of creating a new one.
     if( !m_urlId.isEmpty() )
@@ -127,32 +125,30 @@ SqlPlaylist::saveToDb( bool tracks )
     if( m_dbId != -1 )
     {
         //update existing
-        QString query = "UPDATE playlists SET parent_id=%1, name='%2', description='%3' WHERE id=%4;";
+        QString query = "UPDATE playlists SET parent_id=%1, name='%2' WHERE id=%3;";
         query = query.arg( QString::number( parentId ) )
                 .arg( sql->escape( m_name ) )
-                .arg( sql->escape( m_description ) )
                 .arg( QString::number( m_dbId ) );
-        CollectionManager::instance()->sqlStorage()->query( query );
+        StorageManager::instance()->sqlStorage()->query( query );
 
         if( tracks )
         {
             //delete existing tracks and insert all
             query = "DELETE FROM playlist_tracks where playlist_id=%1;";
             query = query.arg( QString::number( m_dbId ) );
-            CollectionManager::instance()->sqlStorage()->query( query );
+            StorageManager::instance()->sqlStorage()->query( query );
             saveTracks();
         }
     }
     else
     {
         //insert new
-        QString query = "INSERT INTO playlists ( parent_id, name, description, urlid ) "
-                        "VALUES ( %1, '%2', '%3', '%4' );";
+        QString query = "INSERT INTO playlists ( parent_id, name, urlid ) "
+                        "VALUES ( %1, '%2', '%3' );";
         query = query.arg( QString::number( parentId ) )
                 .arg( sql->escape( m_name ) )
-                .arg( sql->escape( m_description ) )
                 .arg( sql->escape( m_urlId ) );
-        m_dbId = CollectionManager::instance()->sqlStorage()->insert( query, "playlists" );
+        m_dbId = StorageManager::instance()->sqlStorage()->insert( query, "playlists" );
         if( tracks )
             saveTracks();
     }
@@ -177,12 +173,13 @@ void
 SqlPlaylist::saveTracks()
 {
     int trackNum = 1;
-    SqlStorage *sql = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sql = StorageManager::instance()->sqlStorage();
 
     foreach( Meta::TrackPtr trackPtr, m_tracks )
     {
         if( trackPtr )
         {
+            // keep this in sync with SqlTrack::updatePlaylistsToDb()!
             debug() << "saving track with url " << trackPtr->uidUrl();
             QString query = "INSERT INTO playlist_tracks ( playlist_id, track_num, url, title, "
                             "album, artist, length, uniqueid ) VALUES ( %1, %2, '%3', '%4', '%5', "
@@ -206,30 +203,13 @@ SqlPlaylist::trackCount() const
 {
     if( m_tracksLoaded )
         return m_tracks.count();
-
-    QString query = "SELECT COUNT(id) FROM playlist_tracks WHERE playlist_id=%1;";
-
-    SqlStorage *sql = CollectionManager::instance()->sqlStorage();
-    Q_ASSERT( sql );
-
-    QStringList results = sql->query( query.arg( m_dbId ) );
-    debug() << results;
-    if( results.isEmpty() )
-    {
-        error() << "no results for COUNT query on playlist_tracks table!";
+    else
         return -1;
-    }
-    int trackCount = results.first().toInt();
-    return trackCount;
 }
 
 Meta::TrackList
 SqlPlaylist::tracks()
 {
-    //If you do not load before, m_tracks
-    //can be empty before usage.
-    triggerTrackLoad();
-
     return m_tracks;
 }
 
@@ -238,6 +218,7 @@ SqlPlaylist::triggerTrackLoad()
 {
     if( !m_tracksLoaded )
         loadTracks();
+    notifyObserversTracksLoaded();
 }
 
 void
@@ -275,7 +256,7 @@ SqlPlaylist::loadTracks()
                     "playlist_tracks WHERE playlist_id=%1 ORDER BY track_num";
     query = query.arg( QString::number( m_dbId ) );
 
-    QStringList result = CollectionManager::instance()->sqlStorage()->query( query );
+    QStringList result = StorageManager::instance()->sqlStorage()->query( query );
 
     int resultRows = result.count() / 7;
 
@@ -284,13 +265,12 @@ SqlPlaylist::loadTracks()
         QStringList row = result.mid( i*7, 7 );
         KUrl url = KUrl( row[2] );
 
-        MetaProxy::Track *proxyTrack = new MetaProxy::Track( url );
+        MetaProxy::TrackPtr proxyTrack( new MetaProxy::Track( url ) );
 
-        proxyTrack->setName( row[3] );
+        proxyTrack->setTitle( row[3] );
         proxyTrack->setAlbum( row[4] );
         proxyTrack->setArtist( row[5] );
-        Meta::TrackPtr trackPtr = Meta::TrackPtr( proxyTrack );
-        m_tracks << trackPtr;
+        m_tracks << Meta::TrackPtr( proxyTrack.data() );
     }
 
     m_tracksLoaded = true;
@@ -308,11 +288,11 @@ SqlPlaylist::removeFromDb()
 {
     QString query = "DELETE FROM playlist_tracks WHERE playlist_id=%1";
     query = query.arg( QString::number( m_dbId ) );
-    CollectionManager::instance()->sqlStorage()->query( query );
+    StorageManager::instance()->sqlStorage()->query( query );
 
     query = "DELETE FROM playlists WHERE id=%1";
     query = query.arg( QString::number( m_dbId ) );
-    CollectionManager::instance()->sqlStorage()->query( query );
+    StorageManager::instance()->sqlStorage()->query( query );
 }
 
 } //namespace Playlists
