@@ -16,9 +16,9 @@
 
 #include "SqlPlaylistGroup.h"
 
-#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/storage/StorageManager.h"
 #include "core/support/Debug.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core/storage/SqlStorage.h>
 
 #include <typeinfo>
 
@@ -62,7 +62,7 @@ SqlPlaylistGroup::save()
     if ( m_parent )
         parentId = m_parent->id();
 
-    SqlStorage* sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage* sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 
@@ -71,8 +71,8 @@ SqlPlaylistGroup::save()
         //update existing
         QString query = "UPDATE playlist_groups SET parent_id=%1, name='%2', \
                 description='%3' WHERE id=%4;";
-        query = query.arg( QString::number( parentId ) ).arg( m_name ).arg(
-            m_description ).arg( QString::number( m_dbId ) );
+        query = query.arg( QString::number( parentId ), m_name,
+                           m_description, QString::number( m_dbId ) );
         sqlStorage->query( query );
     }
     else
@@ -80,8 +80,7 @@ SqlPlaylistGroup::save()
         //insert new
         QString query = "INSERT INTO playlist_groups ( parent_id, name, \
                 description) VALUES ( %1, '%2', '%3' );";
-        query = query.arg( QString::number( parentId ) ).arg( m_name ).arg(
-            m_description );
+        query = query.arg( QString::number( parentId ), m_name, m_description );
         m_dbId = sqlStorage->insert( query, NULL );
     }
 }
@@ -103,7 +102,7 @@ SqlPlaylistGroup::setDescription( const QString &description )
 void
 SqlPlaylistGroup::removeFromDb()
 {
-    SqlStorage* sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage* sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
 
@@ -140,7 +139,7 @@ SqlPlaylistGroup::setParent( SqlPlaylistGroupPtr parent )
 SqlPlaylistGroupList
 SqlPlaylistGroup::childSqlGroups() const
 {
-    SqlStorage* sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage* sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return SqlPlaylistGroupList();
 
@@ -172,7 +171,7 @@ SqlPlaylistGroup::childSqlGroups() const
 SqlPlaylistList
 SqlPlaylistGroup::childSqlPlaylists() const
 {
-    SqlStorage* sqlStorage = CollectionManager::instance()->sqlStorage();
+    SqlStorage* sqlStorage = StorageManager::instance()->sqlStorage();
     if( !sqlStorage )
         return SqlPlaylistList();
 
