@@ -15,11 +15,14 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "ServiceAlbumCoverDownloader"
+
 #include "ServiceAlbumCoverDownloader.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
 #include "amarokconfig.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
+#include "covermanager/CoverCache.h"
 
 #include <QDir>
 #include <QImage>
@@ -29,28 +32,23 @@ using namespace Meta;
 
 Meta::ServiceAlbumWithCover::ServiceAlbumWithCover( const QString &name )
     : ServiceAlbum( name )
-    , m_cover( 0 )
     , m_hasFetchedCover( false )
     , m_isFetchingCover ( false )
-    , m_coverDownloader( 0 )
 {}
 
 Meta::ServiceAlbumWithCover::ServiceAlbumWithCover( const QStringList &resultRow )
     : ServiceAlbum( resultRow )
-    , m_cover( 0 )
     , m_hasFetchedCover( false )
     , m_isFetchingCover ( false )
-    , m_coverDownloader( 0 )
 {}
 
 Meta::ServiceAlbumWithCover::~ServiceAlbumWithCover()
 {
-    delete m_coverDownloader;
-    delete m_cover;
+    CoverCache::invalidateAlbum( this );
 }
 
-QPixmap
-ServiceAlbumWithCover::image( int size )
+QImage
+ServiceAlbumWithCover::image( int size ) const
 {
     if( size > 1000 )
     {
@@ -58,59 +56,52 @@ ServiceAlbumWithCover::image( int size )
         return Meta::Album::image( size );
     }
 
-    QString artist;
+    const QString artist = hasAlbumArtist() ?
+        albumArtist()->name() :
+        QLatin1String("NULL"); //no need to translate, only used as a caching key/temp filename
 
-    if ( hasAlbumArtist() )
-        artist = albumArtist()->name();
-    else
-        artist = "NULL"; //no need to translate, only used as a caching key/temp filename
-
-    QString coverName = downloadPrefix() + '_' + artist+ '_' + name() + "_cover.png";
-
-    QDir cacheCoverDir = QDir( Amarok::saveLocation( "albumcovers/cache/" ) );
+    const QString coverName = QString( "%1_%2_%3_cover.png" ).arg( downloadPrefix(), artist, name() );
+    const QString saveLocation = Amarok::saveLocation( "albumcovers/cache/" );
+    const QDir cacheCoverDir = QDir( saveLocation );
 
     //make sure that this dir exists
-    if ( !cacheCoverDir.exists() )
-        cacheCoverDir.mkpath( Amarok::saveLocation( "albumcovers/cache/" ) );
+    if( !cacheCoverDir.exists() )
+        cacheCoverDir.mkpath( saveLocation );
 
-    if ( size <= 1 )
+    if( size <= 1 )
         size = 100;
-    QString sizeKey = QString::number( size ) + '@';
 
-    QPixmap pixmap;
+    const QString sizeKey = QString::number( size ) + QLatin1Char('@');
+    const QString cacheCoverPath = cacheCoverDir.filePath( sizeKey + coverName );
 
-    if( QFile::exists( cacheCoverDir.filePath( sizeKey + coverName ) ) )
+    if( QFile::exists( cacheCoverPath ) )
     {
-        pixmap = QPixmap( cacheCoverDir.filePath( sizeKey + coverName ) );
-        return pixmap;
+        return QImage( cacheCoverPath );
     }
-    else if ( m_hasFetchedCover && m_cover && !m_cover->isNull() )
+    else if( m_hasFetchedCover && !m_cover.isNull() )
     {
-        pixmap = m_cover->scaled( size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
-        pixmap.save( cacheCoverDir.filePath( sizeKey + coverName ), "PNG" );
-        return pixmap;
-
+        QImage image( m_cover.scaled( size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation ) );
+        image.save( cacheCoverPath, "PNG" );
+        return image;
     }
-    else if ( !m_isFetchingCover && !coverUrl().isEmpty() )
+    else if( !m_isFetchingCover && !coverUrl().isEmpty() )
     {
         m_isFetchingCover = true;
 
-        if ( m_coverDownloader == 0 )
-            m_coverDownloader = new ServiceAlbumCoverDownloader();
-        m_coverDownloader->downloadCover( this );
-
+        ( new ServiceAlbumCoverDownloader )->downloadCover(
+            ServiceAlbumWithCoverPtr(const_cast<ServiceAlbumWithCover*>(this)) );
     }
 
-    return Album::image( size );
+    return Meta::Album::image( size );
 }
 
 void
-ServiceAlbumWithCover::setImage( const QPixmap& pixmap )
+ServiceAlbumWithCover::setImage( const QImage& image )
 {
-    delete m_cover;
-    m_cover = new QPixmap( pixmap );
+    m_cover = image;
     m_hasFetchedCover = true;
     m_isFetchingCover = false;
+    CoverCache::invalidateAlbum( this );
 
     notifyObservers();
 }
@@ -128,11 +119,10 @@ ServiceAlbumWithCover::imageDownloadCanceled() const
 ///////////////////////////////////////////////////////////////////////////////
 
 ServiceAlbumCoverDownloader::ServiceAlbumCoverDownloader()
-    : m_album( 0 )
-    , m_albumDownloadJob( 0 )
+    : m_albumDownloadJob( 0 )
 {
     m_tempDir = new KTempDir();
-    m_tempDir->setAutoRemove( false );
+    m_tempDir->setAutoRemove( true );
 }
 
 ServiceAlbumCoverDownloader::~ServiceAlbumCoverDownloader()
@@ -141,7 +131,7 @@ ServiceAlbumCoverDownloader::~ServiceAlbumCoverDownloader()
 }
 
 void
-ServiceAlbumCoverDownloader::downloadCover( ServiceAlbumWithCover * album )
+ServiceAlbumCoverDownloader::downloadCover( ServiceAlbumWithCoverPtr album )
 {
     m_album = album;
 
@@ -153,36 +143,37 @@ ServiceAlbumCoverDownloader::downloadCover( ServiceAlbumWithCover * album )
 
     m_albumDownloadJob = KIO::file_copy( downloadUrl, KUrl( m_coverDownloadPath ), -1, KIO::Overwrite | KIO::HideProgressInfo );
 
-    connect( m_albumDownloadJob, SIGNAL( result( KJob* ) ), SLOT( coverDownloadComplete( KJob* ) ) );
-    connect( m_albumDownloadJob, SIGNAL( canceled( KJob* ) ), SLOT( coverDownloadCanceled( KJob * ) ) );
+    connect( m_albumDownloadJob, SIGNAL(result(KJob*)), SLOT(coverDownloadComplete(KJob*)) );
+    connect( m_albumDownloadJob, SIGNAL(canceled(KJob*)), SLOT(coverDownloadCanceled(KJob*)) );
 }
 
 void
 ServiceAlbumCoverDownloader::coverDownloadComplete( KJob * downloadJob )
 {
+    if( !m_album ) // album was removed in between
+    {
+        debug() << "Bad album pointer";
+        return;
+    }
 
-
-    if( !downloadJob || !downloadJob->error() == 0 )
+    if( !downloadJob || downloadJob->error() != 0 )
     {
         debug() << "Download Job failed!";
 
         //we could not download, so inform album
-        if( m_album )
-            m_album->imageDownloadCanceled();
-
+        m_album->imageDownloadCanceled();
         return;
     }
 
     if ( downloadJob != m_albumDownloadJob )
         return; //not the right job, so let's ignore it
 
-    const QPixmap cover = QPixmap( m_coverDownloadPath );
+    const QImage cover = QImage( m_coverDownloadPath );
     if ( cover.isNull() )
     {
         debug() << "file not a valid image";
         //the file wasn't an image, so inform album
         m_album->imageDownloadCanceled();
-
         return;
     }
 
@@ -190,7 +181,7 @@ ServiceAlbumCoverDownloader::coverDownloadComplete( KJob * downloadJob )
 
     downloadJob->deleteLater();
 
-    m_tempDir->unlink();
+    deleteLater();
 }
 
 void
@@ -199,8 +190,12 @@ ServiceAlbumCoverDownloader::coverDownloadCanceled( KJob *downloadJob )
     Q_UNUSED( downloadJob );
     DEBUG_BLOCK
 
+    if( !m_album ) // album was removed in between
+        return;
+
     debug() << "Cover download cancelled";
     m_album->imageDownloadCanceled();
+    deleteLater();
 }
 
 #include "ServiceAlbumCoverDownloader.moc"
