@@ -17,6 +17,7 @@
 
 #include "ServiceMetaBase.h"
 
+#include "core/meta/Statistics.h"
 #include "core/support/Amarok.h"
 #include "core/support/Debug.h"
 #include "core-impl/meta/default/DefaultMetaTypes.h"
@@ -128,7 +129,6 @@ ServiceTrack::ServiceTrack( const QString & name )
     , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
-    , m_provider( 0 )
     , m_genre( 0 )
     , m_composer( 0 )
     , m_year( 0 )
@@ -148,7 +148,6 @@ ServiceTrack::ServiceTrack( const QStringList & resultRow )
     , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
-    , m_provider( 0 )
     , m_genre( 0 )
     , m_composer( 0 )
     , m_year( 0 )
@@ -168,7 +167,6 @@ ServiceTrack::ServiceTrack( const QStringList & resultRow )
 
 ServiceTrack::~ServiceTrack()
 {
-    delete m_provider;
 }
 
 void
@@ -239,6 +237,18 @@ ServiceTrack::prettyUrl() const
     return m_displayUrl;
 }
 
+QString
+ServiceTrack::notPlayableReason() const
+{
+    if( !m_playableUrl.isEmpty() && playableUrl().isLocalFile() )
+        return localFileNotPlayableReason( m_playableUrl );
+
+    if( !m_downloadableUrl.isEmpty() )
+        return networkNotPlayableReason();
+
+    return i18n( "Neither playable nor downloadable url specified." );
+}
+
 void
 ServiceTrack::setUidUrl(const QString & url)
 {
@@ -253,18 +263,6 @@ ServiceTrack::setDownloadableUrl(const QString & url)
     m_downloadableUrl = url;
 }
 
-bool
-ServiceTrack::isPlayable() const
-{
-    return true;
-}
-
-bool
-ServiceTrack::isEditable() const
-{
-    return false;
-}
-
 AlbumPtr
 ServiceTrack::album() const
 {
@@ -372,38 +370,6 @@ ServiceTrack::setDescription( const QString &newDescription )
     m_description = newDescription;
 }
 
-double
-ServiceTrack::score() const
-{
-    if( m_provider )
-        return m_provider->score();
-    else
-        return 0.0;
-}
-
-void
-ServiceTrack::setScore( double newScore )
-{
-    if( m_provider )
-        m_provider->setScore( newScore );
-}
-
-int
-ServiceTrack::rating() const
-{
-    if( m_provider )
-        return m_provider->rating();
-    else
-        return 0;
-}
-
-void
-ServiceTrack::setRating( int newRating )
-{
-    if( m_provider )
-        m_provider->setRating( newRating );
-}
-
 qint64
 ServiceTrack::length() const
 {
@@ -452,43 +418,6 @@ ServiceTrack::setDiscNumber( int newDiscNumber )
     Q_UNUSED( newDiscNumber )
 }
 
-int
-ServiceTrack::playCount() const
-{
-    if( m_provider )
-        return m_provider->playCount();
-    else
-        return 0;
-}
-
-QDateTime
-ServiceTrack::lastPlayed() const
-{
-    if( m_provider )
-        return m_provider->lastPlayed();
-    else
-        return QDateTime();
-}
-
-QDateTime
-ServiceTrack::firstPlayed() const
-{
-    if( m_provider )
-        return m_provider->firstPlayed();
-    else
-        return QDateTime();
-}
-
-void
-ServiceTrack::finishedPlaying( double playedFraction )
-{
-    if( m_provider )
-    {
-        m_provider->played( playedFraction, TrackPtr( this ) );
-        notifyObservers();
-    }
-}
-
 QString
 ServiceTrack::type() const
 {
@@ -527,12 +456,9 @@ ServiceTrack::setYear( YearPtr year )
 }
 
 void
-ServiceTrack::setStatisticsProvider( Statistics::StatisticsProvider *provider )
+ServiceTrack::setStatisticsProvider( StatisticsPtr provider )
 {
-    if( m_provider )
-        delete m_provider;
-
-    m_provider = provider;
+    m_statsStore = provider;
 }
 
 void
@@ -553,6 +479,15 @@ void ServiceTrack::processInfoOf(InfoParserBase * infoParser)
     infoParser->getInfo( TrackPtr( this ) );
 }
 
+StatisticsPtr
+ServiceTrack::statistics()
+{
+    if( m_statsStore )
+        return m_statsStore;
+    return Meta::Track::statistics();
+}
+
+
 
 //ServiceArtist
 ServiceArtist::ServiceArtist( const QString & name )
@@ -576,12 +511,9 @@ ServiceArtist::ServiceArtist(const QStringList & resultRow)
     , BookmarkThisProvider()
     , m_tracks()
 {
-
     m_id = resultRow[0].toInt();
     m_name = resultRow[1];
     m_description = resultRow[2];
-
-
 }
 
 ServiceArtist::~ServiceArtist()
@@ -620,13 +552,6 @@ ServiceArtist::tracks()
     return m_tracks;
 }
 
-AlbumList
-ServiceArtist::albums()
-{
-    //TODO
-    return Meta::AlbumList();
-}
-
 void
 ServiceArtist::addTrack( TrackPtr track )
 {
@@ -745,6 +670,17 @@ ServiceAlbum::isCompilation() const
     return m_isCompilation;
 }
 
+bool ServiceAlbum::canUpdateCompilation() const
+{
+    return true;
+}
+
+void
+ServiceAlbum::setCompilation( bool compilation )
+{
+    m_isCompilation = compilation;
+}
+
 bool
 ServiceAlbum::hasAlbumArtist() const
 {
@@ -775,12 +711,6 @@ ServiceAlbum::setAlbumArtist( ArtistPtr artist )
     m_albumArtist = artist;
 }
 
-void
-ServiceAlbum::setIsCompilation( bool compilation )
-{
-    m_isCompilation = compilation;
-}
-
 void ServiceAlbum::setDescription(const QString &description)
 {
     m_description = description;
