@@ -16,22 +16,48 @@
  ****************************************************************************************/
 
 #define DEBUG_PREFIX "ServicePluginManager"
- 
+
 #include "ServicePluginManager.h"
 
-#include "core/support/Amarok.h"
+#include "browsers/servicebrowser/ServiceBrowser.h"
 #include "core/support/Debug.h"
-#include "PluginManager.h"
-#include "ServiceBase.h"
-#include "ServiceBrowser.h"
+#include "services/ServiceBase.h"
 
 #include <KService>
 
-ServicePluginManager::ServicePluginManager( QObject *parent )
-    : QObject( parent )
-    , m_serviceBrowser( ServiceBrowser::instance() )
+#include <QSet>
+#include <QCoreApplication>
+
+ServicePluginManager *ServicePluginManager::s_instance = 0;
+
+ServicePluginManager *
+ServicePluginManager::instance()
+{
+    if( !s_instance ) {
+        s_instance = new ServicePluginManager();
+    }
+
+    return s_instance;
+}
+
+
+void
+ServicePluginManager::destroy()
+{
+    if( s_instance ) {
+        delete s_instance;
+        s_instance = 0;
+    }
+}
+
+
+ServicePluginManager::ServicePluginManager()
+    : QObject()
 {
     DEBUG_BLOCK
+    // ensure this object is created in a main thread
+    Q_ASSERT( thread() == QCoreApplication::instance()->thread() );
+
     setObjectName( "ServicePluginManager" );
 }
 
@@ -40,78 +66,37 @@ ServicePluginManager::~ServicePluginManager()
 {
 }
 
-ServiceBrowser *
-ServicePluginManager::browser() const
-{
-    return m_serviceBrowser;
-}
 
 void
-ServicePluginManager::setBrowser( ServiceBrowser * browser )
+ServicePluginManager::setFactories( const QList<Plugins::PluginFactory*> &factories )
 {
-    m_serviceBrowser = browser;
-}
+    QSet<Plugins::PluginFactory*> newFactories = factories.toSet();
+    QSet<Plugins::PluginFactory*> oldFactories = m_factories.toSet();
 
-void
-ServicePluginManager::init( const QList<Plugins::PluginFactory*> &factories )
-{
-    DEBUG_BLOCK
-    foreach( Plugins::PluginFactory* pFactory, factories )
-    {
-        ServiceFactory *factory = qobject_cast<ServiceFactory*>( pFactory );
-        if( !factory )
-            continue;
-        if( factory->isInitialized() )
-            continue;
-
-        //check if this service is enabled
-        QString pluginName = factory->info().pluginName();
-        debug() << "PLUGIN CHECK:" << pluginName;
-        if( Amarok::config( "Plugins" ).readEntry( pluginName + "Enabled", true ) )
-            initFactory( factory );
-    }
-}
-
-void
-ServicePluginManager::checkEnabledStates( const QList<Plugins::PluginFactory*> &factories )
-{
-    DEBUG_BLOCK
-    foreach( Plugins::PluginFactory* pFactory, factories )
+    // remove old factories
+    foreach( Plugins::PluginFactory* pFactory, oldFactories - newFactories )
     {
         ServiceFactory *factory = qobject_cast<ServiceFactory*>( pFactory );
         if( !factory )
             continue;
 
-        //check if this service is enabled
-        const QString pluginName = factory->info().pluginName();
-        bool enabledInConfig = Amarok::config( "Plugins" ).readEntry( pluginName + "Enabled", true );
-        bool isLastActive = !factory->activeServices().isEmpty();
-        if( enabledInConfig == isLastActive )
+        foreach( ServiceBase * service, factory->activeServices() )
+            ServiceBrowser::instance()->removeCategory( service );
+        factory->clearActiveServices();
+    }
+
+    // create new factories
+    foreach( Plugins::PluginFactory* pFactory, newFactories - oldFactories )
+    {
+        ServiceFactory *factory = qobject_cast<ServiceFactory*>( pFactory );
+        if( !factory )
             continue;
 
-        debug() << "PLUGIN CHECK:" << pluginName << enabledInConfig << isLastActive;
-        if( enabledInConfig && !isLastActive )
-        {
-            initFactory( factory );
-        }
-        else if( !enabledInConfig && isLastActive )
-        {
-            foreach( ServiceBase * service, factory->activeServices() )
-                m_serviceBrowser->removeCategory( service );
-            factory->clearActiveServices();
-        }
+        connect( factory, SIGNAL(newService(ServiceBase*)), SLOT(slotNewService(ServiceBase*)) );
+        connect( factory, SIGNAL(removeService(ServiceBase*)), SLOT(slotRemoveService(ServiceBase*)) );
     }
-}
 
-void
-ServicePluginManager::initFactory( ServiceFactory *factory )
-{
-    DEBUG_BLOCK
-    debug() << "initializing:" << factory->info().pluginName();
-    disconnect( factory, 0, this, 0 );
-    connect( factory, SIGNAL(newService(ServiceBase*)), SLOT(slotNewService(ServiceBase*)) );
-    connect( factory, SIGNAL(removeService(ServiceBase*)), SLOT(slotRemoveService(ServiceBase*)) );
-    factory->init();
+    m_factories = factories;
 }
 
 void
@@ -119,7 +104,7 @@ ServicePluginManager::slotNewService( ServiceBase *newService )
 {
     DEBUG_BLOCK
     debug() << "new service:" << newService->name();
-    m_serviceBrowser->addCategory( newService );
+    ServiceBrowser::instance()->addCategory( newService );
 }
 
 void
@@ -127,14 +112,14 @@ ServicePluginManager::slotRemoveService( ServiceBase *removedService )
 {
     DEBUG_BLOCK
     debug() << "removed service:" << removedService->name();
-    m_serviceBrowser->removeCategory( removedService );
+    ServiceBrowser::instance()->removeCategory( removedService );
 }
 
 QStringList
 ServicePluginManager::loadedServices() const
 {
     QStringList names;
-    foreach( Plugins::PluginFactory *pFactory, The::pluginManager()->factories(QLatin1String("Service")) )
+    foreach( Plugins::PluginFactory *pFactory, m_factories )
     {
         ServiceFactory *factory = qobject_cast<ServiceFactory*>( pFactory );
         if( !factory )
@@ -149,23 +134,23 @@ ServicePluginManager::loadedServices() const
 QStringList
 ServicePluginManager::loadedServiceNames() const
 {
-    return m_serviceBrowser->categories().keys();
+    return ServiceBrowser::instance()->categories().keys();
 }
 
 QString
 ServicePluginManager::serviceDescription( const QString & serviceName )
 {
     //get named service
-    if ( !m_serviceBrowser->categories().contains( serviceName ) )
+    if ( !ServiceBrowser::instance()->categories().contains( serviceName ) )
     {
         return i18n( "No service named %1 is currently loaded", serviceName );
     }
 
-    ServiceBase * service = dynamic_cast<ServiceBase *>( m_serviceBrowser->categories().value( serviceName ) );
+    ServiceBase * service = dynamic_cast<ServiceBase *>( ServiceBrowser::instance()->categories().value( serviceName ) );
 
     if ( service == 0 )
         return QString();
-    
+
     return service->shortDescription();
 }
 
@@ -173,12 +158,12 @@ QString
 ServicePluginManager::serviceMessages( const QString & serviceName )
 {
     //get named service
-    if ( !m_serviceBrowser->categories().contains( serviceName ) )
+    if ( !ServiceBrowser::instance()->categories().contains( serviceName ) )
     {
         return i18n( "No service named %1 is currently loaded", serviceName );
     }
 
-    ServiceBase * service = dynamic_cast<ServiceBase *>( m_serviceBrowser->categories().value( serviceName ) );
+    ServiceBase * service = dynamic_cast<ServiceBase *>( ServiceBrowser::instance()->categories().value( serviceName ) );
 
     if ( service == 0 )
         return QString();
@@ -190,12 +175,12 @@ QString
 ServicePluginManager::sendMessage( const QString & serviceName, const QString & message )
 {
     //get named service
-    if ( !m_serviceBrowser->categories().contains( serviceName ) )
+    if ( !ServiceBrowser::instance()->categories().contains( serviceName ) )
     {
         return i18n( "No service named %1 is currently loaded", serviceName );
     }
 
-    ServiceBase * service = dynamic_cast<ServiceBase *>( m_serviceBrowser->categories().value( serviceName ) );
+    ServiceBase * service = dynamic_cast<ServiceBase *>( ServiceBrowser::instance()->categories().value( serviceName ) );
 
     if ( service == 0 )
         return QString();
