@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2011 Sven Krohlas <sven@getamarok.com>                                 *
+ * Copyright (c) 2011, 2012 Sven Krohlas <sven@asbest-online.de>                        *
  * The Amazon store in based upon the Magnatune store in Amarok,                        *
  * Copyright (c) 2006,2007 Nikolaj Hald Nielsen <nhn@kde.org>                           *
  *                                                                                      *
@@ -25,12 +25,14 @@
 #include "AmazonShoppingCart.h"
 #include "AmazonShoppingCartDialog.h"
 #include "AmazonUrlRunner.h"
+#include "AmazonWantCountryWidget.h"
 
 #include "amarokurls/AmarokUrlHandler.h"
 #include "browsers/CollectionTreeItem.h"
 #include "browsers/SingleCollectionTreeItemModel.h"
 #include "core/interfaces/Logger.h"
 #include "core/support/Components.h"
+#include "core-impl/collections/support/CollectionManager.h"
 #include "playlist/PlaylistController.h"
 #include "widgets/SearchWidget.h"
 
@@ -65,7 +67,6 @@ AmazonServiceFactory::init()
 {
     DEBUG_BLOCK
     AmazonStore* service = new AmazonStore( this, "MP3 Music Store" );
-    m_activeServices << service;
     m_initialized = true;
     emit newService( service );
 }
@@ -91,6 +92,7 @@ AmazonServiceFactory::config()
 
 AmazonStore::AmazonStore( AmazonServiceFactory* parent, const char *name )
     : ServiceBase( name, parent, false )
+    , m_wantCountryWidget(0)
 {
     DEBUG_BLOCK
     setObjectName( name );
@@ -110,22 +112,21 @@ AmazonStore::AmazonStore( AmazonServiceFactory* parent, const char *name )
     m_collection = new Collections::AmazonCollection( this, "amazon", "MP3 Music Store" );
     polish();
     setPlayableTracks( true );
-    m_serviceready = true;
 
-    m_lastSearch = QString();
+    m_lastSearch.clear();
 
     // add the collection, exclude it from global queries
-    CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
+    CollectionManager::instance()->addTrackProvider( m_collection );
 
-    connect( m_searchWidget, SIGNAL( filterChanged( const QString ) ), this, SLOT( newSearchRequest( const QString ) ) );
+    connect( m_searchWidget, SIGNAL(filterChanged(QString)), this, SLOT(newSearchRequest(QString)) );
 
-    emit( ready() );
+    setServiceReady( true );
     newSearchRequest( QLatin1String( "" ) ); // to get some default content
 }
 
 AmazonStore::~AmazonStore()
 {
-    CollectionManager::instance()->removeUnmanagedCollection( m_collection );
+    CollectionManager::instance()->removeTrackProvider( m_collection );
     delete m_collection;
 }
 
@@ -138,14 +139,19 @@ AmazonStore::polish()
         m_polished = true;
 
         initTopPanel();
+        initBottomPanel();
         initView();
 
-        connect( m_itemView, SIGNAL( itemSelected( QModelIndex ) ), this, SLOT( itemSelected( QModelIndex ) ) );
-        connect( m_itemView, SIGNAL( itemDoubleClicked( QModelIndex ) ), this, SLOT( itemDoubleClicked( QModelIndex ) ) );
-        connect( m_itemView, SIGNAL( searchForAlbum( QModelIndex ) ), this, SLOT( searchForAlbum( QModelIndex ) ) );
+        connect( m_itemView, SIGNAL(itemSelected(QModelIndex)), this, SLOT(itemSelected(QModelIndex)) );
+        connect( m_itemView, SIGNAL(itemDoubleClicked(QModelIndex)), this, SLOT(itemDoubleClicked(QModelIndex)) );
+        connect( m_itemView, SIGNAL(searchForAlbum(QModelIndex)), this, SLOT(searchForAlbum(QModelIndex)) );
+
+        m_amazonInfoParser = new AmazonInfoParser();
+        setInfoParser( m_amazonInfoParser );
+        m_amazonInfoParser->showFrontPage();
 
         AmazonUrlRunner *runner = new AmazonUrlRunner();
-        connect( runner, SIGNAL( search( const QString ) ), this, SLOT( newSearchRequest( QString ) ) );
+        connect( runner, SIGNAL(search(QString)), this, SLOT(newSearchRequest(QString)) );
         The::amarokUrlHandler()->registerRunner( runner, runner->command() );
     }
 }
@@ -157,13 +163,12 @@ void
 AmazonStore::addToCart()
 {
     QString asin, name, price;
-    int id = 0;
+    int id = m_itemModel->idForIndex( m_selectedIndex );;
 
     // get item from collection
     if( m_itemModel->isAlbum( m_selectedIndex ) ) // album
     {
         Meta::AmazonAlbum* album;
-        id = m_itemModel->idForIndex( m_selectedIndex );
 
         album = dynamic_cast<Meta::AmazonAlbum*>( m_collection->albumById( id ).data() );
 
@@ -177,7 +182,6 @@ AmazonStore::addToCart()
     else // track
     {
         Meta::AmazonTrack* track;
-        id = m_itemModel->idForIndex( m_selectedIndex );
         track = dynamic_cast<Meta::AmazonTrack*>( m_collection->trackById( id ).data() );
 
         if( !track )
@@ -189,7 +193,6 @@ AmazonStore::addToCart()
     }
 
     AmazonShoppingCart::instance()->add( asin, price, name );
-    Amarok::Components::logger()->shortMessage( i18n( "<em>%1</em> has been added to your shopping cart.", name ) );
     m_checkoutButton->setEnabled( true );
 }
 
@@ -211,6 +214,8 @@ AmazonStore::checkout()
         m_checkoutButton->setEnabled( false );
         AmazonShoppingCart::instance()->clear();
     }
+
+    Amarok::Components::logger()->longMessage( i18n( "<b>MP3 Music Store</b><br/><br/>You are now being redirected to Amazon for the checkout process.<br/>To simplify that process please click <a href=\"%1\">this link</a> to tell Amazon that you have a downloader application for their MP3s installed.", Amazon::createCookieUrl().toString() ) );
 }
 
 void
@@ -245,12 +250,11 @@ AmazonStore::itemDoubleClicked( QModelIndex index )
     // for albums: search for the album ASIN to get details about it
     // for tracks: add it to the playlist
 
-    int id = 0;
+    int id = m_itemModel->idForIndex( index );
 
     if( m_itemModel->isAlbum( index ) ) // album
     {
         Meta::AmazonAlbum* album;
-        id = m_itemModel->idForIndex( index );
         album = dynamic_cast<Meta::AmazonAlbum*>( m_collection->albumById( id ).data() );
 
         if( !album )
@@ -261,7 +265,6 @@ AmazonStore::itemDoubleClicked( QModelIndex index )
     else // track
     {
         Meta::AmazonTrack* track;
-        id = m_itemModel->idForIndex( index );
         track = dynamic_cast<Meta::AmazonTrack*>( m_collection->trackById( id ).data() );
 
         if( !track )
@@ -269,7 +272,7 @@ AmazonStore::itemDoubleClicked( QModelIndex index )
 
         Meta::TrackPtr trackPtr( track );
 
-        The::playlistController()->instance()->insertOptioned( trackPtr, Playlist::Append );
+        The::playlistController()->instance()->insertOptioned( trackPtr, Playlist::OnDoubleClickOnSelectedItems );
     }
 }
 
@@ -278,6 +281,16 @@ AmazonStore::itemSelected( QModelIndex index )
 {
     m_addToCartButton->setEnabled( true );
     m_selectedIndex = index;
+
+    int id = m_itemModel->idForIndex( index );
+    Meta::AlbumPtr album;
+
+    if( m_itemModel->isAlbum( index ) )
+        album = m_collection->albumById( id ).data();
+    else // track
+        album = m_collection->trackById( id ).data()->album();
+
+    m_amazonInfoParser->getInfo( album );
 }
 
 void
@@ -285,21 +298,6 @@ AmazonStore::newSearchRequest( const QString request )
 {
     DEBUG_BLOCK
 
-    // make sure we know where to search
-    if( AmazonConfig::instance()->country().isEmpty() )
-    {
-        KCMultiDialog KCM;
-
-        KCM.setWindowTitle( i18n( "Select your Amazon locale - Amarok" ) );
-        KCM.addModule( KCModuleInfo( QString( "amarok_service_amazonstore_config.desktop" ) ) );
-        KCM.setButtons( KCMultiDialog::Ok | KCMultiDialog::Cancel | KCMultiDialog::Default );
-        KCM.adjustSize();
-
-        // if the user selects an option we continue our quest for search results
-        if( !(KCM.exec() == QDialog::Accepted) )
-            return;
-    }
-
     if( AmazonConfig::instance()->country() == QLatin1String( "none" ) || AmazonConfig::instance()->country().isEmpty() )
     {
         // user explicitly said we are not in a supported country or refused to supply one
@@ -335,14 +333,14 @@ AmazonStore::newSearchRequest( const QString request )
 
     if( !tempFile.open() )
     {
-        Amarok::Components::logger()->shortMessage( i18n( "Error: Unable to write temporary file. :-(" ) );
+        Amarok::Components::logger()->longMessage( i18n( "<b>MP3 Music Store</b><br/><br/>Error: Unable to write temporary file. :-(" ) );
         return;
     }
 
     m_searchWidget->searchStarted();
     KIO::FileCopyJob *requestJob = KIO::file_copy( requestUrl, KUrl( tempFile.fileName() ), 0700 , KIO::HideProgressInfo | KIO::Overwrite );
 
-    connect( requestJob, SIGNAL( result( KJob * ) ), this, SLOT( parseReply( KJob * ) ) );
+    connect( requestJob, SIGNAL(result(KJob*)), this, SLOT(parseReply(KJob*)) );
     requestJob->start();
 }
 
@@ -423,8 +421,8 @@ AmazonStore::initTopPanel()
     navigationToolbar->setToolButtonStyle( Qt::ToolButtonIconOnly );
     navigationToolbar->setIconDimensions( 16 );
 
-    m_backwardAction = KStandardAction::back( this, SLOT( back() ), topPanel );
-    m_forwardAction = KStandardAction::forward( this, SLOT( forward() ), topPanel );
+    m_backwardAction = KStandardAction::back( this, SLOT(back()), topPanel );
+    m_forwardAction = KStandardAction::forward( this, SLOT(forward()), topPanel );
     m_backwardAction->setEnabled( false );
     m_forwardAction->setEnabled( false );
 
@@ -440,7 +438,19 @@ AmazonStore::initTopPanel()
     navigationToolbar->addAction( m_forwardAction );
     m_searchWidget->toolBar()->addWidget( m_resultpageSpinBox );
 
-    connect( m_resultpageSpinBox, SIGNAL( valueChanged( int ) ), this, SLOT( newSpinBoxSearchRequest( int ) ) );
+    connect( m_resultpageSpinBox, SIGNAL(valueChanged(int)), this, SLOT(newSpinBoxSearchRequest(int)) );
+}
+
+void
+AmazonStore::initBottomPanel()
+{
+    QString country(AmazonConfig::instance()->country());
+    if( country.isEmpty() || country == QLatin1String( "none" ) )
+    {
+        m_wantCountryWidget = new AmazonWantCountryWidget(m_bottomPanel);
+        connect(m_wantCountryWidget, SIGNAL(countrySelected()),
+                SLOT(countryUpdated()));
+    }
 }
 
 void
@@ -481,11 +491,30 @@ AmazonStore::initView()
     m_checkoutButton->setParent( bottomPanelLayout );
     m_checkoutButton->setIcon( KIcon( "download-amarok" ) );
 
-    connect( m_addToCartButton, SIGNAL( clicked() ), this, SLOT( addToCart() ) );
-    connect( m_itemView, SIGNAL( addToCart() ), this, SLOT( addToCart() ) );
-    connect( m_itemView, SIGNAL( directCheckout() ), this, SLOT( directCheckout() ) );
-    connect( m_viewCartButton, SIGNAL( clicked() ), this, SLOT( viewCart() ) );
-    connect( m_checkoutButton, SIGNAL( clicked() ), this, SLOT( checkout() ) );
+    connect( m_addToCartButton, SIGNAL(clicked()), this, SLOT(addToCart()) );
+    connect( m_itemView, SIGNAL(addToCart()), this, SLOT(addToCart()) );
+    connect( m_itemView, SIGNAL(directCheckout()), this, SLOT(directCheckout()) );
+    connect( m_viewCartButton, SIGNAL(clicked()), this, SLOT(viewCart()) );
+    connect( m_checkoutButton, SIGNAL(clicked()), this, SLOT(checkout()) );
+}
+
+QString AmazonStore::iso3166toAmazon( const QString& country )
+{
+    static QHash<QString, QString> table;
+    if( table.isEmpty() )
+    {
+        table["at"] = "de";
+        table["ch"] = "de";
+        table["de"] = "de";
+        table["es"] = "es";
+        table["fr"] = "fr";
+        table["it"] = "it";
+        table["jp"] = "co.jp";
+        table["gb"] = "co.uk";
+        table["us"] = "com";
+    }
+
+    return table.value( country, "none" );
 }
 
 /* private slots */
@@ -496,7 +525,7 @@ AmazonStore::parseReply( KJob* requestJob )
     DEBUG_BLOCK
     if( requestJob->error() )
     {
-        Amarok::Components::logger()->shortMessage( i18n( "Error: Querying MP3 Music Store database failed. :-(" ) );
+        Amarok::Components::logger()->longMessage( i18n( "<b>MP3 Music Store</b><br/><br/>Error: Querying MP3 Music Store database failed. :-(" ) );
         debug() << requestJob->errorString();
         requestJob->deleteLater();
         m_searchWidget->searchEnded();
@@ -511,8 +540,8 @@ AmazonStore::parseReply( KJob* requestJob )
 
     // create parser thread
     AmazonParser *parser = new AmazonParser( tempFileName, m_collection, m_metaFactory );
-    connect( parser, SIGNAL( done( ThreadWeaver::Job* ) ), this, SLOT( parsingDone( ThreadWeaver::Job* ) ) );
-    connect( parser, SIGNAL( failed( ThreadWeaver::Job* ) ), this, SLOT( parsingFailed( ThreadWeaver::Job* ) ) );
+    connect( parser, SIGNAL(done(ThreadWeaver::Job*)), this, SLOT(parsingDone(ThreadWeaver::Job*)) );
+    connect( parser, SIGNAL(failed(ThreadWeaver::Job*)), this, SLOT(parsingFailed(ThreadWeaver::Job*)) );
     ThreadWeaver::Weaver::instance()->enqueue( parser );
 
     requestJob->deleteLater();
@@ -531,7 +560,7 @@ void
 AmazonStore::parsingFailed( ThreadWeaver::Job* parserJob )
 {
     Q_UNUSED( parserJob )
-    Amarok::Components::logger()->shortMessage( i18n( "Error: Received an invalid reply. :-(" ) );
+    Amarok::Components::logger()->longMessage( i18n( "<b>MP3 Music Store</b><br/><br/>Error: Received an invalid reply. :-(" ) );
     m_searchWidget->searchEnded();
 }
 
@@ -558,3 +587,19 @@ AmazonStore::forward()
     m_isNavigation = true;
     m_searchWidget->setSearchString( request );
 }
+
+void
+AmazonStore::countryUpdated()
+{
+    QString country( AmazonConfig::instance()->country() );
+    if( country.isEmpty() || country == QLatin1String( "none" ) )
+        return;
+
+    if( m_wantCountryWidget )
+    {
+        m_wantCountryWidget->setParent( 0 );
+        m_wantCountryWidget->deleteLater();
+        m_wantCountryWidget = 0;
+    }
+    newSearchRequest( QString() );
+}
