@@ -26,7 +26,7 @@
 #include "core/interfaces/Logger.h"
 #include "browsers/SingleCollectionTreeItemModel.h"
 #include "core-impl/collections/support/CollectionManager.h"
-#include <config-amarok.h>
+#include <config.h>
 #include "core/support/Debug.h"
 
 #ifdef HAVE_LIBLASTFM
@@ -56,9 +56,6 @@ void AmpacheServiceFactory::init()
     {
         AmpacheServerEntry server = servers.at( i );
         ServiceBase* service = new AmpacheService( this, "Ampache (" + server.name + ')', server.url, server. username, server.password );
-        m_activeServices << service;
-        debug() << "Emitting service!!!!!!";
-        connect( service, SIGNAL( ready() ), this, SLOT( slotServiceReady() ) );
         emit newService( service );
     }
 }
@@ -108,7 +105,7 @@ AmpacheService::AmpacheService( AmpacheServiceFactory* parent, const QString & n
 
 AmpacheService::~AmpacheService()
 {
-    CollectionManager::instance()->removeUnmanagedCollection( m_collection );
+    CollectionManager::instance()->removeTrackProvider( m_collection );
     delete m_collection;
     m_ampacheLogin->deleteLater();
 }
@@ -127,6 +124,8 @@ void
 AmpacheService::reauthenticate()
 {
     m_ampacheLogin->reauthenticate();
+    // it would make sense here to clean the complete cache
+    // information from a server might get outdated.
 }
 
 
@@ -134,14 +133,13 @@ void
 AmpacheService::onLoginSuccessful()
 {
     m_collection = new Collections::AmpacheServiceCollection( this, m_ampacheLogin->server(), m_ampacheLogin->sessionId() );
-    // connect( m_collection, SIGNAL( authenticationNeeded() ), SLOT( authenticate() ) );
-    
-    CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
+    // connect( m_collection, SIGNAL(authenticationNeeded()), SLOT(authenticate()) );
+
+    CollectionManager::instance()->addTrackProvider( m_collection );
     QList<CategoryId::CatMenuId> levels;
     levels << CategoryId::Artist << CategoryId::Album;
     setModel( new SingleCollectionTreeItemModel( m_collection, levels ) );
-    m_serviceready = true;
-    emit ready();
+    setServiceReady( true );
 }
 
 #include "AmpacheService.moc"
