@@ -1,5 +1,6 @@
 /****************************************************************************************
  * Copyright (c) 2007 Shane King <kde@dontletsstart.com>                                *
+ * Copyright (c) 2013 Vedant Agarwala <vedant.kota@gmail.com>                           *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -18,24 +19,21 @@
 
 #include "LastFmServiceSettings.h"
 
+#include "core/collections/QueryMaker.h"
+#include "core/meta/Meta.h"
 #include "core/support/Amarok.h"
 #include "core/support/Debug.h"
-#include "NetworkAccessManagerProxy.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "network/NetworkAccessManagerProxy.h"
 #include "ui_LastFmConfigWidget.h"
 
-#include <lastfm/Audioscrobbler> // from liblastfm
-#include <lastfm/ws.h>
-#include <lastfm/XmlQuery>
-
-#include <QCryptographicHash>
-#include <QNetworkAccessManager>
-#include <QNetworkReply>
-#include <QVBoxLayout>
-#include <QRegExpValidator>
-
 #include <KMessageBox>
 #include <KPluginFactory>
 
+#include <QCryptographicHash>
+#include <QRegExpValidator>
+
+#include <XmlQuery.h>
 
 K_PLUGIN_FACTORY( LastFmServiceSettingsFactory, registerPlugin<LastFmServiceSettings>(); )
 K_EXPORT_PLUGIN( LastFmServiceSettingsFactory( "kcm_amarok_lastfm" ) )
@@ -49,42 +47,58 @@ QString md5( const QByteArray& src )
 
 LastFmServiceSettings::LastFmServiceSettings( QWidget *parent, const QVariantList &args )
     : KCModule( LastFmServiceSettingsFactory::componentData(), parent, args )
+    , m_config( LastFmServiceConfig::instance() )
 {
-    debug() << "Creating Last.fm config object";
-
-    QVBoxLayout* l = new QVBoxLayout( this );
-    QWidget *w = new QWidget;
     m_configDialog = new Ui::LastFmConfigWidget;
-    m_configDialog->setupUi( w );
-    l->addWidget( w );
+    m_configDialog->setupUi( this );
 
-    // whenever this gets opened, we'll assume the user wants to change something,
-    // so blow away the saved session key
-    m_config.clearSessionKey();
+    connect( m_config.data(), SIGNAL(updated()), SLOT(onConfigUpdated()) );
 
-    connect( m_configDialog->kcfg_ScrobblerUsername, SIGNAL( textChanged( const QString & ) ), this, SLOT( settingsChanged() ) );
-    connect( m_configDialog->kcfg_ScrobblerPassword, SIGNAL( textChanged( const QString & ) ), this, SLOT( settingsChanged() ) );
-    connect( m_configDialog->kcfg_SubmitPlayedSongs, SIGNAL( stateChanged( int ) ), this, SLOT( settingsChanged() ) );
-    connect( m_configDialog->kcfg_RetrieveSimilarArtists, SIGNAL( stateChanged( int ) ), this, SLOT( settingsChanged() ) );
-    connect( m_configDialog->testLogin, SIGNAL( clicked() ), this, SLOT( testLogin() ) );
+    connect( m_configDialog->kcfg_ScrobblerUsername, SIGNAL(textChanged(QString)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_ScrobblerPassword, SIGNAL(textChanged(QString)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_SubmitPlayedSongs, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_RetrieveSimilarArtists, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_ScrobbleComposer, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+    connect( m_configDialog->kcfg_UseFancyRatingTags, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+    connect( m_configDialog->kcfg_AnnounceCorrections, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_FilterByLabel, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->kcfg_FilteredLabel, SIGNAL(currentIndexChanged(QString)), this, SLOT(settingsChanged()) );
+    connect( m_configDialog->testLogin, SIGNAL(clicked()), this, SLOT(testLogin()) );
 
-    load();
+    using namespace Collections;
+
+    QueryMaker *query = CollectionManager::instance()->queryMaker();
+    query->setQueryType( QueryMaker::Label );
+    connect( query, SIGNAL(newResultReady(Meta::LabelList)), this, SLOT(addNewLabels(Meta::LabelList)) );
+    query->setAutoDelete( true );
+    query->run();
 }
 
-
 LastFmServiceSettings::~LastFmServiceSettings()
 {
+    delete m_configDialog;
 }
 
-
 void
 LastFmServiceSettings::save()
 {
-    m_config.setUsername( m_configDialog->kcfg_ScrobblerUsername->text() );
-    m_config.setPassword( m_configDialog->kcfg_ScrobblerPassword->text() );
-    m_config.setScrobble( m_configDialog->kcfg_SubmitPlayedSongs->isChecked() );
-    m_config.setFetchSimilar( m_configDialog->kcfg_RetrieveSimilarArtists->isChecked() );
-    m_config.save();
+    QString dialogUser = m_configDialog->kcfg_ScrobblerUsername->text();
+    QString dialogPass = m_configDialog->kcfg_ScrobblerPassword->text();
+
+    // clear the session key if credentials changed
+    if( m_config->username() != dialogUser || m_config->password() != dialogPass )
+        m_config->setSessionKey( QString() );
+
+    m_config->setUsername( dialogUser );
+    m_config->setPassword( dialogPass );
+    m_config->setScrobble( m_configDialog->kcfg_SubmitPlayedSongs->isChecked() );
+    m_config->setFetchSimilar( m_configDialog->kcfg_RetrieveSimilarArtists->isChecked() );
+    m_config->setScrobbleComposer( m_configDialog->kcfg_ScrobbleComposer->isChecked() );
+    m_config->setUseFancyRatingTags( m_configDialog->kcfg_UseFancyRatingTags->isChecked() );
+    m_config->setAnnounceCorrections( m_configDialog->kcfg_AnnounceCorrections->isChecked() );
+    m_config->setFilterByLabel( m_configDialog->kcfg_FilterByLabel->isChecked() );
+    m_config->setFilteredLabel( m_configDialog->kcfg_FilteredLabel->currentText() );
+    m_config->save();
 
     KCModule::save();
 }
@@ -92,14 +106,12 @@ LastFmServiceSettings::save()
 void
 LastFmServiceSettings::testLogin()
 {
-    DEBUG_BLOCK
-
     m_configDialog->testLogin->setEnabled( false );
     m_configDialog->testLogin->setText( i18n( "Testing..." ) );
     // set the global static Lastfm::Ws stuff
     lastfm::ws::ApiKey = Amarok::lastfmApiKey();
-    lastfm::ws::SharedSecret = "fe0dcde9fcd14c2d1d50665b646335e9";
-    lastfm::ws::Username = qstrdup( m_configDialog->kcfg_ScrobblerUsername->text().toLatin1().data() );
+    lastfm::ws::SharedSecret = Amarok::lastfmApiSharedSecret();
+    lastfm::ws::Username = m_configDialog->kcfg_ScrobblerUsername->text();
     if( lastfm::nam() != The::networkAccessManager() )
         lastfm::setNetworkAccessManager( The::networkAccessManager() );
 
@@ -115,16 +127,15 @@ LastFmServiceSettings::testLogin()
     query[ "authToken" ] = authToken;
     m_authQuery = lastfm::ws::post( query );
 
-    connect( m_authQuery, SIGNAL( finished() ), SLOT( onAuthenticated() ) );
-    connect( m_authQuery, SIGNAL( error( QNetworkReply::NetworkError ) ), SLOT( onError( QNetworkReply::NetworkError ) ) );
+    connect( m_authQuery, SIGNAL(finished()), SLOT(onAuthenticated()) );
+    connect( m_authQuery, SIGNAL(error(QNetworkReply::NetworkError)), SLOT(onError(QNetworkReply::NetworkError)) );
 }
 
 void
 LastFmServiceSettings::onAuthenticated()
 {
-    DEBUG_BLOCK
-
-    lastfm::XmlQuery lfm = lastfm::XmlQuery( m_authQuery->readAll() );
+    lastfm::XmlQuery lfm;
+    lfm.parse( m_authQuery->readAll() );
 
     switch( m_authQuery->error() )
     {
@@ -160,8 +171,6 @@ LastFmServiceSettings::onAuthenticated()
 void
 LastFmServiceSettings::onError( QNetworkReply::NetworkError code )
 {
-    DEBUG_BLOCK
-
     if( code == QNetworkReply::NoError )
         return;
 
@@ -179,34 +188,43 @@ LastFmServiceSettings::onError( QNetworkReply::NetworkError code )
     m_authQuery->deleteLater();
 }
 
+void
+LastFmServiceSettings::onConfigUpdated()
+{
+    load();
+}
 
 void
 LastFmServiceSettings::load()
 {
-    m_config.load();
-    m_configDialog->kcfg_ScrobblerUsername->setText( m_config.username() );
-    m_configDialog->kcfg_ScrobblerPassword->setText( m_config.password() );
-    m_configDialog->kcfg_SubmitPlayedSongs->setChecked( m_config.scrobble() );
-    m_configDialog->kcfg_RetrieveSimilarArtists->setChecked( m_config.fetchSimilar() );
+    m_configDialog->kcfg_ScrobblerUsername->setText( m_config->username() );
+    m_configDialog->kcfg_ScrobblerPassword->setText( m_config->password() );
+    m_configDialog->kcfg_SubmitPlayedSongs->setChecked( m_config->scrobble() );
+    m_configDialog->kcfg_RetrieveSimilarArtists->setChecked( m_config->fetchSimilar() );
+    m_configDialog->kcfg_ScrobbleComposer->setChecked( m_config->scrobbleComposer() );
+    m_configDialog->kcfg_UseFancyRatingTags->setChecked( m_config->useFancyRatingTags() );
+    m_configDialog->kcfg_AnnounceCorrections->setChecked( m_config->announceCorrections() );
+    m_configDialog->kcfg_FilterByLabel->setChecked( m_config->filterByLabel() );
+    m_configDialog->kcfg_FilteredLabel->setCurrentIndex( filteredLabelComboIndex( m_config->filteredLabel() ) );
 
-    if( !m_config.username().isEmpty() && !m_config.password().isEmpty() )
+    if( !m_config->username().isEmpty() && !m_config->password().isEmpty() )
         m_configDialog->kcfg_SubmitPlayedSongs->setEnabled( true );
 
     KCModule::load();
 }
 
-
 void
 LastFmServiceSettings::defaults()
 {
-    m_config.reset();
-
-    // By default this checkboxes is:
-    m_configDialog->kcfg_SubmitPlayedSongs->setChecked( false );
-    m_configDialog->kcfg_RetrieveSimilarArtists->setChecked( false );
+    m_configDialog->kcfg_SubmitPlayedSongs->setChecked( m_config->defaultScrobble() );
+    m_configDialog->kcfg_RetrieveSimilarArtists->setChecked( m_config->defaultFetchSimilar() );
+    m_configDialog->kcfg_ScrobbleComposer->setChecked( m_config->defaultScrobbleComposer() );
+    m_configDialog->kcfg_UseFancyRatingTags->setChecked( m_config->defaultUseFancyRatingTags() );
+    m_configDialog->kcfg_AnnounceCorrections->setChecked( m_config->defaultAnnounceCorrections() );
+    m_configDialog->kcfg_FilterByLabel->setChecked( m_config->defaultFilterByLabel() );
+    m_configDialog->kcfg_FilteredLabel->setCurrentIndex( filteredLabelComboIndex( m_config->defaultFilteredLabel() ) );
 }
 
-
 void
 LastFmServiceSettings::settingsChanged()
 {
@@ -214,6 +232,27 @@ LastFmServiceSettings::settingsChanged()
     //with error reporting
 
     m_configDialog->testLogin->setText( i18n( "&Test Login" ) );
+    m_configDialog->testLogin->setEnabled( true );
 
     emit changed( true );
 }
+
+void
+LastFmServiceSettings::addNewLabels( const Meta::LabelList &labels )
+{
+    foreach( const Meta::LabelPtr &label , labels )
+        m_configDialog->kcfg_FilteredLabel->addItem( label->name() );
+}
+
+int
+LastFmServiceSettings::filteredLabelComboIndex( const QString &label )
+{
+    int index = m_configDialog->kcfg_FilteredLabel->findText( label );
+    if( index == -1)
+    {
+        m_configDialog->kcfg_FilteredLabel->addItem( label );
+        return m_configDialog->kcfg_FilteredLabel->findText( label );
+    }
+    else
+        return index;
+}
