@@ -18,14 +18,9 @@
 #ifndef LASTFM_BIAS_H
 #define LASTFM_BIAS_H
 
-#include "dynamic/Bias.h"
-#include "dynamic/BiasFactory.h"
 #include "dynamic/biases/TagMatchBias.h"
 
 #include <QMutex>
-#include <QNetworkReply>
-
-class KJob;
 
 namespace Dynamic
 {
@@ -56,9 +51,8 @@ namespace Dynamic
 
             virtual QWidget* widget( QWidget* parent = 0 );
 
-            virtual Dynamic::TrackSet matchingTracks( int position,
-                                                      const Meta::TrackList& playlist,
-                                                      int contextCount,
+            virtual Dynamic::TrackSet matchingTracks( const Meta::TrackList& playlist,
+                                                      int contextCount, int finalCount,
                                                       Dynamic::TrackCollectionPtr universe ) const;
 
             virtual bool trackMatches( int position,
@@ -78,8 +72,9 @@ namespace Dynamic
 
             void similarArtistQueryDone();
             void similarTrackQueryDone();
+            void queryFailed( const char *message );
 
-            void selectionChanged( int );
+            void setMatchTypeArtist( bool matchArtist );
 
         private:
             /** The pair is used for the tracks */
@@ -93,12 +88,10 @@ namespace Dynamic
             void readSimilarArtists( QXmlStreamReader *reader );
             TitleArtistPair readTrack( QXmlStreamReader *reader );
             void readSimilarTracks( QXmlStreamReader *reader );
-            void loadFromFile();
+            void loadDataFromFile();
 
             mutable QString m_currentArtist;
             mutable QString m_currentTrack;
-            QNetworkReply* m_artistQuery;
-            QNetworkReply* m_trackQuery;
 
             MatchType m_match;
 
