@@ -17,36 +17,18 @@
  ****************************************************************************************/
 
 #include "LastFmMeta.h"
-#include "LastFmMeta_p.h"
-#include "LastFmMeta_p.moc"
-#include "LastFmCapabilityImpl_p.h"
-#include "LastFmCapabilityImpl_p.moc"
-#include "MultiPlayableCapabilityImpl_p.h"
-#include "MultiPlayableCapabilityImpl_p.moc"
-#include "CurrentTrackActionsCapabilityImpl_p.h"
-#include "CurrentTrackActionsCapabilityImpl_p.moc"
-#include "ServiceCapabilities.h"
 
-#include "LastFmService.h"
-#include "LastFmStreamInfoCapability.h"
-#include "ScrobblerAdapter.h"
+#include "EngineController.h"
+#include "services/lastfm/meta/LastFmMeta_p.h"
+#include "services/lastfm/meta/LastFmMultiPlayableCapability.h"
+#include "services/lastfm/meta/LastFmStreamInfoCapability.h"
 
-#include "Debug.h"
+#include <KIcon>
 
-#include <QPointer>
+#include <QCoreApplication>
+#include <QThread>
 
-#include <KSharedPtr>
-#include <KStandardDirs>
-
-#include <lastfm/Track>
-
-namespace LastFm {
-
-class LastFmArtist;
-class LastFmAlbum;
-class LastFmGenre;
-class LastFmComposer;
-class LastFmYear;
+using namespace LastFm;
 
 Track::Track( const QString &lastFmUri )
     : QObject()
@@ -56,7 +38,6 @@ Track::Track( const QString &lastFmUri )
     d->lastFmUri = QUrl( lastFmUri );
     d->t = this;
 
-
     init();
 }
 
@@ -75,7 +56,7 @@ Track::Track( lastfm::Track track )
 
     d->trackFetch = lastfm::ws::post( params );
 
-    connect( d->trackFetch, SIGNAL( finished() ), SLOT( slotResultReady() ) );
+    connect( d->trackFetch, SIGNAL(finished()), SLOT(slotResultReady()) );
 }
 
 
@@ -90,23 +71,31 @@ void Track::init( int id /* = -1*/ )
         d->lastFmUri = QUrl( "lastfm://play/tracks/" + QString::number( id ) );
     d->length = 0;
 
-    d->albumPtr = Meta::AlbumPtr( new LastFmAlbum( QPointer<Track::Private>( d ) ) );
-    d->artistPtr = Meta::ArtistPtr( new LastFmArtist( QPointer<Track::Private>( d ) ) );
-    d->genrePtr = Meta::GenrePtr( new LastFmGenre( QPointer<Track::Private>( d ) ) );
-    d->composerPtr = Meta::ComposerPtr( new LastFmComposer( QPointer<Track::Private>( d ) ) );
-    d->yearPtr = Meta::YearPtr( new LastFmYear( QPointer<Track::Private>( d ) ) );
+    d->albumPtr = Meta::AlbumPtr( new LastFmAlbum( d ) );
+    d->artistPtr = Meta::ArtistPtr( new LastFmArtist( d ) );
+    d->genrePtr = Meta::GenrePtr( new LastFmGenre( d ) );
+    d->composerPtr = Meta::ComposerPtr( new LastFmComposer( d ) );
+    d->yearPtr = Meta::YearPtr( new LastFmYear( d ) );
 
-    QAction * banAction = new QAction( KIcon( "remove-amarok" ), i18n( "Last.fm: &Ban" ), this );
+    QAction *banAction = new QAction( KIcon( "remove-amarok" ), i18n( "Last.fm: &Ban" ), this );
     banAction->setShortcut( i18n( "Ctrl+B" ) );
     banAction->setStatusTip( i18n( "Ban this track" ) );
-    connect( banAction, SIGNAL( triggered() ), this, SLOT( ban() ) );
-    m_currentTrackActions.append( banAction );
+    connect( banAction, SIGNAL(triggered()), this, SLOT(ban()) );
+    m_trackActions.append( banAction );
 
-    QAction * skipAction = new QAction( KIcon( "media-seek-forward-amarok" ), i18n( "Last.fm: &Skip" ), this );
+    QAction *skipAction = new QAction( KIcon( "media-seek-forward-amarok" ), i18n( "Last.fm: &Skip" ), this );
     skipAction->setShortcut( i18n( "Ctrl+S" ) );
     skipAction->setStatusTip( i18n( "Skip this track" ) );
-    connect( skipAction, SIGNAL( triggered() ), this, SLOT( skip() ) );
-    m_currentTrackActions.append( skipAction );
+    connect( skipAction, SIGNAL(triggered()), this, SIGNAL(skipTrack()) );
+    m_trackActions.append( skipAction );
+
+    QThread *mainThread = QCoreApplication::instance()->thread();
+    bool foreignThread = QThread::currentThread() != mainThread;
+    if( foreignThread )
+    {
+        moveToThread( mainThread ); // the actions are children and are moved together with parent
+        d->moveToThread( mainThread );
+    }
 }
 
 QString
@@ -122,21 +111,6 @@ Track::name() const
     }
 }
 
-QString
-Track::prettyName() const
-{
-    return name();
-}
-
-QString
-Track::fullPrettyName() const
-{
-    if( d->track.isEmpty() || d->artist.isEmpty() )
-        return prettyName();
-    else
-        return i18n("%1 - %2", d->artist, d->track );
-}
-
 QString
 Track::sortableName() const
 {
@@ -144,17 +118,6 @@ Track::sortableName() const
     return name();
 }
 
-QString LastFm::Track::fixedName() const
-{
-
-    //unless this is a stream for a single track, return the generic name
-    if ( streamName() != i18n( "Track Radio" ) )
-        return streamName();
-    else
-        return name();
-}
-
-
 KUrl
 Track::playableUrl() const
 {
@@ -179,11 +142,10 @@ Track::uidUrl() const
     return d->lastFmUri.toString();
 }
 
-bool
-Track::isPlayable() const
+QString
+Track::notPlayableReason() const
 {
-    //we could check connectivity here...
-    return !d->trackPath.isEmpty();
+    return networkNotPlayableReason();
 }
 
 Meta::AlbumPtr
@@ -216,7 +178,7 @@ Track::year() const
     return d->yearPtr;
 }
 
-float
+qreal
 Track::bpm() const
 {
     return -1.0;
@@ -228,44 +190,6 @@ Track::comment() const
     return QString();
 }
 
-double
-Track::score() const
-{
-    if( d->statisticsProvider )
-        return d->statisticsProvider->score();
-    else
-        return 0.0;
-}
-
-void
-Track::setScore( double newScore )
-{
-    if( d->statisticsProvider )
-    {
-        d->statisticsProvider->setScore( newScore );
-        notifyObservers();
-    }
-}
-
-int
-Track::rating() const
-{
-    if( d->statisticsProvider )
-        return d->statisticsProvider->rating();
-    else
-        return 0;
-}
-
-void
-Track::setRating( int newRating )
-{
-    if( d->statisticsProvider )
-    {
-        d->statisticsProvider->setRating( newRating );
-        notifyObservers();
-    }
-}
-
 int
 Track::trackNumber() const
 {
@@ -302,58 +226,11 @@ Track::bitrate() const
     return 0; //does the engine deliver this??
 }
 
-uint
-Track::lastPlayed() const
-{
-    if( d->statisticsProvider )
-    {
-        QDateTime dt = d->statisticsProvider->lastPlayed();
-        if( dt.isValid() )
-            return dt.toTime_t();
-        else
-            return 0;
-    }
-    else
-        return 0;
-}
-
-uint
-Track::firstPlayed() const
-{
-    if( d->statisticsProvider )
-    {
-        QDateTime dt = d->statisticsProvider->firstPlayed();
-        if( dt.isValid() )
-            return dt.toTime_t();
-        else
-            return 0;
-    }
-    else
-        return 0;
-}
-
-int
-Track::playCount() const
-{
-    if( d->statisticsProvider )
-        return d->statisticsProvider->playCount();
-    return 0;
-}
-
 QString
 Track::type() const
 {
     return "stream/lastfm";
 }
-void
-Track::finishedPlaying( double playedFraction )
-{
-    if( d->statisticsProvider )
-    {
-        d->statisticsProvider->played( playedFraction );
-        notifyObservers();
-    }
-}
 
 bool
 Track::inCollection() const
@@ -361,7 +238,7 @@ Track::inCollection() const
     return false;
 }
 
-Amarok::Collection*
+Collections::Collection*
 Track::collection() const
 {
     return 0;
@@ -382,9 +259,7 @@ Track::streamName() const
     QStringList elements = d->lastFmUri.toString().split( '/', QString::SkipEmptyParts );
     if( elements.size() >= 2 && elements[0] == "lastfm:" )
     {
-        QString customPart = elements[2];
-        customPart = customPart.replace( "%20", " " );
-
+        QString customPart = QUrl::fromPercentEncoding( elements[2].toUtf8() );
 
         if( elements[1] == "globaltags" )
         {
@@ -423,9 +298,9 @@ Track::streamName() const
                 else if( elements[3] == "personal" )
                     return i18n( "%1's Personal Radio", elements[2] );
 
-                // lastfm://user/<user>/loved
-                else if( elements[3] == "loved" )
-                    return i18n( "%1's Loved Radio", elements[2] );
+                // lastfm://user/<user>/mix
+                else if( elements[3] == "mix" )
+                    return i18n( "%1's Mix Radio", elements[2] );
 
                 // lastfm://user/<user>/recommended
                 else if( elements.size() < 5 && elements[3] == "recommended" )
@@ -460,41 +335,23 @@ Track::streamName() const
     return d->lastFmUri.toString();
 }
 
-void
-Track::love()
-{
-    DEBUG_BLOCK
-
-    debug() << "info:" << d->lastFmTrack.artist() << d->lastFmTrack.title();
-    d->wsReply = lastfm::MutableTrack( d->lastFmTrack ).love();
-    connect( d->wsReply, SIGNAL( finished() ), this, SLOT( slotWsReply() ) );
-}
-
 void
 Track::ban()
 {
     DEBUG_BLOCK
     d->wsReply = lastfm::MutableTrack( d->lastFmTrack ).ban();
-    connect( d->wsReply, SIGNAL( finished() ), this, SLOT( slotWsReply() ) );
-    emit( skipTrack() );
-
-}
-
-void
-Track::skip()
-{
-    DEBUG_BLOCK
-    //MutableTrack( d->lastFmTrack ).skip();
-    emit( skipTrack() );
+    connect( d->wsReply, SIGNAL(finished()), this, SLOT(slotWsReply()) );
+    if( The::engineController()->currentTrack() == this )
+        emit skipTrack();
 }
 
 void Track::slotResultReady()
 {
     if( d->trackFetch->error() == QNetworkReply::NoError )
     {
-        try
+        lastfm::XmlQuery lfm;
+        if( lfm.parse( d->trackFetch->readAll() ) )
         {
-            lastfm::XmlQuery lfm( d->trackFetch->readAll() );
             QString id = lfm[ "track" ][ "id" ].text();
             QString streamable = lfm[ "track" ][ "streamable" ].text();
             if( streamable.toInt() == 1 )
@@ -502,9 +359,10 @@ void Track::slotResultReady()
             else
                 init();
 
-        } catch( lastfm::ws::ParseError& e )
+        }
+        else
         {
-            debug() << "Got exception in parsing from last.fm:" << e.what();
+            debug() << "Got exception in parsing from last.fm:" << lfm.parseError().message();
         }
     } else
     {
@@ -522,41 +380,44 @@ Track::slotWsReply()
         //debug() << "successfully completed WS transaction";
     } else
     {
-        debug() << "ERROR in last.fm skip or ban!" << d->wsReply->error();
+        debug() << "ERROR in last.fm ban!" << d->wsReply->error();
     }
 }
 
 bool
-Track::hasCapabilityInterface( Meta::Capability::Type type ) const
+Track::hasCapabilityInterface( Capabilities::Capability::Type type ) const
 {
-    return type == Meta::Capability::LastFm
-                || type == Meta::Capability::MultiPlayable
-                || type == Meta::Capability::SourceInfo
-                || type == Meta::Capability::CurrentTrackActions
-                || type == Meta::Capability::StreamInfo;
+    return type == Capabilities::Capability::MultiPlayable ||
+           type == Capabilities::Capability::SourceInfo ||
+           type == Capabilities::Capability::Actions ||
+           type == Capabilities::Capability::StreamInfo;
 }
 
-Meta::Capability*
-Track::createCapabilityInterface( Meta::Capability::Type type )
+Capabilities::Capability*
+Track::createCapabilityInterface( Capabilities::Capability::Type type )
 {
     switch( type )
     {
-        case Meta::Capability::LastFm:
-            return new LastFmCapabilityImpl( this );
-        case Meta::Capability::MultiPlayable:
-            return new MultiPlayableCapabilityImpl( this );
-        case Meta::Capability::SourceInfo:
+        case Capabilities::Capability::MultiPlayable:
+            return new LastFmMultiPlayableCapability( this );
+        case Capabilities::Capability::SourceInfo:
             return new ServiceSourceInfoCapability( this );
-        case Meta::Capability::CurrentTrackActions:
-            return new CurrentTrackActionsCapabilityImpl( this );
-        case Meta::Capability::StreamInfo:
+        case Capabilities::Capability::Actions:
+            return new Capabilities::ActionsCapability( m_trackActions );
+        case Capabilities::Capability::StreamInfo:
             return new LastFmStreamInfoCapability( this );
         default:
             return 0;
     }
 }
 
-} // namespace LastFm
+Meta::StatisticsPtr
+Track::statistics()
+{
+    if( d->statsStore )
+        return d->statsStore;
+    return Meta::Track::statistics();
+}
 
 QString LastFm::Track::sourceName()
 {
@@ -584,12 +445,5 @@ QString LastFm::Track::scalableEmblem()
         return QString();
 }
 
-QList< QAction * > LastFm::Track::nowPlayingActions() const
-{
-    return m_currentTrackActions;
-}
-
 #include "LastFmMeta.moc"
-
-
-
+#include "LastFmMeta_p.moc"
