@@ -16,9 +16,9 @@
 
 #include "MagnatuneDatabaseHandler.h"
 
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Debug.h"
-#include "core/collections/support/SqlStorage.h"
+#include <core-impl/storage/StorageManager.h>
+#include <core/support/Debug.h>
+#include <core/storage/SqlStorage.h>
 
 using namespace Meta;
 
@@ -33,7 +33,7 @@ void
 MagnatuneDatabaseHandler::createDatabase( )
 {
     //Get database instance
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
 
     QString autoIncrement = "AUTO_INCREMENT";
 
@@ -118,14 +118,15 @@ MagnatuneDatabaseHandler::createDatabase( )
 void
 MagnatuneDatabaseHandler::destroyDatabase( )
 {
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
-    QStringList result = sqlDb->query( "DROP TABLE magnatune_tracks;" );
-    result = sqlDb->query( "DROP TABLE magnatune_albums;" );
-    result = sqlDb->query( "DROP TABLE magnatune_artists;" );
-    result = sqlDb->query( "DROP TABLE magnatune_genre;" );
-    result = sqlDb->query( "DROP TABLE magnatune_moods;" );
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
+    QStringList result = sqlDb->query( "DROP TABLE IF EXISTS magnatune_tracks;" );
+    result = sqlDb->query( "DROP TABLE IF EXISTS magnatune_albums;" );
+    result = sqlDb->query( "DROP TABLE IF EXISTS magnatune_artists;" );
+    result = sqlDb->query( "DROP TABLE IF EXISTS magnatune_genre;" );
+    result = sqlDb->query( "DROP TABLE IF EXISTS magnatune_moods;" );
 
 
+    /* that would only work for db2/oracle. Other databases connect the index to the table (which we just dropped)
     result = sqlDb->query( "DROP INDEX magnatune_tracks_artist_id;");
     result = sqlDb->query( "DROP INDEX magnatune_tracks_album_id;");
     result = sqlDb->query( "DROP INDEX magnatune_album_name;");
@@ -133,6 +134,7 @@ MagnatuneDatabaseHandler::destroyDatabase( )
     result = sqlDb->query( "DROP INDEX magnatune_artist_name;");
     result = sqlDb->query( "DROP INDEX magnatune_genre_album_id;");
     result = sqlDb->query( "DROP INDEX magnatune_genre_name;");
+    */
 
    /* if ( sqlDb->type() == DbConnection::postgresql )
     {
@@ -148,7 +150,7 @@ MagnatuneDatabaseHandler::insertTrack( ServiceTrack *track )
 {
     MagnatuneTrack * mTrack = static_cast<MagnatuneTrack *> ( track );
 
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
     QString queryString = "INSERT INTO magnatune_tracks ( name, track_number, length, "
             "album_id, artist_id, preview_lofi, preview_ogg, preview_url ) VALUES ( '"
                           + sqlDb->escape( mTrack->name()) + "', "
@@ -176,7 +178,7 @@ MagnatuneDatabaseHandler::insertAlbum( ServiceAlbum *album )
     MagnatuneAlbum * mAlbum = static_cast<MagnatuneAlbum *> ( album );
 
     QString queryString;
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
     queryString = "INSERT INTO magnatune_albums ( name, year, artist_id, "
                   "album_code, cover_url, description ) VALUES ( '"
                   + sqlDb->escape( sqlDb->escape( mAlbum->name() ) ) + "', "
@@ -199,7 +201,7 @@ MagnatuneDatabaseHandler::insertArtist( ServiceArtist *artist )
     MagnatuneArtist * mArtist = static_cast<MagnatuneArtist *> ( artist );
 
     QString queryString;
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
     queryString = "INSERT INTO magnatune_artists ( name, artist_page, description, "
                   "photo_url ) VALUES ( '"
                   + sqlDb->escape( mArtist->name() ) + "', '"
@@ -217,7 +219,7 @@ void
 MagnatuneDatabaseHandler::begin( )
 {
 
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
 
     QString queryString = "BEGIN;";
 
@@ -227,18 +229,18 @@ MagnatuneDatabaseHandler::begin( )
 void
 MagnatuneDatabaseHandler::commit( )
 {
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
     QString queryString = "COMMIT;";
 
     sqlDb->query( queryString );
-
+    sqlDb->query( "FLUSH TABLES;" );
 }
 
 void MagnatuneDatabaseHandler::insertMoods(int trackId, const QStringList &moods)
 {
 
     QString queryString;
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
 
     foreach( const QString &mood, moods ) {
         queryString = "INSERT INTO magnatune_moods ( track_id, mood ) VALUES ( "
@@ -254,7 +256,7 @@ void MagnatuneDatabaseHandler::insertMoods(int trackId, const QStringList &moods
 int MagnatuneDatabaseHandler::getArtistIdByExactName(const QString & name)
 {
 
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
 
     QString queryString = "SELECT id from magnatune_artists WHERE name='" + sqlDb->escape( name ) + "';";
     QStringList result = sqlDb->query( queryString );
@@ -272,7 +274,7 @@ int MagnatuneDatabaseHandler::getArtistIdByExactName(const QString & name)
 
 int MagnatuneDatabaseHandler::getAlbumIdByAlbumCode(const QString & albumcode)
 {
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
 
     QString queryString = "SELECT id from magnatune_albums WHERE album_code='" + sqlDb->escape( albumcode ) + "';";
     QStringList result = sqlDb->query( queryString );
@@ -290,7 +292,7 @@ int MagnatuneDatabaseHandler::getAlbumIdByAlbumCode(const QString & albumcode)
 int MagnatuneDatabaseHandler::insertGenre(ServiceGenre * genre)
 {
     QString queryString;
-    SqlStorage *sqlDb = CollectionManager::instance()->sqlStorage();
+    SqlStorage *sqlDb = StorageManager::instance()->sqlStorage();
     queryString = "INSERT INTO magnatune_genre ( album_id, name "
                   ") VALUES ( "
                   + QString::number ( genre->albumId() ) + ", '"
