@@ -26,6 +26,7 @@
 #include "MagnatuneConfig.h"
 #include "MagnatuneDatabaseWorker.h"
 #include "MagnatuneInfoParser.h"
+#include "MagnatuneNeedUpdateWidget.h"
 #include "browsers/InfoProxy.h"
 #include "MagnatuneUrlRunner.h"
 
@@ -69,7 +70,6 @@ void MagnatuneServiceFactory::init()
 {
     DEBUG_BLOCK
     MagnatuneStore* service = new MagnatuneStore( this, "Magnatune.com" );
-    m_activeServices << service;
     m_initialized = true;
     emit newService( service );
 }
@@ -92,6 +92,7 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
         : ServiceBase( name, parent )
         , m_downloadHandler( 0 )
         , m_redownloadHandler( 0 )
+        , m_needUpdateWidget( 0 )
         , m_downloadInProgress( 0 )
         , m_currentAlbum( 0 )
         , m_streamType( MagnatuneMetaFactory::OGG )
@@ -133,14 +134,13 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
     metaFactory->setStreamType( m_streamType );
     m_registry = new ServiceSqlRegistry( metaFactory );
     m_collection = new Collections::MagnatuneSqlCollection( "magnatune", "Magnatune.com", metaFactory, m_registry );
-    m_serviceready = true;
-    CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
-    emit( ready() );
+    CollectionManager::instance()->addTrackProvider( m_collection );
+    setServiceReady( true );
 }
 
 MagnatuneStore::~MagnatuneStore()
 {
-    CollectionManager::instance()->removeUnmanagedCollection( m_collection );
+    CollectionManager::instance()->removeTrackProvider( m_collection );
     delete m_registry;
     delete m_collection;
 }
@@ -171,7 +171,7 @@ void MagnatuneStore::download( )
     {
         m_downloadHandler = new MagnatuneDownloadHandler();
         m_downloadHandler->setParent( this );
-        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
+        connect( m_downloadHandler, SIGNAL(downloadCompleted(bool)), this, SLOT(downloadCompleted(bool)) );
     }
 
     if ( m_currentAlbum != 0 )
@@ -203,7 +203,7 @@ void MagnatuneStore::download( Meta::MagnatuneAlbum * album )
     {
         m_downloadHandler = new MagnatuneDownloadHandler();
         m_downloadHandler->setParent( this );
-        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
+        connect( m_downloadHandler, SIGNAL(downloadCompleted(bool)), this, SLOT(downloadCompleted(bool)) );
     }
 
     m_downloadHandler->downloadAlbum( album );
@@ -216,19 +216,19 @@ void MagnatuneStore::initTopPanel( )
     QMenu *filterMenu = new QMenu( 0 );
 
     QAction *action = filterMenu->addAction( i18n("Artist") );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByArtist() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(sortByArtist()) );
 
     action = filterMenu->addAction( i18n( "Artist / Album" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByArtistAlbum() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(sortByArtistAlbum()) );
 
     action = filterMenu->addAction( i18n( "Album" ) ) ;
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByAlbum() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(sortByAlbum()) );
 
     action = filterMenu->addAction( i18n( "Genre / Artist" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByGenreArtist() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(sortByGenreArtist()) );
 
     action = filterMenu->addAction( i18n( "Genre / Artist / Album" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( sortByGenreArtistAlbum() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(sortByGenreArtistAlbum()) );
 
     KAction *filterMenuAction = new KAction( KIcon( "preferences-other" ), i18n( "Sort Options" ), this );
     filterMenuAction->setMenu( filterMenu );
@@ -243,10 +243,10 @@ void MagnatuneStore::initTopPanel( )
     QMenu * actionsMenu = new QMenu( 0 );
 
     action = actionsMenu->addAction( i18n( "Re-download" ) );
-    connect( action, SIGNAL( triggered( bool) ), SLOT( processRedownload() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(processRedownload()) );
 
     m_updateAction = actionsMenu->addAction( i18n( "Update Database" ) );
-    connect( m_updateAction, SIGNAL( triggered( bool) ), SLOT( updateButtonClicked() ) );
+    connect( m_updateAction, SIGNAL(triggered(bool)), SLOT(updateButtonClicked()) );
 
     KAction *actionsMenuAction = new KAction( KIcon( "list-add" ), i18n( "Tools" ), this );
     actionsMenuAction->setMenu( actionsMenu );
@@ -283,8 +283,16 @@ void MagnatuneStore::initBottomPanel()
     m_downloadAlbumButton->setObjectName( "downloadButton" );
     m_downloadAlbumButton->setIcon( KIcon( "download-amarok" ) );
     
+    connect( m_downloadAlbumButton, SIGNAL(clicked()) , this, SLOT(download()) );
 
-    connect( m_downloadAlbumButton, SIGNAL( clicked() ) , this, SLOT( download() ) );
+    if ( !config.lastUpdateTimestamp() )
+    {
+        m_needUpdateWidget = new MagnatuneNeedUpdateWidget(m_bottomPanel);
+
+        connect( m_needUpdateWidget, SIGNAL(wantUpdate()), SLOT(updateButtonClicked()) );
+
+        m_downloadAlbumButton->setParent(0);
+    }
 }
 
 
@@ -292,6 +300,9 @@ void MagnatuneStore::updateButtonClicked()
 {
     DEBUG_BLOCK
     m_updateAction->setEnabled( false );
+    if ( m_needUpdateWidget )
+        m_needUpdateWidget->disable();
+
     updateMagnatuneList();
 }
 
@@ -306,7 +317,7 @@ bool MagnatuneStore::updateMagnatuneList()
 
     KTemporaryFile tempFile;
     tempFile.setSuffix( ".bz2" );
-    tempFile.setAutoRemove( false );  //file must be removed later
+    tempFile.setAutoRemove( false );  // file will be removed in MagnatuneXmlParser
     if( !tempFile.open() )
     {
         return false; //error
@@ -315,10 +326,10 @@ bool MagnatuneStore::updateMagnatuneList()
     m_tempFileName = tempFile.fileName();
 
     m_listDownloadJob = KIO::file_copy( KUrl( "http://magnatune.com/info/album_info_xml.bz2" ),  KUrl( m_tempFileName ), 0700 , KIO::HideProgressInfo | KIO::Overwrite );
-    Amarok::Components::logger()->newProgressOperation( m_listDownloadJob, i18n( "Downloading Magnatune.com database..." ), this, SLOT( listDownloadCancelled() ) );
+    Amarok::Components::logger()->newProgressOperation( m_listDownloadJob, i18n( "Downloading Magnatune.com database..." ), this, SLOT(listDownloadCancelled()) );
 
-    connect( m_listDownloadJob, SIGNAL( result( KJob * ) ),
-            this, SLOT( listDownloadComplete( KJob * ) ) );
+    connect( m_listDownloadJob, SIGNAL(result(KJob*)),
+            this, SLOT(listDownloadComplete(KJob*)) );
 
 
     return true;
@@ -336,7 +347,8 @@ void MagnatuneStore::listDownloadComplete( KJob * downLoadJob )
     }
 
     m_updateAction->setEnabled( true );
-    if ( !downLoadJob->error() == 0 )
+
+    if ( downLoadJob->error() != 0 )
     {
         debug() << "Got an error, bailing out: " << downLoadJob->errorString();
         //TODO: error handling here
@@ -347,7 +359,7 @@ void MagnatuneStore::listDownloadComplete( KJob * downLoadJob )
     Amarok::Components::logger()->shortMessage( i18n( "Updating the local Magnatune database."  ) );
     MagnatuneXmlParser * parser = new MagnatuneXmlParser( m_tempFileName );
     parser->setDbHandler( new MagnatuneDatabaseHandler() );
-    connect( parser, SIGNAL( doneParsing() ), SLOT( doneParsing() ) );
+    connect( parser, SIGNAL(doneParsing()), SLOT(doneParsing()) );
 
     ThreadWeaver::Weaver::instance()->enqueue( parser );
 }
@@ -362,6 +374,8 @@ void MagnatuneStore::listDownloadCancelled( )
     debug() << "Aborted xml download";
 
     m_updateAction->setEnabled( true );
+    if ( m_needUpdateWidget )
+        m_needUpdateWidget->enable();
 }
 
 
@@ -380,6 +394,15 @@ void MagnatuneStore::doneParsing()
         config.setLastUpdateTimestamp( m_magnatuneTimestamp );
 
     config.save();
+
+    if ( m_needUpdateWidget )
+    {
+        m_needUpdateWidget->setParent(0);
+        m_needUpdateWidget->deleteLater();
+        m_needUpdateWidget = 0;
+
+        m_downloadAlbumButton->setParent(m_bottomPanel);
+    }
 }
 
 
@@ -447,7 +470,7 @@ void MagnatuneStore::addMoodyTracksToPlaylist( const QString &mood, int count )
 {
     MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
     databaseWorker->fetchTrackswithMood( mood, count, m_registry );
-    connect( databaseWorker, SIGNAL( gotMoodyTracks( Meta::TrackList ) ), this, SLOT( moodyTracksReady(Meta::TrackList ) ) );
+    connect( databaseWorker, SIGNAL(gotMoodyTracks(Meta::TrackList)), this, SLOT(moodyTracksReady(Meta::TrackList)) );
 
     ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
 }
@@ -471,17 +494,17 @@ void MagnatuneStore::polish()
         setInfoParser( m_magnatuneInfoParser );
         setModel( new SingleCollectionTreeItemModel( m_collection, levels ) );
 
-        connect( m_contentView, SIGNAL( itemSelected( CollectionTreeItem * ) ), this, SLOT( itemSelected( CollectionTreeItem * ) ) );
+        connect( m_contentView, SIGNAL(itemSelected(CollectionTreeItem*)), this, SLOT(itemSelected(CollectionTreeItem*)) );
 
         //add a custom url runner
 
         MagnatuneUrlRunner * runner = new MagnatuneUrlRunner();
 
-        connect( runner, SIGNAL( showFavorites() ), this, SLOT( showFavoritesPage() ) );
-        connect( runner, SIGNAL( showHome() ), this, SLOT( showHomePage() ) );
-        connect( runner, SIGNAL( showRecommendations() ), this, SLOT( showRecommendationsPage() ) );
-        connect( runner, SIGNAL( buyOrDownload( const QString & ) ), this, SLOT( download( const QString & ) ) );
-        connect( runner, SIGNAL( removeFromFavorites( const QString & ) ), this, SLOT( removeFromFavorites( const QString & ) ) );
+        connect( runner, SIGNAL(showFavorites()), this, SLOT(showFavoritesPage()) );
+        connect( runner, SIGNAL(showHome()), this, SLOT(showHomePage()) );
+        connect( runner, SIGNAL(showRecommendations()), this, SLOT(showRecommendationsPage()) );
+        connect( runner, SIGNAL(buyOrDownload(QString)), this, SLOT(download(QString)) );
+        connect( runner, SIGNAL(removeFromFavorites(QString)), this, SLOT(removeFromFavorites(QString)) );
 
         The::amarokUrlHandler()->registerRunner( runner, runner->command() );
     }
@@ -497,10 +520,11 @@ void MagnatuneStore::polish()
 
     MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
     databaseWorker->fetchMoodMap();
-    connect( databaseWorker, SIGNAL( gotMoodMap(QMap< QString, int >) ), this, SLOT( moodMapReady(QMap< QString, int >) ) );
+    connect( databaseWorker, SIGNAL(gotMoodMap(QMap<QString,int>)), this, SLOT(moodMapReady(QMap<QString,int>)) );
     ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
 
-    checkForUpdates();
+    if ( MagnatuneConfig().autoUpdateDatabase() )
+        checkForUpdates();
 }
 
 
@@ -552,55 +576,10 @@ void MagnatuneStore::setStreamType( int type )
     m_streamType = type;
 }
 
-
-void MagnatuneStore::downloadCurrentTrackAlbum()
-{
-    //get current track
-    Meta::TrackPtr track = The::engineController()->currentTrack();
-
-    //check if this is indeed a magnatune track
-    Capabilities::SourceInfoCapability *sic = track->create<Capabilities::SourceInfoCapability>();
-    if( sic )
-    {
-        //is the source defined
-        QString source = sic->sourceName();
-        if ( source != "Magnatune.com" ) {
-            //not a Magnatune track, so don't bother...
-            delete sic;
-            return;
-        }
-        delete sic;
-    } else {
-        //not a Magnatune track, so don't bother...
-        return;
-    }
-
-    //so far so good...
-    //now the casting begins:
-
-    Meta::MagnatuneTrack * magnatuneTrack = dynamic_cast<Meta::MagnatuneTrack *> ( track.data() );
-    if ( !magnatuneTrack )
-        return;
-
-    Meta::MagnatuneAlbum * magnatuneAlbum = dynamic_cast<Meta::MagnatuneAlbum *> ( magnatuneTrack->album().data() );
-    if ( !magnatuneAlbum )
-        return;
-
-    if ( !m_downloadHandler )
-    {
-        m_downloadHandler = new MagnatuneDownloadHandler();
-        m_downloadHandler->setParent( this );
-        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
-    }
-
-    m_downloadHandler->downloadAlbum( magnatuneAlbum );
-}
-
-
 void MagnatuneStore::checkForUpdates()
 {
     m_updateTimestampDownloadJob = KIO::storedGet( KUrl( "http://magnatune.com/info/last_update_timestamp" ), KIO::Reload, KIO::HideProgressInfo );
-    connect( m_updateTimestampDownloadJob, SIGNAL( result( KJob * ) ), SLOT( timestampDownloadComplete( KJob *  ) ) );
+    connect( m_updateTimestampDownloadJob, SIGNAL(result(KJob*)), SLOT(timestampDownloadComplete(KJob*)) );
 }
 
 
@@ -608,7 +587,7 @@ void MagnatuneStore::timestampDownloadComplete( KJob *  job )
 {
     DEBUG_BLOCK
 
-    if ( !job->error() == 0 )
+    if ( job->error() != 0 )
     {
         //TODO: error handling here
         return ;
@@ -704,7 +683,7 @@ void MagnatuneStore::download( const QString &sku )
     debug() << "sku: " << sku;
     MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
     databaseWorker->fetchAlbumBySku( sku, m_registry );
-    connect( databaseWorker, SIGNAL( gotAlbumBySku( Meta::MagnatuneAlbum * ) ), this, SLOT( download( Meta::MagnatuneAlbum * ) ) );
+    connect( databaseWorker, SIGNAL(gotAlbumBySku(Meta::MagnatuneAlbum*)), this, SLOT(download(Meta::MagnatuneAlbum*)) );
 
     ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
 }
@@ -723,7 +702,7 @@ void MagnatuneStore::addToFavorites( const QString &sku )
     debug() << "favorites url: " << url;
 
     m_favoritesJob = KIO::storedGet( KUrl( url ), KIO::Reload, KIO::HideProgressInfo );
-    connect( m_favoritesJob, SIGNAL( result( KJob * ) ), SLOT( favoritesResult( KJob *  ) ) );
+    connect( m_favoritesJob, SIGNAL(result(KJob*)), SLOT(favoritesResult(KJob*)) );
 }
 
 void MagnatuneStore::removeFromFavorites( const QString &sku )
@@ -740,7 +719,7 @@ void MagnatuneStore::removeFromFavorites( const QString &sku )
     debug() << "favorites url: " << url;
 
     m_favoritesJob = KIO::storedGet( KUrl( url ), KIO::Reload, KIO::HideProgressInfo );
-    connect( m_favoritesJob, SIGNAL( result( KJob * ) ), SLOT( favoritesResult( KJob *  ) ) );
+    connect( m_favoritesJob, SIGNAL(result(KJob*)), SLOT(favoritesResult(KJob*)) );
 }
 
 void MagnatuneStore::favoritesResult( KJob* addToFavoritesJob )
