@@ -16,8 +16,9 @@
 
 #include "OpmlDirectoryInfoParser.h"
 
-#include "statusbar/StatusBar.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
+#include "core/support/Components.h"
+#include "core/interfaces/Logger.h"
 #include "OpmlDirectoryMeta.h"
 
 #include <KLocale>
@@ -61,8 +62,9 @@ void OpmlDirectoryInfoParser::getInfo( TrackPtr track )
     debug() << "OpmlDirectoryInfoParser: getInfo about feed: " << feed->uidUrl();
 
     m_rssDownloadJob = KIO::storedGet( feed->uidUrl(), KIO::Reload, KIO::HideProgressInfo );
-    The::statusBar()->newProgressOperation( m_rssDownloadJob, i18n( "Fetching Podcast Info" ) );
-    connect( m_rssDownloadJob, SIGNAL(result(KJob *)), SLOT( rssDownloadComplete( KJob*) ) );
+    Amarok::Components::logger()->newProgressOperation( m_rssDownloadJob,
+                                                        i18n( "Fetching Podcast Info" ) );
+    connect( m_rssDownloadJob, SIGNAL(result(KJob*)), SLOT(rssDownloadComplete(KJob*)) );
 }
 
 void OpmlDirectoryInfoParser::rssDownloadComplete(KJob * downLoadJob)
