@@ -17,11 +17,11 @@
 #include "ScriptableServiceManager.h"
 
 
-#include "collection/support/MemoryCollection.h"
-#include "Debug.h"
+#include "core-impl/collections/support/MemoryCollection.h"
+#include "core/support/Debug.h"
 #include "ScriptableServiceCollection.h"
 #include "ScriptableServiceMeta.h"
-#include "ScriptManager.h"
+#include "scripting/scriptmanager/ScriptManager.h"
 #include "../ServiceMetaBase.h"
 #include "browsers/servicebrowser/ServiceBrowser.h"
 
@@ -67,7 +67,6 @@ int ScriptableServiceManager::insertItem( const QString &serviceName, int level,
                                           const QString & albumOverride, const QString & artistOverride, const QString & genreOverride,
                                           const QString & composerOverride, int yearOverride, const QString &coverUrl)
 {
-    DEBUG_BLOCK
     if ( !m_serviceMap.contains( serviceName ) ) {
         //invalid service name
         return -1;
@@ -109,10 +108,8 @@ void ScriptableServiceManager::removeRunningScript(const QString & name)
         return;
     }
 
-    m_serviceMap.take( name );
-
     //service gets deleted by serviceBrowser
-    ServiceBrowser::instance()->removeCategory( name );
+    ServiceBrowser::instance()->removeCategory( m_serviceMap.take( name ) );
 }
 
 void ScriptableServiceManager::setIcon( const QString & serviceName, const QPixmap & icon )
