@@ -30,7 +30,6 @@
 #include "amarokconfig.h"
 #include "amarokurls/AmarokUrl.h"
 #include "amarokurls/AmarokUrlHandler.h"
-#include "browsers/collectionbrowser/CollectionWidget.h"
 #include "core/capabilities/ActionsCapability.h"
 #include "core/capabilities/BookmarkThisCapability.h"
 #include "core/meta/Meta.h"
@@ -280,13 +279,6 @@ MainToolbar::muteStateChanged( bool mute )
     m_volume->setMuted( mute );
 }
 
-void
-MainToolbar::filter( const QString &string )
-{
-    if( CollectionWidget::instance() )
-        CollectionWidget::instance()->setFilter( string );
-}
-
 void
 MainToolbar::layoutProgressBar()
 {
@@ -757,6 +749,7 @@ MainToolbar::showEvent( QShowEvent *ev )
     layoutTrackBar();
     layoutProgressBar();
     m_playPause->setPlaying( The::engineController()->isPlaying() );
+    trackPositionChanged( engine->trackPositionMs(), false ); // Refresh slider on restore
 }
 
 
