@@ -27,7 +27,7 @@ class VolumeDial;
 
 namespace Amarok { class TimeSlider; }
 
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 #include <QToolBar>
 
 class MainToolbar : public QToolBar
@@ -61,7 +61,6 @@ private slots:
     void muteStateChanged( bool mute );
     void volumeChanged( int percent );
     void addBookmark( const QString &name, int milliSeconds );
-    void filter( const QString &string );
     void layoutProgressBar();
     void layoutTrackBar();
     void setLabelTime( int ms );
