@@ -42,9 +42,8 @@ AssistantDialog::AssistantDialog( const QStringList &playableFileTypes, bool sav
     ui.setupUi( uiBase );
     setModal( true );
     setWindowTitle( i18n( "Transcode Tracks" ) );
-    setMinimumSize( 590, 340 );
-    setMaximumWidth( 590 );
-    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::MinimumExpanding );
+    setMinimumSize( 620, 480 );
+    setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );
 
     setButtons( Ok|Cancel );
     button( Ok )->setText( i18n( "Transc&ode" ) );
