@@ -17,9 +17,9 @@
 
 #include "AnalyzerWidget.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
 #include "amarokconfig.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "analyzerbase.h"
 #include "socketserver.h"
 
