@@ -1,6 +1,6 @@
 /****************************************************************************************
  * Copyright (c) 2003 Christian Muehlhaeuser <chris@chris.de>                           *
- * Copyright (c) 2008,2009 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2008-2013 Mark Kretschmann <kretschmann@kde.org>                       *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -18,20 +18,16 @@
 #ifndef AMAROK_OSD_H
 #define AMAROK_OSD_H
 
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 
 #include <QImage>
-#include <QList>
 #include <QPixmap>
 #include <QString>
 #include <QWidget> //baseclass
 
 #define OSD_WINDOW_OPACITY 0.74
 
-namespace Plasma
-{
-    class PanelSvg;
-}
+class QTimeLine;
 
 class OSDWidget : public QWidget
 {
@@ -40,20 +36,24 @@ class OSDWidget : public QWidget
     public:
         enum Alignment { Left, Middle, Center, Right };
 
-        explicit OSDWidget( QWidget *parent, const char *name = "osd" );
+        /** shadow size in every direction */
+        static const int SHADOW_SIZE = 5;
 
-        /** resets the colours to defaults */
-        void unsetColors();
-
-      public slots:
+        static const int FADING_DURATION = 400; //ms
+         
+    public slots:
         /** calls setText() then show(), after setting image if needed */
-        void show( const QString &text, QImage newImage = QImage() );
+        void show( const QString &text, const QImage &newImage = QImage() );
+
         void ratingChanged( const short rating );
         void ratingChanged( const QString& path, int rating );
         void volumeChanged( int volume );
 
-        /** reimplemented, shows the OSD */
-        virtual void show() { QWidget::show(); };
+        /** reimplemented, show the OSD */
+        virtual void show();
+        /** reimplemented, hide the OSD */
+        virtual void hide();
+
         virtual void setVisible( bool visible );
 
         /**
@@ -64,44 +64,65 @@ class OSDWidget : public QWidget
          * To force an update call show();
          */
         void setDuration( int ms ) { m_duration = ms; }
-        void setTextColor( const QColor &color )
-        {
-            QPalette palette = this->palette();
-            palette.setColor( QPalette::Active, QPalette::WindowText, color );
-            setPalette(palette);
-        }
-        void setOffset( int y ) { m_y = y; }
+        void setTextColor( const QColor &color );
+
+        inline uint yOffset() const { return m_yOffset; }
+        void setYOffset( int y ) { m_yOffset = y; }
+
+        inline int alignment() const { return m_alignment; }
         void setAlignment( Alignment alignment ) { m_alignment = alignment; }
-        void setImage( const QImage &image ) { m_cover = image; }
+
+        inline int screen() const { return m_screen; }
         void setScreen( int screen );
+
+        void setPaused( bool paused ) { m_paused = paused; }
+        void setImage( const QImage &image ) { m_cover = image; }
         void setText( const QString &text ) { m_text = text; }
-        void setRating( const short rating ) { if ( isEnabled() ) m_rating = rating; }
-        void setTranslucent( bool enabled ) { setWindowOpacity( enabled ? OSD_WINDOW_OPACITY : 1.0 ); }
-        void setFontScale( int scale ) { m_fontScale = static_cast<double>(scale) / 100.0; }
+        void setRating( const short rating ) { m_rating = rating; }
+        void setTranslucent( bool enabled ) { m_translucent = enabled; setWindowOpacity( maxOpacity() ); }
+        void setFadeOpacity( qreal value );
+        void setFontScale( int scale );
+        void setHideWhenFullscreenWindowIsActive( bool hide );
 
     protected:
+        explicit OSDWidget( QWidget *parent, const char *name = "osd" );
         virtual ~OSDWidget();
 
+        // work-around to get default point size on this platform, Qt API does not offer this directly
+        inline qreal defaultPointSize() const { return QFont(font().family()).pointSizeF(); }
+
+        inline qreal maxOpacity() const { return m_translucent ? OSD_WINDOW_OPACITY : 1.0; }
+
         /** determine new size and position */
         QRect determineMetrics( const int marginMetric );
 
+        /**
+         * @short Checks if the OSD is temporary disabled.
+         * This is usually the case if the OSD should not be shown
+         * if a fullscreen application is active (@see m_hideWhenFullscreenWindowIsActive)
+         * (where the OSD could steal focus).
+         */
+        bool isTemporaryDisabled() const;
+
+        /** resets the colours to defaults */
+        void unsetColors();
+
         // Reimplemented from QWidget
         virtual void paintEvent( QPaintEvent* );
         virtual void mousePressEvent( QMouseEvent* );
-        void resizeEvent( QResizeEvent *e );
-        virtual bool event( QEvent* );
+        virtual void changeEvent( QEvent* );
 
         /** distance from screen edge */
         static const int MARGIN = 15;
 
-        uint        m_m;
+    private:
+        uint        m_margin;
         QSize       m_size;
         int         m_duration;
         QTimer     *m_timer;
         Alignment   m_alignment;
         int         m_screen;
-        uint        m_y;
-        bool        m_drawShadow;
+        uint        m_yOffset;
         short       m_rating;
         int         m_volume;
         bool        m_showVolume;
@@ -109,7 +130,9 @@ class OSDWidget : public QWidget
         QImage      m_cover;
         QPixmap     m_scaledCover;
         bool        m_paused;
-        double      m_fontScale;
+        bool        m_hideWhenFullscreenWindowIsActive;
+        QTimeLine*  m_fadeTimeLine;
+        bool        m_translucent;
 };
 
 
@@ -120,22 +143,15 @@ class OSDPreviewWidget : public OSDWidget
 public:
     OSDPreviewWidget( QWidget *parent );
 
-    int screen()    { return m_screen; }
-    int alignment() { return m_alignment; }
-    int y()         { return m_y; }
-
 public slots:
     void setTextColor( const QColor &color ) { OSDWidget::setTextColor( color ); doUpdate(); }
-    //void setFont( const QFont &font ) { OSDWidget::setFont( font ); doUpdate(); }
     void setScreen( int screen ) { OSDWidget::setScreen( screen ); doUpdate(); }
     void setFontScale( int scale ) { OSDWidget::setFontScale( scale ); doUpdate(); }
-    void setUseCustomColors( const bool use, const QColor &fg )
-    {
-        if( use ) OSDWidget::setTextColor( fg );
-        else      unsetColors();
-        doUpdate();
-    }
-    void setTranslucent( bool enabled ) { setWindowOpacity( enabled ? OSD_WINDOW_OPACITY : 1.0 ); doUpdate(); }
+    void setTranslucent( bool enabled ) { OSDWidget::setTranslucent( enabled ); doUpdate(); }
+
+    void setUseCustomColors( const bool use, const QColor &fg );
+
+    virtual void hide() { QWidget::hide(); }
 
 private:
     inline void doUpdate() { if( !isHidden() ) show(); }
@@ -150,7 +166,7 @@ protected:
 
 private:
     bool   m_dragging;
-    QPoint m_dragOffset;
+    QPoint m_dragYOffset;
 };
 
 
@@ -190,6 +206,7 @@ namespace Amarok
         ~OSD();
 
         static OSD* s_instance;
+
         Meta::TrackPtr m_currentTrack;
     };
 }
