@@ -21,7 +21,6 @@
 #include "EngineController.h"
 #include "SliderWidget.h"
 #include "TimeLabel.h"
-#include "amarokconfig.h"
 #include "amarokurls/AmarokUrl.h"
 #include "amarokurls/AmarokUrlHandler.h"
 #include "core/meta/Meta.h"
@@ -118,8 +117,8 @@ ProgressWidget::updateTimeLabelTooltips()
     TimeLabel *elapsedLabel = AmarokConfig::leftTimeDisplayRemaining() ? m_timeLabelRight : m_timeLabelLeft;
     TimeLabel *remainingLabel = AmarokConfig::leftTimeDisplayRemaining() ? m_timeLabelLeft : m_timeLabelRight;
 
-    elapsedLabel->setToolTip( i18n( "The amount of time elapsed in current song" ) );
-    remainingLabel->setToolTip( i18n( "The amount of time remaining in current song" ) );
+    elapsedLabel->setToolTip( i18n( "The amount of time elapsed in current track" ) );
+    remainingLabel->setToolTip( i18n( "The amount of time remaining in current track" ) );
 }
 
 void
@@ -136,7 +135,7 @@ ProgressWidget::drawTimeDisplay( int ms )  //SLOT
     int remainingSeconds = qMax(0, int((trackLength - ms) / 1000));
 
     QString sSeconds = Meta::secToPrettyTime( seconds );
-    QString sRemainingSeconds = "-" + Meta::secToPrettyTime( remainingSeconds );
+    QString sRemainingSeconds = '-' + Meta::secToPrettyTime( remainingSeconds );
 
     if( AmarokConfig::leftTimeDisplayRemaining() )
     {
