@@ -1,5 +1,6 @@
 /****************************************************************************************
  * Copyright (c) 2007 Dan Meltzer <parallelgrapefruit@gmail.com>                        *
+ * Copyright (c) 2011 Sven Krohlas <sven@asbest-online.de>                              *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -15,51 +16,29 @@
  ****************************************************************************************/
 
 #include "SearchWidget.h"
-#include "EditFilterDialog.h"
+#include "core/support/Debug.h"
+#include "dialogs/EditFilterDialog.h"
 
+#include <KLocale>
+
+#include <QAction>
 #include <QToolBar>
 #include <QVBoxLayout>
 
 #include <KIcon>
-#include <KLocale>
 #include <KLineEdit>
 #include <KHBox>
 #include <KPushButton>
 
+#include <kstandarddirs.h>
 
 SearchWidget::SearchWidget( QWidget *parent, bool advanced )
     : QWidget( parent )
     , m_sw( 0 )
     , m_filterAction( 0 )
+    , m_timeout( 500 )
+    , m_runningSearches( 0 )
 {
-    init( parent, advanced );
-}
-
-SearchWidget::SearchWidget( QWidget *parent, QWidget *caller, bool advanced )
-    : QWidget( parent )
-    , m_sw( 0 )
-    , m_filterAction( 0 )
-{
-    init( parent, advanced );
-    setup( caller );
-}
-
-void
-SearchWidget::setup( QObject* caller )
-{
-    connect( m_sw, SIGNAL( editTextChanged( const QString & ) ), caller,
-             SLOT( slotSetFilterTimeout() ) );
-    connect( this, SIGNAL( filterNow() ), caller,
-             SLOT( slotFilterNow() ) );
-    connect( m_sw, SIGNAL( returnPressed() ), caller, SLOT( slotFilterNow() ) );
-    connect( m_sw, SIGNAL( downPressed() ), caller, SLOT( setFocus() ) );
-}
-
-///Private
-void
-SearchWidget::init( QWidget *parent, bool advanced )
-{
-    Q_UNUSED( parent )
     setContentsMargins( 0, 0, 0, 0 );
     KHBox *searchBox = new KHBox( this );
     searchBox->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
@@ -71,8 +50,11 @@ SearchWidget::init( QWidget *parent, bool advanced )
     m_sw->completionObject()->setIgnoreCase( true );
     m_sw->setToolTip( i18n( "Enter space-separated terms to search." ) );
     m_sw->addItem( KStandardGuiItem::find().icon(), QString() );
-    connect( m_sw, SIGNAL(returnPressed(const QString&)), SLOT(addCompletion(const QString&)) );
-    connect( m_sw, SIGNAL(activated(int)), SLOT(onComboItemActivated(int)));
+    connect( m_sw, SIGNAL(activated(int)), SLOT(onComboItemActivated(int)) );
+    connect( m_sw, SIGNAL(editTextChanged(QString)), SLOT(resetFilterTimeout()) );
+    connect( m_sw, SIGNAL(returnPressed()), SLOT(filterNow()) ); // filterNow() calls addCompletion()
+    connect( m_sw, SIGNAL(returnPressed()), SIGNAL(returnPressed()) );
+    connect( m_sw, SIGNAL(downPressed()), SLOT(advanceFocus()) );
 
     QVBoxLayout *layout = new QVBoxLayout();
     layout->addWidget( searchBox );
@@ -83,20 +65,40 @@ SearchWidget::init( QWidget *parent, bool advanced )
     m_toolBar = new QToolBar( searchBox );
     m_toolBar->setFixedHeight( m_sw->sizeHint().height() );
 
-    if ( advanced ) {
+    if( advanced )
+    {
         m_filterAction = new QAction( KIcon( "document-properties" ), i18n( "Edit filter" ), this );
         m_filterAction->setObjectName( "filter" );
         m_toolBar->addAction( m_filterAction );
 
-        connect ( m_filterAction, SIGNAL( triggered() ), this, SLOT( slotShowFilterEditor() ) );
+        connect( m_filterAction, SIGNAL(triggered()), this, SLOT(slotShowFilterEditor()) );
     }
+
+    m_filterTimer.setSingleShot( true );
+    connect( &m_filterTimer, SIGNAL(timeout()), SLOT(filterNow()) );
+
+    m_animationTimer.setInterval( 500 );
+    connect( &m_animationTimer, SIGNAL(timeout()), this, SLOT(nextAnimationTick()) );
 }
 
 void
-SearchWidget::setSearchString( const QString &searchString )
+SearchWidget::resetFilterTimeout()
 {
-    m_sw->setEditText( searchString );
-    emit filterNow();
+    m_filterTimer.start( m_timeout );
+}
+
+void
+SearchWidget::filterNow()
+{
+    m_filterTimer.stop();
+    addCompletion( m_sw->currentText() );
+    emit filterChanged( m_sw->currentText() );
+}
+
+void
+SearchWidget::advanceFocus()
+{
+    focusNextChild();
 }
 
 void
@@ -108,10 +110,9 @@ SearchWidget::addCompletion( const QString &text )
         m_sw->addItem( KStandardGuiItem::find().icon(), text );
         m_sw->completionObject()->addItem( text );
     }
-    else
-    {
-        m_sw->setCurrentIndex( index );
-    }
+
+    index = m_sw->findText( text );
+    m_sw->setCurrentIndex( index );
 }
 
 void
@@ -132,7 +133,7 @@ SearchWidget::slotShowFilterEditor()
     fd->setAttribute( Qt::WA_DeleteOnClose );
     m_filterAction->setEnabled( false );
 
-    connect( fd, SIGNAL(filterChanged(const QString&) ), m_sw, SLOT(setEditText(const QString&)) );
+    connect( fd, SIGNAL(filterChanged(QString)), m_sw, SLOT(setEditText(QString)) );
     connect( fd, SIGNAL(finished(int)), this, SLOT(slotFilterEditorFinished(int)) );
 
     fd->show();
@@ -147,19 +148,23 @@ SearchWidget::slotFilterEditorFinished( int result )
         addCompletion( m_sw->currentText() );
 }
 
-QToolBar * SearchWidget::toolBar()
+QToolBar *
+SearchWidget::toolBar()
 {
     return m_toolBar;
 }
 
-void SearchWidget::showAdvancedButton(bool show)
+void
+SearchWidget::showAdvancedButton( bool show )
 {
-    if ( show ) {
-        if ( m_filterAction != 0 ) {
+    if( show )
+    {
+        if( m_filterAction != 0 )
+        {
             m_filterAction = new QAction( KIcon( "document-properties" ), i18n( "Edit filter" ), this );
             m_filterAction->setObjectName( "filter" );
             m_toolBar->addAction( m_filterAction );
-            connect ( m_filterAction, SIGNAL( triggered() ), this, SLOT( slotShowFilterEditor() ) );
+            connect( m_filterAction, SIGNAL(triggered()), this, SLOT(slotShowFilterEditor()) );
         }
     }
     else
@@ -169,10 +174,113 @@ void SearchWidget::showAdvancedButton(bool show)
     }
 }
 
-void SearchWidget::setClickMessage( const QString &message )
+void
+SearchWidget::setClickMessage( const QString &message )
 {
     KLineEdit *edit = qobject_cast<KLineEdit*>( m_sw->lineEdit() );
     edit->setClickMessage( message );
 }
 
+void
+SearchWidget::setTimeout( quint16 newTimeout )
+{
+    m_timeout = newTimeout;
+}
+
+// public slots:
+
+void
+SearchWidget::setSearchString( const QString &searchString )
+{
+    if( searchString != currentText() ) {
+        m_sw->setEditText( searchString );
+        filterNow();
+    }
+}
+
+void
+SearchWidget::searchStarted()
+{
+    m_runningSearches++;
+
+    // start the animation
+    if( !m_animationTimer.isActive() )
+    {
+        m_sw->setItemIcon( m_sw->currentIndex(), QIcon( KStandardDirs::locate( "data", "amarok/images/loading1.png" ) ) );
+        m_currentFrame = 0;
+        m_animationTimer.start();
+    }
+
+    // If another search is running it might still have a part of the animation set as its icon.
+    // As the currentIndex() has changed we don't know which one. We now have to iterate through
+    // all of them and set the icon correctly. It's not as bad as it sounds: the number is quite
+    // limited.
+
+    for( int i = 0; i < m_sw->count(); i++ )
+    {
+        if( i != m_sw->currentIndex() ) // not the current one, which should be animated!
+            m_sw->setItemIcon( i, KStandardGuiItem::find().icon() );
+    }
+}
+
+void
+SearchWidget::searchEnded()
+{
+    if( m_runningSearches > 0 ) // just to be sure...
+        m_runningSearches--;
+
+    // stop the animation
+    if( m_runningSearches == 0 )
+    {
+        m_animationTimer.stop();
+        saveLineEditStatus();
+        m_sw->setItemIcon( m_sw->currentIndex(), KStandardGuiItem::find().icon() );
+        restoreLineEditStatus();
+    }
+}
+
+
+// private slots:
+
+void
+SearchWidget::nextAnimationTick()
+{
+    saveLineEditStatus();
+
+    // switch frames
+    if( m_currentFrame == 0 )
+        m_sw->setItemIcon( m_sw->currentIndex(), QIcon( KStandardDirs::locate( "data", "amarok/images/loading2.png" ) ) );
+    else
+        m_sw->setItemIcon( m_sw->currentIndex(), QIcon( KStandardDirs::locate( "data", "amarok/images/loading1.png" ) ) );
+
+    restoreLineEditStatus();
+    m_currentFrame = !m_currentFrame;
+}
+
+
+// private:
+
+void
+SearchWidget::restoreLineEditStatus()
+{
+    // restore text changes made by the user
+    m_sw->setEditText( m_text );
+
+    if( m_hasSelectedText )
+        m_sw->lineEdit()->setSelection( m_selectionStart, m_selectionLength ); // also sets cursor
+    else
+        m_sw->lineEdit()->setCursorPosition( m_cursorPosition );
+}
+
+void
+SearchWidget::saveLineEditStatus()
+{
+    // save text changes made by the user
+    m_text = m_sw->lineEdit()->text();
+    m_cursorPosition = m_sw->cursorPosition();
+    m_hasSelectedText = m_sw->lineEdit()->hasSelectedText();
+    m_selectionStart = m_sw->lineEdit()->selectionStart();
+    m_selectionLength = m_sw->lineEdit()->selectedText().length();
+}
+
 #include "SearchWidget.moc"
