@@ -17,7 +17,7 @@
 
 #include "TokenWithLayout.h"
 
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "TokenDropTarget.h"
 #include "playlist/layouts/LayoutEditDialog.h"
 
@@ -86,18 +86,17 @@ const QString ActionAlignLeftName = QLatin1String( "ActionAlignLeft" );
 const QString ActionAlignCenterName = QLatin1String( "ActionAlignCenter" );
 const QString ActionAlignRightName = QLatin1String( "ActionAlignRight" );
 
-Token * TokenWithLayoutFactory::createToken( const QString &text, const QString &iconName, int value, QWidget *parent )
+Token * TokenWithLayoutFactory::createToken( const QString &text, const QString &iconName, qint64 value, QWidget *parent ) const
 {
     return new TokenWithLayout( text, iconName, value, parent );
 }
 
-QPointer<LayoutEditDialog> TokenWithLayout::m_dialog = 0L;
+QWeakPointer<LayoutEditDialog> TokenWithLayout::m_dialog;
 
-TokenWithLayout::TokenWithLayout( const QString &text, const QString &iconName, int value, QWidget *parent )
+TokenWithLayout::TokenWithLayout( const QString &text, const QString &iconName, qint64 value, QWidget *parent )
     : Token( text, iconName, value, parent  )
     , m_width( 0.0 ), m_wrenchTimer( 0 )
 {
-    m_widthForced = m_width > 0.0;
     m_alignment = Qt::AlignCenter;
     m_bold = false;
     m_italic = false;
@@ -105,7 +104,7 @@ TokenWithLayout::TokenWithLayout( const QString &text, const QString &iconName,
     m_wrench = new Wrench( this );
     m_wrench->installEventFilter( this );
     m_wrench->hide();
-    connect ( m_wrench, SIGNAL( clicked() ), this, SLOT( showConfig() ) );
+    connect ( m_wrench, SIGNAL(clicked()), this, SLOT(showConfig()) );
     setFocusPolicy( Qt::ClickFocus );
 }
 
@@ -152,19 +151,19 @@ void TokenWithLayout::leaveEvent( QEvent *e )
 
 void TokenWithLayout::showConfig()
 {
-    if ( !m_dialog )
+    if( !m_dialog )
         m_dialog = new LayoutEditDialog( window() );
-    m_dialog->setToken( this );
-    if ( !m_dialog->isVisible() )
+    m_dialog.data()->setToken( this );
+    if( !m_dialog.data()->isVisible() )
     {
-        m_dialog->adjustSize();
+        m_dialog.data()->adjustSize();
         QPoint pt = mapToGlobal( rect().bottomLeft() );
         pt.setY( pt.y() + 9 );
         if ( parentWidget() )
-            pt.setX( parentWidget()->mapToGlobal( QPoint( 0, 0 ) ).x() + ( parentWidget()->width() - m_dialog->QDialog::width() ) / 2 );
-        m_dialog->move( pt );
+            pt.setX( parentWidget()->mapToGlobal( QPoint( 0, 0 ) ).x() + ( parentWidget()->width() - m_dialog.data()->QDialog::width() ) / 2 );
+        m_dialog.data()->move( pt );
     }
-    m_dialog->show(); // ensures raise in doubt
+    m_dialog.data()->show(); // ensures raise in doubt
     QTimerEvent te( m_wrenchTimer );
     timerEvent( &te ); // it's not like we'd get a leave event when the child dialog pops in between...
 }
@@ -200,24 +199,6 @@ void TokenWithLayout::setAlignment( Qt::Alignment alignment )
     emit changed();
 }
 
-void TokenWithLayout::setAlignLeft( bool b )
-{
-    if (b)
-        setAlignment( Qt::AlignLeft );
-}
-
-void TokenWithLayout::setAlignCenter( bool b )
-{
-    if (b)
-        setAlignment( Qt::AlignCenter );
-}
-
-void TokenWithLayout::setAlignRight( bool b )
-{
-    if (b)
-        setAlignment( Qt::AlignRight );
-}
-
 bool TokenWithLayout::bold() const
 {
     return m_bold;
@@ -260,17 +241,10 @@ void TokenWithLayout::setSuffix( const QString& string )
 void TokenWithLayout::setWidth( int size )
 {
     m_width = qMax( qMin( 1.0, size/100.0 ), 0.0 ) ;
-    if ( m_width > 0.0 )
-        m_widthForced = true;
 
     emit changed();
 }
 
-void TokenWithLayout::setWidthForced( bool on )
-{
-    m_widthForced = on;
-}
-
 qreal TokenWithLayout::width() const
 {
     return m_width;
