@@ -50,8 +50,7 @@ void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackNa
     Meta::TrackPtr trackPtr( track );
     EXPECT_CALL( *track, name() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName ) );
     EXPECT_CALL( *track, prettyName() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName ) );
-    EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + "_" + artistName + "_" + albumName ) );
-    EXPECT_CALL( *track, isPlayable() ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
+    EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + '_' + artistName + '_' + albumName ) );
     EXPECT_CALL( *track, playableUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( KUrl( '/' + track->uidUrl() ) ) );
     EXPECT_CALL( *track, composer() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::ComposerPtr() ) );
     EXPECT_CALL( *track, genre() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::GenrePtr() ) );
@@ -130,16 +129,20 @@ TestSingleCollectionTreeItemModel::~TestSingleCollectionTreeItemModel()
 }
 
 void
-TestSingleCollectionTreeItemModel::init()
+TestSingleCollectionTreeItemModel::initTestCase()
 {
-
-
+    qRegisterMetaType<Meta::GenreList>();
+    qRegisterMetaType<Meta::ComposerList>();
+    qRegisterMetaType<Meta::YearList>();
+    qRegisterMetaType<Meta::LabelList>();
 }
 
-#define loadChildren( itemModel, idx ) { \
-    if( itemModel->canFetchMore( idx ) ) \
+#define loadChildren( itemModel, idx ) \
+{ \
+    if( itemModel->canFetchMore( idx ) ) { \
         itemModel->fetchMore( idx ); \
-    QTest::qWait( 30 ); \
+        QVERIFY( QTest::kWaitForSignal( itemModel, SIGNAL(allQueriesFinished(bool)), 5000 ) ); \
+    } \
 }
 
 void
