@@ -18,21 +18,22 @@
 #include "TestSqlScanManager.h"
 
 #include "amarokconfig.h"
-#include "shared/MetaTagLib.h"
-#include "core-impl/collections/db/ScanManager.h"
+#include "MetaTagLib.h"
+#include "scanner/GenericScanManager.h"
 #include "core-impl/collections/db/sql/SqlCollection.h"
-#include "core-impl/collections/db/sql/SqlCollectionFactory.h"
 #include "core-impl/collections/db/sql/SqlQueryMaker.h"
 #include "core-impl/collections/db/sql/SqlRegistry.h"
-#include "core-impl/collections/db/sql/mysqlecollection/MySqlEmbeddedStorage.h"
+#include "core-impl/storage/sql/mysqlestorage/MySqlEmbeddedStorage.h"
 
 #include "config-amarok-test.h"
 #include "SqlMountPointManagerMock.h"
 
-#include <QTest>
-
 #include <qtest_kde.h>
 
+#include <QTest>
+#include <QScopedPointer>
+#include <ThreadWeaver/Weaver>
+
 QTEST_KDEMAIN_CORE( TestSqlScanManager )
 
 TestSqlScanManager::TestSqlScanManager()
@@ -48,15 +49,24 @@ TestSqlScanManager::initTestCase()
     // Amarok does not force LC_ALL=C but obviously the test does it which
     // will prevent scanning of files with umlauts.
 
+    //Tell GenericScanManager that we want to use the recently built scanner, not an installed version.
+    const QString overridePath = QString( AMAROK_OVERRIDE_UTILITIES_PATH );
+    qApp->setProperty( "overrideUtilitiesPath", overridePath );
+
     // that is the original mp3 file that we use to generate the "real" tracks
     m_sourcePath = QDir::toNativeSeparators( QString( AMAROK_TEST_DIR ) + "/data/audio/Platz 01.mp3" );
     QVERIFY( QFile::exists( m_sourcePath ) );
 
     m_tmpDatabaseDir = new KTempDir();
     QVERIFY( m_tmpDatabaseDir->exists() );
-    m_storage = new MySqlEmbeddedStorage( m_tmpDatabaseDir->name() );
-    m_collection = Collections::SqlCollectionFactory().createSqlCollection( "testId", "testscanmanager", m_storage );
-    connect( m_collection, SIGNAL( updated() ), this, SLOT( slotCollectionUpdated() ) );
+    m_storage = new MySqlEmbeddedStorage();
+    QVERIFY( m_storage->init( m_tmpDatabaseDir->name() ) );
+
+    m_collection = new Collections::SqlCollection( m_storage );
+    connect( m_collection, SIGNAL(updated()), this, SLOT(slotCollectionUpdated()) );
+
+    // TODO: change the mock mount point manager so that it doesn't pull
+    //       in all the devices. Not much of a mock like this.
     SqlMountPointManagerMock *mock = new SqlMountPointManagerMock( this, m_storage );
     m_collection->setMountPointManager( mock );
     m_scanManager = m_collection->scanManager();
@@ -87,7 +97,14 @@ TestSqlScanManager::initTestCase()
 void
 TestSqlScanManager::cleanupTestCase()
 {
+    // aborts a ThreadWeaver job that would otherwise cause next statement to stall
     delete m_collection;
+
+    // we cannot simply call WeaverInterface::finish(), it stops event loop
+    if( !ThreadWeaver::Weaver::instance()->isIdle() )
+        QVERIFY2( QTest::kWaitForSignal( ThreadWeaver::Weaver::instance(),
+                SIGNAL(finished()), 5000 ), "threads did not finish in timeout" );
+
     //m_storage is deleted by SqlCollection
     delete m_tmpDatabaseDir;
 }
@@ -128,10 +145,8 @@ void
 TestSqlScanManager::testScanSingle()
 {
     m_collectionUpdatedCount = 0;
-
     createSingleTrack();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     QVERIFY( m_collectionUpdatedCount > 0 );
 
@@ -160,17 +175,17 @@ TestSqlScanManager::testScanSingle()
     QVERIFY( track->createDate().secsTo( aDate ) >= 0 );
     QVERIFY( track->modifyDate().secsTo( aDate ) < 5 ); // I just wrote the file
     QVERIFY( track->modifyDate().secsTo( aDate ) >= 0 );
-    qFuzzyCompare( track->score(), 0.875 );
-    QCOMPARE( track->playCount(), 5 );
-    QVERIFY( !track->firstPlayed().isValid() );
-    QVERIFY( !track->lastPlayed().isValid() );
+    Meta::StatisticsPtr statistics = track->statistics();
+    qFuzzyCompare( statistics->score(), 0.875 );
+    QCOMPARE( statistics->playCount(), 5 );
+    QVERIFY( !statistics->firstPlayed().isValid() );
+    QVERIFY( !statistics->lastPlayed().isValid() );
     QVERIFY( track->createDate().isValid() );
 
     // -- check that a further scan doesn't change anything
     m_collectionUpdatedCount = 0;
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     QCOMPARE( m_collectionUpdatedCount, 0 );
 }
@@ -179,8 +194,7 @@ void
 TestSqlScanManager::testScanDirectory()
 {
     createAlbum();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album;
@@ -210,8 +224,7 @@ TestSqlScanManager::testDuplicateUid()
     values.insert( Meta::valTitle, QVariant("Track 2") );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit (the database needs to have been updated correctly)
     m_collection->registry()->emptyCache();
@@ -241,8 +254,7 @@ TestSqlScanManager::testLongUid()
     values.insert( Meta::valTitle, QVariant("Track 2") );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit (the database needs to have been updated correctly)
     m_collection->registry()->emptyCache();
@@ -302,8 +314,7 @@ TestSqlScanManager::testCompilation()
     values.insert( Meta::valAlbum, QVariant("The Last Unicorn") );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album;
@@ -341,44 +352,18 @@ TestSqlScanManager::testCompilation()
     QVERIFY( !album->isCompilation() );
 }
 
-
-void
-TestSqlScanManager::testRestartScanner()
-{
-#ifndef QT_NO_DEBUG
-    createAlbum();
-
-    // the scanner crashes at a special file:
-    Meta::FieldHash values;
-    values.clear();
-    values.insert( Meta::valUniqueId, QVariant("c6c29f50279ab9523a0f44928bc1e96b") );
-    values.insert( Meta::valUrl, QVariant("Thriller/crash_amarok_here.ogg") );
-    createTrack( values );
-
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
-
-    // -- check the commit
-    Meta::AlbumPtr album;
-    album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
-    QVERIFY( album );
-    QCOMPARE( album->tracks().count(), 9 );
-
-#else
-    QSKIP( "Collection scanner only crashes in debug build.", SkipAll );
-#endif
-}
-
 void
 TestSqlScanManager::testBlock()
 {
+    /** TODO: do we need blocking at all?
+
     createSingleTrack();
     Meta::TrackPtr track;
 
     m_scanManager->blockScan(); // block the incremental scanning
     m_scanManager->requestFullScan();
 
-    QTest::qWait( 2000 );
+    QTest::qWait( 100 );
     track = m_collection->registry()->getTrack( 1 );
     QVERIFY( !track );
     QVERIFY( !m_scanManager->isRunning() );
@@ -388,18 +373,17 @@ TestSqlScanManager::testBlock()
     // it might or might not continue with the old scan
 
     waitScannerFinished(); // in case it does continue after all
+    */
 }
 
 void
 TestSqlScanManager::testAddDirectory()
 {
     createAlbum();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     createCompilation();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -419,8 +403,7 @@ TestSqlScanManager::testRemoveDir()
 
     createAlbum();
     createCompilation();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -434,14 +417,14 @@ TestSqlScanManager::testRemoveDir()
     QVERIFY( album->isCompilation() );
 
     // -- remove one album
+
     album = m_collection->registry()->getAlbum( "Top Gun", QString() );
     QVERIFY( album );
     foreach( Meta::TrackPtr t, album->tracks() )
         QVERIFY( QFile::remove( t->playableUrl().path() ) );
-    QVERIFY( QDir( m_tmpCollectionDir->name() ).rmpath( QFileInfo( album->tracks().first()->playableUrl().path() ).path() ) );
+    QVERIFY( QDir( m_tmpCollectionDir->name() ).rmdir( QFileInfo( album->tracks().first()->playableUrl().path() ).path() ) );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // this one is still here
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -454,7 +437,6 @@ TestSqlScanManager::testRemoveDir()
     QVERIFY( album );
     QCOMPARE( album->tracks().count(), 0 );
 
-
     // -- remove the second album
     // this time it's a directory inside a directory
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -462,13 +444,12 @@ TestSqlScanManager::testRemoveDir()
     QCOMPARE( album->tracks().count(), 9 );
     foreach( Meta::TrackPtr t, album->tracks() )
         QVERIFY( QFile::remove( t->playableUrl().path() ) );
-    QVERIFY( QDir( m_tmpCollectionDir->name() ).rmpath( QFileInfo( album->tracks().first()->playableUrl().path() ).path() ) );
-    // QVERIFY( QDir( m_tmpCollectionDir->name() ).rmpath( "Pop" ) );
 
-    m_scanManager->requestIncrementalScan();
-    waitScannerFinished();
+    QVERIFY( QDir( m_tmpCollectionDir->name() ).rmdir( QFileInfo( album->tracks().first()->playableUrl().path() ).path() ) );
 
-    // this both are gone
+    incrementalScanAndWait();
+
+    // this time both are gone
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
     QVERIFY( album );
     QCOMPARE( album->tracks().count(), 0 );
@@ -482,8 +463,7 @@ void
 TestSqlScanManager::testUidChangeMoveDirectoryIncrementalScan()
 {
     createAlbum();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     Meta::AlbumPtr album;
     Meta::TrackList tracks;
@@ -510,7 +490,7 @@ TestSqlScanManager::testUidChangeMoveDirectoryIncrementalScan()
 
         KUrl url = track->playableUrl();
         QVERIFY( url.isLocalFile() );
-        Meta::Tag::writeTags( url.path(), uidChange );
+        Meta::Tag::writeTags( url.path(), uidChange, true );
     }
 
     // move album directory
@@ -519,8 +499,7 @@ TestSqlScanManager::testUidChangeMoveDirectoryIncrementalScan()
     QVERIFY( QFile::rename( base, base + "Albums" ) );
 
     // do an incremental scan
-    m_scanManager->requestIncrementalScan();
-    waitScannerFinished();
+    incrementalScanAndWait();
 
     // recheck album
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -544,8 +523,7 @@ TestSqlScanManager::testRemoveTrack()
     QDateTime aDate = QDateTime::currentDateTime();
 
     createAlbum();
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -554,14 +532,13 @@ TestSqlScanManager::testRemoveTrack()
     QVERIFY( !album->isCompilation() );
     track = album->tracks().first(); // the tracks are sorted, so this is always the same track
     QCOMPARE( track->trackNumber(), 1 );
-    QVERIFY( !track->firstPlayed().isValid() );
+    QVERIFY( !track->statistics()->firstPlayed().isValid() );
     static_cast<Meta::SqlTrack*>(track.data())->setFirstPlayed( aDate );
 
     // -- remove one track
     QVERIFY( QFile::remove( track->playableUrl().path() ) );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check that the track is really gone
     QCOMPARE( album->tracks().count(), 8 );
@@ -580,8 +557,7 @@ TestSqlScanManager::testMove()
     Meta::TrackPtr track;
     QDateTime aDate = QDateTime::currentDateTime();
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
@@ -598,13 +574,12 @@ TestSqlScanManager::testMove()
     const QString targetPath = m_tmpCollectionDir->name() + "moved.mp3";
     QVERIFY( QFile::rename( track->playableUrl().path(), targetPath ) );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check that the track is moved
     QVERIFY( createDate == track->createDate() ); // create date should not have changed
     QVERIFY( modifyDate == track->modifyDate() ); // we just changed the track. it should have changed
-    QCOMPARE( track->firstPlayed(), aDate );
+    QCOMPARE( track->statistics()->firstPlayed(), aDate );
     QCOMPARE( track->playableUrl().path(), targetPath );
 
     // --- move a directory
@@ -618,8 +593,7 @@ TestSqlScanManager::testMove()
                             m_tmpCollectionDir->name() + "Top Gun - Soundtrack" ) );
 
     // do an incremental scan
-    m_scanManager->requestIncrementalScan();
-    waitScannerFinished();
+    incrementalScanAndWait();
 
     // check that the track is now moved (but still the old object)
     QCOMPARE( album->tracks().count(), 10 ); // no doublicate tracks
@@ -640,8 +614,7 @@ TestSqlScanManager::testFeat()
     values.insert( Meta::valAlbum, QVariant("The Last Unicorn") );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album;
@@ -672,8 +645,7 @@ TestSqlScanManager::testAlbumImage()
     targetPath = m_tmpCollectionDir->name() + "Various Artists/Big Screen Adventures/28 - Theme From Armageddon.mp3";
     Meta::Tag::setEmbeddedCover( targetPath, QImage( 200, 200, QImage::Format_RGB32 ) );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album;
@@ -709,8 +681,7 @@ TestSqlScanManager::testMerges()
     createTrack( values );
 
     // -- check the commit
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     Meta::TrackPtr track = m_collection->registry()->getTrack( 1 );
     QVERIFY( track );
@@ -726,7 +697,6 @@ TestSqlScanManager::testMerges()
     values.insert( Meta::valFormat, QVariant("1") );
     values.insert( Meta::valTitle, QVariant("Theme From Armageddon") );
     values.insert( Meta::valArtist, QVariant("Soundtrack & Theme Orchestra") );
-    values.insert( Meta::valAlbumArtist, QVariant("Various Artists") );
     values.insert( Meta::valAlbum, QVariant("Big Screen Adventures") );
     values.insert( Meta::valComposer, QVariant("Unknown Composer") );
     values.insert( Meta::valComment, QVariant("Amazon.com Song ID: 210541237") );
@@ -737,8 +707,7 @@ TestSqlScanManager::testMerges()
     values.insert( Meta::valPlaycount, QVariant(5) );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     QCOMPARE( track->name(), QString("Theme From Armageddon") );
@@ -746,7 +715,6 @@ TestSqlScanManager::testMerges()
     QCOMPARE( track->artist()->name(), QString("Soundtrack & Theme Orchestra") );
     QVERIFY( track->album() );
     QCOMPARE( track->album()->name(), QString("Big Screen Adventures") );
-    QVERIFY( track->album()->isCompilation() ); // the track is by various artists
     QCOMPARE( track->composer()->name(), QString("Unknown Composer") );
     QCOMPARE( track->comment(), QString("Amazon.com Song ID: 210541237") );
     QCOMPARE( track->year()->year(), 2009 );
@@ -756,24 +724,23 @@ TestSqlScanManager::testMerges()
     QCOMPARE( track->length(), qint64(12000) );
     QCOMPARE( track->sampleRate(), 44100 );
     QCOMPARE( track->filesize(), 389679 );
-    qFuzzyCompare( track->score(), 0.875 );
-    QCOMPARE( track->playCount(), 5 );
-    QVERIFY( !track->firstPlayed().isValid() );
-    QVERIFY( !track->lastPlayed().isValid() );
+    Meta::StatisticsPtr statistics = track->statistics();
+    qFuzzyCompare( statistics->score(), 0.875 );
+    QCOMPARE( statistics->playCount(), 5 );
+    QVERIFY( !statistics->firstPlayed().isValid() );
+    QVERIFY( !statistics->lastPlayed().isValid() );
     QVERIFY( track->createDate().isValid() );
 
 
     // -- now do an incremental scan
-    createAlbum();
-    m_scanManager->requestIncrementalScan();
-    waitScannerFinished();
+    createAlbum(); // add a new album
+    incrementalScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album;
 
     // the old track is still there
-    // and it's still has an album artist "various artists" so it's still a compilation
-    album = m_collection->registry()->getAlbum( "Big Screen Adventures", QString() );
+    album = m_collection->registry()->getAlbum( "Big Screen Adventures", "Soundtrack & Theme Orchestra" );
     QVERIFY( album );
     QCOMPARE( album->tracks().count(), 1 );
 
@@ -787,6 +754,9 @@ TestSqlScanManager::testMerges()
 void
 TestSqlScanManager::testLargeInsert()
 {
+    if( qgetenv("AMAROK_RUN_LONG_TESTS").isNull() )
+        QSKIP( "takes too long to be run by default;\nDefine AMAROK_RUN_LONG_TESTS "
+               "environment variable to run all tests.", SkipAll );
     // the old large insert test was misleading as the problems with
     // the insertion started upwards of 20000 tracks.
     //
@@ -814,7 +784,7 @@ TestSqlScanManager::testLargeInsert()
     {
         writer.writeStartElement( "directory" );
         writer.writeTextElement( "path", QString::number(dirId) );
-        writer.writeTextElement( "rpath", "/" + QString::number(dirId) );
+        writer.writeTextElement( "rpath", '/' + QString::number(dirId) );
         writer.writeTextElement( "mtime", QString::number(aDate.toTime_t()) );
 
         for( int trackId = 0; trackId < 20; trackId++ )
@@ -887,8 +857,7 @@ TestSqlScanManager::testLargeInsert()
     aDate = QDateTime::currentDateTime();
     // -- feed the scanner in batch mode
     buffer->seek( 0 );
-    m_scanManager->requestImport( buffer );
-    waitScannerFinished();
+    importAndWait( buffer );
 
     qDebug() << "performance test secs:"<< aDate.secsTo( QDateTime::currentDateTime() );
 
@@ -916,8 +885,7 @@ TestSqlScanManager::testLargeInsert()
     // -- feed the scanner in batch mode
     buffer = new QBuffer(&byteArray); // the old scanner deleted the old buffer.
     buffer->open(QIODevice::ReadWrite);
-    m_scanManager->requestImport( buffer );
-    waitScannerFinished();
+    importAndWait( buffer );
 
     qDebug() << "performance test secs:"<< aDate.secsTo( QDateTime::currentDateTime() );
 
@@ -971,8 +939,7 @@ TestSqlScanManager::testIdentifyCompilationInMultipleDirectories()
     values.insert( Meta::valAlbum, QVariant("Top Gun") );
     createTrack( values );
 
-    m_scanManager->requestFullScan();
-    waitScannerFinished();
+    fullScanAndWait();
 
     // -- check the commit
     Meta::AlbumPtr album = m_collection->registry()->getAlbum( "Top Gun", QString() );
@@ -982,18 +949,182 @@ TestSqlScanManager::testIdentifyCompilationInMultipleDirectories()
     QVERIFY( album->isCompilation() );
 }
 
+void
+TestSqlScanManager::testAlbumArtistMerges()
+{
+    // three tracks with the same artist but different album artist.
+    // (one is unset)
+    // Those should end up in different albums.
+
+    Meta::FieldHash values;
+
+    values.insert( Meta::valUniqueId, QVariant("1ef9fede5b3f98deb088b33428b0398e") );
+    values.insert( Meta::valUrl, QVariant("test1/song1.mp3") );
+    values.insert( Meta::valTitle, QVariant("title1") );
+    values.insert( Meta::valArtist, QVariant("artist") );
+    values.insert( Meta::valAlbumArtist, QVariant("albumArtist1") );
+    values.insert( Meta::valAlbum, QVariant("test1") );
+    createTrack( values );
+
+    values.clear();
+    values.insert( Meta::valUniqueId, QVariant("2ef9fede5b3f98deb088b33428b0398b") );
+    values.insert( Meta::valUrl, QVariant("test1/song2.mp3") );
+    values.insert( Meta::valTitle, QVariant("title2") );
+    values.insert( Meta::valArtist, QVariant("artist") );
+    values.insert( Meta::valAlbumArtist, QVariant("albumArtist2") );
+    values.insert( Meta::valAlbum, QVariant("test1") );
+    createTrack( values );
+
+    values.clear();
+    values.insert( Meta::valUniqueId, QVariant("3ef9fede5b3f98deb088b33428b0398c") );
+    values.insert( Meta::valUrl, QVariant("test1/song3.mp3") );
+    values.insert( Meta::valTitle, QVariant("title3") );
+    values.insert( Meta::valArtist, QVariant("artist") );
+    values.insert( Meta::valAlbum, QVariant("test1") );
+    createTrack( values );
+
+    fullScanAndWait();
+
+    // -- check the commit
+    Meta::AlbumPtr album;
+
+    album = m_collection->registry()->getAlbum( "test1", QString() );
+    QVERIFY( album );
+    QCOMPARE( album->name(), QString("test1") );
+    QCOMPARE( album->tracks().count(), 1 );
+    QVERIFY( album->isCompilation() );
+
+    album = m_collection->registry()->getAlbum( "test1", QString("albumArtist1") );
+    QVERIFY( album );
+    QCOMPARE( album->name(), QString("test1") );
+    QCOMPARE( album->tracks().count(), 1 );
+    QVERIFY( !album->isCompilation() );
+
+    album = m_collection->registry()->getAlbum( "test1", QString("albumArtist2") );
+    QVERIFY( album );
+    QCOMPARE( album->name(), QString("test1") );
+    QCOMPARE( album->tracks().count(), 1 );
+    QVERIFY( !album->isCompilation() );
+}
+
+void
+TestSqlScanManager::testCrossRenaming()
+{
+    createAlbum();
+
+    // we use the created and first played attributes for identifying the moved tracks.
+    // currently those are not written back to the track
+
+    Meta::AlbumPtr album;
+    Meta::TrackPtr track;
+
+    fullScanAndWait();
+
+    // -- check the commit
+    album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
+    QVERIFY( album );
+    QCOMPARE( album->tracks().count(), 9 );
+    QVERIFY( !album->isCompilation() );
+
+    // --- cross-rename two track
+    track = album->tracks().at( 0 );
+    static_cast<Meta::SqlTrack*>(track.data())->setRating( 1 );
+    QString path1 = track->playableUrl().path();
+
+    track = album->tracks().at( 1 );
+    static_cast<Meta::SqlTrack*>(track.data())->setRating( 2 );
+    QString path2 = track->playableUrl().path();
+
+    QString targetPath = m_tmpCollectionDir->name() + "moved.mp3";
+    QVERIFY( QFile::rename( path2, targetPath ) );
+    QVERIFY( QFile::rename( path1, path2 ) );
+    QVERIFY( QFile::rename( targetPath, path1 ) );
+
+    fullScanAndWait();
+
+    // -- check that the tracks are moved correctly
+    album = m_collection->registry()->getAlbum( "Thriller", "Michael Jackson" );
+    QVERIFY( album );
+    QCOMPARE( album->tracks().count(), 9 );
+
+    track = album->tracks().at( 0 );
+    QCOMPARE( track->statistics()->rating(), 1 );
+    QCOMPARE( track->playableUrl().path(), path2 );
+
+    track = album->tracks().at( 1 );
+    QCOMPARE( track->statistics()->rating(), 2 );
+    QCOMPARE( track->playableUrl().path(), path1 );
+}
+
+
 void
 TestSqlScanManager::slotCollectionUpdated()
 {
     m_collectionUpdatedCount++;
 }
 
+void
+TestSqlScanManager::fullScanAndWait()
+{
+    QScopedPointer<Capabilities::CollectionScanCapability> csc( m_collection->create<Capabilities::CollectionScanCapability>());
+    if( csc )
+    {
+        csc->startFullScan();
+        waitScannerFinished();
+    }
+}
+
+void
+TestSqlScanManager::incrementalScanAndWait()
+{
+    // incremental scans use the modification time of the file system.
+    // this time is only in seconds, so to be sure that the incremental scan
+    // works we need to wait at least one second.
+    QTest::qWait( 1000 );
+
+    QScopedPointer<Capabilities::CollectionScanCapability> csc( m_collection->create<Capabilities::CollectionScanCapability>());
+    if( csc )
+        csc->startIncrementalScan();
+
+    waitScannerFinished();
+}
+
+void
+TestSqlScanManager::importAndWait( QIODevice* input )
+{
+    QScopedPointer<Capabilities::CollectionImportCapability> csc( m_collection->create<Capabilities::CollectionImportCapability>());
+    if( csc )
+        csc->import( input, 0 );
+
+    waitScannerFinished();
+}
+
 void
 TestSqlScanManager::waitScannerFinished()
 {
     QVERIFY( m_scanManager->isRunning() );
-    QVERIFY2( QTest::kWaitForSignal( m_scanManager, SIGNAL(scanDone(ScannerJob*)), 60*1000 ),
-              "ScanManager didn't finish scan within timeout" );
+    QSignalSpy succeedSpy( m_scanManager, SIGNAL(succeeded()) );
+    QSignalSpy failSpy( m_scanManager, SIGNAL(failed(QString)) );
+
+    // connect the result signal *after* the spies to ensure they are updated first
+    connect( m_scanManager, SIGNAL(succeeded()), this, SIGNAL(scanManagerResult()) );
+    connect( m_scanManager, SIGNAL(failed(QString)), this, SIGNAL(scanManagerResult()));
+    const bool ok = QTest::kWaitForSignal( this, SIGNAL(scanManagerResult()), 60*1000 );
+    disconnect( m_scanManager, SIGNAL(succeeded()), this, SIGNAL(scanManagerResult()) );
+    disconnect( m_scanManager, SIGNAL(failed(QString)), this, SIGNAL(scanManagerResult()) );
+    QVERIFY2( ok, "Scan Manager timed out without a result" );
+
+    if( failSpy.count() > 0 )
+    {
+        QStringList errors;
+        foreach( const QList<QVariant> &arguments, static_cast<QList<QList<QVariant> > >( failSpy ) )
+            errors << arguments.value( 0 ).toString();
+        // this will fire each time:
+        qWarning() << "ScanManager failed with an error:" << errors.join( ", " );
+    }
+    QCOMPARE( qMakePair( succeedSpy.count(), failSpy.count() ), qMakePair( 1, 0 ) );
+
+    QVERIFY( !m_scanManager->isRunning() );
 }
 
 void
@@ -1007,7 +1138,7 @@ TestSqlScanManager::createTrack( const Meta::FieldHash &values )
     QVERIFY( QFile::copy( m_sourcePath, targetPath ) );
 
     // -- set all the values that we need
-    Meta::Tag::writeTags( targetPath, values );
+    Meta::Tag::writeTags( targetPath, values, true );
 }
 
 void
