@@ -16,10 +16,9 @@
 
 #include "TestSqlTrack.h"
 
-#include "core/support/Debug.h"
 #include "DefaultSqlQueryMakerFactory.h"
 #include "core/meta/Meta.h"
-#include "mysqlecollection/MySqlEmbeddedStorage.h"
+#include "core-impl/storage/sql/mysqlestorage/MySqlEmbeddedStorage.h"
 
 #include "SqlCollection.h"
 #include "SqlMeta.h"
@@ -47,8 +46,9 @@ void
 TestSqlTrack::initTestCase()
 {
     m_tmpDir = new KTempDir();
-    m_storage = new MySqlEmbeddedStorage( m_tmpDir->name() );
-    m_collection = new Collections::SqlCollection( "testId", "testtrack", m_storage );
+    m_storage = new MySqlEmbeddedStorage();
+    QVERIFY( m_storage->init( m_tmpDir->name() ) );
+    m_collection = new Collections::SqlCollection( m_storage );
     m_collection->setMountPointManager( new SqlMountPointManagerMock( this, m_storage ) );
 
     // I just need the table and not the whole playlist manager
@@ -98,10 +98,12 @@ TestSqlTrack::init()
     m_storage->query( "INSERT INTO years(id, name) VALUES (2, '2');" );
     m_storage->query( "INSERT INTO years(id, name) VALUES (3, '3');" );
 
-    m_storage->query( "INSERT INTO urls(id,deviceid,rpath,uniqueid) VALUES (1, -1, './IDoNotExist.mp3','1');" );
-    m_storage->query( "INSERT INTO urls(id,deviceid,rpath,uniqueid) VALUES (2, -1, './IDoNotExistAsWell.mp3','2');" );
-    m_storage->query( "INSERT INTO urls(id,deviceid,rpath,uniqueid) VALUES (3, -1, './MeNeither.mp3','3');" );
-    m_storage->query( "INSERT INTO urls(id,deviceid,rpath,uniqueid) VALUES (4, -1, './NothingHere.mp3','4');" );
+    m_storage->query( "INSERT INTO directories(id, deviceid, dir) VALUES (1, -1, './');" );
+
+    m_storage->query( "INSERT INTO urls(id, deviceid, rpath, directory, uniqueid) VALUES (1, -1, './IDoNotExist.mp3', 1, '1');" );
+    m_storage->query( "INSERT INTO urls(id, deviceid, rpath, directory, uniqueid) VALUES (2, -1, './IDoNotExistAsWell.mp3', 1, '2');" );
+    m_storage->query( "INSERT INTO urls(id, deviceid, rpath, directory, uniqueid) VALUES (3, -1, './MeNeither.mp3', 1, '3');" );
+    m_storage->query( "INSERT INTO urls(id, deviceid, rpath, directory, uniqueid) VALUES (4, -1, './NothingHere.mp3', 1, '4');" );
 
     m_storage->query( "INSERT INTO tracks(id,url,title,comment,artist,album,genre,year,composer) "
                       "VALUES(1,1,'track1','comment1',1,1,1,1,1);" );
@@ -121,6 +123,7 @@ TestSqlTrack::cleanup()
     m_storage->query( "TRUNCATE TABLE artists;" );
     m_storage->query( "TRUNCATE TABLE tracks;" );
     m_storage->query( "TRUNCATE TABLE urls;" );
+    m_storage->query( "TRUNCATE TABLE directories;" );
     m_storage->query( "TRUNCATE TABLE statistics;" );
     m_storage->query( "TRUNCATE TABLE labels;" );
     m_storage->query( "TRUNCATE TABLE urls_labels;" );
@@ -268,7 +271,7 @@ TestSqlTrack::testSetAllValuesSingleNotExisting()
 {
     {
         // get a new track
-        Meta::TrackPtr track1 = m_collection->registry()->getTrack( -1, "./IamANewTrack.mp3", 0, "mb-1e34fb213489" );
+        Meta::TrackPtr track1 = m_collection->registry()->getTrack( -1, "./IamANewTrack.mp3", 1, "1e34fb213489" );
 
         QSignalSpy spy( m_collection, SIGNAL(updated()));
         MetaNotificationSpy metaSpy;
@@ -374,12 +377,12 @@ TestSqlTrack::testSetAllValuesBatch()
         MetaNotificationSpy metaSpy;
         metaSpy.subscribeTo( track1 );
 
-        sqlTrack1->beginMetaDataUpdate();
+        sqlTrack1->beginUpdate();
 
         setAllValues( sqlTrack1 );
         QCOMPARE( metaSpy.notificationsFromTracks().count(), 1 ); // add label does one notify
 
-        sqlTrack1->endMetaDataUpdate();
+        sqlTrack1->endUpdate();
         QCOMPARE( metaSpy.notificationsFromTracks().count(), 2 ); // only one notificate for all the changes
 
         getAllValues( sqlTrack1 );
@@ -483,9 +486,9 @@ TestSqlTrack::testAlbumRemaingsNonCompilationAfterChangingAlbumName()
     Meta::SqlTrack *sqlTrack1 = static_cast<Meta::SqlTrack*>( track1.data() );
     sqlTrack1->setAlbum( "album2" );
     Meta::SqlTrack *sqlTrack2 = static_cast<Meta::SqlTrack*>( track2.data() );
-    sqlTrack2->beginMetaDataUpdate();
+    sqlTrack2->beginUpdate();
     sqlTrack2->setAlbum( "album2" );
-    sqlTrack2->endMetaDataUpdate();
+    sqlTrack2->endUpdate();
 
     QCOMPARE( track1->album()->name(), QString( "album2" ) );
     QVERIFY( track1->album()->hasAlbumArtist() );
@@ -514,9 +517,9 @@ TestSqlTrack::testAlbumRemainsCompilationAfterChangingAlbumName()
     Meta::SqlTrack *sqlTrack1 = static_cast<Meta::SqlTrack*>( track1.data() );
     Meta::SqlTrack *sqlTrack2 = static_cast<Meta::SqlTrack*>( track2.data() );
     sqlTrack1->setAlbum( "album2" );
-    sqlTrack2->beginMetaDataUpdate();
+    sqlTrack2->beginUpdate();
     sqlTrack2->setAlbum( "album2" );
-    sqlTrack2->endMetaDataUpdate();
+    sqlTrack2->endUpdate();
 
     QCOMPARE( track1->album()->name(), QString( "album2" ) );
     QVERIFY( track1->album()->isCompilation() );
