@@ -21,6 +21,8 @@
 #include "SafeFileSaver.h"
 
 //Taglib
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wnon-virtual-dtor"
 #include <apetag.h>
 #include <fileref.h>
 #include <flacfile.h>
@@ -35,6 +37,7 @@
 #include <uniquefileidentifierframe.h>
 #include <vorbisfile.h>
 #include <xiphcomment.h>
+#pragma GCC diagnostic pop
 
 #include <QtDebug>
 #include <QCryptographicHash>
@@ -82,7 +85,7 @@ AFTTagger::AFTTagger( int &argc, char **argv )
                 "This program makes use of multiple libraries not written by the author, and as such neither\n"
                 "the author nor the Amarok project can or do take any responsibility for any damage that may\n"
                 "occur to your files through the use of this program.\n\n"
-                "If you want more information, please see http://amarok.kde.org/wiki/AFT\n\n"
+                "If you want more information, please see http://community.kde.org/Amarok/Development/AFT\n\n"
                 "If you agree to be bound by these terms of use, enter 'y' or 'Y', or anything else to exit:\n" ) );
 
         m_textStream.flush();
@@ -613,8 +616,8 @@ AFTTagger::handleMP4( TagLib::MP4::File *file )
         for( TagLib::MP4::ItemListMap::Iterator it = itemsMap.begin(); it != itemsMap.end(); ++it )
         {
             TagLib::String key = it->first;
-            TagLib::String ukey = key.upper();
-            if( ukey.find( "AMAROK - REDISCOVER YOUR MUSIC" ) != -1 )
+            const QString qkey = TStringToQString( key ).toUpper();
+            if( qkey.contains( "AMAROK - REDISCOVER YOUR MUSIC" ) )
             {
                 nothingfound = false;
 
@@ -625,7 +628,7 @@ AFTTagger::handleMP4( TagLib::MP4::File *file )
                 if( !m_delete )
                     newUid = true;
             }
-            else if( ukey.find( "AMAROK 2 AFT" ) != -1 )
+            else if( qkey.contains( "AMAROK 2 AFT" ) )
             {
                 nothingfound = false;
 
@@ -640,13 +643,15 @@ AFTTagger::handleMP4( TagLib::MP4::File *file )
                 }
                 else
                 {
-                    int version = TStringToQString( ukey ).at( 13 ).digitValue();
+                    int version = qkey.at( qkey.indexOf( "AMAROK 2 AFT" ) + 13 ).digitValue();
                     if( m_verbose )
                         m_textStream << tr( "INFO: AFT identifier is version %1" ).arg( version ) << endl;
                     if( version < s_currentVersion )
                     {
                         if( m_verbose )
-                            m_textStream << tr( "INFO: Upgrading AFT identifier from version %1 to version %2" ).arg( version, s_currentVersion ) << endl;
+                            m_textStream << tr( "INFO: Upgrading AFT identifier from version %1 to version %2" )
+                            .arg( QString::number( version ), QString::number( s_currentVersion ) ) 
+                            << endl;
                         uid = upgradeUID( version, TStringToQString( itemsMap[ key ].toStringList().toString() ) );
                         if( m_verbose )
                             m_textStream << tr( "INFO: Removing current AFT frame" ) << endl;
